@@ -36,6 +36,7 @@
 #include "kategrepdialog.h"
 #include "katemailfilesdialog.h"
 #include "katemainwindowiface.h"
+#include "kateexternaltools.h"
 
 #include <kmdichildview.h>
 #include <dcopclient.h>
@@ -69,10 +70,12 @@
 #include <kurldrag.h>
 #include <kdesktopfile.h>
 #include <khelpmenu.h>
+#include <kmultitabbar.h>
 #include <ktip.h>
 #include <kmenubar.h>
 
 #include <qlayout.h>
+#include <qptrvector.h>
 
 #include <assert.h>
 #include <unistd.h>
@@ -85,6 +88,7 @@ KateMainWindow::KateMainWindow(KateDocManager *_m_docManager, KatePluginManager
 	KateProjectManager *projectMan, KMdi::MdiMode guiMode) :
 	KMdiMainFrm (0,(QString("__KateMainWindow#%1").arg(uniqueID)).latin1(),guiMode)
 {
+  setToolviewStyle(KMultiTabBar::KDEV3ICON);
   // first the very important id
   myID = uniqueID;
   uniqueID++;
@@ -155,6 +159,12 @@ KateMainWindow::KateMainWindow(KateDocManager *_m_docManager, KatePluginManager
     console->loadConsoleIfNeeded();
 
   setAcceptDrops(true);
+
+  // activate the first restored project, if any
+  if (m_projectManager->projects() > 0)
+    activateProject(m_projectManager->project(0));
+  else
+    activateProject(0);
 }
 
 KateMainWindow::~KateMainWindow()
@@ -256,6 +266,9 @@ void KateMainWindow::setupActions()
   a=closeCurrentViewSpace = new KAction( i18n("Close &Current View"), "view_remove", CTRL+SHIFT+Key_R, m_viewManager, SLOT( slotCloseCurrentViewSpace() ), actionCollection(), "view_close_current_space");
   a->setWhatsThis(i18n("Close the currently active splitted view"));
 
+  externalTools = new KateExternalToolsMenuAction( i18n("External Tools"), actionCollection(), "tools_external", this );
+  externalTools->setWhatsThis( i18n("Launch external helper applications") );
+
   showFullScreenAction = KStdAction::fullScreen( 0, 0, actionCollection(),this);
   connect( showFullScreenAction,SIGNAL(toggled(bool)), this,SLOT(slotFullScreen(bool)));
 
@@ -291,6 +304,10 @@ void KateMainWindow::setupActions()
   settingsConfigure = KStdAction::preferences(this, SLOT(slotConfigure()), actionCollection(), "settings_configure");
   settingsConfigure->setWhatsThis(i18n("Configure various aspects of this application and the editing component."));
 
+  // pipe to terminal action
+  if (kapp->authorize("shell_access"))
+    new KAction(i18n("&Pipe to Console"), "pipe", 0, this, SLOT(slotPipeToConsole()), actionCollection(), "tools_pipe_to_terminal");
+
   // tip of the day :-)
   KStdAction::tipOfDay( this, SLOT( tipOfTheDay() ), actionCollection() )->setWhatsThis(i18n("This shows useful tips on the use of this application."));
 
@@ -301,6 +318,7 @@ void KateMainWindow::setupActions()
   }
 
   connect(m_viewManager,SIGNAL(viewChanged()),this,SLOT(slotWindowActivated()));
+  connect(m_viewManager,SIGNAL(viewChanged()),this,SLOT(slotUpdateOpenWith()));
   connect(m_docManager,SIGNAL(documentChanged()),this,SLOT(slotDocumentChanged()));
 
   slotWindowActivated ();
@@ -332,7 +350,7 @@ bool KateMainWindow::queryClose()
   if ( m_projectManager->queryCloseAll () &&
        m_docManager->queryCloseDocuments (this) )
   {
-    KConfig scfg ("katesessionrc", false);
+    KConfig scfg("katesessionrc", false);
 
     KConfig *config = kapp->config();
     config->setGroup("General");
@@ -376,6 +394,8 @@ void KateMainWindow::readOptions(KConfig *config)
   config->setGroup("General");
   syncKonsole =  config->readBoolEntry("Sync Konsole", true);
   modNotification = config->readBoolEntry("Modified Notification", false);
+  m_docManager->setSaveMetaInfos(config->readBoolEntry("Save Meta Infos", true));
+  m_docManager->setDaysMetaInfos(config->readNumEntry("Days Meta Infos", 30));
 
   m_viewManager->setShowFullPath(config->readBoolEntry("Show Full Path in Title", false));
 
@@ -400,6 +420,10 @@ void KateMainWindow::saveOptions(KConfig *config)
   else
     config->writeEntry("Show Console", false);
 
+  config->writeEntry("Save Meta Infos", m_docManager->getSaveMetaInfos());
+
+  config->writeEntry("Days Meta Infos", m_docManager->getDaysMetaInfos());
+
   config->writeEntry("Show Full Path in Title", m_viewManager->getShowFullPath());
 
   config->writeEntry("Sync Konsole", syncKonsole);
@@ -433,8 +457,6 @@ void KateMainWindow::slotWindowActivated ()
 
   if (m_viewManager->activeView())
   {
-    documentOpenWith->setEnabled (!m_viewManager->activeView()->document()->url().isEmpty());
-
     if (console && syncKonsole)
     {
       QString newPath = m_viewManager->activeView()->getDoc()->url().directory();
@@ -448,10 +470,6 @@ void KateMainWindow::slotWindowActivated ()
 
     updateCaption (m_viewManager->activeView()->getDoc());
   }
-  else
-  {
-    documentOpenWith->setEnabled (false);
-  }
 
   if (m_viewManager->viewSpaceCount() == 1)
     closeCurrentViewSpace->setEnabled(false);
@@ -459,6 +477,14 @@ void KateMainWindow::slotWindowActivated ()
     closeCurrentViewSpace->setEnabled(true);
 }
 
+void KateMainWindow::slotUpdateOpenWith()
+{
+  if (m_viewManager->activeView())
+    documentOpenWith->setEnabled(!m_viewManager->activeView()->document()->url().isEmpty());
+  else
+    documentOpenWith->setEnabled(false);
+}
+
 void KateMainWindow::documentMenuAboutToShow()
 {
   documentMenu->clear ();
@@ -518,6 +544,8 @@ void KateMainWindow::editKeys()
   for( QPtrListIterator<KXMLGUIClient> it( clients ); it.current(); ++it )
     dlg.insert ( (*it)->actionCollection(), (*it)->instance()->aboutData()->programName() );
 
+  dlg.insert( externalTools->actionCollection(), i18n("External Tools") );
+
   dlg.configure();
 
   QPtrList<Kate::Document>  l=m_docManager->documentList();
@@ -532,6 +560,8 @@ void KateMainWindow::editKeys()
 	}
 
   }
+
+  externalTools->actionCollection()->writeShortcutSettings( "Shortcuts", new KConfig("externaltools", false, false, "appdata") );
 }
 
 void KateMainWindow::openURL (const QString &name)
@@ -576,11 +606,22 @@ KURL KateMainWindow::activeDocumentUrl()
   return KURL();
 }
 
-void KateMainWindow::fileSelected(const KFileItem *file)
+void KateMainWindow::fileSelected(const KFileItem * /*file*/)
 {
-  m_viewManager->openURL( file->url() );
+  /*if (file) {
+  	m_viewManager->openURL( file->url() );
+ } else */{
+ 	const KFileItemList *list=fileselector->dirOperator()->selectedItems();
+	KFileItem *tmp;
+	for (KFileItemListIterator it(*list); (tmp = it.current()); ++it) {
+		m_viewManager->openURL(tmp->url());
+		fileselector->dirOperator()->view()->setSelected(tmp,false);
+	}
+ }
+ 	//fileSelector->dirOperator()->
 }
 
+// TODO make this work
 void KateMainWindow::mSlotFixOpenWithMenu()
 {
   //kdDebug(13001)<<"13000"<<"fixing open with menu"<<endl;
@@ -730,6 +771,37 @@ void KateMainWindow::slotFullScreen(bool t)
 
 bool KateMainWindow::eventFilter( QObject *o, QEvent *e )
 {
+  if ( e->type() == QEvent::WindowActivate && o == this ) {
+    Kate::Document *doc;
+    typedef QPtrVector<Kate::Document> docvector;
+    docvector list( m_docManager->documents() );
+    uint cnt = 0;
+    for( doc = m_docManager->firstDocument(); doc; doc = m_docManager->nextDocument() )
+    {
+      if ( m_docManager->documentInfo( doc )->modifiedOnDisc )
+      {
+        list.insert( cnt, doc );
+        cnt++;
+      }
+    }
+
+    if ( cnt )
+    {
+      list.resize( cnt );
+
+      // TODO
+      // display a dialog with a list of modified documents,
+      // and options to reload/disguard all, or handle individually
+      for ( uint i=0; i < cnt; i++ )
+      {
+        Kate::DocumentExt *ext = Kate::documentExt (list.at( i ));
+
+        if (ext)
+          ext->slotModifiedOnDisk( activeView );
+      }
+    }
+  }
+
   if ( o == greptool && e->type() == QEvent::Show && activeView )
   {
     if ( activeView->getDoc()->url().isLocalFile() )
@@ -825,6 +897,9 @@ void KateMainWindow::activateProject (Kate::Project *project)
   else
     m_projectNumber = 0;
 
+  saveProject->setEnabled(project != 0);
+  closeProject->setEnabled(project != 0);
+
   emit m_mainWindow->projectChanged ();
 }
 
@@ -866,6 +941,7 @@ void KateMainWindow::slotDocumentCreated (Kate::Document *doc)
 {
   connect(doc,SIGNAL(modStateChanged(Kate::Document *)),this,SLOT(updateCaption(Kate::Document *)));
   connect(doc,SIGNAL(nameChanged(Kate::Document *)),this,SLOT(updateCaption(Kate::Document *)));
+  connect(doc,SIGNAL(nameChanged(Kate::Document *)),this,SLOT(slotUpdateOpenWith()));
 
   updateCaption (doc);
 }
@@ -911,9 +987,9 @@ void KateMainWindow::openConstURLProject (const KURL&url)
 }
 
 void KateMainWindow::saveProperties(KConfig *config) {
-	kdDebug(13000)<<"KateMainWindow::saveProperties()********************************************"<<endl
-		      <<config->group()<<endl
-		      <<"****************************************************************************"<<endl;
+  kdDebug(13000)<<"KateMainWindow::saveProperties()**********************"<<endl
+  <<config->group()<<endl
+  <<"******************************************************"<<endl;
   assert(config);
 
   kdDebug(13000)<<"preparing session saving"<<endl;
@@ -964,3 +1040,19 @@ void KateMainWindow::saveGlobalProperties( KConfig* sessionConfig )
   m_projectManager->saveProjectList (sessionConfig);
   m_docManager->saveDocumentList (sessionConfig);
 }
+
+void KateMainWindow::slotPipeToConsole ()
+{
+  if (!console)
+    return;
+
+  Kate::View *v = m_viewManager->activeView();
+
+  if (!v)
+    return;
+
+  if (v->getDoc()->hasSelection ())
+    console->sendInput (v->getDoc()->selection());
+  else
+    console->sendInput (v->getDoc()->text());
+}
