@@ -65,7 +65,7 @@ KateViewManager::KateViewManager (QWidget *parent, KateDocManager *m_docManager,
   m_viewManager = new Kate::ViewManager (this);
 
   m_blockViewCreationAndActivation=false;
-  
+
   useOpaqueResize = KGlobalSettings::opaqueResize();
 
   // no memleaks
@@ -83,6 +83,7 @@ KateViewManager::KateViewManager (QWidget *parent, KateDocManager *m_docManager,
   m_grid->addWidget( vs, 0, 0);
   m_viewSpaceList.append(vs);
   connect( this, SIGNAL(viewChanged()), this, SLOT(slotViewChanged()) );
+  connect(m_docManager, SIGNAL(initialDocumentReplaced()), this, SIGNAL(viewChanged()));
 }
 
 KateViewManager::~KateViewManager ()
@@ -110,6 +111,7 @@ bool KateViewManager::createView ( Kate::Document *doc )
   // popup menu
   view->installPopup ((QPopupMenu*)(m_mainWindow->factory()->container("ktexteditor_popup", m_mainWindow)) );
 
+  connect(view->getDoc(),SIGNAL(nameChanged(Kate::Document *)),this,SLOT(statusMsg()));
   connect(view,SIGNAL(cursorPositionChanged()),this,SLOT(statusMsg()));
   connect(view,SIGNAL(newStatus()),this,SLOT(statusMsg()));
   connect(view->getDoc(), SIGNAL(undoChanged()), this, SLOT(statusMsg()));
@@ -449,7 +451,7 @@ void KateViewManager::slotDocumentOpen ()
   uint lastID = 0;
   for (KURL::List::Iterator i=r.URLs.begin(); i != r.URLs.end(); ++i)
     lastID = openURL( *i, r.encoding, false );
-  
+
   if (lastID > 0)
     activateView (lastID);
 }
@@ -457,15 +459,30 @@ void KateViewManager::slotDocumentOpen ()
 void KateViewManager::slotDocumentSaveAll()
 {
   for( QPtrListIterator<Kate::View> it( m_viewList ); it.current(); ++it )
-    it.current()->save();
+    if ( it.current()->getDoc()->isModified() )
+       it.current()->save();
 }
 
 void KateViewManager::slotDocumentClose ()
 {
+  // no active view, do nothing
   if (!activeView()) return;
 
+  // prevent close document if only one view alive and the document of
+  // it is not modified and empty !!!
+  if ( (m_viewList.count() == 1)
+       && !activeView()->getDoc()->isModified()
+       && activeView()->getDoc()->url().isEmpty()
+       && (activeView()->getDoc()->length() == 0) )
+  {
+    activeView()->getDoc()->closeURL();
+    return;
+  }
+
+  // close document
   m_docManager->closeDocument (activeView()->getDoc());
 
+  // create new one, if none alive
   openNewIfEmpty();
 }
 
@@ -492,7 +509,7 @@ uint KateViewManager::openURL (const KURL &url, const QString& encoding, bool ac
 
   if (activate)
     activateView( id );
-    
+
   return id;
 }
 
