@@ -6,13 +6,14 @@
 #include <qcolor.h>
 
 
-#include <kuniqueapp.h>
+#include <kuniqueapplication.h>
 #include <kwinmodule.h>
 
 
 #include <X11/Xlib.h>
 #define explicit int_explicit        // avoid compiler name clash in XKBlib.h
 #include <X11/XKBlib.h>
+#undef explicit
 
 
 class KAccessApp : public KUniqueApplication
@@ -25,7 +26,7 @@ public:
 
   bool x11EventFilter(XEvent *event);
   
-  int newInstance() { readSettings(); return 0; };
+  int newInstance();
 
 
 protected:
@@ -68,7 +69,7 @@ class VisualBell : public QWidget
 public:
 
   VisualBell(int pause) 
-    : QWidget(0, 0, WStyle_NoBorder|WStyle_Tool|WStyle_Customize), _pause(pause)
+    : QWidget(0, 0, WX11BypassWM), _pause(pause)
     {};
 
   
