@@ -31,12 +31,14 @@
 #include <qradiobutton.h>
 #include <qslider.h>
 #include <qtimer.h>
+#include <qtooltip.h>
 #include <qwhatsthis.h>
 #include <qapplication.h>
 
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kfiledialog.h>
+#include <kfilemetainfo.h>
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <kimageio.h>
@@ -71,8 +73,6 @@ BGDialog::BGDialog(QWidget* parent, KConfig* _config, bool _multidesktop)
    {
       m_pDesktopLabel->hide();
       m_comboDesktop->hide();
-      m_buttonSetupWallpapers->hide();
-      m_radioSlideShow->hide();
    }
 
    // preview monitor
@@ -87,15 +87,18 @@ BGDialog::BGDialog(QWidget* parent, KConfig* _config, bool _multidesktop)
            SLOT(slotSelectDesk(int)));
 
    // background image settings
-   m_urlWallpaper->setFilter(KImageIO::pattern());
-   m_urlWallpaper->comboBox()->setSizePolicy(
-      QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ) );
+   QIconSet iconSet = SmallIconSet(QString::fromLatin1("fileopen"));
+   QPixmap pixMap = iconSet.pixmap( QIconSet::Small, QIconSet::Normal );
+   m_urlWallpaperButton->setIconSet( iconSet );
+   m_urlWallpaperButton->setFixedSize( pixMap.width()+8, pixMap.height()+8 );
+   QToolTip::add(m_urlWallpaperButton, i18n("Open file dialog"));
+
    connect(m_buttonGroupBackground, SIGNAL(clicked(int)),
            SLOT(slotWallpaperTypeChanged(int)));
-   connect(m_urlWallpaper->comboBox(), SIGNAL(activated(int)),
+   connect(m_urlWallpaperBox, SIGNAL(activated(int)),
            SLOT(slotWallpaper(int)));
-   connect(m_urlWallpaper, SIGNAL(urlSelected(const QString &)),
-           SLOT(slotImageDropped(const QString &)));
+   connect(m_urlWallpaperButton, SIGNAL(clicked()),
+           SLOT(slotWallpaperSelection()));
    connect(m_comboWallpaperPos, SIGNAL(activated(int)),
            SLOT(slotWallpaperPos(int)));
    connect(m_buttonSetupWallpapers, SIGNAL(clicked()),
@@ -149,7 +152,7 @@ BGDialog::BGDialog(QWidget* parent, KConfig* _config, bool _multidesktop)
 
    if (KGlobal::dirs()->isRestrictedResource("wallpaper"))
    {
-      m_urlWallpaper->button()->hide();
+      m_urlWallpaperButton->hide();
       m_buttonSetupWallpapers->hide();
       m_radioSlideShow->hide();
    }
@@ -176,10 +179,11 @@ void BGDialog::makeReadOnly()
     m_comboPattern->setEnabled( false );
     m_radioNoPicture->setEnabled( false );
     m_radioPicture->setEnabled( false );
-    m_urlWallpaper->setEnabled( false );
+    m_radioSlideShow->setEnabled( false );
+    m_urlWallpaperBox->setEnabled( false );
+    m_urlWallpaperButton->setEnabled( false );
     m_comboWallpaperPos->setEnabled( false );
     m_buttonSetupWallpapers->setEnabled( false );
-    m_buttonAdvanced->setEnabled( false );
     m_comboBlend->setEnabled( false );
     m_sliderBlend->setEnabled( false );
     m_cbBlendReverse->setEnabled( false );
@@ -308,7 +312,7 @@ void BGDialog::initUI()
    m_Patterns = KBackgroundPattern::list();
    m_Patterns.sort(); // Defined order
    QStringList::Iterator it;
-   for (it=m_Patterns.begin(); it != m_Patterns.end(); it++)
+   for (it=m_Patterns.begin(); it != m_Patterns.end(); ++it)
    {
       KBackgroundPattern pat(*it);
       if (pat.isAvailable())
@@ -322,28 +326,44 @@ void BGDialog::initUI()
    QStringList lst = m_pDirs->findAllResources("wallpaper", "*", false, true);
    for (QStringList::ConstIterator it = lst.begin(); it != lst.end(); ++it)
    {
-      int slash = (*it).findRev('/') + 1;
-      int endDot = (*it).findRev('.');
+      // First try to see if we have a comment describing the image.  If we do
+      // just use the first line of said comment.
 
-      QString s;
-      // strip the extension if it exists
-      if (endDot != -1 && endDot > slash)
-         s = (*it).mid(slash, endDot - slash);
-      else
-         s = (*it).mid(slash);
+      KFileMetaInfo metaInfo(*it);
+      QString imageCaption;
+
+      if (metaInfo.isValid() && metaInfo.item("Comment").isValid())
+         imageCaption = metaInfo.item("Comment").string().section('\n', 0, 0);
+
+      if (imageCaption.isEmpty())
+      {
+         int slash = (*it).findRev('/') + 1;
+         int endDot = (*it).findRev('.');
+
+         // strip the extension if it exists
+         if (endDot != -1 && endDot > slash)
+            imageCaption = (*it).mid(slash, endDot - slash);
+         else
+            imageCaption = (*it).mid(slash);
+
+         imageCaption.replace('_', ' ');
+         imageCaption = KStringHandler::capwords(imageCaption);
+      }
+      
+      imageCaption = KStringHandler::rEmSqueeze(imageCaption, m_urlWallpaperBox->fontMetrics(), 11);
 
       // avoid name collisions
-      QString rs = s;
+      QString rs = imageCaption;
       QString lrs = rs.lower();
       for (int n = 1; papers.find(lrs) != papers.end(); ++n)
       {
-         rs = s + " (" + QString::number(n) + ')';
+         rs = imageCaption + " (" + QString::number(n) + ')';
          lrs = rs.lower();
       }
       papers[lrs] = qMakePair(rs, *it);
    }
 
-   KComboBox *comboWallpaper = m_urlWallpaper->comboBox();
+   KComboBox *comboWallpaper = m_urlWallpaperBox;
    int i = 0;
    for (QMap<QString, QPair<QString, QString> >::Iterator it = papers.begin();
         it != papers.end();
@@ -365,6 +385,7 @@ void BGDialog::initUI()
 
    // Blend modes: make sure these match with kdesktop/bgrender.cc !!
    m_comboBlend->insertItem(i18n("No Blending"));
+   m_comboBlend->insertItem(i18n("Flat"));
    m_comboBlend->insertItem(i18n("Horizontal"));
    m_comboBlend->insertItem(i18n("Vertical"));
    m_comboBlend->insertItem(i18n("Pyramid"));
@@ -378,7 +399,7 @@ void BGDialog::initUI()
 
 void BGDialog::setWallpaper(const QString &s)
 {
-   KComboBox *comboWallpaper = m_urlWallpaper->comboBox();
+   KComboBox *comboWallpaper = m_urlWallpaperBox;
    comboWallpaper->blockSignals(true);
 
    if (m_Wallpaper.find(s) == m_Wallpaper.end())
@@ -397,10 +418,47 @@ void BGDialog::setWallpaper(const QString &s)
    {
       comboWallpaper->setCurrentItem(m_Wallpaper[s]);
    }
-   m_urlWallpaper->fileDialog()->setURL(s);
    comboWallpaper->blockSignals(false);
 }
 
+void BGDialog::slotWallpaperSelection()
+{
+   KFileDialog dlg( QString::null, QString::null, this,
+                    "file dialog", true );
+
+   dlg.setFilter(KImageIO::pattern());
+   dlg.setMode( KFile::File | KFile::ExistingOnly | KFile::LocalOnly );
+   dlg.setCaption( i18n("Select Wallpaper") );
+
+   int j = m_urlWallpaperBox->currentItem();
+   QString uri;
+   for(QMap<QString,int>::ConstIterator it = m_Wallpaper.begin();
+       it != m_Wallpaper.end();
+       ++it)
+   {
+      if (it.data() == j)
+      {
+         uri = it.key();
+         break;
+      }
+   }
+
+   if ( !uri.isEmpty() ) {
+      dlg.setSelection( uri );
+   }
+
+   if ( dlg.exec() == QDialog::Accepted )
+   {
+      setWallpaper(dlg.selectedFile());
+
+      int optionID = m_buttonGroupBackground->id(m_radioPicture);
+      m_buttonGroupBackground->setButton( optionID );
+      slotWallpaperTypeChanged( optionID );
+
+      emit changed(true);
+   }
+}
+
 void BGDialog::updateUI()
 {
    KBackgroundRenderer *r = m_Renderer[m_eDesk];
@@ -423,18 +481,20 @@ void BGDialog::updateUI()
       // No wallper
       if (wallpaperMode == KBackgroundSettings::NoWallpaper )
       {
-         m_urlWallpaper->setEnabled(false);
+         m_urlWallpaperBox->setEnabled(false);
+         m_urlWallpaperButton->setEnabled(false);
          m_buttonSetupWallpapers->setEnabled(false);
          m_comboWallpaperPos->setEnabled(false);
          m_lblWallpaperPos->setEnabled(false);
          m_buttonGroupBackground->setButton(
             m_buttonGroupBackground->id(m_radioNoPicture) );
       }
-      
+
       // 1 Picture
       else
       {
-         m_urlWallpaper->setEnabled(true);
+         m_urlWallpaperBox->setEnabled(true);
+         m_urlWallpaperButton->setEnabled(true);
          m_buttonSetupWallpapers->setEnabled(false);
          m_comboWallpaperPos->setEnabled(true);
          m_lblWallpaperPos->setEnabled(true);
@@ -443,11 +503,12 @@ void BGDialog::updateUI()
             m_buttonGroupBackground->id(m_radioPicture) );
       }
    }
-   
+
    // Slide show
    else
    {
-      m_urlWallpaper->setEnabled(false);
+      m_urlWallpaperBox->setEnabled(false);
+      m_urlWallpaperButton->setEnabled(false);
       m_buttonSetupWallpapers->setEnabled(true);
       m_comboWallpaperPos->setEnabled(true);
       m_lblWallpaperPos->setEnabled(true);
@@ -520,6 +581,9 @@ void BGDialog::slotPreviewDone(int desk_done)
 
    KBackgroundRenderer *r = m_Renderer[m_eDesk];
 
+   if (!r->image())
+       return;
+
    KPixmap pm;
    if (QPixmap::defaultDepth() < 15)
       pm.convertFromImage(*r->image(), KPixmap::LowColor);
@@ -536,9 +600,6 @@ void BGDialog::slotImageDropped(const QString &uri)
    int optionID = m_buttonGroupBackground->id(m_radioPicture);
    m_buttonGroupBackground->setButton( optionID );
    slotWallpaperTypeChanged( optionID );
-
-   m_copyAllDesktops = true;
-   emit changed(true);
 }
 
 void BGDialog::slotWallpaperTypeChanged(int i)
@@ -549,7 +610,8 @@ void BGDialog::slotWallpaperTypeChanged(int i)
    // No picture
    if (i == m_buttonGroupBackground->id(m_radioNoPicture))  //None
    {
-      m_urlWallpaper->setEnabled(false);
+      m_urlWallpaperBox->setEnabled(false);
+      m_urlWallpaperButton->setEnabled(false);
       m_buttonSetupWallpapers->setEnabled(false);
       m_comboWallpaperPos->setEnabled(false);
       m_lblWallpaperPos->setEnabled(false);
@@ -566,7 +628,8 @@ void BGDialog::slotWallpaperTypeChanged(int i)
    // Slide show
    else if (i == m_buttonGroupBackground->id(m_radioSlideShow))
    {
-      m_urlWallpaper->setEnabled(false);
+      m_urlWallpaperBox->setEnabled(false);
+      m_urlWallpaperButton->setEnabled(false);
       m_buttonSetupWallpapers->setEnabled(true);
       m_comboWallpaperPos->setEnabled(true);
       m_lblWallpaperPos->setEnabled(true);
@@ -589,7 +652,8 @@ void BGDialog::slotWallpaperTypeChanged(int i)
    // 1 Picture
    else if (i == m_buttonGroupBackground->id(m_radioPicture))
    {
-      m_urlWallpaper->setEnabled(true);
+      m_urlWallpaperBox->setEnabled(true);
+      m_urlWallpaperButton->setEnabled(true);
       m_buttonSetupWallpapers->setEnabled(false);
       m_lblWallpaperPos->setEnabled(true);
       m_comboWallpaperPos->setEnabled(true);
@@ -600,12 +664,7 @@ void BGDialog::slotWallpaperTypeChanged(int i)
       else
          r->setMultiWallpaperMode(KBackgroundSettings::NoMultiRandom);
 
-      r->setWallpaperMode(m_wallpaperPos);
-      m_comboWallpaperPos->blockSignals(true);
-      m_comboWallpaperPos->setCurrentItem(m_wallpaperPos-1);
-      m_comboWallpaperPos->blockSignals(false);
-
-      int j = m_urlWallpaper->comboBox()->currentItem();
+      int j = m_urlWallpaperBox->currentItem();
       QString uri;
       for(QMap<QString,int>::ConstIterator it = m_Wallpaper.begin();
           it != m_Wallpaper.end();
@@ -617,8 +676,26 @@ void BGDialog::slotWallpaperTypeChanged(int i)
             break;
          }
       }
+
+      KFileMetaInfo metaInfo(uri);
+      if (metaInfo.isValid() && metaInfo.item("Dimensions").isValid())
+      {
+         // If the image is greater than 800x600 default to using scaled mode,
+         // otherwise default to tiled.
+
+         QSize s = metaInfo.item("Dimensions").value().toSize();
+         if (s.width() >= 800 && s.height() >= 600)
+            m_wallpaperPos = KBackgroundSettings::Scaled;
+         else
+            m_wallpaperPos = KBackgroundSettings::Tiled;
+      }
+
+      r->setWallpaperMode(m_wallpaperPos);
+      m_comboWallpaperPos->blockSignals(true);
+      m_comboWallpaperPos->setCurrentItem(m_wallpaperPos-1);
+      m_comboWallpaperPos->blockSignals(false);
+
       r->setWallpaper(uri);
-      m_urlWallpaper->fileDialog()->setURL(uri);
    }
 
    r->start(true);
@@ -780,6 +857,12 @@ void BGDialog::slotAdvanced()
     m_previewUpdates = false;
     BGAdvancedDialog dlg(r, topLevelWidget(), m_multidesktop);
 
+    if (!m_pMonitor->isEnabled()) {
+       dlg.makeReadOnly();
+       dlg.exec();
+       return;
+    }
+
     dlg.setTextColor(m_pGlobals->textColor());
     dlg.setTextBackgroundColor(m_pGlobals->textBackgroundColor());
     dlg.setShadowEnabled(m_pGlobals->shadowEnabled());
