@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Malte Starostik <malte.starostik@t-online.de>
+ * Copyright (c) 2000 Malte Starostik <malte@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,21 +13,20 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 
 #ifndef __SEARCHPROVIDERDLG_H___
 #define __SEARCHPROVIDERDLG_H___
 
-#include <kdialog.h>
+#include <kdialogbase.h>
 
-class KLineEdit;
-class KComboBox;
-class QPushButton;
 class SearchProvider;
+class SearchProviderDlgUI;
 
-class SearchProviderDialog : public KDialog {
+class SearchProviderDialog : public KDialogBase
+{
     Q_OBJECT
 
 public:
@@ -37,18 +36,11 @@ public:
 
 protected slots:
     void slotChanged();
-    virtual void accept();
+    void slotOk();
 
 private:
     SearchProvider *m_provider;
-
-    KLineEdit *m_name;
-    KLineEdit *m_query;
-    KLineEdit *m_keys;
-    KComboBox *m_charset;
-
-    QPushButton *m_ok;
-    QPushButton *m_cancel;
+    SearchProviderDlgUI *m_dlg;
 };
 
 #endif
