@@ -6,8 +6,6 @@
 //
 // Ported to kcontrol2 by Geert Jansen.
 
-/* $Id: fonts.cpp,v 1.83 2003/11/08 23:43:50 mkretz Exp $ */
-
 #include <qlayout.h>
 #include <qhbox.h>
 #include <qlabel.h>
@@ -29,6 +27,7 @@
 #include <kmessagebox.h>
 #include <kgenericfactory.h>
 #include <stdlib.h>
+#include "../krdb/krdb.h"
 
 #include "fonts.h"
 #include "fonts.moc"
@@ -153,7 +152,7 @@ void FontUseItem::readFont()
     config = KGlobal::config();
   else
   {
-    config = new KSimpleConfig(locate("config", _rcfile), true);
+    config = new KConfig(_rcfile, true);
     deleteme = true;
   }
 
@@ -199,6 +198,277 @@ void FontUseItem::applyFontDiff( const QFont &fnt, int fontDiffFlags )
   setFont( _font, isFixedOnly() );
 }
 
+/**** FontAASettings ****/
+
+FontAASettings::FontAASettings(QWidget *parent)
+              : KDialogBase(parent, "FontAASettings", true, i18n("Configure Anti-Alias Settings"), Ok|Cancel, Ok, true),
+                changesMade(false)
+{
+  QWidget     *mw=new QWidget(this);
+  QGridLayout *layout=new QGridLayout(mw, 1, 1, KDialog::marginHint(), KDialog::spacingHint());
+
+  excludeRange=new QCheckBox(i18n("E&xclude range:"), mw),
+  layout->addWidget(excludeRange, 0, 0);
+  excludeFrom=new KDoubleNumInput(0, 72, 8.0, 1, 1, mw),
+  excludeFrom->setSuffix(i18n(" pt"));
+  layout->addWidget(excludeFrom, 0, 1);
+  excludeToLabel=new QLabel(i18n(" to "), mw);
+  layout->addWidget(excludeToLabel, 0, 2);
+  excludeTo=new KDoubleNumInput(0, 72, 15.0, 1, 1, mw);
+  excludeTo->setSuffix(i18n(" pt"));
+  layout->addWidget(excludeTo, 0, 3);
+
+  useSubPixel=new QCheckBox(i18n("&Use sub-pixel hinting:"), mw);
+  layout->addWidget(useSubPixel, 1, 0);
+
+  QWhatsThis::add(useSubPixel, i18n("If you have a TFT or LCD screen you"
+       " can further improve the quality of displayed fonts by selecting"
+       " this option.<br>Sub-pixel hinting is also known as ClearType(tm).<br>"
+       "<br><b>This will not work with CRT monitors.</b>"));
+
+  subPixelType=new QComboBox(false, mw);
+  layout->addMultiCellWidget(subPixelType, 1, 1, 1, 3);
+
+  QWhatsThis::add(subPixelType, i18n("In order for sub-pixel hinting to"
+       " work correctly you need to know how the sub-pixels of your display"
+       " are aligned.<br>"
+       " On TFT or LCD displays a single pixel is actually composed of"
+       " three sub-pixels, red, green and blue. Most displays"
+       " have a linear ordering of RGB sub-pixel, some have BGR."));
+
+  for(int t=KXftConfig::SubPixel::None+1; t<=KXftConfig::SubPixel::Vbgr; ++t)
+    subPixelType->insertItem(aaPixmaps[t-1], i18n(KXftConfig::description((KXftConfig::SubPixel::Type)t).utf8()));
+
+#ifdef HAVE_FONTCONFIG
+  QLabel *hintingLabel=new QLabel(i18n("Hinting style: "), mw);
+  layout->addWidget(hintingLabel, 2, 0);
+  hintingStyle=new QComboBox(false, mw);
+  layout->addMultiCellWidget(hintingStyle, 2, 2, 1, 3);
+  for(int s=KXftConfig::Hint::NotSet+1; s<=KXftConfig::Hint::Full; ++s)
+    hintingStyle->insertItem(i18n(KXftConfig::description((KXftConfig::Hint::Style)s).utf8()));
+
+  QString hintingText(i18n("Hinting is a process used to enhance the quality of fonts at small sizes."));
+  QWhatsThis::add(hintingStyle, hintingText);
+  QWhatsThis::add(hintingLabel, hintingText);
+#endif
+  load();
+  enableWidgets();
+  setMainWidget(mw);
+
+  connect(excludeRange, SIGNAL(toggled(bool)), SLOT(changed()));
+  connect(useSubPixel, SIGNAL(toggled(bool)), SLOT(changed()));
+  connect(excludeFrom, SIGNAL(valueChanged(double)), SLOT(changed()));
+  connect(excludeTo, SIGNAL(valueChanged(double)), SLOT(changed()));
+  connect(subPixelType, SIGNAL(activated(const QString &)), SLOT(changed()));
+#ifdef HAVE_FONTCONFIG
+  connect(hintingStyle, SIGNAL(activated(const QString &)), SLOT(changed()));
+#endif
+}
+
+void FontAASettings::load()
+{
+  double     from, to;
+#ifdef HAVE_FONTCONFIG
+  KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType|KXftConfig::HintStyle);
+#else
+  KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType);
+#endif
+
+  if(xft.getExcludeRange(from, to))
+     excludeRange->setChecked(true);
+  else
+  {
+    excludeRange->setChecked(false);
+    from=8.0;
+    to=15.0;
+  }
+
+  excludeFrom->setValue(from);
+  excludeTo->setValue(to);
+
+  KXftConfig::SubPixel::Type spType;
+
+  if(!xft.getSubPixelType(spType) || KXftConfig::SubPixel::None==spType)
+    useSubPixel->setChecked(false);
+  else
+  {
+    int idx=getIndex(spType);
+
+    if(idx>-1)
+    {
+      useSubPixel->setChecked(true);
+      subPixelType->setCurrentItem(idx);
+    }
+    else
+      useSubPixel->setChecked(false);
+  }
+
+#ifdef HAVE_FONTCONFIG
+  KXftConfig::Hint::Style hStyle;
+
+  if(!xft.getHintStyle(hStyle) || KXftConfig::Hint::NotSet==hStyle)
+  {
+    KConfig kglobals("kdeglobals", false, false);
+
+    kglobals.setGroup("General");
+    hStyle=KXftConfig::Hint::Medium;
+    xft.setHintStyle(hStyle);
+    xft.apply();  // Save this setting
+    kglobals.writeEntry("XftHintStyle", KXftConfig::toStr(hStyle));
+    kglobals.sync();
+    runRdb(KRdbExportXftSettings);
+  }
+
+  hintingStyle->setCurrentItem(getIndex(hStyle));
+#endif
+
+  enableWidgets();
+}
+
+bool FontAASettings::save()
+{
+#ifdef HAVE_FONTCONFIG
+  KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType|KXftConfig::HintStyle);
+#else
+  KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType);
+#endif
+  KConfig    kglobals("kdeglobals", false, false);
+
+  kglobals.setGroup("General");
+
+  if(excludeRange->isChecked())
+    xft.setExcludeRange(excludeFrom->value(), excludeTo->value());
+  else
+    xft.setExcludeRange(0, 0);
+
+  if(useSubPixel->isChecked())
+  {
+    KXftConfig::SubPixel::Type spType(getSubPixelType());
+
+    xft.setSubPixelType(spType);
+    kglobals.writeEntry("XftSubPixel", KXftConfig::toStr(spType));
+  }
+  else
+  {
+    xft.setSubPixelType(KXftConfig::SubPixel::None);
+    kglobals.writeEntry("XftSubPixel", "");
+  }
+
+  bool mod=false;
+#ifdef HAVE_FONTCONFIG
+  KXftConfig::Hint::Style hStyle(getHintStyle());
+
+  xft.setHintStyle(hStyle);
+
+  QString hs(KXftConfig::toStr(hStyle));
+
+  if(!hs.isEmpty() && hs!=kglobals.readEntry("XftHintStyle"))
+  {
+    kglobals.writeEntry("XftHintStyle", hs);
+    mod=true;
+  }
+#endif
+  kglobals.sync();
+
+  if(!mod)
+    mod=xft.changed();
+
+  xft.apply();
+
+  return mod;
+}
+
+void FontAASettings::defaults()
+{
+  excludeRange->setChecked(true);
+  excludeFrom->setValue(8.0);
+  excludeTo->setValue(15.0);
+  useSubPixel->setChecked(false);
+#ifdef HAVE_FONTCONFIG
+  hintingStyle->setCurrentItem(getIndex(KXftConfig::Hint::Medium));
+#endif
+  enableWidgets();
+}
+
+int FontAASettings::getIndex(KXftConfig::SubPixel::Type spType)
+{
+  int pos=-1;
+  int index;
+
+  for(index=0; index<subPixelType->count(); ++index)
+    if(subPixelType->text(index)==i18n(KXftConfig::description(spType).utf8()))
+    {
+      pos=index;
+      break;
+    }
+
+  return pos;
+}
+
+KXftConfig::SubPixel::Type FontAASettings::getSubPixelType()
+{
+  int t;
+
+  for(t=KXftConfig::SubPixel::None; t<=KXftConfig::SubPixel::Vbgr; ++t)
+    if(subPixelType->currentText()==i18n(KXftConfig::description((KXftConfig::SubPixel::Type)t).utf8()))
+      return (KXftConfig::SubPixel::Type)t;
+
+  return KXftConfig::SubPixel::None;
+}
+
+#ifdef HAVE_FONTCONFIG
+int FontAASettings::getIndex(KXftConfig::Hint::Style hStyle)
+{
+  int pos=-1;
+  int index;
+
+  for(index=0; index<hintingStyle->count(); ++index)
+    if(hintingStyle->text(index)==i18n(KXftConfig::description(hStyle).utf8()))
+    {
+      pos=index;
+      break;
+    }
+
+  return pos;
+}
+
+
+KXftConfig::Hint::Style FontAASettings::getHintStyle()
+{
+  int s;
+
+  for(s=KXftConfig::Hint::NotSet; s<=KXftConfig::Hint::Full; ++s)
+    if(hintingStyle->currentText()==i18n(KXftConfig::description((KXftConfig::Hint::Style)s).utf8()))
+      return (KXftConfig::Hint::Style)s;
+
+  return KXftConfig::Hint::Medium;
+}
+#endif
+
+void FontAASettings::enableWidgets()
+{
+  excludeFrom->setEnabled(excludeRange->isChecked());
+  excludeTo->setEnabled(excludeRange->isChecked());
+  excludeToLabel->setEnabled(excludeRange->isChecked());
+  subPixelType->setEnabled(useSubPixel->isChecked());
+}
+
+void FontAASettings::changed()
+{
+    changesMade=true;
+    enableWidgets();
+}
+
+int FontAASettings::exec()
+{
+    int i=KDialogBase::exec();
+
+    if(!i)
+        load(); // Reset settings...
+
+    return i && changesMade;
+}
+
 /**** KFonts ****/
 
 static QCString desktopConfigName()
@@ -324,77 +594,30 @@ KFonts::KFonts(QWidget *parent, const char *name, const QStringList &)
    lay->addWidget( fontAdjustButton );
    connect(fontAdjustButton, SIGNAL(clicked()), SLOT(slotApplyFontDiff()));
 
-   QGroupBox *aaBox=new QGroupBox(i18n("An&ti-Aliasing"), this);
+   layout->addSpacing(KDialog::spacingHint());
 
-   aaBox->setColumnLayout(1, Qt::Horizontal);
-   aaBox->layout()->setSpacing(KDialog::spacingHint());
-
-   cbAA = new QCheckBox( i18n( "Use a&nti-aliasing for fonts" ), aaBox);
+   lay = new QHBoxLayout(layout, KDialog::spacingHint());
+   cbAA = new QCheckBox( i18n( "Use a&nti-aliasing for fonts" ), this);
    QWhatsThis::add(cbAA, i18n("If this option is selected, KDE will smooth the edges of curves in "
                               "fonts."));
-
-   QHBox *hbox =  new QHBox(aaBox);
-   QSpacerItem *spacer = new QSpacerItem( 20, 0, QSizePolicy::Fixed, QSizePolicy::Minimum );
-   hbox->layout()->addItem(spacer);
-   aaExcludeRange=new QCheckBox(i18n("E&xclude range:"), hbox),
-   aaExcludeFrom=new KDoubleNumInput(0, 72, 8.0, 1, 1, hbox),
-   aaExcludeFrom->setSuffix(i18n(" pt"));
-   aaExcludeToLabel = new QLabel(i18n(" to "), hbox);
-   aaExcludeTo=new KDoubleNumInput(0, 72, 15.0, 1, 1, hbox);
-   aaExcludeTo->setSuffix(i18n(" pt"));
-   QWidget *dummy = new QWidget(hbox);
-   hbox->setStretchFactor(dummy, 1);
-
-   hbox =  new QHBox(aaBox);
-   spacer = new QSpacerItem( 20, 0, QSizePolicy::Fixed, QSizePolicy::Minimum );
-   hbox->layout()->addItem(spacer);
-   aaUseSubPixel=new QCheckBox(i18n("&Use sub-pixel hinting:"), hbox);
-
-   QWhatsThis::add(aaUseSubPixel, i18n("If you have a TFT or LCD screen you"
-	" can further improve the quality of displayed fonts by selecting"
-	" this option.<br>Sub-pixel hinting is also known as ClearType(tm).<br>"
-	"<br><b>This will not work with CRT monitors.</b>"));
-
-   aaSubPixelType=new QComboBox(false, hbox);
-
-   QWhatsThis::add(aaSubPixelType, i18n("In order for sub-pixel hinting to"
-	" work correctly you need to know how the sub-pixels of your display"
-	" are aligned.<br>"
-	" On TFT or LCD displays a single pixel is actually composed of"
-	" three sub-pixels, red, green and blue. Most displays" 
-	" have a linear ordering of RGB sub-pixel, some have BGR."));
-
-   dummy = new QWidget(hbox);
-   hbox->setStretchFactor(dummy, 1);
-
-   for(int t=KXftConfig::SubPixel::None+1; t<=KXftConfig::SubPixel::Vbgr; ++t)
-       aaSubPixelType->insertItem(aaPixmaps[t-1], KXftConfig::description((KXftConfig::SubPixel::Type)t));
-
-   setAaWidgets();
-
-   connect(aaExcludeRange, SIGNAL(toggled(bool)),
-           this, SLOT(slotAaChange()));
-   connect(aaUseSubPixel, SIGNAL(toggled(bool)),
-           this, SLOT(slotAaChange()));
-   connect(aaExcludeFrom, SIGNAL(valueChanged(double)),
-           this, SLOT(slotAaChange()));
-   connect(aaExcludeTo, SIGNAL(valueChanged(double)),
-           this, SLOT(slotAaChange()));
-   connect(aaSubPixelType, SIGNAL(activated(const QString &)),
-           this, SLOT(slotAaChange()));
-
-   layout->addWidget(aaBox);
+   lay->addStretch();
+   QPushButton *aaSettingsButton = new QPushButton( i18n( "Configure..." ), this);
+   connect(aaSettingsButton, SIGNAL(clicked()), SLOT(slotCfgAa()));
+   connect(cbAA, SIGNAL(toggled(bool)), aaSettingsButton, SLOT(setEnabled(bool)));
+   lay->addWidget( cbAA );
+   lay->addWidget( aaSettingsButton );
 
    layout->addStretch(1);
 
+   aaSettings=new FontAASettings(this);
+
    connect(cbAA, SIGNAL(clicked()), SLOT(slotUseAntiAliasing()));
 
    useAA = QSettings().readBoolEntry("/qt/useXft");
    useAA_original = useAA;
 
-   enableAaWidgets();
-
    cbAA->setChecked(useAA);
+   aaSettingsButton->setEnabled(useAA);
 }
 
 KFonts::~KFonts()
@@ -416,13 +639,7 @@ void KFonts::defaults()
 
   useAA = false;
   cbAA->setChecked(useAA);
-
-  aaExcludeRange->setChecked(true);
-  aaExcludeFrom->setValue(8.0);
-  aaExcludeTo->setValue(15.0);
-  aaUseSubPixel->setChecked(false);
-  enableAaWidgets();
-
+  aaSettings->defaults();
   _changed = true;
   emit changed(true);
 }
@@ -451,7 +668,7 @@ void KFonts::load()
   kdDebug(1208) << "AA:" << useAA << endl;
   cbAA->setChecked(useAA);
 
-  setAaWidgets();
+  aaSettings->load();
 
   _changed = true;
   emit changed(false);
@@ -490,20 +707,7 @@ void KFonts::save()
 
   kapp->processEvents(); // Process font change ourselves
 
-  KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType);
-
-  if(aaExcludeRange->isChecked())
-      xft.setExcludeRange(aaExcludeFrom->value()-1,
-                          aaExcludeTo->value()+1);
-  else
-      xft.setExcludeRange(0, 0);
-
-  if(aaUseSubPixel->isChecked())
-      xft.setSubPixelType(getAaSubPixelType());
-  else
-      xft.setSubPixelType(KXftConfig::SubPixel::None);
-
-  if((useAA != useAA_original) || xft.changed()) {
+  if(aaSettings->save() || (useAA != useAA_original) ) {
     KMessageBox::information(this,
       i18n(
         "<p>You have changed anti-aliasing related settings. This change will only affect newly started applications.</p>"
@@ -511,7 +715,8 @@ void KFonts::save()
     useAA_original = useAA;
   }
 
-  xft.apply();
+  runRdb(KRdbExportXftSettings);
+
   emit changed(false);
 }
 
@@ -538,90 +743,17 @@ void KFonts::slotApplyFontDiff()
 void KFonts::slotUseAntiAliasing()
 {
     useAA = cbAA->isChecked();
-    enableAaWidgets();
     _changed = true;
     emit changed(true);
 }
 
-void KFonts::slotAaChange()
+void KFonts::slotCfgAa()
 {
-    enableAaWidgets();
-
+  if(aaSettings->exec())
+  {
     _changed = true;
     emit changed(true);
-}
-
-void KFonts::setAaWidgets()
-{
-   double aaFrom, aaTo;
-   KXftConfig     xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType);
-
-   if(xft.getExcludeRange(aaFrom, aaTo))
-       aaExcludeRange->setChecked(true);
-   else
-   {
-       aaExcludeRange->setChecked(false);
-       aaFrom=8.0;
-       aaTo=15.0;
-   }
-
-   aaExcludeFrom->setValue(aaFrom+1);
-   aaExcludeTo->setValue(aaTo-1);
-
-   KXftConfig::SubPixel::Type aaSpType;
-
-   if(!xft.getSubPixelType(aaSpType) || KXftConfig::SubPixel::None==aaSpType)
-       aaUseSubPixel->setChecked(false);
-   else
-   {
-       int idx=getIndex(aaSpType);
-
-       if(idx>-1)
-       {
-           aaUseSubPixel->setChecked(true);
-           aaSubPixelType->setCurrentItem(idx);
-       }
-       else
-           aaUseSubPixel->setChecked(false);
-   }
-
-   enableAaWidgets();
-}
-
-int KFonts::getIndex(KXftConfig::SubPixel::Type aaSpType)
-{
-    int pos=-1;
-    int index;
-
-    for(index=0; index<aaSubPixelType->count(); ++index)
-        if(aaSubPixelType->text(index)==KXftConfig::description(aaSpType))
-        {
-            pos=index;
-            break;
-        }
-
-    return pos;
-}
-
-KXftConfig::SubPixel::Type KFonts::getAaSubPixelType()
-{
-    int t;
-
-    for(t=KXftConfig::SubPixel::None; t<=KXftConfig::SubPixel::Vbgr; ++t)
-        if(aaSubPixelType->currentText()==KXftConfig::description((KXftConfig::SubPixel::Type)t))
-            return (KXftConfig::SubPixel::Type)t;
-
-    return KXftConfig::SubPixel::None;
-}
-
-void KFonts::enableAaWidgets()
-{
-    aaExcludeRange->setEnabled(useAA);
-    aaExcludeFrom->setEnabled(aaExcludeRange->isChecked() && useAA);
-    aaExcludeTo->setEnabled(aaExcludeRange->isChecked() && useAA);
-    aaExcludeToLabel->setEnabled(aaExcludeRange->isChecked() && useAA);
-    aaUseSubPixel->setEnabled(useAA);
-    aaSubPixelType->setEnabled(aaUseSubPixel->isChecked() && useAA);
+  }
 }
 
 // vim:ts=2:sw=2:tw=78
