@@ -144,8 +144,8 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
     connect( tab1->cbAutoSelect, SIGNAL( clicked() ), this, SLOT( slotClick() ) );
 
     // Only allow setting reversing scroll polarity if we have scroll buttons
-    unsigned char map[5];
-    if ( XGetPointerMapping(kapp->getDisplay(), map, 5) == 5 )
+    unsigned char map[20];
+    if ( XGetPointerMapping(kapp->getDisplay(), map, 20) >= 5 )
     {
       tab1->cbScrollPolarity->setEnabled( true );
       tab1->cbScrollPolarity->show();
@@ -170,10 +170,8 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
     QBoxLayout *lay = new QVBoxLayout(tab2, KDialog::marginHint(),
               KDialog::spacingHint());
 
-    accel = new KDoubleNumInput(2, tab2);
+    accel = new KDoubleNumInput(1, 20, 2, 0.1, 1, tab2);
     accel->setLabel(i18n("Pointer acceleration:"));
-    accel->setPrecision(1);
-    accel->setRange(1,20);
     accel->setSuffix("x");
     lay->addWidget(accel);
     connect(accel, SIGNAL(valueChanged(double)), this, SLOT(changed()));
@@ -187,7 +185,7 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
          " movements of the mouse pointer on the screen even when"
          " you only make a small movement with the physical device."
          " Selecting very high values may result in the mouse pointer"
-         " flying across the screen, making it hard to control!");
+         " flying across the screen, making it hard to control.");
     QWhatsThis::add( accel, wtstr );
 
     thresh = new KIntNumInput(accel, 20, tab2);
@@ -201,8 +199,8 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
     wtstr = i18n("The threshold is the smallest distance that the"
          " mouse pointer must move on the screen before acceleration"
          " has any effect. If the movement is smaller than the threshold,"
-         " the mouse pointer moves as if the acceleration was set to 1X.<p>"
-         " Thus, when you make small movements with the physical device,"
+         " the mouse pointer moves as if the acceleration was set to 1X;<p>"
+         " thus, when you make small movements with the physical device,"
          " there is no acceleration at all, giving you a greater degree"
          " of control over the mouse pointer. With larger movements of"
          " the physical device, you can move the mouse pointer"
@@ -564,20 +562,19 @@ void MouseSettings::load(KConfig *config)
 
   // get settings from X server
   int h = RIGHT_HANDED;
-  bool revScroll = false;
-  unsigned char map[5];
-  num_buttons = XGetPointerMapping(kapp->getDisplay(), map, 5);
+  unsigned char map[20];
+  num_buttons = XGetPointerMapping(kapp->getDisplay(), map, 20);
 
   handedEnabled = true;
 
   // ## keep this in sync with KGlobalSettings::mouseSettings
-  switch (num_buttons)
-    {
-    case 1:
+  if( num_buttons == 1 )
+  {
       /* disable button remapping */
       handedEnabled = false;
-      break;
-    case 2:
+  }
+  else if( num_buttons == 2 )
+  {
       if ( (int)map[0] == 1 && (int)map[1] == 2 )
         h = RIGHT_HANDED;
       else if ( (int)map[0] == 2 && (int)map[1] == 1 )
@@ -585,9 +582,9 @@ void MouseSettings::load(KConfig *config)
       else
         /* custom button setup: disable button remapping */
         handedEnabled = false;
-      break;
-    case 3:
-    case 5:
+  }
+  else
+  {
       middle_button = (int)map[1];
       if ( (int)map[0] == 1 && (int)map[2] == 3 )
     h = RIGHT_HANDED;
@@ -598,15 +595,7 @@ void MouseSettings::load(KConfig *config)
       /* custom button setup: disable button remapping */
       handedEnabled = false;
     }
-      break;
-    default:
-      /* custom setup with > 3 buttons: disable button remapping */
-      handedEnabled = false;
-      break;
-    }
-  if ( handedEnabled && num_buttons == 5 && ( int )map[3] == 5 && ( int )map[4] == 4 )
-    revScroll = true;
-
+  }
 
   config->setGroup("Mouse");
   double a = config->readDoubleNumEntry("Acceleration",-1);
@@ -629,7 +618,7 @@ void MouseSettings::load(KConfig *config)
   else if (key == NULL)
     handed = h;
   reverseScrollPolarity = config->readBoolEntry( "ReverseScrollPolarity", false );
-  m_handedNeedsApply = ( (handed != h) || (reverseScrollPolarity != revScroll) );
+  m_handedNeedsApply = false;
 
   // SC/DC/AutoSelect/ChangeCursor
   config->setGroup("KDE");
@@ -644,65 +633,62 @@ void MouseSettings::load(KConfig *config)
   changeCursor = config->readBoolEntry("ChangeCursor", KDE_DEFAULT_CHANGECURSOR);
 }
 
-void MouseSettings::apply()
+void MouseSettings::apply(bool force)
 {
   XChangePointerControl( kapp->getDisplay(),
                          true, true, int(qRound(accelRate*10)), 10, thresholdMove);
 
-  unsigned char map[5];
-  int remap=1;
-  if (handedEnabled && m_handedNeedsApply) {
-      switch (num_buttons) {
-      case 1:
+  unsigned char map[20];
+  num_buttons = XGetPointerMapping(kapp->getDisplay(), map, 20);
+  int remap=(num_buttons>=1);
+  if (handedEnabled && (m_handedNeedsApply || force)) {
+      if( num_buttons == 1 )
+      {
           map[0] = (unsigned char) 1;
-          break;
-      case 2:
-          if (handed == RIGHT_HANDED) {
+      }
+      else if( num_buttons == 2 )
+      {
+          if (handed == RIGHT_HANDED)
+          {
               map[0] = (unsigned char) 1;
               map[1] = (unsigned char) 3;
           }
-          else {
+          else
+          {
               map[0] = (unsigned char) 3;
               map[1] = (unsigned char) 1;
           }
-          break;
-      case 3:
-          if (handed == RIGHT_HANDED) {
+      }
+      else // 3 buttons and more
+      {
+          if (handed == RIGHT_HANDED)
+          {
               map[0] = (unsigned char) 1;
               map[1] = (unsigned char) middle_button;
               map[2] = (unsigned char) 3;
           }
-          else {
+          else
+          {
               map[0] = (unsigned char) 3;
               map[1] = (unsigned char) middle_button;
               map[2] = (unsigned char) 1;
           }
-          break;
-      case 5:
-          // Intellimouse case, where buttons 1-3 are left, middle, and
-          // right, and 4-5 are up/down depending on scroll polarity
-          if (handed == RIGHT_HANDED) {
-              map[0] = (unsigned char) 1;
-              map[1] = (unsigned char) 2;
-              map[2] = (unsigned char) 3;
-              map[3] = reverseScrollPolarity ? (unsigned char) 5 : (unsigned char) 4;
-              map[4] = reverseScrollPolarity ? (unsigned char) 4 : (unsigned char) 5;
+          if( num_buttons >= 5 )
+          {
+          // Apps seem to expect logical buttons 4,5 are the vertical wheel.
+          // With mice with more than 3 buttons (not including wheel) the physical
+          // buttons mapped to logical 4,5 may not be physical 4,5 , so keep
+          // this mapping, only possibly reversing them.
+              int pos;
+              for( pos = 0; pos < num_buttons; ++pos )
+                  if( map[pos] == 4 || map[pos] == 5 )
+                      break;
+              if( pos < num_buttons - 1 )
+              {
+                  map[pos] = reverseScrollPolarity ? (unsigned char) 5 : (unsigned char) 4;
+                  map[pos+1] = reverseScrollPolarity ? (unsigned char) 4 : (unsigned char) 5;
+              }
           }
-          else {
-              map[0] = (unsigned char) 3;
-              map[1] = (unsigned char) 2;
-              map[2] = (unsigned char) 1;
-              map[3] = reverseScrollPolarity ? (unsigned char) 5 : (unsigned char) 4;
-              map[4] = reverseScrollPolarity ? (unsigned char) 4 : (unsigned char) 5;
-          }
-          break;
-      default: {
-              //catch-all for mice with four or more than five buttons
-              //Without this, XSetPointerMapping is called with a undefined value
-              //for map
-              remap=0;  //don't remap
-          }
-          break;
       }
       int retval;
       if (remap)
@@ -734,10 +720,7 @@ void MouseSettings::save(KConfig *config)
   config->writeEntry("SingleClick", singleClick, true, true);
   config->writeEntry("AutoSelectDelay", autoSelectDelay, true, true );
   config->writeEntry("VisualActivate", visualActivate, true, true);
-
-  // the following need to be in the local config file so they are read
-  // by kcminit (which only reads local config for speed up)
-  config->writeEntry("ChangeCursor", changeCursor );
+  config->writeEntry("ChangeCursor", changeCursor, true, true);
   config->sync();
   KIPC::sendMessageAll(KIPC::SettingsChanged, KApplication::SETTINGS_MOUSE);
 }
