@@ -68,7 +68,7 @@ KCMIOSlaveInfo::KCMIOSlaveInfo(QWidget *parent, const char *name, const QStringL
    hbox->setStretchFactor(m_info,5);
 
    QStringList protocols=KProtocolInfo::protocols();
-   for (QStringList::Iterator it=protocols.begin(); it!=protocols.end(); it++)
+   for (QStringList::Iterator it=protocols.begin(); it!=protocols.end(); ++it)
    {
       QString proto = *it;
       m_ioslavesLb->insertItem( SmallIcon( KProtocolInfo::icon( proto )), 
@@ -78,6 +78,17 @@ KCMIOSlaveInfo::KCMIOSlaveInfo(QWidget *parent, const char *name, const QStringL
    m_ioslavesLb->setSelected(0, true);
 
    setButtons(buttons ());
+
+   KAboutData *about =
+   new KAboutData(I18N_NOOP("kcmioslaveinfo"),
+	I18N_NOOP("KDE Panel System Information Control Module"),
+	0, 0, KAboutData::License_GPL,
+    	I18N_NOOP("(c) 2001 - 2002 Alexander Neundorf"));
+
+   about->addAuthor("Alexander Neundorf", 0, "neundorf@kde.org");
+   about->addAuthor("George Staikos", 0, "staikos@kde.org");
+   setAboutData( about );
+
    load();
 }
 
@@ -85,21 +96,6 @@ KCMIOSlaveInfo::KCMIOSlaveInfo(QWidget *parent, const char *name, const QStringL
 KCMIOSlaveInfo::~KCMIOSlaveInfo() {
 }
 
-const KAboutData* KCMIOSlaveInfo::aboutData() const
-{
-    KAboutData *about =
-    new KAboutData(I18N_NOOP("kcmioslaveinfo"),
-		I18N_NOOP("KDE Panel System Information Control Module"),
-		0, 0, KAboutData::License_GPL,
-    		I18N_NOOP("(c) 2001 - 2002 Alexander Neundorf"));
-
-    about->addAuthor("Alexander Neundorf", 0, "neundorf@kde.org");
-    about->addAuthor("George Staikos", 0, "staikos@kde.org");
-
-    return about;
-}
-
-
 void KCMIOSlaveInfo::load() {
    emit changed(false);
 }
@@ -164,7 +160,7 @@ void KCMIOSlaveInfo::showInfo(const QString& protocol)
    if (!file.isEmpty())
    {
        helpData.truncate( 0 );
-       m_tfj = KIO::get( QString("help:/kioslave/%1.html").arg( protocol ), true, false );
+       m_tfj = KIO::get( KURL( QString("help:/kioslave/%1.html").arg( protocol ) ), true, false );
        connect( m_tfj, SIGNAL( data( KIO::Job *, const QByteArray &) ), SLOT( slaveHelp( KIO::Job *, const QByteArray &) ) );
        connect( m_tfj, SIGNAL( result( KIO::Job * ) ), SLOT( slotResult( KIO::Job * ) ) );
        return;
