@@ -25,7 +25,6 @@
 #include <kmessagebox.h>
 #include <kinstance.h>
 #include <kaction.h>
-#include <kkeydialog.h>
 
 #include <qtabwidget.h>
 #include <qwhatsthis.h>
@@ -64,6 +63,8 @@ TopLevel::TopLevel(const char* name)
     KCGlobal::setIconSize(KIcon::SizeSmall);
   else if (size == "Large")
     KCGlobal::setIconSize(KIcon::SizeLarge);
+  else if (size == "Huge")
+    KCGlobal::setIconSize(KIcon::SizeHuge);
   else
     KCGlobal::setIconSize(KIcon::SizeMedium);
 
@@ -172,6 +173,9 @@ TopLevel::~TopLevel()
     case KIcon::SizeLarge:
       config->writeEntry("IconSize", "Large");
       break;
+    case KIcon::SizeHuge:
+      config->writeEntry("IconSize", "Huge");
+      break;
     default:
       config->writeEntry("IconSize", "Medium");
       break;
@@ -196,7 +200,8 @@ bool TopLevel::queryClose()
 void TopLevel::setupActions()
 {
   KStdAction::quit(this, SLOT(close()), actionCollection());
-  KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );
+  KStdAction::keyBindings(guiFactory(), SLOT(configureShortcuts()), 
+actionCollection());
   icon_view = new KRadioAction
     (i18n("&Icon View"), 0, this, SLOT(activateIconView()),
      actionCollection(), "activate_iconview");
@@ -222,6 +227,11 @@ void TopLevel::setupActions()
      actionCollection(), "activate_largeicons");
   icon_large->setExclusiveGroup( "iconsize" );
 
+  icon_huge = new KRadioAction
+    (i18n("&Huge"), 0, this, SLOT(activateHugeIcons()),
+     actionCollection(), "activate_hugeicons");
+  icon_huge->setExclusiveGroup( "iconsize" );
+
   about_module = new KAction(i18n("About Current Module"), 0, this, SLOT(aboutModule()), actionCollection(), "help_about_module");
   about_module->setEnabled(false);
 
@@ -238,11 +248,6 @@ void TopLevel::setupActions()
   connect(report_bug, SIGNAL(activated()), SLOT(reportBug()));
 }
 
-void TopLevel::slotConfigureKeys()
-{
-  KKeyDialog::configure( actionCollection(), this );
-}
-
 void TopLevel::activateIconView()
 {
   KCGlobal::setViewMode(Icon);
@@ -251,6 +256,7 @@ void TopLevel::activateIconView()
   icon_small->setEnabled(true);
   icon_medium->setEnabled(true);
   icon_large->setEnabled(true);
+  icon_huge->setEnabled(true);
 
   switch(KCGlobal::iconSize())
     {
@@ -260,6 +266,9 @@ void TopLevel::activateIconView()
     case KIcon::SizeLarge:
       icon_large->setChecked(true);
       break;
+    case KIcon::SizeHuge:
+      icon_huge->setChecked(true);
+      break;
     default:
       icon_medium->setChecked(true);
       break;
@@ -274,6 +283,7 @@ void TopLevel::activateTreeView()
   icon_small->setEnabled(false);
   icon_medium->setEnabled(false);
   icon_large->setEnabled(false);
+  icon_huge->setEnabled(false);
 }
 
 void TopLevel::activateSmallIcons()
@@ -294,6 +304,12 @@ void TopLevel::activateLargeIcons()
   _indextab->reload();
 }
 
+void TopLevel::activateHugeIcons()
+{
+  KCGlobal::setIconSize(KIcon::SizeHuge);
+  _indextab->reload();
+}
+
 void TopLevel::newModule(const QString &name, const QString& docPath, const QString &quickhelp)
 {
     setCaption(name, false);
@@ -329,8 +345,8 @@ void TopLevel::categorySelected(QListViewItem *category)
                   "Do you want to apply the changes before exiting "
                   "the Control Center or discard the changes?"),
                             i18n("Unsaved Changes"),
-                            i18n("&Apply"),
-                            i18n("&Discard"));
+                            KStdGuiItem::apply(),
+                            KStdGuiItem::discard());
         if (res == KMessageBox::Yes)
           _active->module()->applyClicked();
         else if (res == KMessageBox::Cancel)
