@@ -4,7 +4,7 @@
 // Author        : Craig Drummond
 // Project       : K Font Installer
 // Creation Date : 26/04/2003
-// Version       : $Revision: 1.17.2.1 $ $Date: 2004/03/18 21:59:27 $
+// Version       : $Revision: 1.31 $ $Date: 2004/07/18 23:05:43 $
 //
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -23,7 +23,7 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 //
 ////////////////////////////////////////////////////////////////////////////////
-// (C) Craig Drummond, 2003
+// (C) Craig Drummond, 2003, 2004
 ////////////////////////////////////////////////////////////////////////////////
 
 #include "KCmFontInst.h"
@@ -46,60 +46,98 @@
 #include <kcmdlineargs.h>
 #include <kurllabel.h>
 #include <kapplication.h>
+#include <klibloader.h>
 #include <kio/job.h>
 #include <kio/netaccess.h>
 #include <kdirlister.h>
-#ifdef HAVE_FT_CACHE
+#include <kpushbutton.h>
+#include <kguiitem.h>
 #include <qsplitter.h>
-#include <knuminput.h>
-#include "FontPreview.cpp"
-#endif
 
-#define CFG_GROUP       "Main Settings"
-#define CFG_LISTVIEW    "ListView"
-#define CFG_PATH        "Path"
-#define CFG_DIRSIZE     "DirSize"
-#define CFG_PREVIEWSIZE "PreviewSize"
+#define CFG_GROUP          "Main Settings"
+#define CFG_LISTVIEW       "ListView"
+#define CFG_PATH           "Path"
+#define CFG_SPLITTER_SIZES "SplitterSizes"
+#define CFG_SIZE           "Size"
+
+//
+// Remove any fonts:/ status information added to name
+static QString formatName(const QString &name)
+{
+    QString n(name);
+
+    return n.remove(i18n(KIO_FONTS_DISABLED));
+}
 
 typedef KGenericFactory<CKCmFontInst, QWidget> FontInstallFactory;
-K_EXPORT_COMPONENT_FACTORY(kcm_fontinst, FontInstallFactory)
+K_EXPORT_COMPONENT_FACTORY(kcm_fontinst, FontInstallFactory("kcmfontinst"))
 
 CKCmFontInst::CKCmFontInst(QWidget *parent, const char *, const QStringList&)
             : KCModule(parent, "kfontinst"),
-              itsAboutData(NULL),
-              itsTop(CMisc::root() ? "fonts:/" : QString("fonts:/")+i18n(KIO_FONTS_USER)),
-              itsConfig("kcmfontinstuirc")
+              itsTop(CMisc::root() ? QString("fonts:/") : QString("fonts:/")+i18n(KIO_FONTS_USER)),
+              itsPreview(NULL),
+              itsConfig(CGlobal::uiCfgFile())
 {
-    KConfigGroupSaver cfgSaver(&itsConfig, CFG_GROUP);
+    KGlobal::locale()->insertCatalogue("kfontinst");
+
+    KAboutData* about = new KAboutData("kcmfontinst",
+         I18N_NOOP("KDE Font Installer"),
+         0, 0,
+         KAboutData::License_GPL,
+         I18N_NOOP("GUI front end to the fonts:/ ioslave.\n"
+         "(c) Craig Drummond, 2000 - 2004"));
+    about->addAuthor("Craig Drummond", I18N_NOOP("Developer and maintainer"), "craig@kde.org");
+    setAboutData( about );
+
+
     const char *appName=KCmdLineArgs::appName();
 
-    itsEmbeddedAdmin=CMisc::root() && (NULL==appName || strcmp("kcontrol", appName)
-                                       && KCmdLineArgs::parsedArgs()->isSet("embed"));
+    itsEmbeddedAdmin=CMisc::root() && (NULL==appName || strcmp("kcontrol", appName) &&
+                     KCmdLineArgs::parsedArgs()->isSet("embed"));
+    itsKCmshell=!itsEmbeddedAdmin && NULL!=appName && 0==strcmp("kcmshell", appName) &&
+                !KCmdLineArgs::parsedArgs()->isSet("embed");
 
     itsStatusLabel = new QLabel(this);
     itsStatusLabel->setFrameShape(QFrame::Panel);
     itsStatusLabel->setFrameShadow(QFrame::Sunken);
     itsStatusLabel->setLineWidth(1);
 
-#ifdef HAVE_FT_CACHE
-    itsSplitter=new QSplitter(this);
+    itsConfig.setGroup(CFG_GROUP);
 
-    QFrame            *fontsFrame=new QFrame(itsSplitter),
-                      *previewFrame=new QFrame(itsSplitter);
-    QGridLayout       *previewLayout=new QGridLayout(previewFrame, 2, 2, 1, 1);
-#else
-    QFrame            *fontsFrame=new QFrame(this);
-#endif
-    QFrame            *urlFrame=new QFrame(this);
-    QGridLayout       *fontsLayout=new QGridLayout(fontsFrame, 1, 1, 0, 1);
-    QHBoxLayout       *urlLayout=new QHBoxLayout(urlFrame, 1);
-    QVBoxLayout       *layout=new QVBoxLayout(this, 0, KDialog::spacingHint());
-    KToolBar          *toolbar=new KToolBar(this);
+    KLibFactory *factory=KLibLoader::self()->factory("libkfontviewpart");
+    QFrame      *fontsFrame,
+                *urlFrame=new QFrame(this);
+
+    if(factory)
+    {
+        itsSplitter=new QSplitter(this);
+        fontsFrame=new QFrame(itsSplitter),
+        itsPreview=(KParts::ReadOnlyPart *)factory->create(itsSplitter, "kcmfontinst", "KParts::ReadOnlyPart");
+        itsSplitter->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
+
+        QValueList<int> sizes(itsConfig.readIntListEntry(CFG_SPLITTER_SIZES));
+
+        if(2!=sizes.count())
+        {
+            sizes.clear();
+            sizes+=250;
+            sizes+=150;
+        }
+        itsSplitter->setSizes(sizes);
+    }
+    else
+    {
+        fontsFrame=new QFrame(this);
+        fontsFrame->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
+        urlFrame->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
+    }
+
+    QGridLayout *fontsLayout=new QGridLayout(fontsFrame, 1, 1, 0, 1);
+    QHBoxLayout *urlLayout=new QHBoxLayout(urlFrame, 1);
+    QVBoxLayout *layout=new QVBoxLayout(this, 0, KDialog::spacingHint());
+    KToolBar    *toolbar=new KToolBar(this);
 
     fontsFrame->setLineWidth(0);
-#ifdef HAVE_FT_CACHE
-    previewFrame->setFrameStyle(QFrame::StyledPanel|QFrame::Sunken);
-#endif
     urlFrame->setFrameShadow(QFrame::Sunken);
     urlFrame->setFrameShape(QFrame::StyledPanel);
     urlFrame->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
@@ -112,20 +150,11 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const char *, const QStringList&)
     urlLayout->addItem(new QSpacerItem(4, 4));
     toolbar->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
     toolbar->setMovingEnabled(false);
-#ifdef HAVE_FT_CACHE
-    itsSplitter->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
-
-    QValueList<int> sizes;
-
-    sizes.append(itsConfig.readNumEntry(CFG_DIRSIZE, 3));
-    sizes.append(itsConfig.readNumEntry(CFG_PREVIEWSIZE, 2));
-    itsSplitter->setSizes(sizes);
-#endif
 
     QString previousPath=itsConfig.readEntry(CFG_PATH);
     KURL    url(itsTop);
 
-    if(!previousPath.isNull())
+    if(!previousPath.isEmpty())
     {
         KIO::UDSEntry uds;
 
@@ -153,33 +182,18 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const char *, const QStringList&)
 
     itsDirOp->setMimeFilter(mimeTypes);
     itsDirOp->dirLister()->setMainWindow(this);
-    fontsLayout->addWidget(itsDirOp, 0, 0);
+    itsDirOp->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
+    fontsLayout->addMultiCellWidget(itsDirOp, 0, 0, 0, 1);
 
-#ifdef HAVE_FT_CACHE
-    itsPreview = new CFontPreview(previewFrame);
-    itsPreview->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)3, (QSizePolicy::SizeType)3, 0, 0,
-                              itsPreview->sizePolicy().hasHeightForWidth()));
-    itsPreview->setMinimumSize(QSize(96, 64));
-    itsFaceLabel=new QLabel(i18n(" Face:"), previewFrame);
-    itsFaceSelector=new KIntNumInput(1, previewFrame);
-    itsFaceLabel->hide();
-    itsFaceSelector->hide();
-    previewLayout->addMultiCellWidget(itsPreview, 0, 0, 0, 1);
-    previewLayout->addWidget(itsFaceLabel, 1, 0);
-    previewLayout->addWidget(itsFaceSelector, 1, 1);
-#else
-    fontsFrame->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)3, (QSizePolicy::SizeType)3, 0, 0,
-                              fontsFrame->sizePolicy().hasHeightForWidth()));
-    urlFrame->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
-#endif
+    KPushButton *button=new KPushButton(KGuiItem(i18n("Add Fonts..."), "newfont"), fontsFrame);
+    connect(button, SIGNAL(clicked()), SLOT(addFonts()));
+    button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
+    fontsLayout->addWidget(button, 1, 0);
+    fontsLayout->addItem(new QSpacerItem(4, 4, QSizePolicy::Expanding, QSizePolicy::Minimum));
 
     layout->addWidget(toolbar);
     layout->addWidget(urlFrame);
-#ifdef HAVE_FT_CACHE
-    layout->addWidget(itsSplitter);
-#else
-    layout->addWidget(fontsFrame);
-#endif
+    layout->addWidget(itsPreview ? itsSplitter : fontsFrame);
     layout->addWidget(itsStatusLabel);
 
     setButtons(0);
@@ -282,7 +296,8 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const char *, const QStringList&)
     itsEnableAct->setEnabled(false);
     itsEnableAct->plug(toolbar);
     topMnu->insert(itsEnableAct);
-    itsDisableAct=new KAction(i18n("Disable"), "button_cancel", 0, this, SLOT(disable()), itsDirOp->actionCollection(), "disable");
+    itsDisableAct=new KAction(i18n("Disable"), "button_cancel", 0, this, SLOT(disable()), itsDirOp->actionCollection(),
+                              "disable");
     itsDisableAct->setEnabled(false);
     itsDisableAct->plug(toolbar);
     topMnu->insert(itsDisableAct);
@@ -299,6 +314,29 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const char *, const QStringList&)
         setupViewMenu();
     }
 
+    if(itsPreview)
+    {
+        KActionCollection *previewCol=itsPreview->actionCollection();
+
+        if(previewCol && previewCol->count()>0)
+        {
+            toolbar->insertLineSeparator();
+
+            if((act=previewCol->action("zoomIn")))
+                act->plug(toolbar);
+            if((act=previewCol->action("zoomOut")))
+                act->plug(toolbar);
+            if((act=previewCol->action("changeText")))
+                act->plug(toolbar);
+            if((act=previewCol->action("toggleWaterfall")))
+                act->plug(toolbar);
+
+            // For some reason the following always put zoomOut, zoomIn, changeText hmmm :-(
+            //for(unsigned int i=0; i<previewCol->count(); ++i)
+            //    previewCol->action(i)->plug(toolbar);
+        }
+    }
+
     setUpAct();
 
     //
@@ -320,47 +358,34 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const char *, const QStringList&)
     connect(itsDirOp, SIGNAL(dropped(const KFileItem *, QDropEvent *, const KURL::List &)),
                       SLOT(dropped(const KFileItem *, QDropEvent *, const KURL::List &)));
     connect(itsLabel, SIGNAL(leftClickedURL(const QString &)), SLOT(openUrlInBrowser(const QString &)));
-#ifdef HAVE_FT_CACHE
-    connect(itsFaceSelector, SIGNAL(valueChanged(int)), SLOT(showFace(int)));
-#endif
     connect(itsDirOp->dirLister(), SIGNAL(infoMessage(const QString &)), SLOT(infoMessage(const QString &)));
     connect(itsDirOp, SIGNAL(updateInformation(int, int)), SLOT(updateInformation(int, int)));
+
+    if(itsKCmshell)
+    {
+        QSize defSize(450, 380);
+
+        itsSizeHint=itsConfig.readSizeEntry(CFG_SIZE, &defSize);
+    }
 }
 
 CKCmFontInst::~CKCmFontInst()
 {
-#ifdef HAVE_FT_CACHE
-    KConfigGroupSaver         cfgSaver(&itsConfig, CFG_GROUP);
-    QValueList<int>           list=itsSplitter->sizes();
-    QValueList<int>::Iterator it;
-    int                       num;
-
-    for(it=list.begin(), num=0; it!=list.end() && num<2; ++it, num++)
-        itsConfig.writeEntry(0==num ? CFG_DIRSIZE : CFG_PREVIEWSIZE, *it);
-#endif
-    if(itsAboutData)
-        delete itsAboutData;
+    if(itsPreview)
+    {
+        itsConfig.setGroup(CFG_GROUP);
+        itsConfig.writeEntry(CFG_SPLITTER_SIZES, itsSplitter->sizes());
+        if(itsKCmshell)
+            itsConfig.writeEntry(CFG_SIZE, size());
+    }
     delete itsDirOp;
 
     CGlobal::destroy();
 }
 
-const KAboutData * CKCmFontInst::aboutData() const
+QSize CKCmFontInst::sizeHint() const
 {
-    if(!itsAboutData)
-    {
-        CKCmFontInst *that = const_cast<CKCmFontInst *>(this);
-
-        that->itsAboutData=new KAboutData("kcmfontinst",
-                                          I18N_NOOP("KDE Font Installer"),
-                                          0, 0,
-                                          KAboutData::License_GPL,
-                                          I18N_NOOP("GUI front end to the fonts:/ ioslave.\n"
-                                                    "(c) Craig Drummond, 2000 - 2003"));
-        that->itsAboutData->addAuthor("Craig Drummond", I18N_NOOP("Developer and maintainer"), "craig@kde.org");
-    }
-
-    return itsAboutData;
+    return itsKCmshell ? itsSizeHint : KCModule::sizeHint();
 }
 
 QString CKCmFontInst::quickHelp() const
@@ -409,7 +434,7 @@ void CKCmFontInst::listView()
 
     itsDirOp->setView(newView);
     itsListAct->setChecked(true);
-    KConfigGroupSaver cfgSaver(&itsConfig, CFG_GROUP);
+    itsConfig.setGroup(CFG_GROUP);
     itsConfig.writeEntry(CFG_LISTVIEW, true);
     if(itsEmbeddedAdmin)
         itsConfig.sync();
@@ -422,7 +447,7 @@ void CKCmFontInst::iconView()
 
     itsDirOp->setView(newView);
     itsIconAct->setChecked(true);
-    KConfigGroupSaver cfgSaver(&itsConfig, CFG_GROUP);
+    itsConfig.setGroup(CFG_GROUP);
     itsConfig.writeEntry(CFG_LISTVIEW, false);
     if(itsEmbeddedAdmin)
         itsConfig.sync();
@@ -460,8 +485,7 @@ static QString createLocationLabel(const KURL &url)
 
 void CKCmFontInst::urlEntered(const KURL &url)
 {
-    KConfigGroupSaver cfgSaver(&itsConfig, CFG_GROUP);
-
+    itsConfig.setGroup(CFG_GROUP);
     itsConfig.writeEntry(CFG_PATH, url.path());
 
     itsEnableAct->setEnabled(false);
@@ -513,40 +537,19 @@ void CKCmFontInst::fileHighlighted(const KFileItem *item)
         itsDisableAct->setEnabled(false);
     }
 
-#ifdef HAVE_FT_CACHE
-    //
-    // Generate preview...
-    const KFileItem *previewItem=item 
-                                   ? item 
-                                   : list && 1==list->count() 
-                                         ? list->getFirst() 
-                                         : NULL;
-
-    if(previewItem && list && list->contains(previewItem))  // OK, check its been selected - not deselected!!!
+    if(itsPreview)
     {
-        QCString fName(QFile::encodeName(previewItem->url().path()));
+        //
+        // Generate preview...
+        const KFileItem *previewItem=item
+                                       ? item
+                                       : list && 1==list->count()
+                                             ? list->getFirst()
+                                             : NULL;
 
-        if(CFontEngine::isAFont(fName))
-        {
-            bool showFs=false;
-
-            if(CFontEngine::isATtc(fName) && CGlobal::fe().openKioFont(fName, CFontEngine::TEST, true))
-            {
-                if(CGlobal::fe().getNumFaces()>1)
-                {
-                    itsFaceSelector->setRange(1, CGlobal::fe().getNumFaces(), 1, false);
-                    showFs=true;
-                }
-
-                CGlobal::fe().closeFont();
-            }
-
-            itsFaceLabel->setShown(showFs);
-            itsFaceSelector->setShown(showFs);
-            itsPreview->showFont(previewItem->url());
-        }
+        if(previewItem && list && list->contains(previewItem))  // OK, check its been selected - not deselected!!!
+            itsPreview->openURL(previewItem->url());
     }
-#endif
 }
 
 void CKCmFontInst::loadingFinished()
@@ -567,10 +570,11 @@ void CKCmFontInst::loadingFinished()
 
 void CKCmFontInst::addFonts()
 {
-    KURL::List list=KFileDialog::getOpenURLs(QString::null, "application/x-font-ttf application/x-font-otf application/x-font-ttc "
-                                                            "application/x-font-type1 application/x-font-bdf application/x-font-pcf "
+    KURL::List list=KFileDialog::getOpenURLs(QString::null, "application/x-font-ttf application/x-font-otf "
+                                                            "application/x-font-ttc application/x-font-type1 "
+                                                            "application/x-font-bdf application/x-font-pcf "
                                                             "application/x-font-snf application/x-font-speedo",
-                                             this, i18n("Install Fonts"));
+                                             this, i18n("Add Fonts"));
 
     if(list.count())
         addFonts(list, itsDirOp->url());
@@ -579,7 +583,7 @@ void CKCmFontInst::addFonts()
 void CKCmFontInst::removeFonts()
 {
     if(itsDirOp->selectedItems()->isEmpty())
-        KMessageBox::information(this, i18n("You didn't select anything to delete."), i18n("Nothing to delete"));
+        KMessageBox::information(this, i18n("You did not select anything to delete."), i18n("Nothing to Delete"));
     else
     {
         KURL::List            urls;
@@ -588,7 +592,7 @@ void CKCmFontInst::removeFonts()
 
         for(; it.current(); ++it)
         {
-            files.append((*it)->url().prettyURL());
+            files.append(formatName((*it)->text()));
             urls.append((*it)->url());
         }
 
@@ -599,19 +603,15 @@ void CKCmFontInst::removeFonts()
             case 0:
                 break;
             case 1:
-                doIt = KMessageBox::Continue==KMessageBox::warningContinueCancel(this,
+                doIt = KMessageBox::Yes==KMessageBox::warningYesNo(this,
                            i18n("<qt>Do you really want to delete\n <b>'%1'</b>?</qt>").arg(files.first()),
-                           i18n("Delete Item"),
-                           i18n("Delete"),
-                           QString::null, KMessageBox::Dangerous);
+                           i18n("Delete Item"), KGuiItem(i18n("Delete"), "editdelete"));
             break;
             default:
-                doIt = KMessageBox::Continue==KMessageBox::warningContinueCancelList(this,
-                           i18n("translators: not called for n == 1", "Do you really want to delete these %n items?", files.count()),
-                           files,
-                           i18n("Delete Items"),
-                           i18n("Delete"),
-                           QString::null, KMessageBox::Dangerous);
+                doIt = KMessageBox::Yes==KMessageBox::warningYesNoList(this,
+                           i18n("translators: not called for n == 1", "Do you really want to delete these %n items?",
+                                files.count()),
+                           files, i18n("Delete Items"), KGuiItem(i18n("Delete"), "editdelete"));
         }
 
         if(doIt)
@@ -620,16 +620,24 @@ void CKCmFontInst::removeFonts()
             KURL::List::Iterator it;
 
             //
-            // Check if installign a Type1 font, if so look for corresponding .afm file to also delete
+            // Check if deleting a Type1 font, if so look for corresponding .afm file to also delete
             for(it=urls.begin(); it!=urls.end(); ++it)
                 if(CFontEngine::isAType1(QFile::encodeName((*it).path())))
                 {
-                    KURL          afmUrl(*it);
-                    KIO::UDSEntry uds;
+                    const char *others[]={ "afm", "AFM", NULL };
 
-                    afmUrl.setPath(CMisc::changeExt((*it).path(), "afm"));
-                    if(KIO::NetAccess::stat(afmUrl, uds, this))
-                        copy+=afmUrl;
+                    for(int i=0; others[i]; ++i)
+                    {
+                        KURL          afmUrl(*it);
+                        KIO::UDSEntry uds;
+
+                        afmUrl.setPath(CMisc::changeExt((*it).path(), others[i]));
+                        if(KIO::NetAccess::stat(afmUrl, uds, this))
+                        {
+                            copy+=afmUrl;
+                            break;
+                        }
+                    }
                 }
 
             KIO::DeleteJob *job = KIO::del(copy, false, true);
@@ -684,17 +692,6 @@ void CKCmFontInst::openUrlInBrowser(const QString &url)
     }
 }
 
-#ifdef HAVE_FT_CACHE
-void CKCmFontInst::showFace(int face)
-{
-    itsPreview->showFace(face);
-}
-#else
-void CKCmFontInst::showFace(int)
-{
-}
-#endif
-
 void CKCmFontInst::infoMessage(const QString &msg)
 {
     itsStatusLabel->setText(msg);
@@ -732,7 +729,12 @@ void CKCmFontInst::jobResult(KIO::Job *job)
     // Force an update of the view. For some reason the view is not automatically updated when
     // run in embedded mode - e.g. from the "Admin" mode button on KControl.
     if(job && 0==job->error())
+    {
         itsDirOp->dirLister()->updateDirectory(itsDirOp->url());
+        KMessageBox::information(this, i18n("Please note that any open applications will need to be restarted in order "
+                                            "for any changes to be noticed."),
+                                 i18n("Success"), "KFontinst_WarnAboutFontChangesAndOpenApps");
+    }
 }
 
 void CKCmFontInst::setUpAct()
@@ -750,8 +752,9 @@ void CKCmFontInst::enableItems(bool enable)
 
     if (items->isEmpty())
         KMessageBox::information(this,
-                                enable ? i18n("You didn't select anything to enable.") : i18n("You didn't select anything to disable."),
-                                enable ? i18n("Nothing to enable") : i18n("Nothing to disable"));
+                                enable ? i18n("You did not select anything to enable.")
+                                       : i18n("You did not select anything to disable."),
+                                enable ? i18n("Nothing to Enable") : i18n("Nothing to Disable"));
     else
     {
         KFileItemListIterator it(*items);
@@ -764,7 +767,7 @@ void CKCmFontInst::enableItems(bool enable)
             if((enable && disabled) || (!enable && !disabled))
             {
                 urls.append(url);
-                files.append(url.prettyURL());
+                files.append(formatName((*it)->text()));
             }
         }
     }
@@ -776,21 +779,21 @@ void CKCmFontInst::enableItems(bool enable)
         case 0:
             break;
         case 1:
-            doIt = KMessageBox::Continue==KMessageBox::warningContinueCancel(this,
+            doIt = KMessageBox::Yes==KMessageBox::warningYesNo(this,
                        enable ? i18n("<qt>Do you really want to enable\n <b>'%1'</b>?</qt>").arg(files.first())
                               : i18n("<qt>Do you really want to disable\n <b>'%1'</b>?</qt>").arg(files.first()),
                        enable ? i18n("Enable Item") : i18n("Disable Item"),
-                       enable ? i18n("Enable") : i18n("Disable"),
-                       QString::null, KMessageBox::Dangerous);
+                       enable ? KGuiItem(i18n("Enable"), "button_ok") : KGuiItem(i18n("Disable"), "button_cancel"));
             break;
         default:
-            doIt = KMessageBox::Continue==KMessageBox::warningContinueCancelList(this,
-                       enable ? i18n("translators: not called for n == 1", "Do you really want to enable these %n items?", files.count())
-                              : i18n("translators: not called for n == 1", "Do you really want to disable these %n items?", files.count()),
+            doIt = KMessageBox::Yes==KMessageBox::warningYesNoList(this,
+                       enable ? i18n("translators: not called for n == 1", "Do you really want to enable these %n items?",
+                                     files.count())
+                              : i18n("translators: not called for n == 1", "Do you really want to disable these %n items?",
+                                     files.count()),
                        files,
                        enable ? i18n("Enable Items") : i18n("Disable Items"),
-                       enable ? i18n("Enable") : i18n("Disable"),
-                       QString::null, KMessageBox::Dangerous);
+                       enable ? KGuiItem(i18n("Enable"), "button_ok") : KGuiItem(i18n("Disable"), "button_cancel"));
     }
 
     if(doIt)
@@ -800,7 +803,7 @@ void CKCmFontInst::enableItems(bool enable)
         CRenameJob::Entry::List renameList;
 
         //
-        // Check if installign a Type1 font, if so look for corresponding .afm file to also enable/disable
+        // Check if enabling/disabling a Type1 font, if so look for corresponding .afm file to also enable/disable
         for(it=urls.begin(); it!=urls.end(); ++it)
             if(CFontEngine::isAType1(QFile::encodeName((*it).path())))
             {
@@ -847,7 +850,8 @@ void CKCmFontInst::addFonts(const KURL::List &src, const KURL &dest)
                 KIO::UDSEntry uds;
 
                 afmUrl.setPath(afm);
-                if(KIO::NetAccess::stat(afmUrl, uds, this) && !KIO::NetAccess::stat(destUrl, uds, this) && -1==copy.findIndex(afmUrl))
+                if(KIO::NetAccess::stat(afmUrl, uds, this) && !KIO::NetAccess::stat(destUrl, uds, this) &&
+                   -1==copy.findIndex(afmUrl))
                     copy+=afmUrl;
             }
 
