@@ -4,7 +4,7 @@
 // Author        : Craig Drummond
 // Project       : K Font Installer
 // Creation Date : 29/04/2001
-// Version       : $Revision: 1.2 $ $Date: 2003/11/01 18:10:01 $
+// Version       : $Revision: 1.4 $ $Date: 2004/04/22 18:56:18 $
 //
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -37,6 +37,9 @@
 #include <fstream>
 
 #ifdef HAVE_FONT_ENC
+
+#include <unistd.h>
+
 extern "C"
 {
 #include <X11/fonts/fontenc.h>
@@ -634,6 +637,9 @@ CEncodings::CEncodings()
         "iso8859-4",
         "iso8859-5",
         "iso8859-6",
+        "iso8859-6.8",
+        "iso8859-6.8x",
+        "iso8859-6.16",
         "iso8859-7",
         "iso8859-8",
         "iso8859-9",
@@ -643,15 +649,22 @@ CEncodings::CEncodings()
         "iso8859-13",
         "iso8859-14",
         "iso8859-15",
+        "iso8859-16",
         "fcd8859-1",
+        "ansi-1251",
         "koi8-r",
         "koi8-u",
         "koi8-ru",
         "koi8-e",
+        "koi8-uni",
         "tis620-2",
+        "sun.unicode.india-0",
         "adobe-standard",
         "adobe-symbol",
         "ibm-cp437",
+        "ibm-cp850",
+        "ibm-cp852",
+        "ibm-cp866",
         "microsoft-cp1252",
         // Not "adobe-dingbats", as it uses generic glyph names
         "jisx0201.1976-0",
@@ -659,7 +672,11 @@ CEncodings::CEncodings()
         "jisx0208.1990-0",
         "jisx0212.1990-0",
         "big5-0",
+        "big5.eten-0",
+        "big5hkscs-0",
         "gb2312.1980-0",
+        "gb18030.2000-0",
+        "gb18030.2000-1",
         "ksc5601.1987-0",
         "ksc5601.1992-3",
         NULL
@@ -732,9 +749,9 @@ bool CEncodings::createEncodingsDotDir(const QString &dir)
         else
         {
             if(CMisc::fExists(destFile))
-                CMisc::doCmd("rm", "-f", destFile);
+                unlink(QFile::encodeName(destFile).data());
 
-            status=CMisc::doCmd("ln", "-s", sysFile, destFile);
+            status=0==symlink(QFile::encodeName(sysFile).data(), QFile::encodeName(destFile).data());
         }
     }
 
