@@ -11,8 +11,7 @@
 #include <qstrlist.h>
 #include <qcheckbox.h>
 #include <qlineedit.h>
-
-#include <kcmodule.h>
+#include <qcombobox.h>
 
 
 //-----------------------------------------------------------------------------
@@ -26,6 +25,7 @@
 
 #include <qstring.h>
 #include <kconfig.h>
+#include <kcmodule.h>
 class QRadioButton;
 class KIntNumInput;
 
@@ -35,24 +35,33 @@ class KMiscHTMLOptions : public KCModule
 
 public:
     KMiscHTMLOptions(KConfig *config, QString group, QWidget *parent = 0L, const char *name = 0L );
+	~KMiscHTMLOptions();
+    QString quickHelp() const;
     virtual void load();
     virtual void save();
     virtual void defaults();
 
 private slots:
-    void changed();
+    void slotChanged();
+    void launchAdvancedTabDialog();
 
 private:
     KConfig* m_pConfig;
     QString  m_groupname;
 
-    QRadioButton *m_pUnderlineRadio[3];
-    QCheckBox* cbCursor;
+    QComboBox* m_pUnderlineCombo;
+    QComboBox* m_pAnimationsCombo;
+    QCheckBox* m_cbCursor;
     QCheckBox* m_pAutoLoadImagesCheckBox;
-    QCheckBox* m_pEnableFaviconCheckBox;
+    QCheckBox* m_pAutoRedirectCheckBox;
+    QCheckBox* m_pOpenMiddleClick;
+    QCheckBox* m_pBackRightClick;
+    QCheckBox* m_pShowMMBInTabs;
     QCheckBox* m_pFormCompletionCheckBox;
+    QCheckBox* m_pDynamicTabbarHide;
+    QCheckBox* m_pAdvancedAddBookmarkCheckBox;
+    QCheckBox* m_pOnlyMarkedBookmarksCheckBox;
     KIntNumInput* m_pMaxFormCompletionItems;
-
 };
 
 #endif
