@@ -2,8 +2,6 @@
 #ifndef __KHTTPOPTDLG_H
 #define __KHTTPOPTDLG_H
 
-
-#include <qwidget.h>
 #include <qlabel.h>
 #include <qlineedit.h>
 #include <qcheckbox.h>
@@ -43,8 +41,7 @@ Q_OBJECT
     QString defaultCharsets;
 
 private slots:
-
-  void changed();
+    void slotChanged();
 
 };
 
