@@ -364,8 +364,8 @@ void KPluginOptions::dirLoad( KConfig *config )
         paths.append("/usr/lib/netscape/plugins-libc5");
         paths.append("/usr/lib/netscape/plugins-libc6");
         paths.append("/usr/lib/mozilla/plugins");
-        paths.append("/usr/lib64/netscape/plugins");
-        paths.append("/usr/lib64/mozilla/plugins");	
+	paths.append("/usr/lib64/netscape/plugins");
+	paths.append("/usr/lib64/mozilla/plugins");
         paths.append("$MOZILLA_HOME/plugins");
     }
 
@@ -577,7 +577,7 @@ PluginDomainDialog::PluginDomainDialog(QWidget *parent) :
   QBoxLayout *hl = new QHBoxLayout(this,0,6);
   hl->addStretch(10);
 
-  QPushButton *closePB = new QPushButton(i18n("&Close"),this);
+  QPushButton *closePB = new KPushButton(KStdGuiItem::close(),this);
   connect(closePB,SIGNAL(clicked()),SLOT(slotClose()));
   hl->addWidget(closePB);
   thisLayout->addLayout(hl);
