@@ -21,14 +21,11 @@
 #ifndef KCONTROL_NIC_H
 #define KCONTROL_NIC_H
 
-#include <qtabwidget.h>
-#include <kglobal.h>
-#include <qlayout.h>
-#include <qpushbutton.h>
-#include <qlistview.h>
-
 #include <kcmodule.h>
-#include <kaboutdata.h>
+
+class QStringList;
+class QPushButton;
+class QListView;
 
 class KCMNic:public KCModule
 {
@@ -36,7 +33,6 @@ class KCMNic:public KCModule
    public:
       KCMNic(QWidget *parent=0, const char * name=0, const QStringList &list = QStringList( ));
       virtual ~KCMNic() {};
-      const KAboutData* aboutData() const;
 
    protected slots:
       void update();
