@@ -92,7 +92,7 @@ extern "C"
 
     void init_style()
     {
-        uint flags = KRdbExportQtSettings | KRdbExportQtColors;
+        uint flags = KRdbExportQtSettings | KRdbExportQtColors | KRdbExportXftSettings;
         KConfig config("kcmdisplayrc", true /*readonly*/, false /*don't read kdeglobals etc.*/);
         config.setGroup("X11");
 
@@ -141,6 +141,17 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 	KGlobal::dirs()->addResourceType("themes",
 		KStandardDirs::kde_default("data") + "kstyle/themes");
 
+	KAboutData *about =
+		new KAboutData( I18N_NOOP("kcmstyle"),
+						I18N_NOOP("KDE Style Module"),
+						0, 0, KAboutData::License_GPL,
+						I18N_NOOP("(c) 2002 Karol Szwed, Daniel Molkentin"));
+
+	about->addAuthor("Karol Szwed", 0, "gallium@kde.org");
+	about->addAuthor("Daniel Molkentin", 0, "molkentin@kde.org");
+	about->addAuthor("Ralf Nolden", 0, "nolden@kde.org");
+	setAboutData( about );
+
 	// Setup pages and mainLayout
 	mainLayout = new QVBoxLayout( this );
 	tabWidget  = new QTabWidget( this );
@@ -353,11 +364,11 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 	// Page2
 	connect( cbEnableEffects,     SIGNAL(toggled(bool)),    this, SLOT(setEffectsDirty()));
 	connect( cbEnableEffects,     SIGNAL(toggled(bool)),    this, SLOT(setStyleDirty()));
-	connect( comboTooltipEffect,  SIGNAL(highlighted(int)), this, SLOT(setEffectsDirty()));
-	connect( comboComboEffect,    SIGNAL(highlighted(int)), this, SLOT(setEffectsDirty()));
-	connect( comboMenuEffect,     SIGNAL(highlighted(int)), this, SLOT(setStyleDirty()));
-	connect( comboMenuHandle,     SIGNAL(highlighted(int)), this, SLOT(setStyleDirty()));
-	connect( comboMenuEffectType, SIGNAL(highlighted(int)), this, SLOT(setStyleDirty()));
+	connect( comboTooltipEffect,  SIGNAL(activated(int)), this, SLOT(setEffectsDirty()));
+	connect( comboComboEffect,    SIGNAL(activated(int)), this, SLOT(setEffectsDirty()));
+	connect( comboMenuEffect,     SIGNAL(activated(int)), this, SLOT(setStyleDirty()));
+	connect( comboMenuHandle,     SIGNAL(activated(int)), this, SLOT(setStyleDirty()));
+	connect( comboMenuEffectType, SIGNAL(activated(int)), this, SLOT(setStyleDirty()));
 	connect( slOpacity,           SIGNAL(valueChanged(int)),this, SLOT(setStyleDirty()));
 	connect( cbMenuShadow,        SIGNAL(toggled(bool)),    this, SLOT(setStyleDirty()));
 	// Page3
@@ -366,7 +377,7 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 	connect( cbEnableTooltips,      SIGNAL(toggled(bool)),    this, SLOT(setEffectsDirty()));
 	connect( cbIconsOnButtons,      SIGNAL(toggled(bool)),    this, SLOT(setEffectsDirty()));
 	connect( cbTearOffHandles,      SIGNAL(toggled(bool)),    this, SLOT(setEffectsDirty()));
-	connect( comboToolbarIcons,     SIGNAL(highlighted(int)), this, SLOT(setToolbarsDirty()));
+	connect( comboToolbarIcons,     SIGNAL(activated(int)), this, SLOT(setToolbarsDirty()));
 
 	addWhatsThis();
 
@@ -374,7 +385,7 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 	tabWidget->insertTab( page1, i18n("&Style"));
 	tabWidget->insertTab( page2, i18n("&Effects"));
 	tabWidget->insertTab( page3, i18n("&Miscellaneous"));
-	
+
 	//Enable/disable the button for the initial style
 	updateConfigButton();
 }
@@ -430,16 +441,16 @@ void KCMStyle::styleSpecificConfig()
 	//Create the container dialog
 	StyleConfigDialog* dial = new StyleConfigDialog(this, styleEntries[currentStyle()]->name);
 	dial->enableButtonSeparator(true);
-	
+
 	typedef QWidget*(* factoryRoutine)( QWidget* parent );
 
 	//Get the factory, and make the widget.
 	factoryRoutine factory      = (factoryRoutine)(allocPtr); //Grmbl. So here I am on my
 	//"never use C casts" moralizing streak, and I find that one can't go void* -> function ptr
 	//even with a reinterpret_cast.
-	
+
 	QWidget*       pluginConfig = factory( dial );
-	
+
 	//Insert it in...
 	dial->setMainWidget( pluginConfig );
 
@@ -451,14 +462,14 @@ void KCMStyle::styleSpecificConfig()
 	if (dial->exec() == QDialog::Accepted  && dial->isDirty() ) {
 		// Force re-rendering of the preview, to apply settings
 		switchStyle(currentStyle());
-		
+
 		//For now, ask all KDE apps to recreate their styles to apply the setitngs
 		KIPC::sendMessageAll(KIPC::StyleChanged);
-		
+
 		// We call setStyleDirty here to make sure we force style re-creation
 		setStyleDirty();
 	}
-	
+
 	delete dial;
 }
 
@@ -633,9 +644,9 @@ void KCMStyle::save()
 bool KCMStyle::findStyle( const QString& str, int& combobox_item )
 {
 	StyleEntry* se   = styleEntries.find(str.lower());
-	
+
 	QString     name = se ? se->name : str;
-	
+
 	combobox_item = 0;
 
 	//look up name
@@ -695,21 +706,6 @@ void KCMStyle::defaults()
 }
 
 
-const KAboutData* KCMStyle::aboutData() const
-{
-	KAboutData *about =
-		new KAboutData( I18N_NOOP("kcmstyle"),
-						I18N_NOOP("KDE Style Module"),
-						0, 0, KAboutData::License_GPL,
-						I18N_NOOP("(c) 2002 Karol Szwed, Daniel Molkentin"));
-
-	about->addAuthor("Karol Szwed", 0, "gallium@kde.org");
-	about->addAuthor("Daniel Molkentin", 0, "molkentin@kde.org");
-	about->addAuthor("Ralf Nolden", 0, "nolden@kde.org");
-	return about;
-}
-
-
 QString KCMStyle::quickHelp() const
 {
 	return i18n("<h1>Style</h1>"
@@ -751,7 +747,7 @@ void KCMStyle::loadStyle( KConfig& config )
 
 	QString strWidgetStyle;
 	QStringList list = KGlobal::dirs()->findAllResources("themes", "*.themerc", true, true);
-	for (QStringList::iterator it = list.begin(); it != list.end(); it++)
+	for (QStringList::iterator it = list.begin(); it != list.end(); ++it)
 	{
 		KSimpleConfig config( *it, true );
 		if ( !(config.hasGroup("KDE") && config.hasGroup("Misc")) )
@@ -795,7 +791,7 @@ void KCMStyle::loadStyle( KConfig& config )
 				continue;
 
 			styles += entry->name;
-			
+
 			nameToStyleKey[entry->name] = id;
 		}
 		else
@@ -863,12 +859,12 @@ void KCMStyle::switchStyle(const QString& styleName)
 
 	// Prevent Qt from wrongly caching radio button images
 	QPixmapCache::clear();
-	
+
 	setStyleRecursive( stylePreview, style );
-	
+
 	// this flickers, but reliably draws the widgets correctly.
 	stylePreview->resize( stylePreview->sizeHint() );
-	
+
 	delete appliedStyle;
 	appliedStyle = style;
 
@@ -1066,12 +1062,12 @@ void KCMStyle::addWhatsThis()
 							"For best performance, it is advisable to disable all effects.") );
 	QWhatsThis::add( cbEnableEffects, i18n( "If you check this box, you can select several effects "
 							"for different widgets like combo boxes, menus or tooltips.") );
-	QWhatsThis::add( comboComboEffect, i18n( "<p><b>Disable: </b>Don't use any combo box effects.</p>\n"
+	QWhatsThis::add( comboComboEffect, i18n( "<p><b>Disable: </b>do not use any combo box effects.</p>\n"
 							"<b>Animate: </b>Do some animation.") );
-	QWhatsThis::add( comboTooltipEffect, i18n( "<p><b>Disable: </b>Don't use any tooltip effects.</p>\n"
+	QWhatsThis::add( comboTooltipEffect, i18n( "<p><b>Disable: </b>do not use any tooltip effects.</p>\n"
 							"<p><b>Animate: </b>Do some animation.</p>\n"
 							"<b>Fade: </b>Fade in tooltips using alpha-blending.") );
-	QWhatsThis::add( comboMenuEffect, i18n( "<p><b>Disable: </b>Don't use any menu effects.</p>\n"
+	QWhatsThis::add( comboMenuEffect, i18n( "<p><b>Disable: </b>do not use any menu effects.</p>\n"
 							"<p><b>Animate: </b>Do some animation.</p>\n"
 							"<p><b>Fade: </b>Fade in menus using alpha-blending.</p>\n"
 							"<b>Make Translucent: </b>Alpha-blend menus for a see-through effect. (KDE styles only)") );
@@ -1087,7 +1083,7 @@ void KCMStyle::addWhatsThis()
 
 	// Page3
 	QWhatsThis::add( gbToolbarSettings, i18n("<b>Note:</b> that all widgets in this combobox "
-							"do not apply to Qt-only applications!") );
+							"do not apply to Qt-only applications.") );
 	QWhatsThis::add( cbHoverButtons, i18n("If this option is selected, toolbar buttons will change "
 							"their color when the mouse cursor is moved over them." ) );
 	QWhatsThis::add( cbTransparentToolbars, i18n("If you check this box, the toolbars will be "
