@@ -1,6 +1,4 @@
 /*
- * $Id: kdcopwindow.cpp,v 1.41.2.2 2004/02/07 08:23:14 ogoffart Exp $
- *
  * Copyright (C) 2000 by Matthias Kalle Dalheimer <kalle@kde.org>
  *
  * Licensed under the Artistic License.
@@ -20,9 +18,11 @@
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <klineedit.h>
+#include <keditlistbox.h>
 #include <klistbox.h>
 #include <kdialogbase.h>
 #include <kstdaccel.h>
+#include <kcolorbutton.h>
 
 #include <qtimer.h>
 #include <qwidgetstack.h>
@@ -41,17 +41,47 @@
 #include <qclipboard.h>
 #include <qdatetime.h>
 #include <dcopref.h>
+#include <qvbox.h>
+#include <qimage.h>
 
 #include <kdebug.h>
 #include <kkeydialog.h>
 #include <stdio.h>
 #include <kstatusbar.h>
 #include <kurl.h>
+#include <kurlrequester.h>
 
 class DCOPBrowserApplicationItem;
 class DCOPBrowserInterfaceItem;
 class DCOPBrowserFunctionItem;
 
+//------------------------------
+
+class KMultiIntEdit : public QVBox
+{
+public:
+	KMultiIntEdit(QWidget *parent , const char * name=0) : QVBox(parent,name) {}
+	void addField(int key, const QString & caption  )
+	{
+		QHBox *l=new QHBox(this);
+		new QLabel(caption + ": ", l);
+		KLineEdit* e = new KLineEdit( l );
+		m_widgets.insert(key, e ) ;
+        	e->setValidator( new QIntValidator( e ) );
+	}
+	int field(int key)
+	{
+		KLineEdit *e=m_widgets[key];
+		if(!e) return 0;
+		return e->text().toInt();
+	}
+	
+private:
+	QMap<int,KLineEdit*> m_widgets;
+};
+
+//------------------------------
+
 DCOPBrowserItem::DCOPBrowserItem
 (QListView * parent, DCOPBrowserItem::Type type)
   : QListViewItem(parent),
@@ -81,6 +111,35 @@ DCOPBrowserApplicationItem::DCOPBrowserApplicationItem
 {
   setExpandable(true);
   setText(0, QString::fromUtf8(app_));
+  setPixmap(0,  KGlobal::iconLoader()->loadIcon( QString::fromLatin1( "exec" ), KIcon::Small ));
+
+
+	/* Get the icon:  we use the icon from a mainwindow in that class.
+	    a lot of applications has a app-mainwindow#1 object, but others can still have 
+	    a main window with another name. In that case, we search for a main window with the qt object.
+	 * Why don't search with qt dirrectly?
+	    simply because some application that have a 'mainwindow#1' doesn't have a qt object.  And, for 
+	    reason i don't know, some application return stanges result. Some konqueror instance are returning
+	    konqueror-mainwindow#3 while only the #1 exists,  I already seen the same problem with konsole 
+	 * All calls are async to don't block the GUI if the clients does not reply immediatly 
+	 */
+     
+	QRegExp rx( "([^\\-]+)");  // remove the possible processus id   
+	rx.search(app_);           //   konqueror-123  => konqueror-mainwindow#1
+	QString mainWindowName= rx.cap(1) + "-mainwindow#1" ;
+	
+	QByteArray data;
+	int callId=kapp->dcopClient()->callAsync( app_, mainWindowName.utf8(), "icon()", data, this, SLOT(retreiveIcon(int, const QCString&, const QByteArray&)));
+	
+	if(!callId)
+	{
+		//maybe there is another mainwindow registered with another name.
+		QByteArray data;
+		QDataStream arg(data, IO_WriteOnly);
+		arg << QCString( "MainWindow" );
+		
+		kapp->dcopClient()->callAsync( app_, "qt", "find(QCString)", data, this, SLOT(slotGotWindowName(int, const QCString&, const QByteArray& )));
+	} 
 }
 
   void
@@ -116,6 +175,35 @@ DCOPBrowserApplicationItem::populate()
   KApplication::restoreOverrideCursor();
 }
 
+void DCOPBrowserApplicationItem::slotGotWindowName(int /*callId*/, const QCString& /*replyType*/, const QByteArray &replyData)
+{
+	QDataStream reply(replyData, IO_ReadOnly);
+	QCStringList mainswindows;
+	reply >> mainswindows;
+	QStringList sl=QStringList::split("/",mainswindows.first() );
+	if(sl.count() >= 1)
+	{
+		QString mainWindowName=sl[1];
+		if(!mainWindowName.isEmpty())
+		{
+			QByteArray data;
+			kapp->dcopClient()->callAsync( app_, mainWindowName.utf8(), "icon()", data, 
+				this, SLOT(retreiveIcon(int, const QCString&, const QByteArray&)));
+		}
+	}
+}
+
+void DCOPBrowserApplicationItem::retreiveIcon(int /*callId*/,const QCString& /*replyType*/, const QByteArray &replyData)
+{
+	QDataStream reply(replyData, IO_ReadOnly);
+	QPixmap returnQPixmap;
+	reply >> returnQPixmap;
+	if(!returnQPixmap.isNull())
+		setPixmap(0, QPixmap(returnQPixmap.convertToImage().smoothScale(16,16)) );  
+	else
+		kdDebug() << "Unable to retreive the icon" << endl;
+}
+
 // ------------------------------------------------------------------------
 
 DCOPBrowserInterfaceItem::DCOPBrowserInterfaceItem
@@ -221,8 +309,9 @@ KDCOPWindow::KDCOPWindow(QWidget *parent, const char * name)
   // set up the actions
   KStdAction::quit( this, SLOT( close() ), actionCollection() );
   KStdAction::copy( this, SLOT( slotCopy()), actionCollection() );
-  KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );
+  KStdAction::keyBindings( guiFactory(), SLOT( configureShortcuts() ), actionCollection() );
   
+
   (void) new KAction( i18n( "&Reload" ), "reload", KStdAccel::shortcut(KStdAccel::Reload), this, SLOT( slotReload() ), actionCollection(), "reload" );
 
   exeaction =
@@ -284,11 +373,6 @@ void KDCOPWindow::slotCurrentChanged( QListViewItem* i )
 }
 
 
-void KDCOPWindow::slotConfigureKeys()
-{
-  KKeyDialog::configure( actionCollection(), this );
-}
-
 void KDCOPWindow::slotCallFunction()
 {
   slotCallFunction( mainView->lv->currentItem() );
@@ -319,7 +403,7 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
   if (!getParameters(unNormalisedSignature, normalisedSignature, types, names))
   {
     KMessageBox::error
-      (this, i18n("No parameters found"), i18n("DCOP Browser Error"));
+      (this, i18n("No parameters found."), i18n("DCOP Browser Error"));
 
     return;
   }
@@ -331,7 +415,7 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
 
   QDataStream arg(data, IO_WriteOnly);
 
-  KDialogBase mydialog( this, "KDCOP parameter entry", true,
+  KDialogBase mydialog( this, "KDCOP Parameter Entry", true,
     QString::null, KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, true );
 
   mydialog.setCaption
@@ -465,6 +549,26 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
         grid->addWidget( e, i, 2 );
         wl.append( e );
       }
+      else if ( type == "QStringList" )
+      {
+        QLabel* n = new QLabel( names[i-1], frame );
+        grid->addWidget( n, i, 0 );
+        QLabel* l = new QLabel( "QStringList", frame );
+        grid->addWidget( l, i, 1 );
+        KEditListBox* e = new KEditListBox ( frame );
+        grid->addWidget( e, i, 2 );
+        wl.append( e );
+      }
+      else if ( type == "QValueList<QCString>" )
+      {
+        QLabel* n = new QLabel( names[i-1], frame );
+        grid->addWidget( n, i, 0 );
+        QLabel* l = new QLabel( "QValueList<QCString>", frame );
+        grid->addWidget( l, i, 1 );
+        KEditListBox* e = new KEditListBox ( frame );
+        grid->addWidget( e, i, 2 );
+        wl.append( e );
+      }
       else if ( type == "KURL" )
       {
         QLabel* n = new QLabel( names[i-1], frame );
@@ -475,6 +579,64 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
         grid->addWidget( e, i, 2 );
         wl.append( e );
       }
+      else if ( type == "QColor" )
+      {
+        QLabel* n = new QLabel( names[i-1], frame );
+        grid->addWidget( n, i, 0 );
+        QLabel* l = new QLabel( "QColor", frame );
+        grid->addWidget( l, i, 1 );
+        KColorButton* e = new KColorButton( frame );
+        grid->addWidget( e, i, 2 );
+        wl.append( e );
+      }
+      else if ( type == "QSize" )
+      {
+        QLabel* n = new QLabel( names[i-1], frame );
+        grid->addWidget( n, i, 0 );
+        QLabel* l = new QLabel( "QSize", frame );
+        grid->addWidget( l, i, 1 );
+        KMultiIntEdit* e = new KMultiIntEdit( frame );
+        e->addField( 1, i18n("Width") );
+        e->addField( 2, i18n("Height") );
+        grid->addWidget( e, i, 2 );
+        wl.append( e );
+      }
+      else if ( type == "QPoint" )
+      {
+        QLabel* n = new QLabel( names[i-1], frame );
+        grid->addWidget( n, i, 0 );
+        QLabel* l = new QLabel( "QPoint", frame );
+        grid->addWidget( l, i, 1 );
+        KMultiIntEdit* e = new KMultiIntEdit( frame );
+        e->addField( 1, i18n("X") );
+        e->addField( 2, i18n("Y") );
+        grid->addWidget( e, i, 2 );
+        wl.append( e );
+      }
+      else if ( type == "QRect" )
+      {
+        QLabel* n = new QLabel( names[i-1], frame );
+        grid->addWidget( n, i, 0 );
+        QLabel* l = new QLabel( "QRect", frame );
+        grid->addWidget( l, i, 1 );
+        KMultiIntEdit* e = new KMultiIntEdit( frame );
+        e->addField( 1, i18n("Left") );
+        e->addField( 2, i18n("Top") );
+        e->addField( 3, i18n("Width") );
+        e->addField( 4, i18n("Height") );
+        grid->addWidget( e, i, 2 );
+        wl.append( e );
+      }
+      else if( type == "QPixmap" )
+      {
+        QLabel* n = new QLabel( names[i-1], frame );
+        grid->addWidget( n, i, 0 );
+        QLabel* l = new QLabel( "KURL", frame );
+        grid->addWidget( l, i, 1 );
+        KURLRequester* e = new KURLRequester( frame );
+        grid->addWidget( e, i, 2 );
+        wl.append( e );
+      }
       else
       {
         KMessageBox::sorry(this, i18n("Cannot handle datatype %1").arg(type));
@@ -551,11 +713,47 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
         KLineEdit* e = (KLineEdit*)wl.at( i );
         arg << e->text();
       }
+      else if( type == "QStringList" )
+      {
+        KEditListBox* e = (KEditListBox*)wl.at( i );
+        arg << e->items();
+      }
+      else if( type == "QValueList<QCString>" )
+      {
+        KEditListBox* e = (KEditListBox*)wl.at( i );
+        for (int i = 0; i < e->count(); i++)
+          arg << QCString( e->text(i).local8Bit() );
+      }
       else if( type == "KURL" )
       {
         KLineEdit* e = (KLineEdit*)wl.at( i );
         arg << KURL( e->text() );
       }
+      else if( type == "QColor" )
+      {
+       KColorButton* e = (KColorButton*)wl.at( i );
+       arg << e->color();
+      }
+      else if( type == "QSize" )
+      {
+       KMultiIntEdit* e = (KMultiIntEdit*)wl.at( i );
+       arg << QSize(e->field(1) , e->field(2)) ;
+      }
+      else if( type == "QPoint" )
+      {
+       KMultiIntEdit* e = (KMultiIntEdit*)wl.at( i );
+       arg << QPoint(e->field(1) , e->field(2)) ;
+      }
+      else if( type == "QRect" )
+      {
+       KMultiIntEdit* e = (KMultiIntEdit*)wl.at( i );
+       arg << QRect(e->field(1) , e->field(2) , e->field(3) , e->field(4)) ;
+      }
+      else if( type == "QPixmap" )
+      {
+       KURLRequester* e= (KURLRequester*)wl.at( i );
+       arg << QPixmap(e->url());        
+      }
       else
       {
         KMessageBox::sorry(this, i18n("Cannot handle datatype %1").arg(type));
@@ -566,6 +764,8 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
     }
   }
 
+  DCOPRef( fitem->app(), "MainApplication-Interface" ).call( "updateUserTimestamp", kapp->userTimestamp());
+
   // Now do the DCOP call
 
   bool callOk =
@@ -604,8 +804,8 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
         (
          i18n
          (
-          "<p>Application is still registered with DCOP."
-          " I don't know why this call failed.</p>"
+          "<p>Application is still registered with DCOP;"
+          " I do not know why this call failed.</p>"
          )
         );
     }
@@ -859,7 +1059,7 @@ bool KDCOPWindow::demarshal
   else
   {
     ret <<
-      i18n("Don't know how to demarshal %1").arg(QString::fromUtf8(replyType));
+      i18n("Do not know how to demarshal %1").arg(QString::fromUtf8(replyType));
 	isValid = false;
   }
 
