@@ -14,7 +14,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
  */
 
@@ -22,7 +22,7 @@
 #define CONFFILTERS_H
 
 #include <qwidget.h>
-
+#include <qpushbutton.h>
 class KListView;
 
 class ConfFilters : public QWidget
@@ -33,16 +33,16 @@ public:
 
 	void load();
 	void save();
-
 protected slots:
 	void slotAdd();
 	void slotRemove();
 	void slotChange();
 	void slotUp();
 	void slotDown();
-
+	void updateButton();
 private:
 	KListView	*m_filters;
+	QPushButton	*m_add,*m_remove,*m_up,*m_down,*m_change;
 };
 
 #endif
