@@ -14,7 +14,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
  */
 
@@ -22,21 +22,29 @@
 #define CONFSYSTEM_H
 
 #include <qwidget.h>
+#include <qstringlist.h>
 
 class QLineEdit;
-class QButtonGroup;
+class QComboBox;
 
 class ConfSystem : public QWidget
 {
+	Q_OBJECT
 public:
 	ConfSystem(QWidget *parent = 0, const char *name = 0);
 
 	void load();
 	void save();
 
+protected slots:
+	void slotSystemChanged(int);
+	void slotDeviceChanged(int);
+
 private:
-	QButtonGroup	*m_system;
-	QLineEdit	*m_efax, *m_hfax;
+	QComboBox	*m_system, *m_device;
+	QLineEdit	*m_command, *m_server, *m_device_edit;
+	QStringList	m_commands;
+	int		m_current;
 };
 
 #endif
