@@ -17,6 +17,7 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include <kuniqueapplication.h>
 #include <klocale.h>
 #include <dcopclient.h>
@@ -38,10 +39,10 @@
 #include <fcntl.h>
 #include <stdlib.h>
 
-static const char *description =
-        I18N_NOOP("The KDE Desktop.");
+static const char description[] =
+        I18N_NOOP("The KDE desktop");
 
-static const char *version = KDE_VERSION_STRING;
+static const char version[] = VERSION;
 
 static KCmdLineOptions options[] =
 {
