@@ -8,35 +8,16 @@
 #ifndef __XAUTOLOCK_H__
 #define __XAUTOLOCK_H__
 
-#include <time.h>
-#include <qobject.h>
-#include <qqueue.h>
+#include <qwidget.h>
+
 #include <X11/Xlib.h>
 
-//===========================================================================
-//
-// A window and its creation time.
-//
-class WatchedWindow
-{
-public:
-    WatchedWindow(Window w)
-        { mWindow = w; mTime = time(0); }
-
-    time_t created() { return mTime; }
-    Window window() { return mWindow; }
-
-protected:
-    Window mWindow;
-    time_t mTime;
-};
-
 //===========================================================================
 //
 // Detect user inactivity.
 // Named XAutoLock after the program that it is based on.
 //
-class XAutoLock : public QObject
+class XAutoLock : public QWidget
 {
     Q_OBJECT
 public:
@@ -48,7 +29,7 @@ public:
     // The time in seconds of continuous inactivity.
     //
     void setTimeout(int t);
-
+    
     //-----------------------------------------------------------------------
     //
     // Start watching Activity
@@ -61,28 +42,26 @@ public:
     //
     void stop();
 
-public slots:
-    void windowCreated(Window w);
-    void keyPressed();
-
+    // internal
+    void resetTrigger();
+    // internal
+    void setTrigger( time_t );
+    // internal
+    bool ignoreWindow( WId );
+    
 signals:
     void timeout();
 
 protected:
-    void resetTrigger();
-    void processWatched(time_t delay);
-    void selectEvents(Window window, bool substructure_only);
-    void queryPointer();
     virtual void timerEvent(QTimerEvent *ev);
+    virtual bool x11Event( XEvent* );
 
 protected:
     int     mTimerId;
     int     mTimeout;
-    int     mTrigger;
+    time_t  mTrigger;
     bool    mActive;
     time_t  mLastTimeout;
-    QQueue<WatchedWindow>  mWindows;
 };
 
 #endif
-
