@@ -1,10 +1,13 @@
-/* $TOG: auth.c /main/64 1998/02/26 10:02:22 barstow $ */
-/* $Id: auth.c,v 1.23.2.1 2001/10/03 10:47:11 ossi Exp $ */
 /*
 
 Copyright 1988, 1998  The Open Group
+Copyright 2000-2004 Oswald Buddenhagen <ossi@kde.org>
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
@@ -12,25 +15,22 @@ in all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
+IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
-Except as contained in this notice, the name of The Open Group shall
+Except as contained in this notice, the name of a copyright holder shall
 not be used in advertising or otherwise to promote the sale, use or
 other dealings in this Software without prior written authorization
-from The Open Group.
+from the copyright holder.
 
 */
-/* $XFree86: xc/programs/xdm/auth.c,v 3.20 1999/12/27 00:40:08 robin Exp $ */
 
 /*
  * xdm - display manager daemon
  * Author:  Keith Packard, MIT X Consortium
  *
- * auth.c
- *
  * maintain the authorization generation daemon
  */
 
@@ -41,15 +41,10 @@ from The Open Group.
 #include <X11/X.h>
 
 #include <sys/stat.h>
+#include <fcntl.h>
+#include <stdlib.h>
 
-#include <errno.h>
-#ifdef X_NOT_STDC_ENV
-extern int errno;
-#endif
-
-#ifndef ESIX
-# include <sys/ioctl.h>
-#endif /* !ESIX */
+#include <sys/ioctl.h>
 
 #if defined(TCPCONN) || defined(STREAMSCONN)
 # include "dm_socket.h"
@@ -60,8 +55,8 @@ extern int errno;
 #endif
 
 #if (defined(_POSIX_SOURCE) && !defined(AIXV3) && !defined(__QNX__)) || defined(hpux) || defined(USG) || defined(SVR4) || (defined(SYSV) && defined(i386))
-#define NEED_UTSNAME
-#include <sys/utsname.h>
+# define NEED_UTSNAME
+# include <sys/utsname.h>
 #endif
 
 #if defined(SYSV) && defined(i386)
@@ -70,9 +65,6 @@ extern int errno;
 #  include <stropts.h>
 #  include <sys/sioctl.h>
 # endif /* ISC */
-# ifdef ESIX
-#  include <lan/net_ioctl.h>
-# endif /* ESIX */
 #endif /* i386 */
 
 #ifdef SVR4
@@ -90,25 +82,9 @@ extern int errno;
 # include <netdb.h>
 # undef SIOCGIFCONF
 #else /* __GNU__ */
-# ifndef MINIX
-#  include <net/if.h>
-# else
-#  include <net/netlib.h>
-#  include <net/gen/netdb.h>
-# endif /* !MINIX */
+# include <net/if.h>
 #endif /* __GNU__ */
 
-#if ((defined(SVR4) && !defined(sun)) || defined(ISC)) && defined(SIOCGIFCONF)
-# define SYSV_SIOCGIFCONF
-#endif
-
-#ifdef CSRG_BASED
-# include <sys/param.h>
-# if (BSD >= 199103)
-#  define VARIABLE_IFREQ
-# endif
-#endif
-
 #ifdef __EMX__
 # define link rename
 int chown(int a,int b,int c) {}
@@ -117,14 +93,14 @@ int chown(int a,int b,int c) {}
 
 struct AuthProtocol {
     unsigned short  name_length;
-    char	    *name;
-    void	    (*InitAuth)(unsigned short len, char *name);
-    Xauth	    *(*GetAuth)(unsigned short len, char *name);
+    const char	    *name;
+    void	    (*InitAuth)(unsigned short len, const char *name);
+    Xauth	    *(*GetAuth)(unsigned short len, const char *name);
 #ifdef XDMCP
     void	    (*GetXdmcpAuth)(
-    			struct protoDisplay	*pdpy,
-    			unsigned short	authorizationNameLen,
-    			char		*authorizationName);
+			struct protoDisplay	*pdpy,
+			unsigned short		authorizationNameLen,
+			const char		*authorizationName);
 #endif
     int		    inited;
 };
@@ -157,7 +133,7 @@ static struct AuthProtocol AuthProtocols[] = {
 };
 
 static struct AuthProtocol *
-findProtocol (unsigned short name_length, char *name)
+findProtocol (unsigned short name_length, const char *name)
 {
     unsigned	i;
 
@@ -171,7 +147,7 @@ findProtocol (unsigned short name_length, char *name)
 }
 
 int
-ValidAuthorization (unsigned short name_length, char *name)
+ValidAuthorization (unsigned short name_length, const char *name)
 {
     if (findProtocol (name_length, name))
 	return TRUE;
@@ -179,7 +155,7 @@ ValidAuthorization (unsigned short name_length, char *name)
 }
 
 static Xauth *
-GenerateAuthorization (unsigned short name_length, char *name)
+GenerateAuthorization (unsigned short name_length, const char *name)
 {
     struct AuthProtocol	*a;
     Xauth   *auth = 0;
@@ -195,16 +171,16 @@ GenerateAuthorization (unsigned short name_length, char *name)
 	auth = (*a->GetAuth) (name_length, name);
 	if (auth)
 	{
-	    Debug ("Got %p (%d %.*s) %02[*hhx\n", auth,
+	    Debug ("got %p (%d %.*s) %02[*hhx\n", auth,
 		    auth->name_length, auth->name_length, auth->name,
 		    auth->data_length, auth->data);
 	}
 	else
-	    Debug ("Got (null)\n");
+	    Debug ("got (null)\n");
     }
     else
     {
-	Debug ("Unknown authorization %s\n", name);
+	Debug ("unknown authorization %s\n", name);
     }
     return auth;
 }
@@ -215,7 +191,7 @@ void
 SetProtoDisplayAuthorization (
     struct protoDisplay	*pdpy,
     unsigned short	authorizationNameLen,
-    char		*authorizationName)
+    const char		*authorizationName)
 {
     struct AuthProtocol	*a;
     Xauth   *auth;
@@ -241,17 +217,17 @@ SetProtoDisplayAuthorization (
 	    pdpy->xdmcpAuthorization = 0;
 	}
 	if (auth)
-	    Debug ("Got %p (%d %.*s)\n", auth,
+	    Debug ("got %p (%d %.*s)\n", auth,
 		auth->name_length, auth->name_length, auth->name);
 	else
-	    Debug ("Got (null)\n");
+	    Debug ("got (null)\n");
     }
 }
 
 #endif /* XDMCP */
 
 void
-CleanUpFileName (char *src, char *dst, int len)
+CleanUpFileName (const char *src, char *dst, int len)
 {
     while (*src) {
 	if (--len <= 0)
@@ -272,54 +248,62 @@ CleanUpFileName (char *src, char *dst, int len)
     *dst = '\0';
 }
 
-static char authdir[] = "authfiles";
 
-#ifdef SYSV
+static FILE *
+fdOpenW (int fd)
+{
+    FILE *f;
+
+    if (fd >= 0) {
+	if ((f = fdopen (fd, "w")))
+	    return f;
+	close (fd);
+    }
+    return 0;
+}
+
+
+#if defined(SYSV) && !defined(SVR4)
 # define NAMELEN	14
 #else
 # define NAMELEN	255
 #endif
 
-static int
+static FILE *
 MakeServerAuthFile (struct display *d)
 {
+    FILE	*f;
+#ifndef HAS_MKSTEMP
     int		r;
-    struct stat	statb;
-    char	*af, cleanname[NAMELEN];
+#endif
+    char	cleanname[NAMELEN], nambuf[NAMELEN+128];
 
-    if (d->clientAuthFile && *d->clientAuthFile)
-	return ReStr (&d->authFile, d->clientAuthFile);
-
-    if (d->authFile) {
-	free (d->authFile);
-	d->authFile = 0;
-    }
-    r = stat(authDir, &statb);
-    if (r == 0) {
-	if (statb.st_uid != 0)
-	    (void) chown(authDir, 0, statb.st_gid);
-	if ((statb.st_mode & 0077) != 0)
-	    (void) chmod(authDir, statb.st_mode & 0700);
-    } else {
-	if (errno == ENOENT)
-	    r = mkdir(authDir, 0700);
-	if (r < 0)
-	    return FALSE;
-    }
+    /*
+     * Some paranoid, but still not sufficient (DoS was still possible)
+     * checks used to be here. I removed all this stuff because
+     * a) authDir is supposed to be /var/run/xauth (=safe) or similar and
+     * b) even if it's not (say, /tmp), we create files safely (hopefully).
+     */
+    if (mkdir(authDir, 0755) < 0  &&  errno != EEXIST)
+	return 0;
     CleanUpFileName (d->name, cleanname, NAMELEN - 8);
-    if (!(af = malloc ((unsigned) strlen (authDir) + 
-			sizeof (authdir) + strlen (cleanname) + 10)))
-	return FALSE;
-    sprintf (af, "%s/%s", authDir, authdir);
-    r = mkdir(af, 0700);
-    if (r < 0  &&  errno != EEXIST) {
-	free (af);
-	return FALSE;
+#ifdef HAS_MKSTEMP
+    sprintf (nambuf, "%s/A%s-XXXXXX", authDir, cleanname);
+    if ((f = fdOpenW (mkstemp (nambuf)))) {
+	StrDup (&d->authFile, nambuf);
+	return f;
     }
-    sprintf (af, "%s/%s/A%s-XXXXXX", authDir, authdir, cleanname);
-    (void) mktemp (af);
-    d->authFile = af;
-    return TRUE;
+#else
+    for (r = 0; r < 100; r++) {
+	sprintf (nambuf, "%s/A%s-XXXXXX", authDir, cleanname);
+	(void) mktemp (nambuf);
+	if ((f = fdOpenW (open (nambuf, O_WRONLY | O_CREAT | O_EXCL, 0600)))) {
+	    StrDup (&d->authFile, nambuf);
+	    return f;
+	}
+    }
+#endif	
+    return 0;
 }
 
 int
@@ -329,46 +313,44 @@ SaveServerAuthorizations (
     int		    count)
 {
     FILE	*auth_file;
-    int		mask;
-    int		ret;
-    int		i;
+    int		i, ret;
 
-    mask = umask (0077);
-    if (!d->authFile && !MakeServerAuthFile (d))
-	return FALSE;
-    (void) unlink (d->authFile);
-    auth_file = fopen (d->authFile, "w");
-    umask (mask);
-    if (!auth_file) {
-	LogError ("Cannot open server authorization file %s\n", d->authFile);
-	free (d->authFile);
-	d->authFile = NULL;
-	ret = FALSE;
+    if (!d->authFile && d->clientAuthFile && *d->clientAuthFile)
+	StrDup (&d->authFile, d->clientAuthFile);
+    if (d->authFile) {
+	if (!(auth_file = fdOpenW (creat (d->authFile, 0600)))) {
+	    LogError ("Cannot open X server authorization file %s\n", d->authFile);
+	    free (d->authFile);
+	    d->authFile = NULL;
+	    return FALSE;
+	}
+    } else {
+	if (!(auth_file = MakeServerAuthFile (d))) {
+	    LogError ("Cannot create X server authorization file\n");
+	    return FALSE;
+	}
     }
-    else
+    Debug ("file: %s  auth: %p\n", d->authFile, auths);
+    ret = TRUE;
+    for (i = 0; i < count; i++)
     {
-    	Debug ("File: %s auth: %p\n", d->authFile, auths);
-	ret = TRUE;
-	for (i = 0; i < count; i++)
-	{
-	    /*
-	     * User-based auths may not have data until
-	     * a user logs in.  In which case don't write
-	     * to the auth file so xrdb and setup programs don't fail.
-	     */
-	    if (auths[i]->data_length > 0)
-		if (!XauWriteAuth (auth_file, auths[i]) ||
-		    fflush (auth_file) == EOF)
-		{
-		    LogError ("Cannot write server authorization file %s\n",
-			      d->authFile);
-		    ret = FALSE;
-		    free (d->authFile);
-		    d->authFile = NULL;
-		}
-    	}
-	fclose (auth_file);
+	/*
+	 * User-based auths may not have data until
+	 * a user logs in.  In which case don't write
+	 * to the auth file so xrdb and setup programs don't fail.
+	 */
+	if (auths[i]->data_length > 0)
+	    if (!XauWriteAuth (auth_file, auths[i]) ||
+		fflush (auth_file) == EOF)
+	    {
+		LogError ("Cannot write X server authorization file %s\n",
+			  d->authFile);
+		ret = FALSE;
+		free (d->authFile);
+		d->authFile = NULL;
+	    }
     }
+    fclose (auth_file);
     return ret;
 }
 
@@ -386,6 +368,7 @@ SetLocalAuthorization (struct display *d)
 	d->authorizations = (Xauth **) NULL;
 	d->authNum = 0;
     }
+    Debug ("SetLocalAuthorization %s, auths %[s\n", d->name, d->authNames);
     if (!d->authNames)
 	return;
     for (i = 0; d->authNames[i]; i++)
@@ -393,13 +376,13 @@ SetLocalAuthorization (struct display *d)
     d->authNameNum = i;
     if (d->authNameLens)
 	free ((char *) d->authNameLens);
-    d->authNameLens = (unsigned short *) malloc
+    d->authNameLens = (unsigned short *) Malloc
 				(d->authNameNum * sizeof (unsigned short));
     if (!d->authNameLens)
 	return;
     for (i = 0; i < d->authNameNum; i++)
 	d->authNameLens[i] = strlen (d->authNames[i]);
-    auths = (Xauth **) malloc (d->authNameNum * sizeof (Xauth *));
+    auths = (Xauth **) Malloc (d->authNameNum * sizeof (Xauth *));
     if (!auths)
 	return;
     j = 0;
@@ -450,16 +433,11 @@ SetAuthorization (struct display *d)
 }
 
 static int
-openFiles (char *name, char *new_name, FILE **oldp, FILE **newp)
+openFiles (const char *name, char *new_name, FILE **oldp, FILE **newp)
 {
-	int	mask;
-
 	strcat (strcpy (new_name, name), "-n");
-	mask = umask (0077);
-	(void) unlink (new_name);
-	*newp = fopen (new_name, "w");
-	(void) umask (mask);
-	if (!*newp) {
+	if (!(*newp = 
+	      fdOpenW (creat (new_name, 0600)))) {
 		Debug ("can't open new file %s\n", new_name);
 		return 0;
 	}
@@ -468,16 +446,6 @@ openFiles (char *name, char *new_name, FILE **oldp, FILE **newp)
 	return 1;
 }
 
-static int
-binaryEqual (char *a, char *b, unsigned short len)
-{
-	while (len-- > 0)
-		if (*a++ != *b++)
-			return FALSE;
-	return TRUE;
-}
-
-
 struct addrList {
 	unsigned short	family;
 	unsigned short	address_length;
@@ -518,15 +486,11 @@ saveEntry (Xauth *auth)
 {
 	struct addrList	*new;
 
-	new = (struct addrList *) malloc (sizeof (struct addrList));
-	if (!new) {
-		LogOutOfMem ("saveEntry");
+	if (!(new = (struct addrList *) Malloc (sizeof (struct addrList))))
 		return;
-	}
 	if ((new->address_length = auth->address_length) > 0) {
-		new->address = malloc (auth->address_length);
+		new->address = Malloc (auth->address_length);
 		if (!new->address) {
-			LogOutOfMem ("saveEntry");
 			free ((char *) new);
 			return;
 		}
@@ -534,9 +498,8 @@ saveEntry (Xauth *auth)
 	} else
 		new->address = 0;
 	if ((new->number_length = auth->number_length) > 0) {
-		new->number = malloc (auth->number_length);
+		new->number = Malloc (auth->number_length);
 		if (!new->number) {
-			LogOutOfMem ("saveEntry");
 			free (new->address);
 			free ((char *) new);
 			return;
@@ -545,9 +508,8 @@ saveEntry (Xauth *auth)
 	} else
 		new->number = 0;
 	if ((new->name_length = auth->name_length) > 0) {
-		new->name = malloc (auth->name_length);
+		new->name = Malloc (auth->name_length);
 		if (!new->name) {
-			LogOutOfMem ("saveEntry");
 			free (new->number);
 			free (new->address);
 			free ((char *) new);
@@ -569,11 +531,11 @@ checkEntry (Xauth *auth)
 	for (a = addrs; a; a = a->next) {
 		if (a->family == auth->family &&
 		    a->address_length == auth->address_length &&
- 		    binaryEqual (a->address, auth->address, auth->address_length) &&
+		    !memcmp (a->address, auth->address, auth->address_length) &&
 		    a->number_length == auth->number_length &&
- 		    binaryEqual (a->number, auth->number, auth->number_length) &&
+		    !memcmp (a->number, auth->number, auth->number_length) &&
 		    a->name_length == auth->name_length &&
-		    binaryEqual (a->name, auth->name, auth->name_length))
+		    !memcmp (a->name, auth->name, auth->name_length))
 		{
 			return 1;
 		}
@@ -625,7 +587,7 @@ DefineLocal (FILE *file, Xauth *auth)
 	char	displayname[100];
 	char	tmp_displayname[100];
 
-	strcpy(tmp_displayname, "");
+	tmp_displayname[0] = 0;
 
 	/* stolen from xinit.c */
 
@@ -649,7 +611,7 @@ DefineLocal (FILE *file, Xauth *auth)
 	struct utsname name;
 
 	uname(&name);
-	strcpy(displayname, name.nodename);
+	sprintf(displayname, "%.*s", sizeof(displayname) - 1, name.nodename);
 	}
 	writeAddr (FamilyLocal, strlen (displayname), displayname, file, auth);
 
@@ -657,7 +619,7 @@ DefineLocal (FILE *file, Xauth *auth)
 #endif
 
 #if (!defined(NEED_UTSNAME) || defined (hpux))
-        /* AIXV3:
+	/* AIXV3:
 	 * In AIXV3, _POSIX_SOURCE is defined, but uname gives only first
 	 * field of hostname. Thus, we use gethostname instead.
 	 */
@@ -667,7 +629,9 @@ DefineLocal (FILE *file, Xauth *auth)
 	 * is achieved by using gethostname().  For compatability, we must
 	 * also still create the entry using uname() above.
 	 */
-	gethostname(displayname, sizeof(displayname));
+	displayname[0] = 0;
+	if (!gethostname (displayname, sizeof(displayname)))
+	    displayname[sizeof(displayname) - 1] = 0;
 
 	/*
 	 * If gethostname and uname both returned the same name,
@@ -682,11 +646,27 @@ DefineLocal (FILE *file, Xauth *auth)
 /* Argh! this is evil. But ConvertAddr works only with Xdmcp.h */
 #ifdef XDMCP
 
+/*
+ * Call ConvertAddr(), and if it returns an IPv4 localhost, convert it
+ * to a local display name.  Meets the _XTransConvertAddress's localhost
+ * hack.
+ */
+ 
+static int
+ConvertAuthAddr (XdmcpNetaddr saddr, int *len, char **addr)
+{
+    int ret = ConvertAddr(saddr, len, addr);
+    if (ret == FamilyInternet &&
+	((struct in_addr *)*addr)->s_addr == htonl(0x7F000001L))
+	ret = FamilyLocal;
+    return ret;
+}
+
 #ifdef SYSV_SIOCGIFCONF
 
 /* Deal with different SIOCGIFCONF ioctl semantics on SYSV, SVR4 */
 
-static int
+int
 ifioctl (int fd, int cmd, char *arg)
 {
     struct strioctl ioc;
@@ -707,8 +687,8 @@ ifioctl (int fd, int cmd, char *arg)
 	 * ifreq fields to hold all interfaces. The return buffer length
 	 * is placed in the buffer header.
 	 */
-        ((struct ifconf *) ioc.ic_dp)->ifc_len =
-                                         ioc.ic_len - sizeof(struct ifconf);
+	((struct ifconf *) ioc.ic_dp)->ifc_len =
+					 ioc.ic_len - sizeof(struct ifconf);
 #endif
     }
     else
@@ -731,11 +711,61 @@ ifioctl (int fd, int cmd, char *arg)
 #endif
     return(ret);
 }
-#else /* SYSV_SIOCGIFCONF */
-# define ifioctl ioctl
+
 #endif /* SYSV_SIOCGIFCONF */
 
-#if defined(STREAMSCONN) && !defined(SYSV_SIOCGIFCONF) && !defined(NCR)
+#ifdef HAS_GETIFADDRS
+# include <ifaddrs.h>
+
+static void
+DefineSelf(int fd, FILE *file, Xauth *auth)
+{
+    struct ifaddrs *ifap, *ifr;
+    char *addr;
+    int family, len;
+    
+    if (getifaddrs(&ifap) < 0) 
+	return;
+    for (ifr = ifap; ifr; ifr = ifr->ifa_next) {
+	len = sizeof(*(ifr->ifa_addr));
+	family = ConvertAddr((XdmcpNetaddr)(ifr->ifa_addr), &len, &addr);
+	if (family == -1 || family == FamilyLocal) 
+	    continue;
+	/*
+	 * don't write out 'localhost' entries, as
+	 * they may conflict with other local entries.
+	 * DefineLocal will always be called to add
+	 * the local entry anyway, so this one can
+	 * be tossed.
+	 */
+	if (family == FamilyInternet &&
+	    addr[0] == 127 && addr[1] == 0 &&
+	    addr[2] == 0 && addr[3] == 1)
+	{
+	    Debug ("Skipping localhost address\n");
+	    continue;
+	}
+# if defined(IPv6) && defined(AF_INET6)
+	if (family == FamilyInternet6) {
+	    if (IN6_IS_ADDR_LOOPBACK(((struct in6_addr *)addr))) {
+		Debug ("Skipping IPv6 localhost address\n");
+		continue;
+	    }
+	    /* Also skip XDM-AUTHORIZATION-1 */
+	    if (auth->name_length == 19 && 
+		!memcmp (auth->name, "XDM-AUTHORIZATION-1", 19)) {
+		Debug ("Skipping IPv6 XDM-AUTHORIZATION-1\n");
+		continue;
+	    }
+	}
+# endif
+	writeAddr (family, len, addr, file, auth);
+    }
+    freeifaddrs (ifap);
+}
+#else  /* GETIFADDRS */
+
+#if defined(STREAMSCONN) && !defined(SYSV_SIOCGIFCONF) && !defined(WINTCP)
 
 #include <tiuser.h>
 
@@ -761,7 +791,6 @@ DefineSelf (int fd, FILE *file, Xauth *auth)
 
 #ifdef WINTCP /* NCR with Wollongong TCP */
 
-#include <sys/un.h>
 #include <stropts.h>
 #include <tiuser.h>
 
@@ -789,7 +818,7 @@ DefineSelf (int fd, FILE *file, Xauth *auth)
     int	len, ipfd;
 
     if ((ipfd = open ("/dev/ip", O_RDWR, 0 )) < 0) {
-	LogError ("Getting interface configuration\n");
+	LogError ("Trouble getting interface configuration\n");
 	return;
     }
 
@@ -806,7 +835,7 @@ DefineSelf (int fd, FILE *file, Xauth *auth)
 	if (ioctl (ipfd, (int) I_STR, (char *) &str) < 0)
 	{
 	    close (ipfd);
-	    LogError ("Getting interface configuration\n");
+	    LogError ("Trouble getting interface configuration\n");
 	    return;
 	}
 
@@ -819,7 +848,7 @@ DefineSelf (int fd, FILE *file, Xauth *auth)
 	if (ioctl (ipfd, (int) I_STR, (char *) &str) < 0)
 	{
 	    close (ipfd);
-	    LogError ("Getting interface configuration\n");
+	    LogError ("Trouble getting interface configuration\n");
 	    return;
 	}
 
@@ -837,7 +866,30 @@ DefineSelf (int fd, FILE *file, Xauth *auth)
 
 #else /* WINTCP */
 
-#ifdef SIOCGIFCONF
+/* Solaris provides an extended interface SIOCGLIFCONF.  Other systems
+ * may have this as well, but the code has only been tested on Solaris
+ * so far, so we only enable it there.  Other platforms may be added as
+ * needed.
+ *
+ * Test for Solaris commented out  --  TSI @ UQV 2003.06.13
+ */
+#ifdef SIOCGLIFCONF
+/* #if defined(sun) */
+# define USE_SIOCGLIFCONF
+/* #endif */
+#endif
+
+#if defined(SIOCGIFCONF) || defined (USE_SIOCGLIFCONF)
+
+#if !defined(SYSV_SIOCGIFCONF) || defined(USE_SIOCGLIFCONF)
+# define ifioctl ioctl
+#endif
+
+#ifdef USE_SIOCGLIFCONF
+# define ifr_type    struct lifreq
+#else
+# define ifr_type    struct ifreq
+#endif
 
 /* Handle variable length ifreq in BNR2 and later */
 #ifdef VARIABLE_IFREQ
@@ -845,7 +897,25 @@ DefineSelf (int fd, FILE *file, Xauth *auth)
 		      (p->ifr_addr.sa_len > sizeof (p->ifr_addr) ? \
 		       p->ifr_addr.sa_len - sizeof (p->ifr_addr) : 0))
 #else
-# define ifr_size(p) (sizeof (struct ifreq))
+# define ifr_size(p) (sizeof (ifr_type))
+#endif
+
+#ifdef USE_SIOCGLIFCONF
+# define IFC_IOCTL_REQ SIOCGLIFCONF
+# define IFC_REQ(ifc) ifc.lifc_req
+# define IFC_LEN(ifc) ifc.lifc_len
+# define IFR_ADDR(ifr) ifr->lifr_addr
+# define IFR_NAME(ifr) ifr->lifr_name
+#else    
+# define IFC_IOCTL_REQ SIOCGIFCONF
+# ifdef ISC
+#  define IFC_REQ(ifc) ((ifr_type *) ifc.ifc_buf)
+# else
+#  define IFC_REQ(ifc) ifc.ifc_req
+# endif
+# define IFC_LEN(ifc) ifc.ifc_len
+# define IFR_ADDR(ifr) ifr->ifr_addr
+# define IFR_NAME(ifr) ifr->ifr_name
 #endif
 
 /* Define this host for access control.  Find all the hosts the OS knows about 
@@ -855,47 +925,75 @@ static void
 DefineSelf (int fd, FILE *file, Xauth *auth)
 {
     char		buf[2048], *cp, *cplim;
-    struct ifconf	ifc;
     int 		len;
     char 		*addr;
     int 		family;
-    register struct ifreq *ifr;
+    ifr_type		*ifr;
+#ifdef USE_SIOCGLIFCONF
+    int			n;
+    void *		bufptr = buf;
+    size_t		buflen = sizeof(buf);
+    struct lifconf	ifc;
+# ifdef SIOCGLIFNUM
+    struct lifnum	ifn;
+# endif
+#else
+    struct ifconf	ifc;
+#endif
     
+#if defined(SIOCGLIFNUM) && defined(SIOCGLIFCONF)
+    ifn.lifn_family = AF_UNSPEC;
+    ifn.lifn_flags = 0;
+    if (ioctl (fd, (int) SIOCGLIFNUM, (char *) &ifn) < 0)
+	LogError ("Failed getting interface count\n");
+    if (buflen < (ifn.lifn_count * sizeof(struct lifreq))) {
+	buflen = ifn.lifn_count * sizeof(struct lifreq);
+	bufptr = Malloc (buflen);
+    }
+#endif
+
+#ifdef USE_SIOCGLIFCONF
+    ifc.lifc_family = AF_UNSPEC;
+    ifc.lifc_flags = 0;
+    ifc.lifc_len = buflen;
+    ifc.lifc_buf = bufptr;
+#else
     ifc.ifc_len = sizeof (buf);
     ifc.ifc_buf = buf;
-    if (ifioctl (fd, SIOCGIFCONF, (char *) &ifc) < 0) {
+#endif
+    if (ifioctl (fd, IFC_IOCTL_REQ, (char *) &ifc) < 0) {
 	LogError ("Trouble getting network interface configuration\n");
+#if defined(SIOCGLIFNUM) && defined(SIOCGLIFCONF)
+	if (bufptr != buf)
+	    free (bufptr);
+#endif
 	return;
     }
 
-#ifdef ISC
-# define IFC_IFC_REQ (struct ifreq *) ifc.ifc_buf
-#else
-# define IFC_IFC_REQ ifc.ifc_req
-#endif
+    cplim = (char *) IFC_REQ (ifc) + IFC_LEN (ifc);
 
-    cplim = (char *) IFC_IFC_REQ + ifc.ifc_len;
-
-    for (cp = (char *) IFC_IFC_REQ; cp < cplim; cp += ifr_size (ifr))
+    for (cp = (char *) IFC_REQ (ifc); cp < cplim; cp += ifr_size (ifr))
     {
-	ifr = (struct ifreq *) cp;
+	ifr = (ifr_type *) cp;
 #ifdef DNETCONN
 	/*
 	 * this is ugly but SIOCGIFCONF returns decnet addresses in
 	 * a different form from other decnet calls
 	 */
-	if (ifr->ifr_addr.sa_family == AF_DECnet) {
+	if (IFR_ADDR (ifr).sa_family == AF_DECnet) {
 		len = sizeof (struct dn_naddr);
-		addr = (char *)ifr->ifr_addr.sa_data;
+		addr = (char *)IFR_ADDR (ifr).sa_data;
 		family = FamilyDECnet;
 	} else
 #endif
 	{
-	    if (ConvertAddr ((XdmcpNetaddr) &ifr->ifr_addr, &len, &addr) < 0)
+	    family = ConvertAddr ((XdmcpNetaddr) &IFR_ADDR (ifr), &len, &addr);
+	    if (family < 0)
 		continue;
+
 	    if (len == 0)
- 	    {
-		Debug ("Skipping zero length address\n");
+	    {
+		Debug ("skipping zero length address\n");
 		continue;
 	    }
 	    /*
@@ -905,18 +1003,35 @@ DefineSelf (int fd, FILE *file, Xauth *auth)
 	     * the local entry anyway, so this one can
 	     * be tossed.
 	     */
-	    if (len == 4 &&
+	    if (family == FamilyInternet &&
 		addr[0] == 127 && addr[1] == 0 &&
 		addr[2] == 0 && addr[3] == 1)
 	    {
-		    Debug ("Skipping localhost address\n");
+		    Debug ("skipping localhost address\n");
 		    continue;
 	    }
-	    family = FamilyInternet;
+#if defined(IPv6) && defined(AF_INET6)
+	    if (family == FamilyInternet6) {
+		if (IN6_IS_ADDR_LOOPBACK(((struct in6_addr *)addr))) {
+		    Debug ("Skipping IPv6 localhost address\n");
+		    continue;
+		}
+		/* Also skip XDM-AUTHORIZATION-1 */
+		if (auth->name_length == 19 && 
+		    !memcmp (auth->name, "XDM-AUTHORIZATION-1", 19)) {
+		    Debug ("Skipping IPv6 XDM-AUTHORIZATION-1\n");
+		    continue;
+		}
+	    }
+#endif
 	}
 	Debug ("DefineSelf: write network address, length %d\n", len);
 	writeAddr (family, len, addr, file, auth);
     }
+#if defined(SIOCGLIFNUM) && defined(SIOCGLIFCONF)
+    if (bufptr != buf)
+	free (bufptr);
+#endif
 }
 
 #else /* SIOCGIFCONF */
@@ -948,13 +1063,11 @@ DefineSelf (int fd, int file, int auth)
      * see), whereas gethostname() kindly truncates it for me.
      */
     uname(&name);
-    hp = gethostbyname (name.nodename);
-    if (hp != NULL) {
+    if ((hp = gethostbyname (name.nodename))) {
 	saddr.sa.sa_family = hp->h_addrtype;
 	inetaddr = (struct sockaddr_in *) (&(saddr.sa));
 	memmove( (char *) &(inetaddr->sin_addr), (char *) hp->h_addr, (int) hp->h_length);
-	family = ConvertAddr ( &(saddr.sa), &len, &addr);
-	if ( family >= 0) {
+	if ( (family = ConvertAddr ( &(saddr.sa), &len, &addr)) >= 0) {
 	    writeAddr (FamilyInternet, sizeof (inetaddr->sin_addr),
 			(char *) (&inetaddr->sin_addr), file, auth);
 	}
@@ -964,14 +1077,15 @@ DefineSelf (int fd, int file, int auth)
 #endif /* SIOCGIFCONF else */
 #endif /* WINTCP else */
 #endif /* STREAMSCONN && !SYSV_SIOCGIFCONF else */
+#endif /* HAS_GETIFADDRS */
 
 #endif /* XDMCP */
 
 static void
-setAuthNumber (Xauth *auth, char *name)
+setAuthNumber (Xauth *auth, const char *name)
 {
     char	*colon;
-    char	*dot, *number;
+    char	*dot;
 
     Debug ("setAuthNumber %s\n", name);
     colon = strrchr(name, ':');
@@ -982,27 +1096,17 @@ setAuthNumber (Xauth *auth, char *name)
 	    auth->number_length = dot - colon;
 	else
 	    auth->number_length = strlen (colon);
-	number = malloc (auth->number_length + 1);
-	if (number) {
-	    strncpy (number, colon, auth->number_length);
-	    number[auth->number_length] = '\0';
-	} else {
-	    LogOutOfMem ("setAuthNumber");
+	if (!StrNDup (&auth->number, colon, auth->number_length))
 	    auth->number_length = 0;
-	}
-	auth->number = number;
-	Debug ("setAuthNumber: %s\n", number);
+	Debug ("setAuthNumber: %s\n", auth->number);
     }
 }
 
 static void
-writeLocalAuth (FILE *file, Xauth *auth, char *name)
+writeLocalAuth (FILE *file, Xauth *auth, const char *name)
 {
 #ifdef XDMCP
     int	fd;
-#ifdef MINIX
-    char *tcp_device;
-#endif
 #endif
 
     Debug ("writeLocalAuth: %s %.*s\n", name, auth->name_length, auth->name);
@@ -1016,14 +1120,11 @@ writeLocalAuth (FILE *file, Xauth *auth, char *name)
     t_close (fd);
 #endif
 #ifdef TCPCONN
-#ifdef MINIX
-    tcp_device= getenv("TCP_DEVICE");
-    if (tcp_device == NULL)
-    	tcp_device= TCP_DEVICE;
-    fd = open(tcp_device, O_RDWR);
-#else
-    fd = socket (AF_INET, SOCK_STREAM, 0);
+#if defined(IPv6) && defined(AF_INET6)
+    fd = socket (AF_INET6, SOCK_STREAM, 0);
+    if (fd < 0)
 #endif
+    fd = socket (AF_INET, SOCK_STREAM, 0);
     DefineSelf (fd, file, auth);
     close (fd);
 #endif
@@ -1039,7 +1140,7 @@ writeLocalAuth (FILE *file, Xauth *auth, char *name)
 #ifdef XDMCP
 
 static void
-writeRemoteAuth (FILE *file, Xauth *auth, XdmcpNetaddr peer, int peerlen, char *name)
+writeRemoteAuth (FILE *file, Xauth *auth, XdmcpNetaddr peer, int peerlen, const char *name)
 {
     int	    family = FamilyLocal;
     char    *addr;
@@ -1048,12 +1149,12 @@ writeRemoteAuth (FILE *file, Xauth *auth, XdmcpNetaddr peer, int peerlen, char *
     if (!peer || peerlen < 2)
 	return;
     setAuthNumber (auth, name);
-    family = ConvertAddr (peer, &peerlen, &addr);
+    family = ConvertAuthAddr (peer, &peerlen, &addr);
     Debug ("writeRemoteAuth: family %d\n", family);
     if (family != FamilyLocal)
     {
-	Debug ("writeRemoteAuth: %d, %d, %x\n",
-		family, peerlen, *(int *)addr);
+	Debug ("writeRemoteAuth: %d, %02[*:hhx\n",
+		family, peerlen, addr);
 	writeAddr (family, peerlen, addr, file, auth);
     }
     else
@@ -1064,67 +1165,120 @@ writeRemoteAuth (FILE *file, Xauth *auth, XdmcpNetaddr peer, int peerlen, char *
 
 #endif /* XDMCP */
 
+#define NBSIZE 1024
+
+static void
+startUserAuth (char *buf, char *nbuf, 
+	       FILE **old, FILE **new)
+{
+    const char	*home;
+    int		lockStatus;
+
+    initAddrs ();
+    *new = 0;
+    if ((home = getEnv (userEnviron, "HOME")) && strlen (home) < NBSIZE - 12) {
+	sprintf (buf, "%s/.Xauthority", home);
+	Debug ("XauLockAuth %s\n", buf);
+	lockStatus = XauLockAuth (buf, 1, 2, 10);
+	Debug ("lock is %d\n", lockStatus);
+	if (lockStatus == LOCK_SUCCESS)
+	    if (!openFiles (buf, nbuf, old, new))
+		XauUnlockAuth (buf);
+    }
+    if (!*new)
+	LogInfo ("Can't update authorization file in home dir %s\n", home);
+}
+
+static void
+endUserAuth (FILE *old, FILE *new, const char *nname)
+{
+    Xauth	*entry;
+    struct stat	statb;
+
+    if (old) {
+	if (fstat (fileno (old), &statb) != -1)
+	    chmod (nname, (int) (statb.st_mode & 0777));
+	/*SUPPRESS 560*/
+	while ((entry = XauReadAuth (old))) {
+	    if (!checkEntry (entry))
+	    {
+		Debug ("writing an entry\n");
+		writeAuth (new, entry);
+	    }
+	    XauDisposeAuth (entry);
+	}
+	fclose (old);
+    }
+    fclose (new);
+    doneAddrs ();
+}
+
+static char *
+moveUserAuth (const char *name, char *new_name, char *envname)
+{
+    if (unlink (name))
+	Debug ("unlink %s failed\n", name);
+    if (link (new_name, name)) {
+	Debug ("link failed %s %s\n", new_name, name);
+	LogError ("Can't move authorization into place\n");
+	envname = new_name;
+    } else {
+	Debug ("new authorization moved into place\n");
+	unlink (new_name);
+    }
+    XauUnlockAuth (name);
+    return envname;
+}
+
 void
-SetUserAuthorization (struct display *d, struct verify_info *verify)
+SetUserAuthorization (struct display *d)
 {
     FILE	*old, *new;
-    char	home_name[1024], backup_name[1024], new_name[1024];
-    char	*name = 0;
-    char	*home;
-    char	*envname = 0;
-    int	lockStatus;
-    Xauth	*entry, **auths;
-    int		set_env = 0;
-    struct stat	statb;
+    char	*name;
+    char	*envname;
+    Xauth	**auths;
     int		i;
     int		magicCookie;
     int		data_len;
+    char	name_buf[NBSIZE], new_name[NBSIZE + 2];
 
     Debug ("SetUserAuthorization\n");
     auths = d->authorizations;
     if (auths) {
-	home = getEnv (verify->userEnviron, "HOME");
-	lockStatus = LOCK_ERROR;
-	if (home) {
-	    strcpy (home_name, home);
-	    if (home[strlen(home) - 1] != '/')
-		strcat (home_name, "/");
-	    strcat (home_name, ".Xauthority");
-	    Debug ("XauLockAuth %s\n", home_name);
-	    lockStatus = XauLockAuth (home_name, 1, 2, 10);
-	    Debug ("Lock is %d\n", lockStatus);
-	    if (lockStatus == LOCK_SUCCESS) {
-		if (openFiles (home_name, new_name, &old, &new)) {
-		    name = home_name;
-		    set_env = 0;
-		} else {
-		    Debug ("openFiles failed\n");
-		    XauUnlockAuth (home_name);
-		    lockStatus = LOCK_ERROR;
-		}	
+	startUserAuth (name_buf, new_name, &old, &new);
+	if (new) {
+	    envname = 0;
+	    name = name_buf;
+	} else {
+	    if (strlen (d->userAuthDir) >= NBSIZE - 13)
+		return;
+	    /*
+	     * Note, that we don't lock the auth file here, as it's
+	     * temporary - we can assume, that we are the only ones
+	     * knowing about this file anyway.
+	     */
+#ifdef HAS_MKSTEMP
+	    sprintf (name_buf, "%s/.XauthXXXXXX", d->userAuthDir);
+	    new = fdOpenW (mkstemp (name_buf));
+#else
+	    for (i = 0; i < 100; i++) {
+		sprintf (name_buf, "%s/.XauthXXXXXX", d->userAuthDir);
+		(void) mktemp (name_buf);
+		if ((new = 
+		     fdOpenW (open (name_buf, O_WRONLY | O_CREAT | O_EXCL, 
+				    0600))))
+		    break;
 	    }
-	}
-	if (lockStatus != LOCK_SUCCESS) {
-	    sprintf (backup_name, "%s/.XauthXXXXXX", d->userAuthDir);
-	    (void) mktemp (backup_name);	/* XXX insecure */
-	    lockStatus = XauLockAuth (backup_name, 1, 2, 10);
-	    Debug ("backup lock is %d\n", lockStatus);
-	    if (lockStatus == LOCK_SUCCESS) {
-		if (openFiles (backup_name, new_name, &old, &new)) {
-		    name = backup_name;
-		    set_env = 1;
-		} else {
-		    XauUnlockAuth (backup_name);
-		    lockStatus = LOCK_ERROR;
-		}	
+#endif
+	    if (!new) {
+		LogError ("Can't create authorization file in %s\n", 
+			  d->userAuthDir);
+		return;
 	    }
+	    name = 0;
+	    envname = name_buf;
+	    old = 0;
 	}
-	if (lockStatus != LOCK_SUCCESS) {
-	    LogError ("can't lock authorization file %s or backup %s\n",
-			    home_name, backup_name);
-	    return;
-	}
-	initAddrs ();
 	doWrite = 1;
 	Debug ("%d authorization protocols for %s\n", d->authNum, d->name);
 	/*
@@ -1139,11 +1293,11 @@ SetUserAuthorization (struct display *d, struct verify_info *verify)
 		!memcmp (auths[i]->name, "MIT-MAGIC-COOKIE-1", 18))
 	    {
 		magicCookie = i;
-	    	if ((d->displayType & d_location) == Local)
-	    	    writeLocalAuth (new, auths[i], d->name);
+		if ((d->displayType & d_location) == dLocal)
+		    writeLocalAuth (new, auths[i], d->name);
 #ifdef XDMCP
-	    	else
-	    	    writeRemoteAuth (new, auths[i], (XdmcpNetaddr)d->peer.data, 
+		else
+		    writeRemoteAuth (new, auths[i], (XdmcpNetaddr)d->peer.data, 
 				     d->peer.length, d->name);
 #endif
 		break;
@@ -1161,123 +1315,56 @@ SetUserAuthorization (struct display *d, struct verify_info *verify)
 		if (auths[i]->name_length == 14 &&
 		    !strncmp (auths[i]->name, "MIT-KERBEROS-5", 14))
 		    auths[i]->data_length = 0;
-	    	if ((d->displayType & d_location) == Local)
-	    	    writeLocalAuth (new, auths[i], d->name);
+		if ((d->displayType & d_location) == dLocal)
+		    writeLocalAuth (new, auths[i], d->name);
 #ifdef XDMCP
-	    	else
-	    	    writeRemoteAuth (new, auths[i], (XdmcpNetaddr)d->peer.data, 
+		else
+		    writeRemoteAuth (new, auths[i], (XdmcpNetaddr)d->peer.data, 
 				     d->peer.length, d->name);
 #endif
 		auths[i]->data_length = data_len;
 	    }
 	}
-	if (old) {
-	    if (fstat (fileno (old), &statb) != -1)
-		chmod (new_name, (int) (statb.st_mode & 0777));
-	    /*SUPPRESS 560*/
-	    while ((entry = XauReadAuth (old))) {
-		if (!checkEntry (entry))
-		{
-		    Debug ("Writing an entry\n");
-		    writeAuth (new, entry);
-		}
-		XauDisposeAuth (entry);
-	    }
-	    fclose (old);
+	endUserAuth (old, new, new_name);
+	if (name)
+	    envname = moveUserAuth (name, new_name, envname);
+	if (envname) {
+	    userEnviron = setEnv (userEnviron, "XAUTHORITY", envname);
+	    systemEnviron = setEnv (systemEnviron, "XAUTHORITY", envname);
 	}
-	doneAddrs ();
-	fclose (new);
-	if (unlink (name) == -1)
-	    Debug ("unlink %s failed\n", name);
-	envname = name;
-	if (link (new_name, name) == -1) {
-	    Debug ("link failed %s %s\n", new_name, name);
-	    LogError ("Can't move authorization into place\n");
-	    set_env = 1;
-	    envname = new_name;
-	} else {
-	    Debug ("new authorization moved into place\n");
-	    unlink (new_name);
-	}
-	if (set_env) {
-	    verify->userEnviron = setEnv (verify->userEnviron,
-				    "XAUTHORITY", envname);
-	    verify->systemEnviron = setEnv (verify->systemEnviron,
-				    "XAUTHORITY", envname);
-	}
-	XauUnlockAuth (name);
 	/* a chown() used to be here, but this code runs as user anyway */
     }
     Debug ("done SetUserAuthorization\n");
 }
 
 void
-RemoveUserAuthorization (struct display *d, struct verify_info *verify)
+RemoveUserAuthorization (struct display *d)
 {
-    char    *home;
-    Xauth   **auths, *entry;
-    char    name[1024], new_name[1024];
-    int	    lockStatus;
+    Xauth   **auths;
     FILE    *old, *new;
-    struct stat	statb;
     int	    i;
+    char    name[NBSIZE], new_name[NBSIZE + 2];
 
     if (!(auths = d->authorizations))
 	return;
-    home = getEnv (verify->userEnviron, "HOME");
-    if (!home)
-	return;
     Debug ("RemoveUserAuthorization\n");
-    strcpy (name, home);
-    if (home[strlen(home) - 1] != '/')
-	strcat (name, "/");
-    strcat (name, ".Xauthority");
-    Debug ("XauLockAuth %s\n", name);
-    lockStatus = XauLockAuth (name, 1, 2, 10);
-    Debug ("Lock is %d\n", lockStatus);
-    if (lockStatus != LOCK_SUCCESS)
-	return;
-    if (openFiles (name, new_name, &old, &new))
+    startUserAuth (name, new_name, &old, &new);
+    if (new)
     {
-	initAddrs ();
 	doWrite = 0;
 	for (i = 0; i < d->authNum; i++)
 	{
-	    if ((d->displayType & d_location) == Local)
-	    	writeLocalAuth (new, auths[i], d->name);
+	    if ((d->displayType & d_location) == dLocal)
+		writeLocalAuth (new, auths[i], d->name);
 #ifdef XDMCP
 	    else
-	    	writeRemoteAuth (new, auths[i], (XdmcpNetaddr)d->peer.data, 
+		writeRemoteAuth (new, auths[i], (XdmcpNetaddr)d->peer.data, 
 				 d->peer.length, d->name);
 #endif
 	}
 	doWrite = 1;
-	if (old) {
-	    if (fstat (fileno (old), &statb) != -1)
-		chmod (new_name, (int) (statb.st_mode & 0777));
-	    /*SUPPRESS 560*/
-	    while ((entry = XauReadAuth (old))) {
-		if (!checkEntry (entry))
-		{
-		    Debug ("Writing an entry\n");
-		    writeAuth (new, entry);
-		}
-		XauDisposeAuth (entry);
-	    }
-	    fclose (old);
-	}
-	doneAddrs ();
-	fclose (new);
-	if (unlink (name) == -1)
-	    Debug ("unlink %s failed\n", name);
-	if (link (new_name, name) == -1) {
-	    Debug ("link failed %s %s\n", new_name, name);
-	    LogError ("Can't move authorization into place\n");
-	} else {
-	    Debug ("new authorization moved into place\n");
-	    unlink (new_name);
-	}
+	endUserAuth (old, new, new_name);
+	(void) moveUserAuth (name, new_name, 0);
     }
-    XauUnlockAuth (name);
     Debug ("done RemoveUserAuthorization\n");
 }
