@@ -1,7 +1,7 @@
-/* $TOG: verify.c /main/37 1998/02/11 10:00:45 kaleb $ */
 /*
 
 Copyright 1988, 1998  The Open Group
+Copyright 2000-2004 Oswald Buddenhagen <ossi@kde.org>
 
 Permission to use, copy, modify, distribute, and sell this software and its
 documentation for any purpose is hereby granted without fee, provided that
@@ -15,25 +15,22 @@ in all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
+IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
-Except as contained in this notice, the name of The Open Group shall
+Except as contained in this notice, the name of a copyright holder shall
 not be used in advertising or otherwise to promote the sale, use or
 other dealings in this Software without prior written authorization
-from The Open Group.
+from the copyright holder.
 
 */
-/* $XFree86: xc/programs/xdm/greeter/verify.c,v 3.9 2000/06/14 00:16:16 dawes Exp $ */
 
 /*
  * xdm - display manager daemon
  * Author:  Keith Packard, MIT X Consortium
  *
- * verify.c
- *
  * user verification and session initiation.
  */
 
@@ -47,6 +44,7 @@ from The Open Group.
 #ifdef SECURE_RPC
 # include <rpc/rpc.h>
 # include <rpc/key_prot.h>
+extern int key_setnet(struct key_netstarg *arg);
 #endif
 #ifdef K5AUTH
 # include <krb5/krb5.h>
@@ -101,6 +99,7 @@ extern char *crypt(const char *, const char *);
 char *curuser;
 char *curpass;
 char *curtype;
+char *newpass;
 char **userEnviron;
 char **systemEnviron;
 static int curuid;
@@ -124,10 +123,19 @@ static char krbtkfile[MAXPATHLEN];
 # endif
 #endif
 
-#define V_RET(e) \
+#define V_RET_AUTH \
 	do { \
 		PrepErrorGreet (); \
-		GSendInt (e); \
+		GSendInt (V_AUTH); \
+		return 0; \
+	} while(0)
+
+#define V_RET_FAIL(m) \
+	do { \
+		PrepErrorGreet (); \
+		GSendInt (V_MSG_ERR); \
+		GSendStr (m); \
+		GSendInt (V_FAIL); \
 		return 0; \
 	} while(0)
 
@@ -157,7 +165,7 @@ PAM_conv (int num_msg,
     struct pam_response *reply;
     struct pam_data *pd = (struct pam_data *)appdata_ptr;
 
-    if (!(reply = calloc(num_msg, sizeof(*reply))))
+    if (!(reply = Calloc(num_msg, sizeof(*reply))))
 	return PAM_CONV_ERR;
 
     ReInitErrorLog ();
@@ -248,7 +256,7 @@ PAM_conv_null (int num_msg,
     int count;
     struct pam_response *reply;
 
-    if (!(reply = calloc(num_msg, sizeof(*reply))))
+    if (!(reply = Calloc(num_msg, sizeof(*reply))))
 	return PAM_CONV_ERR;
 
     ReInitErrorLog ();
@@ -313,7 +321,7 @@ doPAMAuth (const char *psrv, struct pam_data *pdata)
 	  pam_bail2:
 	    ReInitErrorLog ();
 	    LogError ("PAM error: %s\n", pam_strerror (pamh, pretc));
-	    V_RET (V_ERROR);
+	    V_RET_FAIL (0);
 	}
 	if ((pretc = pam_set_item (pamh, PAM_USER, curuser)) != PAM_SUCCESS)
 	    goto pam_bail;
@@ -359,9 +367,9 @@ doPAMAuth (const char *psrv, struct pam_data *pdata)
 	case PAM_AUTH_ERR:
 	case PAM_MAXTRIES: /* should handle this better ... */
 	case PAM_AUTHINFO_UNAVAIL: /* returned for unknown users ... bogus */
-	    V_RET (V_AUTH);
+	    V_RET_AUTH;
 	default:
-	    V_RET (V_ERROR);
+	    V_RET_FAIL (0);
 	}
     }
     return 1;
@@ -370,9 +378,17 @@ doPAMAuth (const char *psrv, struct pam_data *pdata)
 #endif /* USE_PAM */
 
 static int
+#if defined(USE_PAM) || defined(AIXV3)
 AccNoPass (const char *un)
 {
+    struct passwd *pw = 0;
+#else
+AccNoPass (const char *un, struct passwd *pw)
+{
+#endif
+    struct group *gr;
     char **fp;
+    int hg;
 
     if (!*un)
 	return 0;
@@ -380,9 +396,40 @@ AccNoPass (const char *un)
     if (!strcmp (un, td->autoUser))
 	return 1;
 
-    for (fp = td->noPassUsers; *fp; fp++)
-	if (!strcmp (un, *fp))
+    for (hg = 0, fp = td->noPassUsers; *fp; fp++)
+	if (**fp == '@')
+	    hg = 1;
+	else if (!strcmp (un, *fp))
 	    return 1;
+	else if (!strcmp ("*", *fp)) {
+#if defined(USE_PAM) || defined(AIXV3)
+	    if (!(pw = getpwnam (un)))
+		return 0;
+#endif
+	    if (pw->pw_uid)
+		return 1;
+	}
+
+#if defined(USE_PAM) || defined(AIXV3)
+    if (hg && (pw || (pw = getpwnam (un)))) {
+#else
+    if (hg) {
+#endif
+	for (setgrent (); (gr = getgrent ()); )
+	    for (fp = td->noPassUsers; *fp; fp++)
+		if (**fp == '@' && !strcmp (gr->gr_name, *fp + 1)) {
+		    if (pw->pw_gid == gr->gr_gid) {
+			endgrent ();
+			return 1;
+		    }
+		    for (; *gr->gr_mem; gr->gr_mem++)
+			if (!strcmp (un, *gr->gr_mem)) {
+			    endgrent ();
+			    return 1;
+			}
+		}
+	endgrent ();
+    }
 
     return 0;
 }
@@ -407,6 +454,8 @@ Verify (GConvFunc gconv, int rootok)
 #else
     struct stat		st;
     const char		*nolg;
+    char		*buf;
+    int			fd;
 # ifdef HAVE_GETUSERSHELL
     char		*s;
 # endif
@@ -500,12 +549,15 @@ Verify (GConvFunc gconv, int rootok)
 	    if (msg)
 		free (msg);
 	    loginfailed (curuser, hostname, tty);
-	    V_RET (i == ENOENT || i == ESAD ? V_AUTH : V_ERROR);
+	    if (i == ENOENT || i == ESAD)
+		V_RET_AUTH;
+	    else
+		V_RET_FAIL (0);
 	}
 	if (reenter) {
 	    LogError ("authenticate() requests more data: %s\n", msg);
 	    free (msg);
-	    V_RET (V_ERROR);
+	    V_RET_FAIL (0);
 	}
     } else if (!strcmp (curtype, "generic")) {
 	if (!gconv (GCONV_USER, 0))
@@ -517,7 +569,10 @@ Verify (GConvFunc gconv, int rootok)
 		if (msg)
 		    free (msg);
 		loginfailed (curuser, hostname, tty);
-		V_RET (i == ENOENT || i == ESAD ? V_AUTH : V_ERROR);
+		if (i == ENOENT || i == ESAD)
+		    V_RET_AUTH;
+		else
+		    V_RET_FAIL (0);
 	    }
 	    if (curret)
 		free (curret);
@@ -529,7 +584,7 @@ Verify (GConvFunc gconv, int rootok)
 	}
     } else {
 	LogError ("Unsupported authentication type %\"s requested\n", curtype);
-	V_RET (V_ERROR);
+	V_RET_FAIL (0);
     }
     if (msg) {
 	PrepErrorGreet ();
@@ -544,7 +599,7 @@ Verify (GConvFunc gconv, int rootok)
 
     if (strcmp (curtype, "classic")) {
 	LogError ("Unsupported authentication type %\"s requested\n", curtype);
-	V_RET (V_ERROR);
+	V_RET_FAIL (0);
     }
 
     if (!gconv (GCONV_USER, 0))
@@ -552,12 +607,12 @@ Verify (GConvFunc gconv, int rootok)
 
     if (!(p = getpwnam (curuser))) {
 	Debug ("getpwnam() failed.\n");
-	V_RET (V_AUTH);
+	V_RET_AUTH;
     }
 # ifdef linux	/* only Linux? */
     if (p->pw_passwd[0] == '!' || p->pw_passwd[0] == '*') {
 	Debug ("account is locked\n");
-	V_RET (V_AUTH);
+	V_RET_AUTH;
     }
 # endif
 
@@ -565,18 +620,18 @@ Verify (GConvFunc gconv, int rootok)
     if ((sp = getspnam (curuser)))
 	p->pw_passwd = sp->sp_pwdp;
     else
-	Debug ("getspnam() failed: %s.  Are you root?\n", SysErrorMsg());
+	Debug ("getspnam() failed: %m.  Are you root?\n");
 # endif
 
     if (!*p->pw_passwd) {
 	if (!td->allowNullPasswd) {
 	    Debug ("denying user with empty password\n");
-	    V_RET (V_AUTH);
+	    V_RET_AUTH;
 	}
 	goto nplogin;
     }
 
-    if (AccNoPass (curuser)) {
+    if (AccNoPass (curuser, p)) {
       nplogin:
 	gconv (GCONV_PASS_ND, 0);
 	if (!*curpass) {
@@ -595,7 +650,7 @@ Verify (GConvFunc gconv, int rootok)
 
 	if (krb_get_lrealm (realm, 1)) {
 	    LogError ("Can't get KerberosIV realm.\n");
-	    V_RET (V_ERROR);
+	    V_RET_FAIL (0);
 	}
 
 	sprintf (krbtkfile, "%s.%.*s", TKT_ROOT, MAXPATHLEN - strlen(TKT_ROOT) - 2, td->name);
@@ -611,7 +666,7 @@ Verify (GConvFunc gconv, int rootok)
 	    LogError ("KerberosIV verification failure %\"s for %s\n",
 		      krb_get_err_text (ret), curuser);
 	    krbtkfile[0] = '\0';
-	    V_RET (V_ERROR);
+	    V_RET_FAIL (0);
 	}
 	Debug ("KerberosIV verify failed: %s\n", krb_get_err_text (ret));
     }
@@ -625,7 +680,7 @@ Verify (GConvFunc gconv, int rootok)
 # endif
     {
 	Debug ("password verify failed\n");
-	V_RET (V_AUTH);
+	V_RET_AUTH;
     }
 
   done:
@@ -637,12 +692,12 @@ Verify (GConvFunc gconv, int rootok)
 #if defined(USE_PAM) || defined(AIXV3)
     if (!(p = getpwnam (curuser))) {
 	LogError ("getpwnam(%s) failed.\n", curuser);
-	V_RET (V_ERROR);
+	V_RET_FAIL (0);
     }
 #endif
     if (!p->pw_uid) {
 	if (!rootok && !td->allowRootLogin)
-	    V_RET (V_NOROOT);
+	    V_RET_FAIL ("Root logins are not allowed");
 	return 1;	/* don't deny root to log in */
     }
 
@@ -666,12 +721,8 @@ Verify (GConvFunc gconv, int rootok)
 	} else
 	    GSendInt (V_CHTOK);
 	for (;;) {
-	    /* don't use PAM_CHANGE_EXPIRED_AUTHTOK:
-	       - makes pam_unix ask for a password (why?!)
-	       - the frontend cannot depend on the requested item sequence
-	    */
 	    Debug (" pam_chauthtok() ...\n");
-	    pretc = pam_chauthtok (pamh, 0);
+	    pretc = pam_chauthtok (pamh, PAM_CHANGE_EXPIRED_AUTHTOK);
 	    ReInitErrorLog ();
 	    Debug (" pam_chauthtok() returned: %s\n", pam_strerror (pamh, pretc));
 	    if (pdata.abort)
@@ -679,10 +730,14 @@ Verify (GConvFunc gconv, int rootok)
 	    if (pretc == PAM_SUCCESS)
 		break;
 	    /* effectively there is only PAM_AUTHTOK_ERR */
-	    GSendInt (V_RETRY);
+	    GSendInt (V_FAIL);
 	}
+	if (curpass)
+	    free (curpass);
+	curpass = newpass;
+	newpass = 0;
     } else if (pretc != PAM_SUCCESS)
-	V_RET (V_AUTH);
+	V_RET_AUTH;
 
 #elif defined(AIXV3)	/* USE_PAM */
 
@@ -711,7 +766,7 @@ Verify (GConvFunc gconv, int rootok)
 	if (!(s = getusershell ())) {
 	    Debug ("shell not in /etc/shells\n");
 	    endusershell ();
-	    V_RET (V_BADSHELL);
+	    V_RET_FAIL ("Your login shell is not listed in /etc/shells");
 	}
 	if (!strcmp (s, p->pw_shell)) {
 	    endusershell ();
@@ -727,7 +782,7 @@ Verify (GConvFunc gconv, int rootok)
     lc = login_getpwclass (p);
 #  endif
     if (!lc)
-	V_RET (V_ERROR);
+	V_RET_FAIL (0);
 # endif
 
 
@@ -745,8 +800,23 @@ Verify (GConvFunc gconv, int rootok)
 	 !stat ((nolg = _PATH_NOLOGIN), &st)))
     {
 	PrepErrorGreet ();
-	GSendInt (V_NOLOGIN);
-	GSendStr (nolg);
+	GSendInt (V_MSG_ERR);
+	if (st.st_size && (fd = open (nolg, O_RDONLY)) >= 0) {
+	    if ((buf = Malloc (st.st_size + 1))) {
+		if (read (fd, buf, st.st_size) == st.st_size) {
+		    buf[st.st_size] = 0;
+		    GSendStr (buf);
+		    free (buf);
+		    close (fd);
+		    GSendInt (V_FAIL);
+		    LC_RET0;
+		}
+		free (buf);
+	    }
+	    close (fd);
+	}
+	GSendStr ("Logins are not allowed at the moment.\nTry again later");
+	GSendInt (V_FAIL);
 	LC_RET0;
     }
 
@@ -757,7 +827,9 @@ Verify (GConvFunc gconv, int rootok)
 	struct stat st;
 	if (!*p->pw_dir || stat (p->pw_dir, &st) || st.st_uid != p->pw_uid) {
 	    PrepErrorGreet ();
-	    GSendInt (V_NOHOME);
+	    GSendInt (V_MSG_ERR);
+	    GSendStr ("Home folder not available");
+	    GSendInt (V_FAIL);
 	    LC_RET0;
 	}
     }
@@ -769,7 +841,9 @@ Verify (GConvFunc gconv, int rootok)
 #  ifdef HAVE_AUTH_TIMEOK
     if (!auth_timeok (lc, time (NULL))) {
 	PrepErrorGreet ();
-	GSendInt (V_BADTIME);
+	GSendInt (V_MSG_ERR);
+	GSendStr ("You are not allowed to login at the moment");
+	GSendInt (V_FAIL);
 	LC_RET0;
     }
 #  endif
@@ -811,12 +885,21 @@ Verify (GConvFunc gconv, int rootok)
 #  endif
 	    if (tim > expir) {
 		PrepErrorGreet ();
-		GSendInt (V_AEXPIRED);
+		GSendInt (V_MSG_ERR);
+		GSendStr ("Your account has expired;"
+			  " please contact your system administrator");
+		GSendInt (V_FAIL);
 		LC_RET0;
 	    } else if (tim > (expir - warntime) && !quietlog) {
-		PrepErrorGreet ();
-		GSendInt (V_AWEXPIRE);
-		GSendInt (expir - tim);
+		ASPrintf (&buf,
+			  "Warning: your account will expire in %d day(s)",
+			  expir - tim);
+		if (buf) {
+		    PrepErrorGreet ();
+		    GSendInt (V_MSG_INFO);
+		    GSendStr (buf);
+		    free (buf);
+		}
 	    }
 	}
 
@@ -826,28 +909,41 @@ Verify (GConvFunc gconv, int rootok)
 #  else
 	if (!sp->sp_lstchg) {
 	    PrepErrorGreet ();
-	    GSendInt (V_PFEXPIRED);
+	    GSendInt (V_MSG_ERR);
+	    GSendStr ("You are required to change your password immediately"
+		      " (root enforced)");
 	    /* XXX todo password change */
-	    GSendInt (V_AUTH);
+	    GSendInt (V_FAIL);
 	    LC_RET0;
 	} else if (sp->sp_max != -1) {
 	    expir = sp->sp_lstchg + sp->sp_max;
 	    if (sp->sp_inact != -1 && tim > expir + sp->sp_inact) {
 		PrepErrorGreet ();
-		GSendInt (V_APEXPIRED);
+		GSendInt (V_MSG_ERR);
+		GSendStr ("Your account has expired;"
+			  " please contact your system administrator");
+		GSendInt (V_FAIL);
 		LC_RET0;
 	    }
 #  endif
 	    if (tim > expir) {
 		PrepErrorGreet ();
-		GSendInt (V_PEXPIRED);
+		GSendInt (V_MSG_ERR);
+		GSendStr ("You are required to change your password immediately"
+			  " (password aged)");
 		/* XXX todo password change */
-		GSendInt (V_AUTH);
+		GSendInt (V_FAIL);
 		LC_RET0;
 	    } else if (tim > (expir - warntime) && !quietlog) {
-		PrepErrorGreet ();
-		GSendInt (V_PWEXPIRE);
-		GSendInt (expir - tim);
+		ASPrintf (&buf,
+			  "Warning: your password will expire in %d day(s)",
+			  expir - tim);
+		if (buf) {
+		    PrepErrorGreet ();
+		    GSendInt (V_MSG_INFO);
+		    GSendStr (buf);
+		    free (buf);
+		}
 	    }
 	}
 
@@ -894,14 +990,13 @@ SetGid (const char *name, int gid)
 {
     if (setgid(gid) < 0)
     {
-	LogError("setgid(%d) (user %s) failed: %s\n",
-		 gid, name, SysErrorMsg());
+	LogError("setgid(%d) (user %s) failed: %m\n", gid, name);
 	return 0;
     }
 #ifndef QNX4
     if (initgroups(name, gid) < 0)
     {
-	LogError("initgroups for %s failed: %s\n", name, SysErrorMsg());
+	LogError("initgroups for %s failed: %m\n", name);
 	return 0;
     }
 #endif   /* QNX4 doesn't support multi-groups, no initgroups() */
@@ -913,8 +1008,7 @@ SetUid (const char *name, int uid)
 {
     if (setuid(uid) < 0)
     {
-	LogError("setuid(%d) (user %s) failed: %s\n",
-		 uid, name, SysErrorMsg());
+	LogError("setuid(%d) (user %s) failed: %m\n", uid, name);
 	return 0;
     }
     return 1;
@@ -1004,8 +1098,8 @@ StartClient ()
 #ifdef HAS_SETUSERCONTEXT
     extern char	**environ;
 #endif
-    char	*failsafeArgv[2];
-    int		i, pid;
+    char	*failsafeArgv[2], *lname;
+    int		i, pid, lfd;
 
     if (StrCmp (dmrcuser, curuser)) {
 	if (curdmrc) { free (curdmrc); curdmrc = 0; }
@@ -1046,7 +1140,7 @@ StartClient ()
 
     env = baseEnv (curuser);
     xma = 0;
-    if (td->fifoPath && StrDup (&xma, td->fifoPath)) {
+    if (td->ctrl.fpath && StrDup (&xma, td->ctrl.fpath)) {
 	if ((td->allowShutdown == SHUT_ALL ||
 	     (td->allowShutdown == SHUT_ROOT && !curuser)) &&
 	    StrApp (&xma, ",maysd", (char *)0))
@@ -1063,6 +1157,8 @@ StartClient ()
     } else
         StrDup (&xma, "true");
     StrApp (&xma, ",method=", curtype, (char *)0);
+    if (td_setup)
+	StrApp (&xma, ",auto", (char *)0);
     if (xma)
     {
 	env = setEnv (env, "XDM_MANAGED", xma);
@@ -1127,7 +1223,7 @@ StartClient ()
      * Run system-wide initialization file
      */
     sourceReset = 1;
-    if (source (systemEnviron, td->startup) != 0) {
+    if (source (systemEnviron, td->startup, td_setup) != 0) {
 	LogError ("Cannot execute startup script %\"s\n", td->startup);
 	SessionExit (EX_NORMAL);
     }
@@ -1144,16 +1240,20 @@ StartClient ()
     ReInitErrorLog ();
 #endif    
     removeAuth = 1;
-    if (td->fifoPath)
-	chown (td->fifoPath, curuid, -1);
+    chownCtrl (&td->ctrl, curuid, -1);
     endpwent ();
 #if !defined(USE_PAM) && !defined(AIXV3)
 # ifndef QNX4  /* QNX4 doesn't need endspent() to end shadow passwd ops */
     endspent ();
 # endif
 #endif
+    ClearCloseOnFork (mstrtalk.pipe->wfd);
     switch (pid = Fork ()) {
     case 0:
+	if (Setjmp (mstrtalk.errjmp))
+	    exit (1);
+	GSet (&mstrtalk);
+
 	/* Do system-dependent login setup here */
 #ifdef CSRG_BASED
 	setsid();
@@ -1177,10 +1277,8 @@ StartClient ()
 # endif
 # ifdef USE_PAM
 #  ifdef AIXV3
-	if (!(environ = initStrArr (0))) {
-	    LogOutOfMem("StartSession");
+	if (!(environ = initStrArr (0)))
 	    exit (1);
-	}
 #  endif
 	if ((pretc = pam_setcred (pamh, 0)) != PAM_SUCCESS) {
 	    ReInitErrorLog ();
@@ -1203,7 +1301,7 @@ StartClient ()
 #  if defined(BSD) && (BSD >= 199103)
 	if (setlogin(curuser) < 0)
 	{
-	    LogError("setlogin for %s failed: %s\n", curuser, SysErrorMsg());
+	    LogError("setlogin for %s failed: %m\n", curuser);
 	    exit (1);
 	}
 #  endif
@@ -1216,10 +1314,8 @@ StartClient ()
 	 * We need to do this before setusercontext() because that may
 	 * set or reset some environment variables.
 	 */
-	if (!(environ = initStrArr (0))) {
-	    LogOutOfMem("StartSession");
+	if (!(environ = initStrArr (0)))
 	    exit (1);
-	}
 
 	/*
 	 * Set the user's credentials: uid, gid, groups,
@@ -1227,8 +1323,7 @@ StartClient ()
 	 */
 	if (setusercontext(NULL, p, p->pw_uid, LOGIN_SETALL) < 0)
 	{
-	    LogError("setusercontext for %s failed: %s\n",
-		     curuser, SysErrorMsg());
+	    LogError("setusercontext for %s failed: %m\n", curuser);
 	    exit (1);
 	}
 
@@ -1243,7 +1338,7 @@ StartClient ()
 	 */
 	if (setpcred(curuser, NULL) == -1)
 	{
-	    LogError("setpcred for %s failed: %s\n", curuser, SysErrorMsg());
+	    LogError("setpcred for %s failed: %m\n", curuser);
 	    exit (1);
 	}
 
@@ -1276,6 +1371,7 @@ StartClient ()
 	    int	    nameret, keyret;
 	    int	    len;
 	    int     key_set_ok = 0;
+	    struct  key_netstarg netst;
 
 	    nameret = getnetname (netname);
 	    Debug ("user netname: %s\n", netname);
@@ -1285,6 +1381,11 @@ StartClient ()
 	    keyret = getsecretkey(netname, secretkey, curpass);
 	    Debug ("getsecretkey returns %d, key length %d\n",
 		   keyret, strlen (secretkey));
+	    netst.st_netname = netname;
+	    memcpy (netst.st_priv_key, secretkey, HEXKEYBYTES);
+	    memset (netst.st_pub_key, 0, HEXKEYBYTES);
+	    if (key_setnet (&netst) < 0)
+		Debug ("Could not set secret key.\n");
 	    /* is there a key, and do we have the right password? */
 	    if (keyret == 1)
 	    {
@@ -1319,24 +1420,56 @@ StartClient ()
 	    else
 		NukeAuth (14, "MIT-KERBEROS-5");
 #endif /* K5AUTH */
+	if (td->autoReLogin) {
+	    GSendInt (D_ReLogin);
+	    GSendStr (curuser);
+	    GSendStr (curpass);
+	    GSendStr (newdmrc);
+	}
 	if (curpass)
 	    bzero (curpass, strlen (curpass));
 	SetUserAuthorization (td);
 	home = getEnv (userEnviron, "HOME");
 	if (home) {
 	    if (chdir (home) < 0) {
-		LogError ("Cannot chdir to %s's home %s: %s, using /\n",
-			  curuser, home, SysErrorMsg());
+		LogError ("Cannot chdir to %s's home %s: %m, using /\n",
+			  curuser, home);
 		home = 0;
 		userEnviron = setEnv (userEnviron, "HOME", "/");
-		chdir ("/");
+		goto cdroot;
 	    }
-	} else
+	    ASPrintf (&lname, td->clientLogFile, td->name);
+	    if ((lfd = creat (lname, 0600)) < 0) {
+		LogInfo ("Cannot create session log file %s: %m\n", lname);
+		free (lname);
+		goto tmperr;
+	    }
+	} else {
+	  cdroot:
 	    chdir ("/");
+	  tmperr:
+	    ASPrintf (&lname, "/tmp/xerr-%s-%s", curuser, td->name);
+	    unlink (lname);
+	    if ((lfd = open (lname, O_WRONLY|O_CREAT|O_EXCL, 0600)) < 0) {
+		LogError ("Cannot create fallback session log file %s: %m\n",
+			  lname);
+		goto logerr;
+	    }
+	}
+	dup2 (lfd, 1);
+	dup2 (lfd, 2);
+	close (lfd);
+      logerr:
+	free (lname);
 	if (!*dmrcDir)
 	    mergeSessionArgs (home != 0);
 	if (!(desksess = iniEntry (curdmrc, "Desktop", "Session", 0)))
 	    desksess = "failsafe"; /* only due to OOM */
+	GSendInt (D_User);
+	GSendInt (curuid);
+	GSendStr (curuser);
+	GSendStr (desksess);
+	close (mstrtalk.pipe->wfd);
 	userEnviron = setEnv (userEnviron, "DESKTOP_SESSION", desksess);
 	for (i = 0; td->sessionsDirs[i]; i++) {
 	    fname = 0;
@@ -1359,43 +1492,28 @@ StartClient ()
 	else
 	    sessargs = "";
       gotit:
-	argv = parseArgs ((char **)0, td->session);
-	if (argv && argv[0] && *argv[0]) {
-		argv = addStrArr (argv, sessargs, -1);
-		Debug ("executing session %\"[s\n", argv);
-		execute (argv, userEnviron);
-		LogError ("Session %\"s execution failed: %s\n",
-			  argv[0], SysErrorMsg());
-	} else {
-		LogError ("Session has no command/arguments\n");
-	}
+	if (!(argv = parseArgs ((char **)0, td->session)) ||
+	    !(argv = addStrArr (argv, sessargs, -1)))
+	    exit (1);
+	if (argv[0] && *argv[0]) {
+	    Debug ("executing session %\"[s\n", argv);
+	    execute (argv, userEnviron);
+	    LogError ("Session %\"s execution failed: %m\n", argv[0]);
+	} else
+	    LogError ("Session has no command/arguments\n");
 	failsafeArgv[0] = td->failsafeClient;
 	failsafeArgv[1] = 0;
 	execute (failsafeArgv, userEnviron);
-	LogError ("Failsafe client %\"s execution failed: %s\n",
-		  failsafeArgv[0], SysErrorMsg());
+	LogError ("Failsafe client %\"s execution failed: %m\n",
+		  failsafeArgv[0]);
 	exit (1);
     case -1:
-	LogError ("Forking session on %s failed: %s\n",
-		  td->name, SysErrorMsg());
+	RegisterCloseOnFork (mstrtalk.pipe->wfd);
+	LogError ("Forking session on %s failed: %m\n", td->name);
 	return 0;
     default:
+	RegisterCloseOnFork (mstrtalk.pipe->wfd);
 	Debug ("StartSession, fork succeeded %d\n", pid);
-/* ### right after forking dpy	mstrtalk.pipe = &td->pipe; */
-#ifdef nofork_session
-	if (!nofork_session)
-#endif
-	if (!Setjmp (mstrtalk.errjmp)) {
-	    GSet (&mstrtalk);
-	    GSendInt (D_User);
-	    GSendInt (curuid);
-	    if (td->autoReLogin) {
-		GSendInt (D_ReLogin);
-		GSendStr (curuser);
-		GSendStr (curpass);
-		GSendStr (newdmrc);
-	    }
-	}
 	return pid;
     }
 }
@@ -1414,12 +1532,10 @@ SessionExit (int status)
 	 * run system-wide reset file
 	 */
 	Debug ("source reset program %s\n", td->reset);
-	source (systemEnviron, td->reset);
+	source (systemEnviron, td->reset, td_setup);
     }
     if (removeAuth)
     {
-	if (td->fifoPath)
-	    chown (td->fifoPath, 0, -1);
 #ifdef USE_PAM
 	if (pamh) {
 	    int pretc;
@@ -1488,7 +1604,10 @@ ReadDmrc ()
 	free (fname);
 	return GE_Ok;
     }
-    if (errno != EPERM) {
+
+    /* on POSIX systems only EACCES can be returned - 
+       I leave the EPERM here just because I don't know all non-POSIX systems */
+    if (errno != EACCES && errno != EPERM) {
 	free (fname);
 	return GE_NoFile;
     }
@@ -1519,7 +1638,7 @@ ReadDmrc ()
     if (Reader (pfd[0], &len, sizeof(int)) == sizeof(int)) {
 	if (len == -1)
 	    err = GE_Denied;
-	else if ((curdmrc = malloc(len + 1))) {
+	else if ((curdmrc = Malloc(len + 1))) {
 	    if (Reader (pfd[0], curdmrc, len + 1) == len + 1)
 		err = GE_Ok;
 	    else {
