@@ -1,7 +1,7 @@
-/* $TOG: error.c /main/17 1998/02/09 13:55:13 kaleb $ */
 /*
 
 Copyright 1988, 1998  The Open Group
+Copyright 2000-2004 Oswald Buddenhagen <ossi@kde.org>
 
 Permission to use, copy, modify, distribute, and sell this software and its
 documentation for any purpose is hereby granted without fee, provided that
@@ -15,25 +15,22 @@ in all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
+IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
-Except as contained in this notice, the name of The Open Group shall
+Except as contained in this notice, the name of a copyright holder shall
 not be used in advertising or otherwise to promote the sale, use or
 other dealings in this Software without prior written authorization
-from The Open Group.
+from the copyright holder.
 
 */
-/* $XFree86: xc/programs/xdm/error.c,v 1.2 1998/10/10 15:25:34 dawes Exp $ */
 
 /*
  * xdm - display manager daemon
  * Author:  Keith Packard, MIT X Consortium
  *
- * error.c
- *
  * Log display manager errors to a file as
  * we generally do not have a terminal to talk to
  * or use syslog if it exists
@@ -42,7 +39,6 @@ from The Open Group.
 #include "dm.h"
 #include "dm_error.h"
 
-#include <sys/stat.h>
 #include <unistd.h>
 #include <stdio.h>
 
@@ -85,7 +81,8 @@ Panic (const char *mesg)
 void
 ReInitErrorLog ()
 {
-    InitLog ();
+    if (!(debugLevel & DEBUG_NOSYSLOG))
+	InitLog ();
 }
 #endif
 
@@ -93,7 +90,6 @@ void
 InitErrorLog (const char *errorLogFile)
 {
     int fd;
-    struct stat st;
     char buf[128];
 
 #ifdef USE_SYSLOG
@@ -102,19 +98,7 @@ InitErrorLog (const char *errorLogFile)
     /* We do this independently of using syslog, as we cannot redirect
      * the output of external programs to syslog.
      */
-    if (errorLogFile
-	|| fstat (1, &st) ||
-#ifndef X_NOT_POSIX
-	!(S_ISREG(st.st_mode) || S_ISFIFO(st.st_mode))
-#else
-	!(st.st_mode & (S_IFREG | S_IFIFO))
-#endif
-	|| fstat (2, &st) ||
-#ifndef X_NOT_POSIX
-	!(S_ISREG(st.st_mode) || S_ISFIFO(st.st_mode)))
-#else
-	!(st.st_mode & (S_IFREG | S_IFIFO)))
-#endif
+    if (!errorLogFile || strcmp (errorLogFile, "-"))
     {
 	if (!errorLogFile) {
 	    sprintf (buf, "/var/log/%s.log", prog);
@@ -139,10 +123,8 @@ InitErrorLog (const char *errorLogFile)
 	    if (!lseek (fd, 0, SEEK_END))
 		write (fd, WARNMSG, sizeof(WARNMSG) - 1);
 #endif
-	    if (fd != 1) {
-		dup2 (fd, 1);
-		close (fd);
-	    }
+	    dup2 (fd, 1);
+	    close (fd);
 	    dup2 (1, 2);
 	}
     }
