@@ -1,7 +1,7 @@
-/* $TOG: session.c /main/79 1998/02/09 13:56:17 kaleb $ */
 /*
 
 Copyright 1988, 1998  The Open Group
+Copyright 2000-2004 Oswald Buddenhagen <ossi@kde.org>
 
 Permission to use, copy, modify, distribute, and sell this software and its
 documentation for any purpose is hereby granted without fee, provided that
@@ -15,24 +15,23 @@ in all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
+IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
-Except as contained in this notice, the name of The Open Group shall
+Except as contained in this notice, the name of a copyright holder shall
 not be used in advertising or otherwise to promote the sale, use or
 other dealings in this Software without prior written authorization
-from The Open Group.
+from the copyright holder.
 
 */
-/* $XFree86: xc/programs/xdm/session.c,v 3.23 2000/06/17 00:27:34 dawes Exp $ */
 
 /*
  * xdm - display manager daemon
  * Author:  Keith Packard, MIT X Consortium
  *
- * session.c
+ * subdaemon event loop, etc.
  */
 
 #define NEED_SIGNAL
@@ -47,6 +46,12 @@ from The Open Group.
 #include <ctype.h>
 
 struct display *td;
+const char *td_setup = "auto";
+
+static void DeleteXloginResources (void);
+static void LoadXloginResources (void);
+static void SetupDisplay (const char *arg);
+
 
 static Jmp_buf	pingTime;
 
@@ -136,6 +141,14 @@ AutoLogon ()
 	newdmrc = td->hstent->nargs;
 	td->hstent->nargs = 0;
     } else if (*td->autoUser && tdiff > 0) {
+	unsigned int lmask;
+	Window dummy1, dummy2;
+	int dummy3, dummy4, dummy5, dummy6;
+	XQueryPointer (dpy, DefaultRootWindow (dpy),
+		       &dummy1, &dummy2, &dummy3, &dummy4, &dummy5, &dummy6,
+		       &lmask);
+	if (lmask & ShiftMask)
+	    return 0;
 	StrDup (&curuser, td->autoUser);
 	StrDup (&curpass, td->autoPass);
     } else
@@ -177,27 +190,27 @@ conv_interact (int what, const char *prompt)
 	    tag = GRecvInt();
 	    switch (what) {
 	    case GCONV_USER:
-		/* assert(tag == V_IS_USER); */
+		/* assert(tag & V_IS_USER); */
 		if (curuser)
 		    free (curuser);
 		curuser = ret;
 		break;
 	    case GCONV_PASS:
 	    case GCONV_PASS_ND:
-		/* assert(tag == V_IS_PASSWORD); */
+		/* assert(tag & V_IS_PASSWORD); */
 		if (curpass)
 		    free (curpass);
 		curpass = ret;
 		break;
 	    default:
-		switch (tag) {
-		case V_IS_USER:
+		if (tag & V_IS_USER)
 		    ReStr (&curuser, ret);
-		    break;
-		case V_IS_PASSWORD:
+		else if (tag & V_IS_PASSWORD)
 		    ReStr (&curpass, ret);
-		    break;
-		}
+		else if (tag & V_IS_NEWPASSWORD)
+		    ReStr (&newpass, ret);
+		else if (tag & V_IS_OLDPASSWORD)
+		    ReStr (&ret, curpass);
 	    }
 	}
 	return ret;
@@ -213,7 +226,7 @@ GTalk mstrtalk;	/* make static; see dm.c */
 int
 CtrlGreeterWait (int wreply)
 {
-    int		i, j, cmd, type, rootok;
+    int		i, cmd, type, rootok;
     char	*name, *pass, **avptr;
 #ifdef XDMCP
     ARRAY8Ptr	aptr;
@@ -234,6 +247,19 @@ CtrlGreeterWait (int wreply)
 	    Debug ("G_GetCfg\n");
 	    type = GRecvInt ();
 	    Debug (" index %#x\n", type);
+	    if (type == C_isLocal)
+		i = (td->displayType & d_location) == dLocal;
+	    else if (type == C_hasConsole)
+		i = td->console != 0;
+	    else if (type == C_isAuthorized)
+		i = td->authorizations != 0;
+	    else
+		goto normal;
+	    GSendInt (GE_Ok);
+	    Debug (" -> bool %d\n", i);
+	    GSendInt (i);
+	    break;
+	  normal:
 	    if (!(avptr = FindCfgEnt (td, type))) {
 		Debug (" -> not found\n");
 		GSendInt (GE_NoEnt);
@@ -334,7 +360,7 @@ CtrlGreeterWait (int wreply)
 	  doverify:
 	    if (curuser) { free (curuser); curuser = 0; }
 	    if (curpass) { free (curpass); curpass = 0; }
-	    if (curtype) { free (curtype); curtype = 0; }
+	    if (curtype) free (curtype);
 	    curtype = GRecvStr ();
 	    Debug (" type %\"s\n", curtype);
 	    if (Verify (conv_interact, rootok)) {
@@ -343,22 +369,10 @@ CtrlGreeterWait (int wreply)
 	    } else
 		Debug (" -> failure returned\n");
 	    break;
-	case G_Shutdown:
-	    i = GRecvInt ();
-	    j = GRecvInt ();
-#ifdef nofork_session
-	    if (nofork_session)
-		break;
-#endif
-	    GSet (&mstrtalk);
-	    GSendInt (D_Shutdown);
-	    GSendInt (i);
-	    GSendInt (j);
-	    GSet (&grttalk);
-	    break;
 	case G_SetupDpy:
 	    Debug ("G_SetupDpy\n");
-	    SetupDisplay ();
+	    SetupDisplay (0);
+	    td_setup = 0;
 	    GSendInt (0);
 	    break;
 	default:
@@ -401,7 +415,7 @@ OpenGreeter ()
 
     grttalk.pipe = &grtproc.pipe;
     env = systemEnv ((char *) 0);
-    if (GOpen (&grtproc, (char **)0, "_greet", env, name))
+    if (GOpen (&grtproc, (char **)0, "_greet", env, name, &td->gpipe))
 	SessionExit (EX_UNMANAGE_DPY);
     freeStrArr (env);
     if ((cmd = CtrlGreeterWait (TRUE))) {
@@ -422,7 +436,7 @@ CloseGreeter (int force)
     if (!greeter)
 	return EX_NORMAL;
     greeter = 0;
-    ret = GClose (&grtproc, force);
+    ret = GClose (&grtproc, 0, force);
     Debug ("greeter for %s stopped\n", td->name);
     if (WaitCode(ret) > EX_NORMAL && WaitCode(ret) <= EX_MAX) {
 	Debug ("greeter-initiated session exit, code %d\n", WaitCode(ret));
@@ -472,7 +486,7 @@ catchTerm (int n ATTR_UNUSED)
 static int
 IOErrorHandler (Display *dspl ATTR_UNUSED)
 {
-    LogError("Fatal X server IO error: %s\n", SysErrorMsg());
+    LogError("Fatal X server IO error: %m\n");
     /* The only X interaction during the session are pings, and those
        have an own IOErrorHandler -> not EX_AL_RESERVER_DPY */
     Longjmp (abortSession, EX_RESERVER_DPY);
@@ -509,6 +523,8 @@ ManageSession (struct display *d)
     (void)XSetErrorHandler (ErrorHandler);
     (void)Signal (SIGTERM, catchTerm);
 
+    (void)Signal (SIGHUP, SIG_IGN);
+
     if (Setjmp (grttalk.errjmp))
 	Longjmp (abortSession, EX_RESERVER_DPY);	/* EX_RETRY_ONCE */
 
@@ -573,6 +589,9 @@ ManageSession (struct display *d)
 
     DeleteXloginResources ();
 
+    if (td_setup)
+	SetupDisplay (td_setup);
+
     if (!(clientPid = StartClient ())) {
 	LogError ("Client start failed\n");
 	SessionExit (EX_NORMAL); /* XXX maybe EX_REMANAGE_DPY? -- enable in dm.c! */
@@ -618,31 +637,29 @@ void
 LoadXloginResources ()
 {
     char	**args;
-    char	**env = 0;
+    char	**env;
 
     if (!xResLoaded && td->resources[0] && access (td->resources, 4) == 0) {
 	env = systemEnv ((char *) 0);
-	args = parseArgs ((char **) 0, td->xrdb);
-	args = parseArgs (args, td->resources);
-	Debug ("loading resource file: %s\n", td->resources);
-	(void) runAndWait (args, env);
-	freeStrArr (args);
+	if ((args = parseArgs ((char **) 0, td->xrdb)) &&
+	    (args = addStrArr (args, td->resources, -1))) {
+	    Debug ("loading resource file: %s\n", td->resources);
+	    (void) runAndWait (args, env);
+	    freeStrArr (args);
+	}
 	freeStrArr (env);
 	xResLoaded = TRUE;
     }
 }
 
 void
-SetupDisplay ()
+SetupDisplay (const char *arg)
 {
-    char	**env = 0;
+    char **env;
 
-    if (td->setup && td->setup[0])
-    {
-	env = systemEnv ((char *) 0);
-	(void) source (env, td->setup);
-	freeStrArr (env);
-    }
+    env = systemEnv ((char *) 0);
+    (void) source (env, td->setup, arg);
+    freeStrArr (env);
 }
 
 void
@@ -665,21 +682,17 @@ DeleteXloginResources ()
 
 
 int
-source (char **env, char *file)
+source (char **env, const char *file, const char *arg)
 {
-    char	**args, *args_safe[2];
+    char	**args;
     int		ret;
 
     if (file && file[0]) {
 	Debug ("source %s\n", file);
-	args = parseArgs ((char **) 0, file);
-	if (!args)
-	{
-	    args = args_safe;
-	    args[0] = file;
-	    args[1] = NULL;
-	    return runAndWait (args, env);
-	}
+	if (!(args = parseArgs ((char **) 0, file)))
+	    return waitCompose (0,0,3);
+	if (arg && !(args = addStrArr (args, arg, -1)))
+	    return waitCompose (0,0,3);
 	ret = runAndWait (args, env);
 	freeStrArr (args);
 	return ret;
@@ -728,6 +741,9 @@ baseEnv (const char *user)
 		memcmp (td->name, "localhost:", 10) ?
 		td->name : td->name + 9);
 
+    if (td->ctrl.path)
+    	env = setEnv (env, "DM_CONTROL", fifoDir);
+
     return env;
 }
 
