@@ -31,7 +31,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <qpopupmenu.h>
 #include <qbitmap.h>
 
-#include <dcopclient.h>
+#include <dcopref.h>
 
 #include <kconfig.h>
 #include <kglobalsettings.h>
@@ -59,6 +59,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 const int knDesktopPreviewSize = 12;
 const int knBtnSpacing = 1;
 
+// The previews tend to have a 4/3 aspect ratio
+static const int smallHeight = 32;
+static const int smallWidth = 42;
+
 extern "C"
 {
     KPanelApplet* init(QWidget *parent, const QString& configFile)
@@ -96,91 +100,112 @@ void KMiniPagerButton::resizeEvent(QResizeEvent *ev)
 
 void KMiniPagerButton::paintEvent(QPaintEvent *)
 {
-    QPainter p;
-
-    // the following breaks dark color schemes
-    QBrush bg = colorGroup().brush( (!isOn() && !isDown()) ? QColorGroup::Dark : QColorGroup::Base );
     setBackgroundOrigin( AncestorOrigin );
-    QColor fg = ((!isOn() && !isDown()) || pager()->transparent()) ? colorGroup().base() : colorGroup().text();
 
-    if ( pager()->desktopPreview() ) {
-	QPixmap pm( width() - 2, height() - 2 );
-	QBitmap pmask( pm.size(), true /*clear*/ );
-	if (pm.isNull() || pmask.isNull()) return;
-	QPainter pp( &pm, this  );
-	QPainter pmaskp( &pmask );
+    bool active = isOn() || isDown();
+    bool transparent = pager()->transparent();
 
-	if( ! pager()->transparent() )
-	    pp.fillRect( pm.rect(), bg );
-	pp.setPen(fg);
-	int dw = QApplication::desktop()->width();
-	int dh = QApplication::desktop()->height();
-	QValueList<WId>::ConstIterator it;
-	// SELI this needs checking for kwin_iii
-	for ( it = pager()->kwin()->stackingOrder().begin();
-	      it != pager()->kwin()->stackingOrder().end(); ++it )
-        {
-	    KWin::WindowInfo* info = pager()->info( *it );
+    QPixmap buffer( width(), height() );
+    QBitmap mask( buffer.size(), true );
+    QPainter bp( &buffer, this );
+    QPainter mp( &mask );
 
-	    if ( info &&
-                 info->windowType( NET::NormalMask | NET::DesktopMask | NET::DockMask
-	            | NET::ToolbarMask | NET::MenuMask | NET::DialogMask | NET::OverrideMask
-	            | NET::TopMenuMask | NET::UtilityMask | NET::SplashMask ) != NET::Desktop &&
-                 !info->isMinimized() &&
-                 (info->onAllDesktops() || info->desktop() == deskNum ) &&
-                 !(info->state() & NET::SkipPager || info->state() & NET::Shaded ) )
-            {
-		QRect r =  info->frameGeometry();
-		r = QRect( r.x() * pm.width() / dw, 2 + r.y() * pm.height() / dh,
-			   r.width() * pm.width() / dw, r.height() * pm.height() / dh );
-		if ( pager()->kwin()->activeWindow() == (*it) ) {
-		    qDrawShadeRect( &pp, r, colorGroup(), false, 1, 0, &colorGroup().brush( QColorGroup::Highlight ) );
-		} else {
-		    pp.fillRect( r, colorGroup().brush(  QColorGroup::Button ) );
-		    qDrawShadeRect( &pp, r, colorGroup(), true, 1, 0 );
-		}
-		if( pager()->transparent() )
-		    pmaskp.fillRect( r, color1 );
-	    }
-	}
-	pp.end();
-	pmaskp.end();
-	if( pager()->transparent() )
-	{
-	    pm.setMask( pmask );
-	    erase();
-	}
-	p.begin( this );
-	p.drawPixmap( 1, 1, pm );
+    // background...
+    if( ! transparent ) {
+        // fill with a solid background, if active Midlight, else Mid...
+        QBrush background = colorGroup().brush( active ? QColorGroup::Midlight : QColorGroup::Mid );
+        bp.fillRect( buffer.rect(), background );
     } else {
-	if( pager()->transparent() )
-	    erase();
-	p.begin( this );
-	if( ! pager()->transparent() )
-	    p.fillRect( rect(), bg );
+        // transparent windows get an 1 pixel frame...
+        if( active )
+            bp.setPen( colorGroup().midlight() );
+        else
+            bp.setPen( colorGroup().dark() );
+        bp.drawRect( buffer.rect() );
+        mp.setPen( Qt::color1 );
+        mp.drawRect( buffer.rect() );
     }
 
-    p.setPen( fg );
+    // window preview...
+    if( pager()->desktopPreview() ) {
+        int dw = QApplication::desktop()->width();
+        int dh = QApplication::desktop()->height();
+
+        QValueList<WId>::ConstIterator it;
+        // SELI this needs checking for kwin_iii
+        for ( it = pager()->kwin()->stackingOrder().begin();
+                it != pager()->kwin()->stackingOrder().end(); ++it ) {
+            KWin::WindowInfo* info = pager()->info( *it );
+            NET::WindowType type;
+            if( info )
+                 type= info->windowType( NET::NormalMask | NET::DesktopMask | NET::DockMask
+                    | NET::ToolbarMask | NET::MenuMask | NET::DialogMask | NET::OverrideMask
+                    | NET::TopMenuMask | NET::UtilityMask | NET::SplashMask );
+            if ( info &&
+                    type != NET::Desktop && type != NET::Dock &&
+                    !info->isMinimized() &&
+                    (info->onAllDesktops() || info->desktop() == deskNum ) &&
+                    !(info->state() & NET::SkipPager || info->state() & NET::Shaded ) ) {
+                QRect r =  info->frameGeometry();
+                int w = buffer.width(),
+                    h = buffer.height();
+                r = QRect( r.x() * w / dw, r.y() * h / dh,
+                            r.width() * w / dw, r.height() * h / dh );
+
+                QBrush fill;
+                QColor line;
+                if ( pager()->kwin()->activeWindow() == (*it) ) {
+                    fill = QBrush( colorGroup().highlight() );
+                    line = colorGroup().foreground() /*highlightedText()*/;
+                } else {
+                    fill = QBrush( colorGroup().background() );
+                    line = colorGroup().foreground();
+                }
+                bp.setPen( line );
+                bp.drawRect( r );
+                if( r.width() > 2 && r.height() > 2 )
+                    bp.fillRect( r.x()+1, r.y()+1, r.width()-2, r.height()-2, fill );
+                if( pager()->transparent() )
+                    mp.fillRect( r, color1 );
+            }
+        }
+    }
+
+    bp.end();
+    mp.end();
 
     if( pager()->transparent() )
-	p.setRasterOp( XorROP );
+    {
+        buffer.setMask( mask );
+        erase();
+    }
 
+    QPainter p( this );
+    p.drawPixmap( 0, 0, buffer );
+
+    // label...
     if ( pager()->mode() == KMiniPager::LabelNumber ) {
-	p.drawText(0, 0, width(), height(), AlignCenter, QString::number( deskNum ) );
+        p.setPen( colorGroup().background() );
+        p.translate(1,1);
+        p.drawText(0, 0, width()-1, height(), AlignCenter,
+                QString::number( deskNum ) );
+        p.setPen( colorGroup().foreground() );
+        p.translate(-1,-1);
+        p.drawText(0, 0, width(), height(), AlignCenter,
+                QString::number( deskNum ) );
     } else if ( pager()->mode() == KMiniPager::LabelName ) {
-	p.drawText( 0, 0, width(), height(), AlignVCenter | AlignCenter, pager()->kwin()->desktopName( deskNum ) );
+        p.setPen( colorGroup().background() );
+        p.translate(1,1);
+        p.drawText( 0, 0, width(), height(), AlignVCenter | AlignCenter,
+                pager()->kwin()->desktopName( deskNum ) );
+        p.setPen( colorGroup().foreground() );
+        p.translate(-1,-1);
+        p.drawText( 0, 0, width(), height(), AlignVCenter | AlignCenter,
+                pager()->kwin()->desktopName( deskNum ) );
     }
 
-    if(!isOn() && !isDown()){
-	p.setPen(colorGroup().light());
-	p.drawLine(0, 0, width()-1, 0);
-	p.drawLine(0, 0, 0, height()-1);
-	p.setPen(Qt::black);
-	p.drawLine(0, height()-1, width()-1, height()-1);
-	p.drawLine(width()-1, 0, width()-1, height()-1);
-    } else {
-	p.drawRect(rect());
-    }
+
+    p.end();
 }
 
 void KMiniPagerButton::mousePressEvent( QMouseEvent * e)
@@ -271,6 +296,10 @@ KMiniPager::KMiniPager(const QString& configFile, Type type, int actions,
     defFont = conf->readFontEntry("Font", &defFont);
     setFont(defFont);
 
+    m_rows = conf->readUnsignedNumEntry( "NumberOfRows", 0 );
+    if(m_rows > 3)
+        m_rows = 0;
+
     QString ms = conf->readEntry( "Label", "Number" );
     if ( ms == "None" )
 	m_mode = LabelNone;
@@ -355,20 +384,34 @@ void KMiniPager::slotButtonSelected( int desk )
 int KMiniPager::widthForHeight(int h) const
 {
     int deskNum = kwin_module->numberOfDesktops();
-    int deskHalf = (deskNum+1)/2;
-
-    int bw = h < 32 ? h : (h/2);
-    if ( desktopPreview() )
-	bw = (int) ( bw * (double) QApplication::desktop()->width() / QApplication::desktop()->height() );
-    if ( mode() == LabelName ) {
-	for (int i = 1; i <= deskNum; i++ ) {
-	    int sw = fontMetrics().width( kwin_module->desktopName( i ) ) + 6;
-	    if ( sw > bw )
-		bw = sw;
-	}
+    int rowNum = deskNum; // default is the number of desktops...
+    if( m_rows == 0 ) {
+        if( h<=32 || deskNum<=1 )
+            rowNum = 1;
+        else
+            rowNum = 2;
+    } else {
+        rowNum = m_rows;
     }
+    int deskCols = deskNum/rowNum;
+    if(deskNum == 0 || deskNum % rowNum != 0)
+        deskCols += 1;
 
-    int nWd = ( h <= 32 ? deskNum * bw : deskHalf * bw);
+    int nWd;
+    if( mode() != LabelName ) {
+        int bw = (h/rowNum);
+        if ( desktopPreview() )
+            bw = (int) ( bw * (double) QApplication::desktop()->width() / QApplication::desktop()->height() );
+        nWd = ( deskCols * bw);
+    } else {
+        int bw = (h/rowNum);
+        for (int i = 1; i <= deskNum; i++ ) {
+            int sw = fontMetrics().width( kwin_module->desktopName( i ) ) + 16;
+            if ( sw > bw )
+                bw = sw;
+        }
+        nWd = ( deskCols * bw );
+    }
 
     return nWd;
 }
@@ -376,19 +419,27 @@ int KMiniPager::widthForHeight(int h) const
 int KMiniPager::heightForWidth(int w) const
 {
     int deskNum = kwin_module->numberOfDesktops();
-    int deskHalf = (deskNum+1)/2;
+    int rowNum = deskNum; // actually these are columns now... oh well.
+    if( m_rows == 0 ) {
+        if( w<=48 || deskNum<=1 )
+            rowNum = 1;
+        else
+            rowNum = 2;
+    } else {
+        rowNum = m_rows;
+    }
+    int deskCols = deskNum/rowNum;
+    if(deskNum == 0 || deskNum % rowNum != 0)
+        deskCols += 1;
 
-    bool small = w <= 32;
-
-    int bh = small ? w : (w/2);
+    int bh = (w/rowNum);
     if ( desktopPreview() )
 	bh = (int) ( bh *  (double) QApplication::desktop()->height() / QApplication::desktop()->width() );
     else if ( mode() == LabelName ) {
 	bh = fontMetrics().lineSpacing() + 8;
-	small = true;
     }
 
-    int nHg = ( small ? deskNum * bh : deskHalf * bh);
+    int nHg = ( deskCols * bh);
 
     return nHg;
 }
@@ -417,15 +468,30 @@ void KMiniPager::updateDesktopLayout(int o, int x, int y)
    desktopLayoutY = y;
 }
 
+void KMiniPager::paintEvent(QPaintEvent *)
+{
+    // will be shown as 1 Pixel frame because of the grid's spacing...
+    QPainter p(this);
+    p.fillRect( rect(), transparent()?colorGroup().brush( QColorGroup::Background ):(colorGroup().mid()).dark(140) );
+}
+
 void KMiniPager::resizeEvent(QResizeEvent*)
 {
-    int deskNum = btnList.count();
-    int deskHalf = (deskNum+1)/2;
     bool horiz = orientation() == Horizontal;
-    bool small = (horiz && height() <=32) || (!horiz && width() <=32);
 
-    if ( !horiz && mode() == LabelName )
-	small = TRUE;
+    int deskNum = btnList.count();
+    int rowNum = deskNum;
+    if( m_rows == 0 ) {
+        if( ((horiz && height()<=32)||(!horiz && width()<=48)) || deskNum <= 1 )
+            rowNum = 1;
+        else
+            rowNum = 2;
+    } else {
+        rowNum = m_rows;
+    }
+    int deskCols = deskNum/rowNum;
+    if(deskNum == 0 || deskNum % rowNum != 0)
+        deskCols += 1;
 
     if (layout)
     {
@@ -436,48 +502,31 @@ void KMiniPager::resizeEvent(QResizeEvent*)
     int nDX, nDY;
     if (horiz)
     {
-        if (small)
-	    nDX = 1, nDY = deskNum;
-        else
-	    nDX = 2, nDY = deskHalf;
-
+        nDX = rowNum;
+        nDY = deskCols;
         updateDesktopLayout(Qt::Horizontal, -1, nDX);
     }
     else
     {
-        if (small)
-	    nDX = deskNum, nDY = 1;
-        else
-	    nDX = deskHalf, nDY = 2;
+        nDX = deskCols;
+        nDY = rowNum;
         updateDesktopLayout(Qt::Horizontal, nDY, -1);
     }
 
-    layout = new QGridLayout(this, nDX, nDY);
+    // 1 pixel margin and spacing.
+    layout = new QGridLayout(this, nDX, nDY, 1, 1);
 
-    int c = 0;
     QValueList<KMiniPagerButton*>::Iterator it = btnList.begin();
-
-    if (small) {
-	while( it != btnList.end() ) {
-	    if (horiz)
-		layout->addWidget( *it, 0, c);
-	    else
-		layout->addWidget( *it, c, 0);
-	    ++it;
-	    ++c;
-	}
-    } else {
-	int r = 0;
-	while(it != btnList.end())
-	{
-	   c = 0;
-	   while( (it != btnList.end()) && (c < nDY) ) {
-             layout->addWidget( *it, r, c);
-             ++it;
-	     ++c;
-	   }
-	   ++r;
-  	}
+    int c = 0,
+        r = 0;
+    while( it != btnList.end() ) {
+        c = 0;
+        while( (it != btnList.end()) && (c < nDY) ) {
+            layout->addWidget( *it, r, c );
+            ++it;
+            ++c;
+        }
+        ++r;
     }
 
     layout->activate();
@@ -486,17 +535,7 @@ void KMiniPager::resizeEvent(QResizeEvent*)
 
 void KMiniPager::wheelEvent( QWheelEvent* e )
 {
-    if( e->delta() < 0 ) {
-        // this could be "( curDesk - 1 ) + 1" for clarity, but that seems superfluous
-        const int next = curDesk % KWin::numberOfDesktops() + 1;
-        KWin::setCurrentDesktop( next );
-    }
-    else {
-        // again, a little ugly -- the addition of the number of desktops makes sure that we are
-        // not taking the modulo of a negative number and the "curDesk - 2" could be "curDesk - 1 - 1"
-        const int previous = ( KWin::numberOfDesktops() + curDesk - 2 ) % KWin::numberOfDesktops() + 1;
-        KWin::setCurrentDesktop( previous );
-    }
+    DCOPRef("kdesktop", "KDesktopIface").send("switchDesktops", e->delta());
 }
 
 void KMiniPager::allocateButtons()
@@ -632,6 +671,15 @@ void KMiniPager::aboutToShowContextMenu()
 
     QPopupMenu* showMenu = new QPopupMenu( contextMenu );
     showMenu->setCheckable( true );
+    QPopupMenu* rowMenu = new QPopupMenu( showMenu );
+    rowMenu->setCheckable( true );
+    rowMenu->insertItem( i18n("&Automatic"), 0+2000 );
+    rowMenu->insertItem( i18n("one row or column", "&1"), 1+2000 );
+    rowMenu->insertItem( i18n("two rows or columns", "&2"), 2+2000 );
+    rowMenu->insertItem( i18n("three rows or columns", "&3"), 3+2000 );
+    connect(rowMenu, SIGNAL(activated(int)), SLOT(contextMenuActivated(int)));
+    showMenu->insertItem( (orientation()==Horizontal)?i18n("&Rows"):i18n("&Columns"), rowMenu );
+    showMenu->insertSeparator();
     showMenu->insertItem( i18n("N&umber"), LabelNumber );
     showMenu->insertItem( i18n("N&ame"), LabelName );
     showMenu->insertItem( i18n("N&one"), LabelNone );
@@ -642,10 +690,11 @@ void KMiniPager::aboutToShowContextMenu()
     connect(showMenu, SIGNAL(activated(int)), SLOT(contextMenuActivated(int)));
     contextMenu->insertItem( i18n("&Show"),showMenu );
 
-    contextMenu->insertItem( SmallIcon("desktop"), i18n("&Configure Virtual Desktops..."), 98);
+    contextMenu->insertItem( SmallIcon("configure"), i18n("&Configure Virtual Desktops..."), 98);
     contextMenu->insertSeparator();
     contextMenu->insertItem( i18n("&Rename Virtual Desktop"), 99 );
 
+    rowMenu->setItemChecked( m_rows+2000, true );
     contextMenu->setItemChecked( m_mode, showMenu );
     contextMenu->setItemChecked( 97, bDesktopPreview );
     contextMenu->setItemEnabled( 99, m_mode == LabelName );
@@ -682,6 +731,13 @@ void KMiniPager::contextMenuActivated(int result)
 
     KConfig* conf = config();
     conf->setGroup("minipager");
+
+    if (result >= 2000) {
+      m_rows = result-2000;
+      conf->writeEntry( "NumberOfRows", m_rows );
+      resizeEvent(0);
+    }
+
     switch (result)
       {
       case 97:
