@@ -34,11 +34,13 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kconfig.h>
 #include <klocale.h>
 #include <kglobalsettings.h>
+#include <kwin.h>
 #include <kwinmodule.h>
 #include <kapplication.h>
 #include <kdebug.h>
 #include <karrowbutton.h>
 #include <netwm.h>
+#include <fixx11h.h>
 
 #include "kicker.h"
 #include "panelop_mnu.h"
@@ -50,15 +52,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "container_panel.h"
 #include "container_panel.moc"
 
-#include <X11/Xlib.h>
-const int XFocusOut = FocusOut;
-const int XFocusIn = FocusIn;
-#undef FocusOut
-#undef FocusIn
-#undef KeyPress
-#undef KeyRelease
-
-extern Time qt_x_time;
+const int XineramaAllScreens = -2;
 
 KWinModule* PanelContainer::kWinModule = 0;
 
@@ -68,8 +62,13 @@ PanelSettings::PanelSettings()
     _position          = ::Bottom;
     _alignment         = QApplication::reverseLayout() ? ::RightBottom : ::LeftTop;
     _HBwidth           = 14;
-    _showLeftHB        = QApplication::reverseLayout();
-    _showRightHB       = !QApplication::reverseLayout();
+    
+//    _showLeftHB        = QApplication::reverseLayout();
+//    _showRightHB       = !QApplication::reverseLayout();
+
+     _showLeftHB        = false;
+     _showRightHB       = true;
+    
     _hideMode          = Manual;
     _autoHideSwitch    = false;
     _autoHideDelay     = 3;
@@ -121,7 +120,7 @@ void PanelSettings::readConfig( KConfig *c )
     if (sizeSetting < Tiny || sizeSetting > Custom) _size = Normal;
     else _size = static_cast<Size>(sizeSetting);
 
-    if (_customSize < 1) _customSize = 24;
+    if (_customSize < 1) _customSize = Kicker::MIN_DIMENSION;
 
     if (_HBwidth < 3) _HBwidth = 3;
     else if (_HBwidth > 24) _HBwidth = 24;
@@ -183,22 +182,37 @@ PanelContainer::PanelContainer(QWidget *parent, const char *name)
     connect( _popupWidgetFilter, SIGNAL(popupWidgetHiding()), SLOT(maybeStartAutoHideTimer()) );
 
     // layout
-    _layout = new QBoxLayout( this, orientation() == Horizontal ? QBoxLayout::LeftToRight
-			      : QBoxLayout::TopToBottom, 0, 0);
+   
+    /* This is actually a work arround some miss implementations in QT
+        in the constructor, we passed the direction, however QT is "smart"
+	enough to reverse it in RTL desktops. We need this widget to be "LTR"
+	always. (remeber that we are in reverseLayout).
+	
+	QT4 will have a special API to tell a layout "you shall be always LTR". Now,
+	we hack it.
+    */    
+    QBoxLayout::Direction dir;
+    
+    if (QApplication::reverseLayout())
+	dir = orientation() == Horizontal ? QBoxLayout::RightToLeft  : QBoxLayout::TopToBottom;
+    else
+	dir = orientation() == Horizontal ? QBoxLayout::LeftToRight  : QBoxLayout::TopToBottom;
+    
+    _layout = new QBoxLayout( this, dir, 0, 0);
     _layout->setResizeMode( QLayout::FreeResize );
-
+    
     // left/top hide button
     _ltHB = new KArrowButton(this);
     _ltHB->installEventFilter( this );
     connect( _ltHB, SIGNAL( clicked() ), this, SLOT( hideLeft() ) );
-
+    
     _layout->addWidget( _ltHB );
 
     // right/bottom hide button
     _rbHB = new KArrowButton(this);
     _rbHB->installEventFilter( this );
     connect( _rbHB, SIGNAL( clicked() ), this, SLOT( hideRight() ) );
-
+    
     _layout->addWidget( _rbHB );
 
     // instantiate the autohide timer
@@ -226,10 +240,10 @@ int PanelContainer::xineramaScreen() const
     // sanitize at runtime only, since many Xinerama users
     // turn it on and off and don't want kicker to lose their configs
 
-		/* -2 means all screens, -1 primary screens, the rest are valid screen numbers */
-		if(-2 >= _settings._xineramaScreen < QApplication::desktop()->numScreens())
-      return _settings._xineramaScreen;
-		else
+    /* -2 means all screens, -1 primary screens, the rest are valid screen numbers */
+    if (XineramaAllScreens <= _settings._xineramaScreen && _settings._xineramaScreen < QApplication::desktop()->numScreens())
+        return _settings._xineramaScreen;
+    else
       /* force invalid screen locations onto the primary screen */
       return QApplication::desktop()->primaryScreen();
 }
@@ -392,10 +406,10 @@ void PanelContainer::resetLayout()
     QRect g = initialGeometry( position(), alignment(), xineramaScreen(), autoHidden(), userHidden() );
 
     setGeometry( g );
-
+    
     // layout
     if( orientation() == Horizontal )
-	_layout->setDirection( QBoxLayout::LeftToRight );
+	_layout->setDirection( QApplication::reverseLayout()?QBoxLayout::RightToLeft:QBoxLayout::LeftToRight );
     else
 	_layout->setDirection( QBoxLayout::TopToBottom );
 
@@ -504,7 +518,6 @@ void PanelContainer::blockUserInput( bool block )
 // are printed! WTF?! This happens if you drag an applet handle outside
 // the panel and wait for the timeout. I am completely mystified
 // by this.
-
 void PanelContainer::unhideTriggered( UnhideTrigger::Trigger tr, int XineramaScreen )
 {
     if( _settings._hideMode == PanelSettings::Manual )
@@ -524,8 +537,7 @@ void PanelContainer::unhideTriggered( UnhideTrigger::Trigger tr, int XineramaScr
     // only unhide when the cursor position is within the widget geometry.
     // We can't just do geometry().contains(QCursor::pos()) because
     // now we hide the panel completely off screen.
-
-    if(XineramaScreen != xineramaScreen())
+    if (xineramaScreen() != XineramaAllScreens && XineramaScreen != xineramaScreen())
         return;
 
     int x = QCursor::pos().x();
@@ -655,30 +667,6 @@ bool PanelContainer::eventFilter( QObject*, QEvent * e)
     }
 
     switch ( e->type() ) {
-#if QT_VERSION < 0x030200
-    // The next case is necessary for tooltips to be displayed.
-    // Ratio: Qt only shows tooltips for active windows, meaning
-    // windows that have the keyboard focus. This is usually desireable,
-    // but breaks for always-active desktop utilities like kicker. In order
-    // to see tooltips _always_ we fake a XFocusIn event for Qt whenever we
-    // get the mouse pointer and QApplication does not have an active window.
-    case QEvent::Enter:
-	if  ( !qApp->activeWindow() ) {
-	    XEvent ev;
-	    memset(&ev, 0, sizeof(ev));
-	    ev.xfocus.display = qt_xdisplay();
-	    ev.xfocus.type = XFocusIn;
-	    ev.xfocus.window = topLevelWidget()->winId();
-	    ev.xfocus.mode = NotifyNormal;
-	    ev.xfocus.detail = NotifyAncestor;
-	    Time time = qt_x_time;
-	    qt_x_time = 1;
-	    qApp->x11ProcessEvent( &ev );
-	    qt_x_time = time;
-	    setFocus(); // to avoid having a blinking cursor or a focus rectangle somewhere else
-	}
-	break;
-#endif
     case QEvent::MouseButtonPress:
 	{
 	    QMouseEvent* me = (QMouseEvent*) e;
@@ -1202,17 +1190,25 @@ QRect PanelContainer::workArea(int XineramaScreen)
     // panels below this one in the list. Else exclude the
     // struts of all panels.
     if( _settings._hideMode == PanelSettings::Manual ) {
-	it += PanelManager::the()->stackingPosition( this );
+        if (XineramaScreen != XineramaAllScreens) {
+            for( int i = 0 ; i < PanelManager::the()->stackingPosition( this ); ++i, ++it ) {
+                KWin::WindowInfo wi = KWin::windowInfo( ( *it )->winId() );
+                if ( !wi.geometry().intersects( QApplication::desktop()->screenGeometry(XineramaScreen) ) )
+                    list.append( (*it)->winId());
+            }
+        } else {
+            it += PanelManager::the()->stackingPosition( this );
+        }
     }
 
     for( ; it.current(); ++it ) {
 	list.append((*it)->winId());
     }
 
-    if(XineramaScreen == -2) /* special value for all screens */
+    if(XineramaScreen == XineramaAllScreens) /* special value for all screens */
 	return kWinModule->workArea(list);
     else
-	return kWinModule->workArea(list).intersect(QApplication::desktop()->screenGeometry(XineramaScreen));
+	return kWinModule->workArea(list, XineramaScreen).intersect(QApplication::desktop()->screenGeometry(XineramaScreen));
 }
 
 PopupWidgetFilter::PopupWidgetFilter( QObject *parent )
