@@ -1,4 +1,3 @@
-// $Id: imaplist.cc,v 1.6.2.1 2004/02/09 20:10:50 burghard Exp $
 /**********************************************************************
  *
  *   imapinfo.cc  - IMAP4rev1 EXAMINE / SELECT handler
@@ -47,14 +46,16 @@
 #include <kdebug.h>
 
 imapList::imapList ():noInferiors_ (false),
-noSelect_ (false), marked_ (false), unmarked_ (false)
+noSelect_ (false), marked_ (false), unmarked_ (false), 
+hasChildren_ (false), hasNoChildren_ (false)
 {
 }
 
 imapList::imapList (const imapList & lr):hierarchyDelimiter_ (lr.hierarchyDelimiter_),
 name_ (lr.name_),
 noInferiors_ (lr.noInferiors_),
-noSelect_ (lr.noSelect_), marked_ (lr.marked_), unmarked_ (lr.unmarked_)
+noSelect_ (lr.noSelect_), marked_ (lr.marked_), unmarked_ (lr.unmarked_),
+hasChildren_ (lr.hasChildren_), hasNoChildren_ (lr.hasNoChildren_)
 {
 }
 
@@ -70,13 +71,16 @@ imapList & imapList::operator = (const imapList & lr)
   noSelect_ = lr.noSelect_;
   marked_ = lr.marked_;
   unmarked_ = lr.unmarked_;
+  hasChildren_ = lr.hasChildren_;
+  hasNoChildren_ = lr.hasNoChildren_;
 
   return *this;
 }
 
 imapList::imapList (const QString & inStr):noInferiors_ (false),
 noSelect_ (false),
-marked_ (false), unmarked_ (false)
+marked_ (false), unmarked_ (false), hasChildren_ (false),
+hasNoChildren_ (false)  
 {
   parseString s;
   s.data.duplicate(inStr.latin1(), inStr.length());
@@ -87,32 +91,31 @@ marked_ (false), unmarked_ (false)
   s.pos++;  // tie off (
 
   //process the attributes
-  QString
-    attribute;
+  QCString attribute;
 
   while (!s.isEmpty () && s[0] != ')')
   {
-    attribute = imapParser::b2c(imapParser::parseOneWord (s));
-    if (-1 != attribute.find ("\\Noinferiors", 0, false))
+    attribute = imapParser::parseOneWordC(s).lower();
+    if (-1 != attribute.find ("\\noinferiors"))
       noInferiors_ = true;
-    else if (-1 != attribute.find ("\\Noselect", 0, false))
+    else if (-1 != attribute.find ("\\noselect"))
       noSelect_ = true;
-    else if (-1 != attribute.find ("\\Marked", 0, false))
+    else if (-1 != attribute.find ("\\marked"))
       marked_ = true;
-    else if (-1 != attribute.find ("\\Unmarked", 0, false))
+    else if (-1 != attribute.find ("\\unmarked"))
       unmarked_ = true;
-    else if (-1 != attribute.find ("\\HasChildren", 0, false)) {
-      // nothing special
-    } else if (-1 != attribute.find ("\\HasNoChildren", 0, false)) {
-      // nothing special
-    } else
+    else if (-1 != attribute.find ("\\haschildren"))
+      hasChildren_ = true;
+    else if (-1 != attribute.find ("\\hasnochildren"))
+      hasNoChildren_ = true;
+    else
       kdDebug(7116) << "imapList::imapList: bogus attribute " << attribute << endl;
   }
 
   s.pos++;  // tie off )
   imapParser::skipWS (s);
 
-  hierarchyDelimiter_ = imapParser::b2c(imapParser::parseOneWord(s));
+  hierarchyDelimiter_ = imapParser::parseOneWordC(s);
   if (hierarchyDelimiter_ == "NIL")
     hierarchyDelimiter_ = QString::null;
   name_ = rfcDecoder::fromIMAP (imapParser::parseOneWord (s));  // decode modified UTF7
