@@ -15,7 +15,6 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <time.h>
 #include "mimeheader.h"
 #include "mimehdrline.h"
 #include "mailheader.h"
@@ -35,6 +34,7 @@
 mimeHeader::mimeHeader ():
 typeList (17, false), dispositionList (17, false)
 {
+  // Case insensitive hashes are killing us.  Also are they too small?
   originalHdrLines.setAutoDelete (true);
   additionalHdrLines.setAutoDelete (false); // is also in original lines
   nestedParts.setAutoDelete (true);
@@ -50,9 +50,9 @@ mimeHeader::~mimeHeader ()
 }
 
 /*
-QList<mimeHeader> mimeHeader::getAllParts()
+QPtrList<mimeHeader> mimeHeader::getAllParts()
 {
-	QList<mimeHeader> retVal;
+	QPtrList<mimeHeader> retVal;
 
 	// caller is responsible for clearing
 	retVal.setAutoDelete( false );
@@ -148,11 +148,7 @@ mimeHeader::addHdrLine (mimeHdrLine * aHdrLine)
         {
           if (skip > 0)
           {
-            QCString aParm;
-
-            aParm = QCString (aCStr, skip);
-            aParm = aParm.simplifyWhiteSpace ().stripWhiteSpace ();
-            addParameter (aParm, aList);
+            addParameter (QCString (aCStr, skip).simplifyWhiteSpace(), aList);
 //            cout << "-- '" << aParm.data() << "'" << endl;
             mimeValue = QCString (addLine->getValue ().data (), skip);
             aCStr += skip;
@@ -166,7 +162,7 @@ mimeHeader::addHdrLine (mimeHdrLine * aHdrLine)
 }
 
 void
-mimeHeader::addParameter (QCString aParameter, QDict < QString > *aList)
+mimeHeader::addParameter (const QCString& aParameter, QDict < QString > *aList)
 {
   QString *aValue;
   QCString aLabel;
@@ -183,29 +179,28 @@ mimeHeader::addParameter (QCString aParameter, QDict < QString > *aList)
 }
 
 QString
-mimeHeader::getDispositionParm (QCString aStr)
+mimeHeader::getDispositionParm (const QCString& aStr)
 {
   return getParameter (aStr, &dispositionList);
 }
 
 QString
-mimeHeader::getTypeParm (QCString aStr)
+mimeHeader::getTypeParm (const QCString& aStr)
 {
   return getParameter (aStr, &typeList);
 }
 
 void
-mimeHeader::setDispositionParm (QCString aLabel, QString aValue)
+mimeHeader::setDispositionParm (const QCString& aLabel, const QString& aValue)
 {
   setParameter (aLabel, aValue, &dispositionList);
   return;
 }
 
 void
-mimeHeader::setTypeParm (QCString aLabel, QString aValue)
+mimeHeader::setTypeParm (const QCString& aLabel, const QString& aValue)
 {
   setParameter (aLabel, aValue, &typeList);
-  return;
 }
 
 QDictIterator < QString > mimeHeader::getDispositionIterator ()
@@ -218,14 +213,14 @@ QDictIterator < QString > mimeHeader::getTypeIterator ()
   return QDictIterator < QString > (typeList);
 }
 
-QListIterator < mimeHdrLine > mimeHeader::getOriginalIterator ()
+QPtrListIterator < mimeHdrLine > mimeHeader::getOriginalIterator ()
 {
-  return QListIterator < mimeHdrLine > (originalHdrLines);
+  return QPtrListIterator < mimeHdrLine > (originalHdrLines);
 }
 
-QListIterator < mimeHdrLine > mimeHeader::getAdditionalIterator ()
+QPtrListIterator < mimeHdrLine > mimeHeader::getAdditionalIterator ()
 {
-  return QListIterator < mimeHdrLine > (additionalHdrLines);
+  return QPtrListIterator < mimeHdrLine > (additionalHdrLines);
 }
 
 void
@@ -254,7 +249,7 @@ mimeHeader::outputHeader (mimeIO & useIO)
     useIO.outputMimeLine (QCString ("Content-Transfer-Encoding: ") +
                           getEncoding ());
 
-  QListIterator < mimeHdrLine > ait = getAdditionalIterator ();
+  QPtrListIterator < mimeHdrLine > ait = getAdditionalIterator ();
   while (ait.current ())
   {
     useIO.outputMimeLine (ait.current ()->getLabel () + ": " +
@@ -265,7 +260,7 @@ mimeHeader::outputHeader (mimeIO & useIO)
 }
 
 QString
-mimeHeader::getParameter (QCString aStr, QDict < QString > *aDict)
+mimeHeader::getParameter (const QCString& aStr, QDict < QString > *aDict)
 {
   QString retVal, *found;
   if (aDict)
@@ -301,7 +296,7 @@ mimeHeader::getParameter (QCString aStr, QDict < QString > *aDict)
           part++;
         }
         while (found);
-        if (encoded.find ("'") >= 0)
+        if (encoded.find ('\'') >= 0)
         {
           retVal = rfcDecoder::decodeRFC2231String (encoded.local8Bit ());
         }
@@ -327,10 +322,12 @@ mimeHeader::getParameter (QCString aStr, QDict < QString > *aDict)
 }
 
 void
-mimeHeader::setParameter (QCString aLabel, QString aValue,
+mimeHeader::setParameter (const QCString& aLabel, const QString& aValue,
                           QDict < QString > *aDict)
 {
   bool encoded = true;
+  uint vlen, llen;
+  QString val = aValue;
 
   if (aDict)
   {
@@ -338,23 +335,25 @@ mimeHeader::setParameter (QCString aLabel, QString aValue,
     //see if it needs to get encoded
     if (encoded && aLabel.find ('*') == -1)
     {
-      aValue = rfcDecoder::encodeRFC2231String (aValue);
+      val = rfcDecoder::encodeRFC2231String (aValue);
     }
     //see if it needs to be truncated
-    if (aValue.length () + aLabel.length () + 4 > 80)
+    vlen = val.length();
+    llen = aLabel.length();
+    if (vlen + llen + 4 > 80)
     {
-      int limit = 80 - 8 - aLabel.length ();
+      int limit = 80 - 8 - llen;
       int i = 0;
       QString shortValue;
       QCString shortLabel;
 
-      while (!aValue.isEmpty ())
+      while (!val.isEmpty ())
       {
         //don't truncate the encoded chars
         int offset = 0;
-        if (limit > (int)aValue.length())
-          limit = aValue.length ();
-        offset = aValue.findRev ('%', limit);
+        if (limit > int(vlen))
+          limit = vlen;
+        offset = val.findRev ('%', limit);
         if (offset == limit - 1 || offset == limit - 2)
         {
 //          cout << "offset " << offset << "-" << limit << "=" << limit-offset << endl;
@@ -362,10 +361,11 @@ mimeHeader::setParameter (QCString aLabel, QString aValue,
         }
         else
           offset = 0;
-        shortValue = aValue.left (limit - offset);
+        shortValue = val.left (limit - offset);
         shortLabel.setNum (i);
         shortLabel = aLabel + "*" + shortLabel;
-        aValue = aValue.right (aValue.length () - limit + offset);
+        val = val.right (vlen - limit + offset);
+        vlen = vlen - limit + offset;
         if (encoded)
         {
           if (i == 0)
@@ -381,7 +381,7 @@ mimeHeader::setParameter (QCString aLabel, QString aValue,
     }
     else
     {
-      aDict->insert (aLabel, new QString (aValue));
+      aDict->insert (aLabel, new QString (val));
     }
   }
 }
@@ -415,7 +415,7 @@ mimeHeader::outputParameter (QDict < QString > *aDict)
 void
 mimeHeader::outputPart (mimeIO & useIO)
 {
-  QListIterator < mimeHeader > nestedParts = getNestedIterator ();
+  QPtrListIterator < mimeHeader > nestedParts = getNestedIterator ();
   QCString boundary;
   if (!getTypeParm ("boundary").isEmpty ())
     boundary = getTypeParm ("boundary").latin1 ();
@@ -439,7 +439,7 @@ mimeHeader::outputPart (mimeIO & useIO)
 }
 
 int
-mimeHeader::parsePart (mimeIO & useIO, QString boundary)
+mimeHeader::parsePart (mimeIO & useIO, const QString& boundary)
 {
   int retVal = 0;
   bool mbox = false;
@@ -481,7 +481,7 @@ mimeHeader::parsePart (mimeIO & useIO, QString boundary)
 
 int
 mimeHeader::parseBody (mimeIO & useIO, QCString & messageBody,
-                       QString boundary, bool mbox)
+                       const QString& boundary, bool mbox)
 {
   QCString inputStr;
   QCString buffer;
@@ -568,23 +568,22 @@ mimeHeader *
 mimeHeader::bodyPart (const QString & _str)
 {
   // see if it is nested a little deeper
-  if (_str.find (".") != -1)
+  int pt = _str.find('.');
+  if (pt != -1)
   {
     QString tempStr = _str;
     mimeHeader *tempPart;
 
-    tempStr = _str.right (_str.length () - _str.find (".") - 1);
+    tempStr = _str.right (_str.length () - pt - 1);
     if (nestedMessage)
     {
       kdDebug(7116) << "mimeHeader::bodyPart - recursing message" << endl;
-      tempPart =
-        nestedMessage->nestedParts.at (_str.left (_str.find (".")).
-                                       toULong () - 1);
+      tempPart = nestedMessage->nestedParts.at (_str.left(pt).toULong() - 1);
     }
     else
     {
       kdDebug(7116) << "mimeHeader::bodyPart - recursing mixed" << endl;
-      tempPart = nestedParts.at (_str.left (_str.find (".")).toULong () - 1);
+      tempPart = nestedParts.at (_str.left(pt).toULong() - 1);
     }
     if (tempPart)
       tempPart = tempPart->bodyPart (tempStr);
@@ -602,6 +601,30 @@ mimeHeader::bodyPart (const QString & _str)
   return nestedParts.at (_str.toULong () - 1);
 }
 
+void mimeHeader::serialize(QDataStream& stream)
+{
+  int nestedcount = nestedParts.count();
+  if (nestedParts.isEmpty() && nestedMessage) 
+    nestedcount = 1;
+  stream << nestedcount << contentType << QString (getTypeParm ("name")) << _contentDescription 
+    << _contentDisposition << contentEncoding << contentLength << partSpecifier;
+  // serialize nested message
+  if (nestedMessage)
+    nestedMessage->serialize(stream);
+
+  // serialize nested parts
+  if (!nestedParts.isEmpty())
+  {
+    QPtrListIterator < mimeHeader > it(nestedParts);
+    mimeHeader* part;
+    while ( (part = it.current()) != 0 ) 
+    {
+      ++it;
+      part->serialize(stream);
+    }
+  }
+}
+
 #ifdef KMAIL_COMPATIBLE
 // compatibility subroutines
 QString
@@ -658,6 +681,7 @@ mimeHeader::iconName ()
 {
   QString fileName;
 
+  // FIXME: bug?  Why throw away this data?
   fileName =
     KMimeType::mimeType (contentType.lower ())->icon (QString::null, false);
   fileName =
