@@ -1,7 +1,5 @@
 /*  This file is part of the KDE project
 
-    $Id: kio_nfs.cpp,v 1.29 2004/01/05 21:10:38 neundorf Exp $
-
     Copyright (C) 2000 Alexander Neundorf <neundorf@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -431,10 +429,15 @@ void NFSProtocol::openConnection()
          atLeastOnceSucceeded=TRUE;
          NFSFileHandle fh;
          fh=fhStatus.fhstatus_u.fhs_fhandle;
-         m_handleCache.insert(QString("/")+KIO::encodeFileName(exportlist->ex_dir),fh);
-         m_exportedDirs.append(KIO::encodeFileName(exportlist->ex_dir));
-         //cerr<<"appending file -"<<KIO::encodeFileName(exportlist->ex_dir).latin1()<<"- with FH: -"<<fhStatus.fhstatus_u.fhs_fhandle<<"-"<<endl;
-      };
+         QString fname;
+         if ( exportlist->ex_dir[0] == '/' )
+            fname = KIO::encodeFileName(exportlist->ex_dir + 1);
+         else
+            fname = KIO::encodeFileName(exportlist->ex_dir);
+         m_handleCache.insert(QString("/")+fname,fh);
+         m_exportedDirs.append(fname);
+         // kdDebug() <<"appending file -"<<fname<<"- with FH: -"<<fhStatus.fhstatus_u.fhs_fhandle<<"-"<<endl;
+      }
    }
    if (!atLeastOnceSucceeded)
    {
