@@ -460,12 +460,12 @@ int KSshProcess::error(QString& msg) {
 void KSshProcess::kill(int signal) {
     int pid = ssh.pid();
     
-    kdDebug(KSSHPROC) << "KSshProcess::kill(signal:" << signal << "): "
-        "ssh pid is " << pid << endl;
-    kdDebug(KSSHPROC) << "KSshPRocess::kill(): "
-        "we are " << (mConnected ? "" : "not ") << "connected" << endl;
-    kdDebug(KSSHPROC) << "KSshProcess::kill(): "
-        "we are " << (mRunning ? "" : "not ") <<"running a ssh process" << endl;
+    kdDebug(KSSHPROC) << "KSshProcess::kill(signal:" << signal 
+                      << "): ssh pid is " << pid << endl;
+    kdDebug(KSSHPROC) << "KSshPRocess::kill(): we are " 
+                      << (mConnected ? "" : "not ") << "connected" << endl;
+    kdDebug(KSSHPROC) << "KSshProcess::kill(): we are " 
+                      << (mRunning ? "" : "not ") << "running a ssh process" << endl;
 
     if( mRunning && pid > 1 ) {
             // Kill the child process...
@@ -590,7 +590,10 @@ QString KSshProcess::getLine() {
             tv.tv_sec = 60; tv.tv_usec = 0; // 60 second timeout
     
             // Wait for a message from ssh on stderr or the pty.
-            int ret = ::select(maxfd+1, &rfds, NULL, &efds, &tv);
+            int ret = -1;
+            do 
+              ret = ::select(maxfd+1, &rfds, NULL, &efds, &tv);
+            while( ret == -1 && errno == EINTR );
     
             // Handle any errors from select
             if( ret == 0 ) {
