@@ -2,11 +2,9 @@
 
    urlgrabber.h (part of Klipper - Cut & paste history for KDE)
 
-   $Id: urlgrabber.h,v 1.17 2003/09/19 18:00:04 mueller Exp $
-
    (C) 2000 by Carsten Pfeiffer <pfeiffer@kde.org>
 
-   Licensed under the Artistic License
+   Licensed under the GNU GPL Version 2
 
  ------------------------------------------------------------- */
 
@@ -97,7 +95,7 @@ signals:
 
 struct ClipCommand
 {
-    ClipCommand( const QString &, const QString &, bool = true );
+    ClipCommand( const QString &, const QString &, bool = true, const QString & = "" );
     QString command;
     QString description;
     bool isEnabled;
@@ -132,7 +130,7 @@ public:
    */
   void clearCommands() { myCommands.clear(); }
 
-  void  addCommand( const QString& command, const QString& description, bool );
+  void  addCommand( const QString& command, const QString& description, bool, const QString& icon = "" );
   const QPtrList<ClipCommand>& commands() 	const { return myCommands; }
 
   /**
