@@ -27,8 +27,8 @@
 
 #include "kmenuedit.h"
 
-static const char *description = I18N_NOOP("KDE Menu editor");
-static const char *version = "0.5";
+static const char description[] = I18N_NOOP("KDE menu editor");
+static const char version[] = "0.6.1";
 
 extern "C" int kdemain( int argc, char **argv )
 {
@@ -47,7 +47,7 @@ extern "C" int kdemain( int argc, char **argv )
 
     KUniqueApplication app;
 
-    KMenuEdit *menuEdit = new KMenuEdit;
+    KMenuEdit *menuEdit = new KMenuEdit(false);
     menuEdit->show();
 
     app.setMainWidget(menuEdit);
