@@ -17,8 +17,6 @@
    Boston, MA 02111-1307, USA.
 */
 
-// $Id: kfmclient.cc,v 1.110 2003/10/12 19:47:52 binner Exp $
-
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -62,9 +60,9 @@ bool clientApp::m_ok;
 
 static const KCmdLineOptions options[] =
 {
-   { "commands", I18N_NOOP("Show available commands."), 0},
-   { "+command", I18N_NOOP("Command (see --commands)."), 0},
-   { "+[URL(s)]", I18N_NOOP("Arguments for command."), 0},
+   { "commands", I18N_NOOP("Show available commands"), 0},
+   { "+command", I18N_NOOP("Command (see --commands)"), 0},
+   { "+[URL(s)]", I18N_NOOP("Arguments for command"), 0},
    KCmdLineLastOption
 };
 
@@ -89,6 +87,10 @@ extern "C" int kdemain( int argc, char **argv )
                 "            #   component that Konqueror should use. For instance, set it to\n"
                 "            #   text/html for a web page, to make it appear faster\n\n").local8Bit());
 
+    puts(i18n("  kfmclient newTab 'url' ['mimetype']\n"
+                "            # Same as above but opens a new tab with 'url' in an existing Konqueror\n"
+                "            #   window on the current active desktop if possible.\n\n").local8Bit());
+
     puts(i18n("  kfmclient openProfile 'profile' ['url']\n"
                 "            # Opens a window using the given profile.\n"
                 "            #   'profile' is a file under ~/.kde/share/apps/konqueror/profiles.\n"
@@ -171,28 +173,6 @@ extern "C" int kdemain( int argc, char **argv )
 */
 static bool startNewKonqueror( QString url, QString mimetype, const QString& profile )
 {
-    if( url.isEmpty() && !profile.isEmpty())
-    {
-	QString profilepath = locate( "data", QString::fromLatin1("konqueror/profiles/") + profile );
-	if( profilepath.isEmpty())
-	    return true;
-	KConfig cfg( profilepath, true );
-	cfg.setDollarExpansion( true );
-	QMap< QString, QString > entries = cfg.entryMap( QString::fromLatin1( "Profile" ));
-	QRegExp urlregexp( QString::fromLatin1( "^View[0-9]*_URL$" ));
-	QStringList urls;
-	for( QMap< QString, QString >::ConstIterator it = entries.begin();
-	     it != entries.end();
-	     ++it )
-	{
-	    if( urlregexp.search( it.key()) >= 0 && !(*it).isEmpty())
-		urls << *it;
-	}
-	if( urls.count() != 1 )
-	    return true;
-	url = urls.first();
-	mimetype = QString::fromLatin1( "" );
-    }
     KConfig cfg( QString::fromLatin1( "konquerorrc" ), true );
     cfg.setGroup( "Reusing" );
     QStringList allowed_parts;
@@ -207,9 +187,36 @@ static bool startNewKonqueror( QString url, QString mimetype, const QString& pro
         && cfg.readEntry( "SafeParts" ) != QString::fromLatin1( "SAFE" ))
         allowed_parts = cfg.readListEntry( "SafeParts" );
     if( allowed_parts.count() == 1 && allowed_parts.first() == QString::fromLatin1( "ALL" ))
-	return true; // all parts allowed
+	return false; // all parts allowed
+    if( url.isEmpty())
+    {
+        if( profile.isEmpty())
+            return true;
+	QString profilepath = locate( "data", QString::fromLatin1("konqueror/profiles/") + profile );
+	if( profilepath.isEmpty())
+	    return true;
+	KConfig cfg( profilepath, true );
+	cfg.setDollarExpansion( true );
+        cfg.setGroup( "Profile" );
+	QMap< QString, QString > entries = cfg.entryMap( QString::fromLatin1( "Profile" ));
+	QRegExp urlregexp( QString::fromLatin1( "^View[0-9]*_URL$" ));
+	QStringList urls;
+	for( QMap< QString, QString >::ConstIterator it = entries.begin();
+	     it != entries.end();
+	     ++it )
+	{
+            // don't read value from map, dollar expansion is needed
+            QString value = cfg.readEntry( it.key());
+	    if( urlregexp.search( it.key()) >= 0 && !value.isEmpty())
+		urls << value;
+	}
+	if( urls.count() != 1 )
+	    return true;
+	url = urls.first();
+	mimetype = QString::fromLatin1( "" );
+    }
     if( mimetype.isEmpty())
-	mimetype = KMimeType::findByURL( url )->name();
+	mimetype = KMimeType::findByURL( KURL( url ) )->name();
     KTrader::OfferList offers = KTrader::self()->query( mimetype, QString::fromLatin1( "KParts/ReadOnlyPart" ),
 	QString::null, QString::null );
     KService::Ptr serv;
@@ -265,25 +272,41 @@ static QCString konqyToReuse( const QString& url, const QString& mimetype, const
     return ret;
 }
 
-bool clientApp::createNewWindow(const KURL & url, const QString & mimetype)
+bool clientApp::createNewWindow(const KURL & url, bool newTab, const QString & mimetype)
 {
     kdDebug( 1202 ) << "clientApp::createNewWindow " << url.url() << " mimetype=" << mimetype << endl;
 
-	// check if user wants to use external browser
-	KConfig config( QString::fromLatin1("kfmclientrc"));
-	config.setGroup( QString::fromLatin1("Settings"));
-	QString strBrowser = config.readPathEntry("ExternalBrowser");
-	if (!strBrowser.isEmpty())
-	{
-		KProcess proc;
-		proc << strBrowser << url.url();
-		proc.start( KProcess::DontCare );
-		return true;
-	}
+    // check if user wants to use external browser
+    KConfig config( QString::fromLatin1("kfmclientrc"));
+    config.setGroup( QString::fromLatin1("Settings"));
+    QString strBrowser = config.readPathEntry("ExternalBrowser");
+    if (!strBrowser.isEmpty())
+    {
+        KProcess proc;
+        proc << strBrowser << url.url();
+        proc.start( KProcess::DontCare );
+        return true;
+    }
+
+    if (url.protocol().startsWith(QString::fromLatin1("http")))
+    {
+        config.setGroup("General");
+        if (!config.readEntry("BrowserApplication").isEmpty())
+        {
+            clientApp app;
+            KStartupInfo::appStarted();
+            
+            KRun * run = new KRun( url );
+            QObject::connect( run, SIGNAL( finished() ), &app, SLOT( delayedQuit() ));
+            QObject::connect( run, SIGNAL( error() ), &app, SLOT( delayedQuit() ));
+            app.exec();
+            return m_ok;
+        }
+    }
 
     KConfig cfg( QString::fromLatin1( "konquerorrc" ), true );
     cfg.setGroup( "FMSettings" );
-    if ( cfg.readBoolEntry( "KonquerorTabforExternalURL", false ) )
+    if ( newTab || cfg.readBoolEntry( "KonquerorTabforExternalURL", false ) )
     {
         QCString foundApp, foundObj;
         QByteArray data;
@@ -422,7 +445,7 @@ bool clientApp::doIt()
   // read ASN env. variable for non-KApp cases
   startup_id_str = KStartupInfo::currentStartupIdEnv().id();
 
-  if ( command == "openURL" )
+  if ( command == "openURL" || command == "newTab" )
   {
     KInstance inst(appName);
     if( !KApplication::dcopClient()->attach())
@@ -435,15 +458,15 @@ bool clientApp::doIt()
     {
       KURL url;
       url.setPath(QDir::homeDirPath());
-      return createNewWindow( url );
+      return createNewWindow( url, command == "newTab" );
     }
     if ( argc == 2 )
     {
-      return createNewWindow( args->url(1) );
+      return createNewWindow( args->url(1), command == "newTab" );
     }
     if ( argc == 3 )
     {
-      return createNewWindow( args->url(1), QString::fromLatin1(args->arg(2)) );
+      return createNewWindow( args->url(1), command == "newTab", QString::fromLatin1(args->arg(2)) );
     }
   }
   else if ( command == "openProfile" )
@@ -537,7 +560,9 @@ bool clientApp::doIt()
        KFileDialog::getSaveFileName( (argc<2) ? (QString::null) : (args->url(1).filename()) );
     if (dst == QString::null)
        return m_ok; // AK - really okay?
-    KIO::Job * job = KIO::copy( srcLst, dst );
+    KURL dsturl;
+    dsturl.setPath( dst );
+    KIO::Job * job = KIO::copy( srcLst, dsturl );
     connect( job, SIGNAL( result( KIO::Job * ) ), &app, SLOT( slotResult( KIO::Job * ) ) );
     app.exec();
     return m_ok;
