@@ -109,16 +109,14 @@ public:
    */
   void openURL( KonqView * view, const KURL & url,
                 const QString &serviceType = QString::null,
-                KonqOpenURLRequest req = KonqOpenURLRequest(), bool trustedSource = false );
+                KonqOpenURLRequest & req = KonqOpenURLRequest::null, bool trustedSource = false );
 
   /**
    * Called by openURL when it knows the service type (either directly,
    * or using KonqRun)
-   * @param req is not passed by reference because the calling KonqRun
-   * that holds it might get destroyed (by childView->stop())
    */
   bool openView( QString serviceType, const KURL &_url, KonqView *childView,
-                 KonqOpenURLRequest req = KonqOpenURLRequest() );
+                 KonqOpenURLRequest & req = KonqOpenURLRequest::null );
 
 
   void abortLoading();
@@ -145,10 +143,10 @@ public:
   void insertChildView( KonqView *childView );
   void removeChildView( KonqView *childView );
   KonqView *childView( KParts::ReadOnlyPart *view );
-  KonqView *childView( const QString &name, KParts::BrowserHostExtension **hostExtension, KParts::ReadOnlyPart **part );
+  KonqView *childView( KParts::ReadOnlyPart *callingPart, const QString &name, KParts::BrowserHostExtension **hostExtension, KParts::ReadOnlyPart **part );
 
   // dcop idl bug! it can't handle KonqMainWindow *&mainWindow
-  static KonqView *findChildView( const QString &name, KonqMainWindow **mainWindow, KParts::BrowserHostExtension **hostExtension, KParts::ReadOnlyPart **part );
+  static KonqView *findChildView( KParts::ReadOnlyPart *callingPart, const QString &name, KonqMainWindow **mainWindow, KParts::BrowserHostExtension **hostExtension, KParts::ReadOnlyPart **part );
 
   // Total number of views
   int viewCount() const { return m_mapViews.count(); }
@@ -287,8 +285,8 @@ public:
   KonqFrameBase* workingTab()const { return m_pWorkingTab; }
   void setWorkingTab( KonqFrameBase* tab ) { m_pWorkingTab = tab; }
 
-  bool isMimeTypeAssociatedWithSelf( const QString &mimeType );
-  bool isMimeTypeAssociatedWithSelf( const QString &mimeType, const KService::Ptr &offer );
+  static bool isMimeTypeAssociatedWithSelf( const QString &mimeType );
+  static bool isMimeTypeAssociatedWithSelf( const QString &mimeType, const KService::Ptr &offer );
 
   void resetWindow();
 
@@ -300,6 +298,8 @@ public:
   QString currentTitle() const;
   QString currentURL() const;
 
+  QStringList configModules() const;
+
 signals:
   void viewAdded( KonqView *view );
   void viewRemoved( KonqView *view );
@@ -358,13 +358,14 @@ public slots:
   void slotForward();
   void slotHome();
   void slotGoApplications();
+  void slotGoDevices();
+  void slotGoSettings();
   void slotGoDirTree();
   void slotGoTrash();
   void slotGoTemplates();
   void slotGoAutostart();
 
   void slotConfigure();
-  void slotConfigureKeys();
   void slotConfigureToolbars();
     void slotConfigureSpellChecking();
   void slotNewToolbarConfig();
@@ -390,7 +391,7 @@ public slots:
 protected slots:
   void slotViewCompleted( KonqView * view );
 
-  void slotURLEntered( const QString &text, ButtonState state );
+  void slotURLEntered( const QString &text, int );
 
   void slotFileNewAboutToShow();
   void slotLocationLabelActivated();
@@ -420,6 +421,8 @@ protected slots:
 
   void slotActivateNextTab();
   void slotActivatePrevTab();
+  void slotActivateTab();
+
   void slotDumpDebugInfo();
 
   void slotSaveViewProfile();
@@ -563,6 +566,8 @@ private:
 
   bool stayPreloaded();
   bool checkPreloadResourceUsage();
+  
+  QObject* lastFrame( KonqView *view );
 
   KNewMenu * m_pMenuNew;
 
@@ -591,6 +596,7 @@ private:
   KAction *m_paBreakOffTab;
   KAction *m_paRemoveView;
   KAction *m_paRemoveTab;
+  KAction *m_paRemoveOtherTabs;
   KAction *m_paActivateNextTab;
   KAction *m_paActivatePrevTab;
   KAction *m_paDumpDebugInfo;
@@ -601,6 +607,7 @@ private:
   KToggleAction *m_paLockView;
   KToggleAction *m_paLinkView;
   KAction *m_paReload;
+  KAction *m_paReloadAllTabs;
   KAction *m_paUndo;
   KAction *m_paCut;
   KAction *m_paCopy;
@@ -610,6 +617,7 @@ private:
 
   KAction *m_paTrash;
   KAction *m_paDelete;
+  QPtrList<KAction> m_deleteActions;
 
   KAction *m_paCopyFiles;
   KAction *m_paMoveFiles;
@@ -685,8 +693,8 @@ private:
   QPtrList<KAction> m_openWithActions;
   KActionMenu *m_viewModeMenu;
   QPtrList<KAction> m_toolBarViewModeActions; // basically holds two KonqViewActions, one of
-                                           // iconview and one for listview
-  QPtrList<KAction> m_viewModeActions;
+                                              // iconview and one for listview
+  QPtrList<KRadioAction> m_viewModeActions;
   QMap<QString,KService::Ptr> m_viewModeToolBarServices; // similar to m_toolBarViewModeActions
   // it holds a map library name (libkonqiconview/libkonqlistview) ==> service (service for
   // iconview, multicolumnview, treeview, etc .)
