@@ -32,6 +32,7 @@
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kurldrag.h>
+#include <kstringhandler.h>
 
 #include "konq_frame.h"
 #include "konq_view.h"
@@ -52,7 +53,7 @@
 
 KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentContainer,
                              KonqViewManager* viewManager, const char * name)
-  : KTabWidget(parent, name), m_CurrentMaxLength(30), m_rightWidget(0)
+  : KTabWidget(parent, name), m_rightWidget(0), m_leftWidget(0), m_alwaysTabBar(false)
 {
   //kdDebug(1202) << "KonqFrameTabs::KonqFrameTabs()" << endl;
 
@@ -94,12 +95,12 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
                             i18n( "&Reload" ),
                             m_pViewManager->mainWindow(),
                             SLOT( slotReloadPopup() ),
-                            KStdAccel::shortcut(KStdAccel::Reload) );
+                            m_pViewManager->mainWindow()->action("reload")->shortcut() );
   m_pPopupMenu->insertItem( SmallIcon( "reload_all_tabs" ),
                             i18n( "&Reload All Tabs" ),
                             m_pViewManager->mainWindow(),
                             SLOT( slotReloadAllTabs() ),
-                            QKeySequence(),
+                            m_pViewManager->mainWindow()->action("reload_all_tabs")->shortcut(),
                             RELOAD_ALL_ID );
   m_pPopupMenu->insertSeparator();
   m_pSubPopupMenuTab = new QPopupMenu( this );
@@ -111,13 +112,20 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
                             i18n( "Close &Other Tabs" ),
                             m_pViewManager->mainWindow(),
                             SLOT( slotRemoveOtherTabsPopup() ),
-                            QKeySequence(),
+                            m_pViewManager->mainWindow()->action("removeothertabs")->shortcut(),
                             CLOSE_OTHER_ID );
   connect( this, SIGNAL( contextMenu( QWidget *, const QPoint & ) ),
            SLOT(slotContextMenu( QWidget *, const QPoint & ) ) );
 
   KConfig *config = KGlobal::config();
   KConfigGroupSaver cs( config, QString::fromLatin1("FMSettings") );
+
+  m_maxLength = config->readNumEntry("MaximumTabLength", 30);
+  m_minLength = config->readNumEntry("MinimumTabLength", 3);
+  if (m_minLength>m_maxLength)
+    m_maxLength=m_minLength;
+  m_CurrentMaxLength = m_maxLength;
+
   m_permanentCloseButtons = config->readBoolEntry( "PermanentCloseButton", false );
   if (m_permanentCloseButtons) {
     setHoverCloseButton( true );
@@ -125,18 +133,21 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
   }
   else
     setHoverCloseButton( config->readBoolEntry( "HoverCloseButton", false ) );
+  setTabCloseActivatePrevious( config->readBoolEntry("TabCloseActivatePrevious", false) );
+  if (config->readEntry("TabPosition")=="Bottom")
+    setTabPosition(QTabWidget::Bottom);
   connect( this, SIGNAL( closeRequest( QWidget * )), SLOT(slotCloseRequest( QWidget * )));
   connect( this, SIGNAL( removeTabPopup() ),
            m_pViewManager->mainWindow(), SLOT( slotRemoveTabPopup() ) );
 
   if ( config->readBoolEntry( "AddTabButton", true ) ) {
-    QToolButton * leftWidget = new QToolButton( this );
-    connect( leftWidget, SIGNAL( clicked() ),
+    m_leftWidget = new QToolButton( this );
+    connect( m_leftWidget, SIGNAL( clicked() ),
              m_pViewManager->mainWindow(), SLOT( slotAddTab() ) );
-    leftWidget->setIconSet( SmallIcon( "tab_new" ) );
-    leftWidget->adjustSize();
-    QToolTip::add(leftWidget, i18n("Open a new tab"));
-    setCornerWidget( leftWidget, TopLeft );
+    m_leftWidget->setIconSet( SmallIcon( "tab_new" ) );
+    m_leftWidget->adjustSize();
+    QToolTip::add(m_leftWidget, i18n("Open a new tab"));
+    setCornerWidget( m_leftWidget, TopLeft );
   }
   if ( config->readBoolEntry( "CloseTabButton", true ) ) {
     m_rightWidget = new QToolButton( this );
@@ -155,6 +166,8 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
            SLOT( slotMouseMiddleClick() ) );
   connect( this, SIGNAL( mouseMiddleClick( QWidget * ) ),
            SLOT( slotMouseMiddleClick( QWidget * ) ) );
+  connect( this, SIGNAL( mouseDoubleClick() ),
+           m_pViewManager->mainWindow(), SLOT( slotAddTab() ) );
 
   connect( this, SIGNAL( testCanDecode(const QDragMoveEvent *, bool & )),
            SLOT( slotTestCanDecode(const QDragMoveEvent *, bool & ) ) );
@@ -190,7 +203,7 @@ void KonqFrameTabs::saveConfig( KConfig* config, const QString &prefix, bool sav
       newPrefix = QString::fromLatin1( it->frameType() ) + "T" + QString::number(i);
       strlst.append( newPrefix );
       newPrefix.append( '_' );
-      it->saveConfig( config, newPrefix, saveURLs, docContainer, id, depth);
+      it->saveConfig( config, newPrefix, saveURLs, docContainer, id, depth + i );
       i++;
     }
 
@@ -257,8 +270,7 @@ uint KonqFrameTabs::tabBarWidthForMaxChars( uint maxLength )
         newTitle = konqview->frame()->title();
     }
 
-    if ( newTitle.length() > maxLength )
-      newTitle = newTitle.left( maxLength-3 ) + "...";
+    newTitle = KStringHandler::rsqueeze( newTitle, maxLength ).leftJustify( m_minLength, ' ' );
 
     QTab* tab = tabBar()->tabAt( i );
     int lw = fm.width( newTitle );
@@ -278,18 +290,18 @@ void KonqFrameTabs::setTitle( const QString &title , QWidget* sender)
   removeTabToolTip( sender );
 
   uint lcw=0, rcw=0;
-#if QT_VERSION >= 0x030200
+
   int tabBarHeight = tabBar()->sizeHint().height();
   if ( cornerWidget( TopLeft ) && cornerWidget( TopLeft )->isVisible() )
     lcw = QMAX( cornerWidget( TopLeft )->width(), tabBarHeight );
   if ( cornerWidget( TopRight ) && cornerWidget( TopRight )->isVisible() )
     rcw = QMAX( cornerWidget( TopRight )->width(), tabBarHeight );
-#endif
+
   uint maxTabBarWidth = width() - lcw - rcw;
   // kdDebug(1202) << "maxTabBarWidth=" << maxTabBarWidth << endl;
 
-  uint newMaxLength=30;
-  for ( ; newMaxLength > 3; newMaxLength-- ) {
+  uint newMaxLength=m_maxLength;
+  for ( ; newMaxLength > m_minLength; newMaxLength-- ) {
     // kdDebug(1202) << "tabBarWidthForMaxChars(" << newMaxLength
     //               << ")=" << tabBarWidthForMaxChars( newMaxLength ) << endl;
     if ( tabBarWidthForMaxChars( newMaxLength ) < maxTabBarWidth )
@@ -299,10 +311,10 @@ void KonqFrameTabs::setTitle( const QString &title , QWidget* sender)
 
   QString newTitle = title;
   if ( newTitle.length() > newMaxLength )
-    {
       setTabToolTip( sender, newTitle );
-      newTitle = newTitle.left( newMaxLength-3 ) + "...";
-    }
+
+  newTitle = KStringHandler::rsqueeze( newTitle, newMaxLength ).leftJustify( m_minLength, ' ' );
+
   newTitle.replace( '&', "&&" );
   if ( tabLabel( sender ) != newTitle )
     changeTab( sender, newTitle );
@@ -322,10 +334,10 @@ void KonqFrameTabs::setTitle( const QString &title , QWidget* sender)
 
           removeTabToolTip( page( i ) );
           if ( newTitle.length() > newMaxLength )
-            {
               setTabToolTip( page( i ), newTitle );
-              newTitle = newTitle.left( newMaxLength-3 ) + "...";
-            }
+
+          newTitle = KStringHandler::rsqueeze( newTitle, newMaxLength ).leftJustify( m_minLength, ' ' );
+
           newTitle.replace( '&', "&&" );
           if ( newTitle != tabLabel( page( i ) ) )
             changeTab( page( i ), newTitle );
@@ -362,6 +374,7 @@ void KonqFrameTabs::insertChildFrame( KonqFrameBase* frame, int index )
   if (frame)
     {
       //kdDebug(1202) << "Adding frame" << endl;
+      bool showTabBar = (count() == 1);
       insertTab(frame->widget(),"", index);
       frame->setParentContainer(this);
       if (index == -1) m_pChildFrameList->append(frame);
@@ -373,6 +386,10 @@ void KonqFrameTabs::insertChildFrame( KonqFrameBase* frame, int index )
         activeChildView->setCaption( activeChildView->caption() );
         activeChildView->setTabIcon( activeChildView->url().url() );
       }
+      if (showTabBar)
+          this->showTabBar();
+      else if ( count() == 1 )
+          this->hideTabBar();//the first frame inserted (initialization)
     }
   else
     kdWarning(1202) << "KonqFrameTabs " << this << ": insertChildFrame(0L) !" << endl;
@@ -386,6 +403,8 @@ void KonqFrameTabs::removeChildFrame( KonqFrameBase * frame )
     m_pChildFrameList->remove(frame);
     if (m_rightWidget)
       m_rightWidget->setEnabled( m_pChildFrameList->count()>1 );
+    if (count() == 1)
+      hideTabBar();
   }
   else
     kdWarning(1202) << "KonqFrameTabs " << this << ": removeChildFrame(0L) !" << endl;
@@ -480,7 +499,7 @@ void KonqFrameTabs::slotSubPopupMenuTabActivated( int _id)
 void KonqFrameTabs::slotMouseMiddleClick()
 {
   QApplication::clipboard()->setSelectionMode( QClipboard::Selection );
-  KURL filteredURL = KonqMisc::konqFilteredURL( this, QApplication::clipboard()->text() );
+  KURL filteredURL ( KonqMisc::konqFilteredURL( this, QApplication::clipboard()->text() ) );
   if ( !filteredURL.isEmpty() ) {
     KonqView* newView = m_pViewManager->addTab(QString::null, QString::null, false, false);
     if (newView == 0L) return;
@@ -493,7 +512,7 @@ void KonqFrameTabs::slotMouseMiddleClick()
 void KonqFrameTabs::slotMouseMiddleClick( QWidget *w )
 {
   QApplication::clipboard()->setSelectionMode( QClipboard::Selection );
-  KURL filteredURL = KonqMisc::konqFilteredURL( this, QApplication::clipboard()->text() );
+  KURL filteredURL ( KonqMisc::konqFilteredURL( this, QApplication::clipboard()->text() ) );
   if ( !filteredURL.isEmpty() ) {
     KonqFrameBase* frame = dynamic_cast<KonqFrameBase*>(w);
     if (frame) {
@@ -559,4 +578,39 @@ void KonqFrameTabs::slotInitiateDrag( QWidget *w )
   }
 }
 
+void KonqFrameTabs::hideTabBar()
+{
+  if ( !m_alwaysTabBar ) {
+    if (m_leftWidget)
+      m_leftWidget->hide();
+    if (m_rightWidget)
+      m_rightWidget->hide();
+    tabBar()->hide();
+  }
+  m_pPopupMenu->setItemEnabled( BREAKOFF_ID, false );
+  m_pPopupMenu->setItemEnabled( CLOSETAB_ID, false );
+}
+
+void KonqFrameTabs::showTabBar()
+{
+  tabBar()->show();
+  if (m_leftWidget)
+    m_leftWidget->show();
+  if (m_rightWidget)
+    m_rightWidget->show();
+}
+
+void KonqFrameTabs::setAlwaysTabbedMode( bool enable )
+{
+  bool update = ( enable != m_alwaysTabBar );
+
+  m_alwaysTabBar = enable;
+  if ( update ) {
+    if ( m_alwaysTabBar )
+      showTabBar();
+    else
+      hideTabBar();
+  }
+}
+
 #include "konq_tabs.moc"
