@@ -18,43 +18,31 @@
 */
 
 #include "konq_listview.h"
-#include "konq_listviewitems.h"
-#include "konq_listviewwidget.h"
 #include "konq_textviewwidget.h"
 #include "konq_treeviewwidget.h"
-#include <konq_undo.h>
+#include "konq_infolistviewwidget.h"
 
 #include <kaction.h>
-#include <kcolordlg.h>
-#include <kcursor.h>
 #include <kdebug.h>
-#include <kio/job.h>
-#include <klibloader.h>
-#include <klineeditdlg.h>
+#include <kdirlister.h>
+#include <kinputdialog.h>
 #include <klocale.h>
-#include <konq_bgnddlg.h>
-#include <konq_dirlister.h>
 #include <konq_drag.h>
-#include <konq_fileitem.h>
-#include <kpropsdlg.h>
-#include <kprotocolmanager.h>
+#include <kpropertiesdialog.h>
 #include <kstdaction.h>
+#include <kprotocolinfo.h>
 
 #include <qapplication.h>
 #include <qclipboard.h>
-#include <qdragobject.h>
 #include <qheader.h>
-#include <qkeycode.h>
-#include <qlist.h>
 #include <qregexp.h>
-#include <qstringlist.h>
-#include <qtimer.h>
 
 #include <assert.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
+#include <kinstance.h>
 
 KonqListViewFactory::KonqListViewFactory()
 {
@@ -73,13 +61,12 @@ KonqListViewFactory::~KonqListViewFactory()
   s_defaultViewProps = 0;
 }
 
-KParts::Part* KonqListViewFactory::createPart( QWidget *parentWidget, const char *, QObject *parent, const char *name, const char*, const QStringList &args )
+KParts::Part* KonqListViewFactory::createPartObject( QWidget *parentWidget, const char *, QObject *parent, const char *name, const char*, const QStringList &args )
 {
   if( args.count() < 1 )
     kdWarning() << "KonqListView: Missing Parameter" << endl;
 
   KParts::Part *obj = new KonqListView( parentWidget, parent, name, args.first() );
-  emit objectCreated( obj );
   return obj;
 }
 
@@ -104,104 +91,78 @@ KonqPropsView *KonqListViewFactory::s_defaultViewProps = 0;
 
 extern "C"
 {
-  void *init_libkonqlistview()
+  void *init_konq_listview()
   {
     return new KonqListViewFactory;
   }
 }
 
 ListViewBrowserExtension::ListViewBrowserExtension( KonqListView *listView )
- : KParts::BrowserExtension( listView )
+ : KonqDirPartBrowserExtension( listView )
 {
   m_listView = listView;
 }
 
 int ListViewBrowserExtension::xOffset()
 {
-  kdDebug() << "ListViewBrowserExtension::xOffset" << endl;
+  //kdDebug(1202) << k_funcinfo << endl;
   return m_listView->listViewWidget()->contentsX();
 }
 
 int ListViewBrowserExtension::yOffset()
 {
-  kdDebug() << "ListViewBrowserExtension::yOffset" << endl;
+  //kdDebug(1202) << k_funcinfo << endl;
   return m_listView->listViewWidget()->contentsY();
 }
 
 void ListViewBrowserExtension::updateActions()
 {
   // This code is very related to KonqIconViewWidget::slotSelectionChanged
-
-  QValueList<KonqBaseListViewItem*> selection;
-  m_listView->listViewWidget()->selectedItems( selection );
-
-  bool cutcopy, del;
+  int canCopy = 0;
+  int canDel = 0;
   bool bInTrash = false;
-  QValueList<KonqBaseListViewItem*>::ConstIterator it = selection.begin();
-  KFileItem * firstSelectedItem = 0L;
-  for (; it != selection.end(); ++it )
+  KFileItemList lstItems = m_listView->selectedFileItems();
+
+  for (KFileItem *item = lstItems.first(); item; item = lstItems.next())
   {
-    if ( (*it)->item()->url().directory(false) == KGlobalSettings::trashPath() )
+    canCopy++;
+    KURL url = item->url();
+    if ( url.directory(false) == KGlobalSettings::trashPath() )
       bInTrash = true;
-    if ( ! firstSelectedItem )
-        firstSelectedItem = (*it)->item();
+    if (  KProtocolInfo::supportsDeleting(  url ) )
+      canDel++;
   }
-  cutcopy = del = ( selection.count() > 0 );
 
-  emit enableAction( "copy", cutcopy );
-  emit enableAction( "cut", cutcopy );
-  emit enableAction( "trash", del && !bInTrash && m_listView->url().isLocalFile() );
-  emit enableAction( "del", del );
-  emit enableAction( "shred", del );
-
-  KFileItemList lstItems;
-  if ( firstSelectedItem )
-      lstItems.append( firstSelectedItem );
-  emit enableAction( "properties", ( selection.count() == 1 ) &&
-                     KPropertiesDialog::canDisplay( lstItems ) );
-  emit enableAction( "editMimeType", ( selection.count() == 1 ) );
-  emit enableAction( "rename", ( selection.count() == 1 ) );
+  emit enableAction( "copy", canCopy > 0 );
+  emit enableAction( "cut", canDel > 0 );
+  emit enableAction( "trash", canDel > 0 && !bInTrash && m_listView->url().isLocalFile() );
+  emit enableAction( "del", canDel > 0 );
+  emit enableAction( "properties", lstItems.count() > 0 && KPropertiesDialog::canDisplay( lstItems ) );
+  emit enableAction( "editMimeType", ( lstItems.count() == 1 ) );
+  emit enableAction( "rename", ( m_listView->listViewWidget()->currentItem() != 0 )&& !bInTrash );
 }
 
 void ListViewBrowserExtension::copySelection( bool move )
 {
-  QValueList<KonqBaseListViewItem*> selection;
-
-  m_listView->listViewWidget()->selectedItems( selection );
-
-  KURL::List lstURLs;
-
-  QValueList<KonqBaseListViewItem*>::ConstIterator it = selection.begin();
-  QValueList<KonqBaseListViewItem*>::ConstIterator end = selection.end();
-  for (; it != end; ++it )
-  {
-    lstURLs.append( (*it)->item()->url() );
-  }
-
-  KonqDrag *urlData = KonqDrag::newDrag( lstURLs, move );
+  KonqDrag *urlData = KonqDrag::newDrag( m_listView->listViewWidget()->selectedUrls(), move );
   QApplication::clipboard()->setData( urlData );
 }
 
 void ListViewBrowserExtension::paste()
 {
-  QValueList<KonqBaseListViewItem*> selection;
-  /*m_listView->listViewWidget()->selectedItems( selection );
-  assert ( selection.count() <= 1 );
-  KURL pasteURL;
-  if ( selection.count() == 1 )
-    pasteURL = selection.first()->item()->url();
-  else
-    pasteURL = m_listView->url();*/
+  KonqOperations::doPaste( m_listView->listViewWidget(), m_listView->url() );
+}
 
-  KonqOperations::doPaste( m_listView->listViewWidget(), /*pasteURL*/m_listView->url() );
+void ListViewBrowserExtension::pasteTo( const KURL& url )
+{
+  KonqOperations::doPaste( m_listView->listViewWidget(), url );
 }
 
 void ListViewBrowserExtension::rename()
 {
-  QValueList<KonqBaseListViewItem*> selection;
-  m_listView->listViewWidget()->selectedItems( selection );
-  ASSERT ( selection.count() == 1 );
-  m_listView->listViewWidget()->rename( selection.first(), 0 );
+  QListViewItem* item = m_listView->listViewWidget()->currentItem();
+  Q_ASSERT ( item );
+  m_listView->listViewWidget()->rename( item, 0 );
 }
 
 void ListViewBrowserExtension::reparseConfiguration()
@@ -216,29 +177,26 @@ void ListViewBrowserExtension::setSaveViewPropertiesLocally(bool value)
    m_listView->props()->setSaveViewPropertiesLocally( value );
 }
 
-void ListViewBrowserExtension::setNameFilter( QString nameFilter )
+void ListViewBrowserExtension::setNameFilter( const QString &nameFilter )
 {
   m_listView->setNameFilter( nameFilter );
 }
 
 void ListViewBrowserExtension::properties()
 {
-    QValueList<KonqBaseListViewItem*> selection;
-    m_listView->listViewWidget()->selectedItems( selection );
-    assert ( selection.count() == 1 );
-    (void) new KPropertiesDialog( selection.first()->item() );
+  (void) new KPropertiesDialog( m_listView->selectedFileItems() );
 }
 
 void ListViewBrowserExtension::editMimeType()
 {
-    QValueList<KonqBaseListViewItem*> selection;
-    m_listView->listViewWidget()->selectedItems( selection );
-    assert ( selection.count() == 1 );
-    KonqOperations::editMimeType( selection.first()->item()->mimetype() );
+  KFileItemList items = m_listView->selectedFileItems();
+  assert ( items.count() == 1 );
+  KonqOperations::editMimeType( items.first()->mimetype() );
 }
 
 KonqListView::KonqListView( QWidget *parentWidget, QObject *parent, const char *name, const QString& mode )
  : KonqDirPart( parent, name )
+,m_headerTimer(0)
 {
    setInstance( KonqListViewFactory::instance() );
 
@@ -260,6 +218,12 @@ KonqListView::KonqListView( QWidget *parentWidget, QObject *parent, const char *
       setXMLFile( "konq_treeview.rc" );
       m_pListView=new KonqTreeViewWidget(this,parentWidget);
    }
+   else if (mode=="InfoListView")
+   {
+      kdDebug(1202) << "Creating KonqInfoListViewWidget" << endl;
+      setXMLFile( "konq_infolistview.rc" );
+      m_pListView=new KonqInfoListViewWidget(this,parentWidget);
+   }
    else
    {
       kdDebug(1202) << "Creating KonqDetailedListViewWidget" << endl;
@@ -267,17 +231,14 @@ KonqListView::KonqListView( QWidget *parentWidget, QObject *parent, const char *
       m_pListView = new KonqBaseListViewWidget( this, parentWidget);
    }
    setWidget( m_pListView );
+   setDirLister( m_pListView->m_dirLister );
 
-   QTimer * timer = new QTimer( this );
-   connect( timer, SIGNAL( timeout() ), this, SLOT( slotProcessMimeIcons() ) );
-   m_mimeTypeResolver = new KonqMimeTypeResolver<KonqBaseListViewItem,KonqListView>(this,timer);
-   // When our viewport is adjusted (resized or scrolled) we need
-   // to get the mime types for any newly visible icons. (Rikkus)
-   connect( m_pListView, SIGNAL( viewportAdjusted() ),
-            SLOT( slotViewportAdjusted() ) );
+
+   m_mimeTypeResolver = new KMimeTypeResolver<KonqBaseListViewItem,KonqListView>(this);
 
    setupActions();
 
+   m_pListView->confColumns.resize( 11 );
    m_pListView->confColumns[0].setData(I18N_NOOP("MimeType"),"Type",KIO::UDS_MIME_TYPE,-1,FALSE,m_paShowMimeType);
    m_pListView->confColumns[1].setData(I18N_NOOP("Size"),"Size",KIO::UDS_SIZE,-1,FALSE,m_paShowSize);
    m_pListView->confColumns[2].setData(I18N_NOOP("Modified"),"Date",KIO::UDS_MODIFICATION_TIME,-1,FALSE,m_paShowTime);
@@ -291,10 +252,17 @@ KonqListView::KonqListView( QWidget *parentWidget, QObject *parent, const char *
    // Note: File Type is in fact the mimetype comment. We use UDS_FILE_TYPE but that's not what we show in fact :/
    m_pListView->confColumns[10].setData(I18N_NOOP("File Type"),"Type",KIO::UDS_FILE_TYPE,-1,FALSE,m_paShowType);
 
-   QObject::connect( m_pListView, SIGNAL( selectionChanged() ),
-                     m_extension, SLOT( updateActions() ) );
+
+   connect( m_pListView, SIGNAL( selectionChanged() ),
+            m_extension, SLOT( updateActions() ) );
+   connect( m_pListView, SIGNAL( selectionChanged() ),
+            this, SLOT( slotSelectionChanged() ) );
+
+   connect( m_pListView, SIGNAL( currentChanged(QListViewItem*) ),
+            m_extension, SLOT( updateActions() ) );
    connect(m_pListView->header(),SIGNAL(indexChange(int,int,int)),this,SLOT(headerDragged(int,int,int)));
    connect(m_pListView->header(),SIGNAL(clicked(int)),this,SLOT(slotHeaderClicked(int)));
+   connect(m_pListView->header(),SIGNAL(sizeChange(int,int,int)),SLOT(slotHeaderSizeChanged()));
 
    // signals from konqdirpart (for BC reasons)
    connect( this, SIGNAL( findOpened( KonqDirPart * ) ), SLOT( slotKFindOpened() ) );
@@ -312,20 +280,16 @@ void KonqListView::guiActivateEvent( KParts::GUIActivateEvent *event )
    KonqDirPart::guiActivateEvent(event );
    //ReadOnlyPart::guiActivateEvent(event );
    ((ListViewBrowserExtension*)m_extension)->updateActions();
-};
+}
 
-bool KonqListView::openURL( const KURL &url )
+bool KonqListView::doOpenURL( const KURL &url )
 {
-  m_url = url;
-
   KURL u( url );
-
   emit setWindowCaption( u.prettyURL() );
-
   return m_pListView->openURL( url );
 }
 
-bool KonqListView::closeURL()
+bool KonqListView::doCloseURL()
 {
   m_pListView->stop();
   m_mimeTypeResolver->m_lstPendingMimeIconItems.clear();
@@ -354,13 +318,14 @@ void KonqListView::determineIcon( KonqBaseListViewItem * item )
 
 void KonqListView::saveState( QDataStream &stream )
 {
-    kdDebug() << "KonqListView::saveState" << endl;
+    //kdDebug(1202) << k_funcinfo << endl;
     KonqDirPart::saveState( stream );
     m_pListView->saveState( stream );
 }
 
 void KonqListView::restoreState( QDataStream &stream )
 {
+    //kdDebug(1202) << k_funcinfo << endl;
     KonqDirPart::restoreState( stream );
     m_pListView->restoreState( stream );
 }
@@ -372,46 +337,53 @@ void KonqListView::disableIcons( const KURL::List &lst )
 
 void KonqListView::slotSelect()
 {
-   KLineEditDlg l( i18n("Select files:"), "*", m_pListView );
-   if ( l.exec() )
+   bool ok;
+   QString pattern = KInputDialog::getText( QString::null,
+      i18n( "Select files:" ), "*", &ok, m_pListView );
+   if ( !ok )
+      return;
+
+   QRegExp re( pattern, true, true );
+
+   m_pListView->blockSignals( true );
+
+   for (KonqBaseListViewWidget::iterator it = m_pListView->begin(); it != m_pListView->end(); it++ )
    {
-      QString pattern = l.text();
-      if ( pattern.isEmpty() )
-         return;
-
-      QRegExp re( pattern, true, true );
-
-      m_pListView->blockSignals( true );
-
-      for (KonqBaseListViewWidget::iterator it = m_pListView->begin(); it != m_pListView->end(); it++ )
-         if ( re.match( it->text(0) ) != -1 )
-            it->setSelected( TRUE);
-
-      m_pListView->blockSignals( false );
-   }
+      if ((m_pListView->automaticSelection()) && (it->isSelected()))
+      {
+         it->setSelected(FALSE);
+         //the following line is to prevent that more than one item were selected
+         //and now get deselected and automaticSelection() was true, this shouldn't happen
+         //but who knows, aleXXX
+         m_pListView->deactivateAutomaticSelection();
+      };
+      if ( re.exactMatch( it->text(0) ) )
+         it->setSelected( TRUE);
+   };
+   m_pListView->blockSignals( false );
+   m_pListView->deactivateAutomaticSelection();
    emit m_pListView->selectionChanged();
    m_pListView->viewport()->update();
 }
 
 void KonqListView::slotUnselect()
 {
-   KLineEditDlg l( i18n("Unselect files:"), "*", m_pListView );
-   if ( l.exec() )
-   {
-      QString pattern = l.text();
-      if ( pattern.isEmpty() )
-         return;
+   bool ok;
+   QString pattern = KInputDialog::getText( QString::null,
+      i18n( "Unselect files:" ), "*", &ok, m_pListView );
+   if ( !ok )
+      return;
 
-      QRegExp re( pattern, TRUE, TRUE );
+   QRegExp re( pattern, TRUE, TRUE );
 
-      m_pListView->blockSignals(TRUE);
+   m_pListView->blockSignals(TRUE);
 
-      for (KonqBaseListViewWidget::iterator it = m_pListView->begin(); it != m_pListView->end(); it++ )
-         if ( re.match( it->text(0) ) != -1 )
-            it->setSelected(FALSE);
+   for (KonqBaseListViewWidget::iterator it = m_pListView->begin(); it != m_pListView->end(); it++ )
+      if ( re.exactMatch( it->text(0) ) )
+         it->setSelected(FALSE);
 
-      m_pListView->blockSignals(FALSE);
-   }
+   m_pListView->blockSignals(FALSE);
+   m_pListView->deactivateAutomaticSelection();
    emit m_pListView->selectionChanged();
    m_pListView->viewport()->update();
 }
@@ -419,19 +391,27 @@ void KonqListView::slotUnselect()
 void KonqListView::slotSelectAll()
 {
    m_pListView->selectAll(TRUE);
+   m_pListView->deactivateAutomaticSelection();
    emit m_pListView->selectionChanged();
 }
 
 void KonqListView::slotUnselectAll()
 {
     m_pListView->selectAll(FALSE);
+   m_pListView->deactivateAutomaticSelection();
     emit m_pListView->selectionChanged();
 }
 
 
 void KonqListView::slotInvertSelection()
 {
-    m_pListView->invertSelection();
+   if ((m_pListView->automaticSelection())
+       && (m_pListView->currentItem()!=0)
+       && (m_pListView->currentItem()->isSelected()))
+      m_pListView->currentItem()->setSelected(FALSE);
+
+   m_pListView->invertSelection();
+    m_pListView->deactivateAutomaticSelection();
     emit m_pListView->selectionChanged();
     m_pListView->viewport()->update();
 }
@@ -446,25 +426,26 @@ void KonqListView::slotShowDot()
 {
    m_pProps->setShowingDotFiles( m_paShowDot->isChecked() );
    m_pListView->m_dirLister->setShowingDotFiles( m_pProps->isShowingDotFiles() );
+   m_pListView->m_dirLister->emitChanges();
 }
 
 void KonqListView::slotCaseInsensitive()
 {
-   m_pListView->setCaseInsensitiveSort( m_paCaseInsensitive->isChecked() );
+   m_pProps->setCaseInsensitiveSort( m_paCaseInsensitive->isChecked() );
    m_pListView->sort();
 }
 
 void KonqListView::slotColumnToggled()
 {
    kdDebug(1202) << "::slotColumnToggled" << endl;
-   for (unsigned int i=0; i<KonqBaseListViewWidget::NumberOfAtoms; i++)
+   for (uint i=0; i<m_pListView->NumberOfAtoms; i++)
    {
       m_pListView->confColumns[i].displayThisOne=m_pListView->confColumns[i].toggleThisOne->isChecked()&&m_pListView->confColumns[i].toggleThisOne->isEnabled();
       //this column has been enabled, the columns after it slide one column back
       if ((m_pListView->confColumns[i].displayThisOne) && (m_pListView->confColumns[i].displayInColumn==-1))
       {
          int maxColumn(0);
-         for (unsigned int j=0; j<KonqBaseListViewWidget::NumberOfAtoms; j++)
+         for (uint j=0; j<m_pListView->NumberOfAtoms; j++)
             if ((m_pListView->confColumns[j].displayInColumn>maxColumn) && (m_pListView->confColumns[j].displayThisOne))
                maxColumn=m_pListView->confColumns[j].displayInColumn;
          m_pListView->confColumns[i].displayInColumn=maxColumn+1;
@@ -472,7 +453,7 @@ void KonqListView::slotColumnToggled()
       //this column has been disabled, the columns after it slide one column
       if ((!m_pListView->confColumns[i].displayThisOne) && (m_pListView->confColumns[i].displayInColumn!=-1))
       {
-         for (unsigned int j=0; j<KonqBaseListViewWidget::NumberOfAtoms; j++)
+         for (uint j=0; j<m_pListView->NumberOfAtoms; j++)
             if (m_pListView->confColumns[j].displayInColumn>m_pListView->confColumns[i].displayInColumn)
                m_pListView->confColumns[j].displayInColumn--;
          m_pListView->confColumns[i].displayInColumn=-1;
@@ -489,7 +470,7 @@ void KonqListView::slotColumnToggled()
    config->setGroup( groupName );
    QStringList lstColumns;
    int currentColumn(m_pListView->m_filenameColumn+1);
-   for (int i=0; i<KonqBaseListViewWidget::NumberOfAtoms; i++)
+   for (int i=0; i<(int)m_pListView->NumberOfAtoms; i++)
    {
       kdDebug(1202)<<"checking: -"<<m_pListView->confColumns[i].name<<"-"<<endl;
       if ((m_pListView->confColumns[i].displayThisOne) && (currentColumn==m_pListView->confColumns[i].displayInColumn))
@@ -502,13 +483,16 @@ void KonqListView::slotColumnToggled()
    }
    config->writeEntry("Columns",lstColumns);
    config->sync();
+
+   // Update column sizes
+   slotHeaderSizeChanged();
 }
 
 void KonqListView::slotHeaderClicked(int sec)
 {
    kdDebug(1202)<<"section: "<<sec<<" clicked"<<endl;
    int clickedColumn(-1);
-   for (int i=0; i<KonqBaseListViewWidget::NumberOfAtoms; i++)
+   for (uint i=0; i<m_pListView->NumberOfAtoms; i++)
       if (m_pListView->confColumns[i].displayInColumn==sec) clickedColumn=i;
    kdDebug(1202)<<"atom index "<<clickedColumn<<endl;
    QString nameOfSortColumn;
@@ -532,7 +516,7 @@ void KonqListView::slotHeaderClicked(int sec)
    config->writeEntry("SortBy",nameOfSortColumn);
    config->writeEntry("SortOrder",m_pListView->ascending());
    config->sync();
-};
+}
 
 void KonqListView::headerDragged(int sec, int from, int to)
 {
@@ -556,33 +540,70 @@ void KonqListView::slotSaveAfterHeaderDrag()
    config->setGroup( groupName );
    QStringList lstColumns;
 
-   int oldCurrentColumn(-1);
-   for (unsigned int i=0; i<KonqBaseListViewWidget::NumberOfAtoms; i++)
+   for ( int i=0; i < m_pListView->columns(); i++ )
    {
-      int currentColumn(1000);
-      for (unsigned int j=0; j<KonqBaseListViewWidget::NumberOfAtoms; j++)
+      int section = m_pListView->header()->mapToSection( i );
+
+      // look for section
+      for ( uint j=0; j < m_pListView->NumberOfAtoms; j++ )
       {
-         int tmp=m_pListView->header()->mapToIndex(m_pListView->confColumns[j].displayInColumn);
-         if ((tmp>oldCurrentColumn) && (tmp<currentColumn))
-            currentColumn=tmp;
-      };
-      kdDebug(1202)<<"currentColumn: "<<currentColumn<<endl;
-      //everything done
-      if (currentColumn==1000) break;
-      for (unsigned int j=0; j<KonqBaseListViewWidget::NumberOfAtoms; j++)
-      {
-         int tmp=m_pListView->header()->mapToIndex(m_pListView->confColumns[j].displayInColumn);
-         if (tmp==currentColumn)
+         if ( m_pListView->confColumns[j].displayInColumn == section )
          {
-            oldCurrentColumn=currentColumn;
-            lstColumns.append(m_pListView->confColumns[j].name);
-            kdDebug(1202)<<"appending: "<<m_pListView->confColumns[j].name<<endl;
+            lstColumns.append( m_pListView->confColumns[j].name );
+            break;
          }
       }
-
    }
    config->writeEntry("Columns",lstColumns);
    config->sync();
+
+   // Update column sizes
+   slotHeaderSizeChanged();
+}
+
+void KonqListView::slotSaveColumnWidths()
+{
+   QString protocol = url().protocol();
+   KConfig * config = KGlobal::config();
+   config->setGroup( "ListView_" + protocol );
+
+   QValueList<int> lstColumnWidths;
+
+   for ( int i=0; i < m_pListView->columns(); i++ )
+   {
+      int section = m_pListView->header()->mapToSection( i );
+
+      // look for section
+      for ( uint j=0; j < m_pListView->NumberOfAtoms; j++ )
+      {
+         // Save size only if the column is found
+         if ( m_pListView->confColumns[j].displayInColumn == section )
+         {
+            m_pListView->confColumns[j].width = m_pListView->columnWidth(section);
+            lstColumnWidths.append( m_pListView->confColumns[j].width );
+            break;
+         }
+      }
+   }
+   config->writeEntry( "ColumnWidths", lstColumnWidths );
+
+   // size of current filename column
+   config->writeEntry( "FileNameColumnWidth", m_pListView->columnWidth(0) );
+
+   config->sync();
+}
+
+void KonqListView::slotHeaderSizeChanged()
+{
+   if ( !m_headerTimer )
+   {
+      m_headerTimer = new QTimer( this );
+      connect( m_headerTimer, SIGNAL( timeout() ), this, SLOT( slotSaveColumnWidths() ) );
+   }
+   else
+      m_headerTimer->stop();
+
+   m_headerTimer->start( 250, true );
 }
 
 void KonqListView::slotKFindOpened()
@@ -600,30 +621,48 @@ void KonqListView::slotKFindClosed()
 void KonqListView::setupActions()
 {
    m_paShowTime=new KToggleAction(i18n("Show &Modification Time"), 0,this, SLOT(slotColumnToggled()), actionCollection(), "show_time" );
+   m_paShowTime->setCheckedState(i18n("Hide &Modification Time"));
    m_paShowType=new KToggleAction(i18n("Show &File Type"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_type" );
+   m_paShowType->setCheckedState(i18n("Hide &File Type"));
    m_paShowMimeType=new KToggleAction(i18n("Show MimeType"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_mimetype" );
+   m_paShowMimeType->setCheckedState(i18n("Hide MimeType"));
    m_paShowAccessTime=new KToggleAction(i18n("Show &Access Time"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_access_time" );
+   m_paShowAccessTime->setCheckedState(i18n("Hide &Access Time"));
    m_paShowCreateTime=new KToggleAction(i18n("Show &Creation Time"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_creation_time" );
+   m_paShowCreateTime->setCheckedState(i18n("Hide &Creation Time"));
    m_paShowLinkDest=new KToggleAction(i18n("Show &Link Destination"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_link_dest" );
+   m_paShowLinkDest->setCheckedState(i18n("Hide &Link Destination"));
    m_paShowSize=new KToggleAction(i18n("Show Filesize"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_size" );
+   m_paShowSize->setCheckedState(i18n("Hide Filesize"));
    m_paShowOwner=new KToggleAction(i18n("Show Owner"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_owner" );
+   m_paShowOwner->setCheckedState(i18n("Hide Owner"));
    m_paShowGroup=new KToggleAction(i18n("Show Group"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_group" );
+   m_paShowGroup->setCheckedState(i18n("Hide Group"));
    m_paShowPermissions=new KToggleAction(i18n("Show Permissions"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_permissions" );
+   m_paShowPermissions->setCheckedState(i18n("Hide Permissions"));
    m_paShowURL=new KToggleAction(i18n("Show URL"), 0, this, SLOT(slotColumnToggled()),actionCollection(), "show_url" );
 
-   m_paSelect = new KAction( i18n( "&Select..." ), CTRL+Key_Plus, this, SLOT( slotSelect() ), actionCollection(), "select" );
-  m_paUnselect = new KAction( i18n( "&Unselect..." ), CTRL+Key_Minus, this, SLOT( slotUnselect() ), actionCollection(), "unselect" );
-  m_paSelectAll = KStdAction::selectAll( this, SLOT( slotSelectAll() ), this, "selectall" );
-  m_paUnselectAll = new KAction( i18n( "U&nselect All" ), CTRL+Key_U, this, SLOT( slotUnselectAll() ), actionCollection(), "unselectall" );
+   m_paSelect = new KAction( i18n( "Se&lect..." ), CTRL+Key_Plus, this, SLOT( slotSelect() ), actionCollection(), "select" );
+  m_paUnselect = new KAction( i18n( "Unselect..." ), CTRL+Key_Minus, this, SLOT( slotUnselect() ), actionCollection(), "unselect" );
+  m_paSelectAll = KStdAction::selectAll( this, SLOT( slotSelectAll() ), actionCollection(), "selectall" );
+  m_paUnselectAll = new KAction( i18n( "Unselect All" ), CTRL+Key_U, this, SLOT( slotUnselectAll() ), actionCollection(), "unselectall" );
   m_paInvertSelection = new KAction( i18n( "&Invert Selection" ), CTRL+Key_Asterisk, this, SLOT( slotInvertSelection() ), actionCollection(), "invertselection" );
 
   m_paShowDot = new KToggleAction( i18n( "Show &Hidden Files" ), 0, this, SLOT( slotShowDot() ), actionCollection(), "show_dot" );
+//  m_paShowDot->setCheckedState(i18n("Hide &Hidden Files"));
   m_paCaseInsensitive = new KToggleAction(i18n("Case Insensitive Sort"), 0, this, SLOT(slotCaseInsensitive()),actionCollection(), "sort_caseinsensitive" );
-  m_paCaseInsensitive->setChecked( m_pListView->caseInsensitiveSort() );
 
   newIconSize( KIcon::SizeSmall /* default size */ );
 }
 
+void KonqListView::slotSelectionChanged()
+{
+  bool itemSelected = selectedFileItems().count()>0;
+  m_paUnselect->setEnabled( itemSelected );
+  m_paUnselectAll->setEnabled( itemSelected );
+//  m_paInvertSelection->setEnabled( itemSelected );
+}
+
 #include "konq_listview.moc"
 
 
