@@ -95,10 +95,13 @@
                   "abcdefgjijklmnopqrstuvwxyz" \
                   "0123456789./+@"
 
+extern bool argb_visual; // declared in main.cpp and konsole_part.cpp
+
 // scroll increment used when dragging selection at top/bottom of window.
 
 // static
 bool TEWidget::s_antialias = true;
+bool TEWidget::s_standalone = false;
 
 /* ------------------------------------------------------------------------- */
 /*                                                                           */
@@ -136,8 +139,17 @@ static const ColorEntry base_color_table[TABLE_COLORS] =
    IBMPC (rgb) Black   Blue    Green   Cyan    Red     Magenta Yellow  White
 */
 
+void TEWidget::setDefaultBackColor(const QColor& color)
+{
+  defaultBgColor = color;
+  if (qAlpha(blend_color) != 0xff && !backgroundPixmap())
+    setBackgroundColor(getDefaultBackColor());
+}
+
 QColor TEWidget::getDefaultBackColor()
 {
+  if (defaultBgColor.isValid())
+    return defaultBgColor;
   return color_table[DEFAULT_BACK_COLOR].color;
 }
 
@@ -150,7 +162,17 @@ void TEWidget::setColorTable(const ColorEntry table[])
 {
   for (int i = 0; i < TABLE_COLORS; i++) color_table[i] = table[i];
   const QPixmap* pm = backgroundPixmap();
-  if (!pm) setBackgroundColor(color_table[DEFAULT_BACK_COLOR].color);
+  if (!pm)
+    if (!argb_visual || (qAlpha(blend_color) == 0xff))
+      setBackgroundColor(getDefaultBackColor());
+    else {
+      float alpha = qAlpha(blend_color) / 255.;
+      int pixel = qAlpha(blend_color) << 24 |
+                  int(qRed(blend_color) * alpha) << 16 |
+                  int(qGreen(blend_color) * alpha) << 8  |
+                  int(qBlue(blend_color) * alpha);
+      setBackgroundColor(QColor(blend_color, pixel));
+    }
   update();
 }
 
@@ -348,6 +370,7 @@ TEWidget::TEWidget(QWidget *parent, const char *name)
 ,m_cursorCol(0)
 ,m_isIMEdit(false)
 ,m_isIMSel(false)
+,blend_color(qRgba(0,0,0,0xff))
 {
   // The offsets are not yet calculated.
   // Do not calculate these too often to be more smoothly when resizing
@@ -379,9 +402,14 @@ TEWidget::TEWidget(QWidget *parent, const char *name)
 
   setFocusPolicy( WheelFocus );
 
-  // We're just a big pixmap, no need to have a background
-  // Speeds up redraws
-  setBackgroundMode(NoBackground);
+  // im
+  setInputMethodEnabled(true);
+
+  if (!argb_visual)
+  {
+    // Looks better at startup with KRootPixmap based pseudo-transparancy
+    setBackgroundMode(NoBackground);
+  }
 }
 
 //FIXME: make proper destructor
@@ -452,7 +480,26 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
     {
       if (pm || color_table[attr->b].color != color_table[ colorsSwapped ? DEFAULT_FORE_COLOR : DEFAULT_BACK_COLOR ].color
           || clear || (blinking && (attr->r & RE_BLINK)))
-        paint.fillRect(rect, color_table[attr->b].color);
+
+        // draw background colors with 75% opacity
+        if ( argb_visual && qAlpha(blend_color) < 0xff ) {
+          QRgb col = color_table[attr->b].color.rgb();
+
+          Q_UINT8 salpha = 192;
+          Q_UINT8 dalpha = 255 - salpha;
+
+          int a, r, g, b;
+          a = QMIN( (qAlpha (col) * salpha) / 255 + (qAlpha (blend_color) * dalpha) / 255, 255 );
+          r = QMIN( (qRed   (col) * salpha) / 255 + (qRed   (blend_color) * dalpha) / 255, 255 );
+          g = QMIN( (qGreen (col) * salpha) / 255 + (qGreen (blend_color) * dalpha) / 255, 255 );
+          b = QMIN( (qBlue  (col) * salpha) / 255 + (qBlue  (blend_color) * dalpha) / 255, 255 );
+
+          col = a << 24 | r << 16 | g << 8 | b;
+          int pixel = a << 24 | (r * a / 255) << 16 | (g * a / 255) << 8 | (b * a / 255);
+
+          paint.fillRect(rect, QColor(col, pixel));
+        } else
+          paint.fillRect(rect, color_table[attr->b].color);
     }
 
     QString tmpStr = str.simplifyWhiteSpace();
@@ -472,7 +519,7 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
       int w = font_w * (m_imSelEnd - m_imSelStart);
       int h = font_h;
 
-      QRect tmpRect = QRect( x, y, w, h ); 
+      QRect tmpRect = QRect( x, y, w, h );
       if ( str != m_imPreeditText ) {  // ugly hack
         tmpRect.setLeft( tmpRect.left() + font_w );
         tmpRect.setWidth( tmpRect.width() + font_w );
@@ -505,6 +552,11 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
   // Paint text
   if (!(blinking && (attr->r & RE_BLINK)))
   {
+    // ### Disabled for now, since it causes problems with characters
+    // that use the full width and/or height of the character cells.
+    //bool shadow = ( !isPrinting && qAlpha(blend_color) < 0xff
+    //		    && qGray( fColor.rgb() ) > 64 );
+    bool shadow = false;
     paint.setPen(fColor);
     int x = rect.x();
     if (color_table[attr->f].bold && printerBold)
@@ -521,6 +573,12 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
       // The meaning of y differs between different versions of QPainter::drawText!!
       int y = rect.y(); // top of rect
 
+      if ( shadow ) {
+        paint.setPen( Qt::black );
+        drawTextFixed(paint, x+1, y+1, str, attr);
+        paint.setPen(fColor);
+      }
+
       drawTextFixed(paint, x, y, str, attr);
     }
     else
@@ -528,11 +586,13 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
       // The meaning of y differs between different versions of QPainter::drawText!!
       int y = rect.y()+a; // baseline
 
-      if (bidiEnabled)
-        paint.drawText(x,y, str, -1);
-      else
-        paint.drawText(x,y, str, -1, QPainter::LTR);
+      if ( shadow ) {
+        paint.setPen( Qt::black );
+        paint.drawText(x+1,y+1, str, -1, bidiEnabled ? QPainter::Auto : QPainter::LTR );
+        paint.setPen(fColor);
+      }
 
+      paint.drawText(x,y, str, -1, bidiEnabled ? QPainter::Auto : QPainter::LTR );
     }
 
     if (color_table[attr->f].bold && isPrinting)
@@ -695,7 +755,7 @@ HCNT("setImage");
           if ( ( m_imSelStart < m_imSelEnd ) )
             m_isIMSel = true;
 	}
-        
+
         drawAttrStr(paint,
                     QRect(bX+tLx+font_w*x,bY+tLy+font_h*y,font_w*len,font_h),
                     unistr, &ext[x], pm != NULL, true);
@@ -745,7 +805,7 @@ HCNT("setImage");
      QString sizeStr = i18n("Size: %1 x %2").arg(columns).arg(lines);
      mResizeLabel->setText(sizeStr);
      mResizeWidget->move((width()-mResizeWidget->width())/2,
-                         (height()-mResizeWidget->height())/2);
+                         (height()-mResizeWidget->height())/2+20);
      mResizeWidget->show();
      mResizeTimer->start(1000, true);
   }
@@ -951,7 +1011,7 @@ void TEWidget::updateImageSize()
   //NOTE: control flows from the back through the chest right into the eye.
   //      `emu' will call back via `setImage'.
 
-  resizing = true;
+  resizing = (oldlin!=lines) || (oldcol!=columns);
   emit changedContentSizeSignal(contentHeight, contentWidth); // expose resizeEvent
   resizing = false;
 }
@@ -1714,7 +1774,7 @@ void TEWidget::imEndEvent( QIMEvent *e )
                              contentsRect().width(), contentsRect().height() );
   m_imStart = 0;
   m_imPreeditLength = 0;
- 
+
   m_isIMEdit = m_isIMSel = false;
   repaint( repaintRect, true );
 }
@@ -1729,7 +1789,7 @@ bool TEWidget::event( QEvent *e )
     KKey key( ke );
     int keyCodeQt = key.keyCodeQt();
 
-    if ( ke->state() == Qt::ControlButton )
+    if ( !standalone() && (ke->state() == Qt::ControlButton) )
     {
       ke->accept();
       return true;
