@@ -67,8 +67,7 @@ QString KeyTrans::KeyEntry::text()
 */
 
 KeyTrans::KeyTrans(const QString& path)
-:m_hdr("")
-,m_path(path)
+:m_path(path)
 ,m_numb(0)
 ,m_fileRead(false)
 {
@@ -483,6 +482,7 @@ void KeyTransSymbols::defModSyms()
   defModSym("Ansi",       BITS_Ansi         );
   defModSym("NewLine",    BITS_NewLine      );
   defModSym("AppCuKeys",  BITS_AppCuKeys    );
+  defModSym("AppScreen",  BITS_AppScreen    );
 }
 
 void KeyTransSymbols::defKeySyms()
