@@ -29,9 +29,26 @@
 #include <kimageio.h>
 #include <kdebug.h>
 #include <kstandarddirs.h>
+
+#include <config.h>
+
 #include "konsole.h"
 
-static const char *description =
+// COMPOSITE disabled by default because the QApplication constructor
+// needed to enable the ARGB32 visual has undesired side effects.
+#if 0
+#if defined(Q_WS_X11) && defined(HAVE_XRENDER) && QT_VERSION >= 0x030300
+# define COMPOSITE
+#endif
+#endif
+
+#ifdef COMPOSITE
+# include <X11/Xlib.h>
+# include <X11/extensions/Xrender.h>
+# include <fixx11h.h>
+#endif
+
+static const char description[] =
   I18N_NOOP("X terminal for use with KDE.");
 
 //   { "T <title>",       0, 0 },
@@ -49,15 +66,19 @@ static KCmdLineOptions options[] =
    { "notoolbar",       I18N_NOOP("Do not display tab bar"), 0 },
    { "noframe",         I18N_NOOP("Do not display frame"), 0 },
    { "noscrollbar",     I18N_NOOP("Do not display scrollbar"), 0 },
-   { "noxft",           I18N_NOOP("Do not use XFT (Anti-Aliasing)"), 0 },
+   { "noxft",           I18N_NOOP("Do not use Xft (Anti-Aliasing)"), 0 },
+#ifdef COMPOSITE
+   { "noargb",          I18N_NOOP("Do not use the ARGB32 visual (Transparency)"), 0 },
+#endif
    { "vt_sz CCxLL",     I18N_NOOP("Terminal size in columns x lines"), 0 },
    { "noresize",        I18N_NOOP("Terminal size is fixed"), 0 },
    { "type <type>",     I18N_NOOP("Open the given session type instead of the default shell"), 0 },
-   { "types",           I18N_NOOP("List available session types."), 0 },
+   { "types",           I18N_NOOP("List available session types"), 0 },
    { "keytab <name>",   I18N_NOOP("Use given .keytab file"), 0 },
    { "profile <name>",  I18N_NOOP("Start with given sessions profile"), 0 },
-   { "profiles",        I18N_NOOP("List available profiles."), 0 },
+   { "profiles",        I18N_NOOP("List available profiles"), 0 },
    { "schema <name>",   I18N_NOOP("Use given .schema file"), 0 },
+   { "schemas",         I18N_NOOP("List available schemas"), 0 },
    { "script",          I18N_NOOP("Enable extended DCOP functions"), 0 },
    { "workdir <dir>",   I18N_NOOP("Change working directory of the konsole to 'dir'"), 0 },
    { "!e <command>",    I18N_NOOP("Execute 'command' instead of shell"), 0 },
@@ -72,6 +93,8 @@ static bool full_script = false;
 static bool auto_close = true;
 static bool fixed_size = false;
 
+bool argb_visual = false;
+
 const char *konsole_shell(QStrList &args)
 {
   const char* shell = getenv("SHELL");
@@ -148,7 +171,7 @@ extern "C" int kdemain(int argc, char* argv[])
 
   KAboutData aboutData( PACKAGE, I18N_NOOP("Konsole"),
     VERSION, description, KAboutData::License_GPL_V2,
-    "(c) 1997-2002, Lars Doelle");
+    "(c) 1997-2004, Lars Doelle");
   aboutData.addAuthor("Waldo Bastian",I18N_NOOP("Maintainer"), "bastian@kde.org");
   aboutData.addAuthor("Lars Doelle",I18N_NOOP("Author"), "lars.doelle@on-line.de");
   aboutData.addCredit("Stephan Binner",
@@ -213,8 +236,50 @@ extern "C" int kdemain(int argc, char* argv[])
   KCmdLineArgs *qtargs = KCmdLineArgs::parsedArgs("qt");
   has_noxft = !args->isSet("xft");
   TEWidget::setAntialias( !has_noxft );
+  TEWidget::setStandalone( true );
 
+#ifdef COMPOSITE
+  char *display = 0;
+  if ( qtargs->isSet("display"))
+    display = qtargs->getOption( "display" ).data();
+
+  Display *dpy = XOpenDisplay( display );
+  if ( !dpy ) {
+    kdError() << "cannot connect to X server " << display << endl;
+    exit( 1 );
+  }
+
+  int screen = DefaultScreen( dpy );
+  Colormap colormap = 0;
+  Visual *visual = 0;
+  int event_base, error_base;
+
+  if ( args->isSet("argb") && XRenderQueryExtension( dpy, &event_base, &error_base ) )
+  {
+    int nvi;
+    XVisualInfo templ;
+    templ.screen  = screen;
+    templ.depth   = 32;
+    templ.c_class = TrueColor;
+    XVisualInfo *xvi = XGetVisualInfo( dpy, VisualScreenMask | VisualDepthMask
+		  | VisualClassMask, &templ, &nvi );
+
+    for ( int i = 0; i < nvi; i++ ) {
+      XRenderPictFormat *format = XRenderFindVisualFormat( dpy, xvi[i].visual );
+      if ( format->type == PictTypeDirect && format->direct.alphaMask ) {
+        visual = xvi[i].visual;
+        colormap = XCreateColormap( dpy, RootWindow( dpy, screen ), visual, AllocNone );
+        argb_visual = true;
+        break;
+      }
+    }
+  }
+
+  KApplication a( dpy, Qt::HANDLE( visual ), Qt::HANDLE( colormap ) );
+#else
   KApplication a;
+#endif
+
   KImageIO::registerFormats(); // add io for additional image formats
   //2.1 secs
 
@@ -275,7 +340,7 @@ extern "C" int kdemain(int argc, char* argv[])
   if(args->isSet("types")) {
     QStringList types = KGlobal::dirs()->findAllResources("appdata", "*.desktop", false, true);
     types.sort();
-    for(QStringList::ConstIterator it = types.begin(); 
+    for(QStringList::ConstIterator it = types.begin();
         it != types.end(); ++it)
     {
        QString file = *it;
@@ -286,9 +351,20 @@ extern "C" int kdemain(int argc, char* argv[])
     }
     return 0;
   }
-  
-  if(args->isSet("workdir"))
-     QDir::setCurrent( args->getOption("workdir") );
+  if(args->isSet("schemas")) {
+    ColorSchemaList colors;
+    colors.checkSchemas();
+    for(int i = 0; i < (int) colors.count(); i++)
+    {
+       ColorSchema *schema = colors.find(i);
+       QString relPath = schema->relPath();
+       if (!relPath.isEmpty())
+          printf("%s\n", QFile::encodeName(relPath).data());
+    }
+    return 0;
+  }
+
+  QString workDir = QFile::decodeName( args->getOption("workdir") );
 
   QString keytab = "";
   if (args->isSet("keytab"))
@@ -312,7 +388,7 @@ extern "C" int kdemain(int argc, char* argv[])
   {
      QStringList profiles = KGlobal::dirs()->findAllResources("data", "konsole/profiles/*", false, true);
      profiles.sort();
-     for(QStringList::ConstIterator it = profiles.begin(); 
+     for(QStringList::ConstIterator it = profiles.begin();
          it != profiles.end(); ++it)
      {
         QString file = *it;
@@ -448,13 +524,12 @@ extern "C" int kdemain(int argc, char* argv[])
         n++;
         m->activateSession( sessionconfig->readNumEntry("ActiveSession",0) );
         m->show();
-        m->run();
 	m->setAutoClose(auto_close);
     }
   }
   else
   {
-    Konsole*  m = new Konsole(wname,(shell ? QFile::decodeName(shell) : QString::null),eargs,histon,menubaron,tabbaron,frameon,scrollbaron,QString::null,title,type,term);
+    Konsole*  m = new Konsole(wname,(shell ? QFile::decodeName(shell) : QString::null),eargs,histon,menubaron,tabbaron,frameon,scrollbaron,QString::null,title,type,term, false, 0, workDir);
     m->enableFullScripting(full_script);
     m->enableFixedSize(fixed_size);
     //3.8 :-(
@@ -474,7 +549,6 @@ extern "C" int kdemain(int argc, char* argv[])
 
     m->initFullScreen();
     m->show();
-    m->run();
     if (showtip)
       m->showTipOnStart();
     m->setAutoClose(auto_close);
@@ -486,12 +560,23 @@ extern "C" int kdemain(int argc, char* argv[])
 
   // Delete all toplevel widgets that have WDestructiveClose
   QWidgetList *list = QApplication::topLevelWidgets();
+  // remove all toplevel widgets that have a parent (i.e. they
+  // got WTopLevel explicitly), they'll be deleted by the parent
+  list->first();
+  while( list->current())
+  {
+    if( list->current()->parentWidget() != NULL || !list->current()->testWFlags( Qt::WDestructiveClose ) )
+    {
+        list->remove();
+        continue;
+    }
+    list->next();
+  }
   QWidgetListIt it(*list);
   QWidget * w;
   while( (w=it.current()) != 0 ) {
      ++it;
-     if ( w->testWFlags( Qt::WDestructiveClose ) )
-          delete w;
+     delete w;
   }
   delete list;
 
