@@ -1,7 +1,7 @@
 /*
     KSysGuard, the KDE System Guard
    
-	Copyright (c) 2001 Tobias Koenig <tokoe82@yahoo.de>
+	Copyright (c) 2001 Tobias Koenig <tokoe@kde.org>
     
     This program is free software; you can redistribute it and/or
     modify it under the terms of version 2 of the GNU General Public
@@ -14,17 +14,18 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
 */
 
 #ifndef _netstat_h_
 #define _netstat_h_
 
-void initNetStat(void);
+void initNetStat(struct SensorModul* sm);
 void exitNetStat(void);
 
 int updateNetStat(void);
-int updateNetStatTcpUdpRaw(const char *cmd);
+int updateNetStatTcpUdpRaw(const char* cmd);
 int updateNetStatUnix(void);
 
 void printNetStat(const char* cmd);
