@@ -1,63 +1,81 @@
+/*
+ * Laptop KWin Decoration
+ *
+ * Port of this decoration to KDE 3.2, accessibility enhancement are
+ * Copyright (c) 2003 Luciano Montanaro <mikelima@cirulla.net>
+ */
 
 #include <kconfig.h> // up here to avoid X11 header conflict :P
 #include "laptopclient.h"
-#include <qabstractlayout.h>
 #include <qlayout.h>
 #include <qdrawutil.h>
 #include <kpixmapeffect.h>
 #include <kdrawutil.h>
 #include <kglobal.h>
-#include <kapp.h>
+#include <kapplication.h>
 #include <klocale.h>
 #include <qbitmap.h>
-#include "../../workspace.h"
-#include "../../options.h"
+#include <qtooltip.h>
+#include <qlabel.h>
 
-using namespace KWinInternal;
+namespace Laptop {
 
-
-
-static unsigned char iconify_bits[] = {
+static const unsigned char iconify_bits[] = {
     0xff, 0xff, 0x00, 0xff, 0xff, 0x7e, 0x3c, 0x18};
 
-static unsigned char close_bits[] = {
+static const unsigned char close_bits[] = {
     0x42, 0xe7, 0x7e, 0x3c, 0x3c, 0x7e, 0xe7, 0x42};
 
-static unsigned char maximize_bits[] = {
+static const unsigned char maximize_bits[] = {
     0x18, 0x3c, 0x7e, 0xff, 0xff, 0x00, 0xff, 0xff };
 
-static unsigned char minmax_bits[] = {
+static const unsigned char minmax_bits[] = {
     0x0c, 0x18, 0x33, 0x67, 0xcf, 0x9f, 0x3f, 0x3f};
 
-static unsigned char question_bits[] = {
+static const unsigned char question_bits[] = {
     0x3c, 0x66, 0x60, 0x30, 0x18, 0x00, 0x18, 0x18};
 
-static unsigned char unsticky_bits[] = {
+static const unsigned char unsticky_bits[] = {
    0x3c, 0x42, 0x99, 0xbd, 0xbd, 0x99, 0x42, 0x3c};
 
-static unsigned char sticky_bits[] = {
+static const unsigned char sticky_bits[] = {
    0x3c, 0x42, 0x81, 0x81, 0x81, 0x81, 0x42, 0x3c};
 
-static QPixmap *titlePix=0;
-static KPixmap *aUpperGradient=0;
-static KPixmap *iUpperGradient=0;
+static QPixmap *titlePix;
+static KPixmap *aUpperGradient;
+static KPixmap *iUpperGradient;
 // buttons active, inactive, up, down, and 2 sizes :P
-static KPixmap *btnPix1=0;
-static KPixmap *iBtnPix1=0;
-static KPixmap *btnDownPix1=0;
-static KPixmap *iBtnDownPix1=0;
-static KPixmap *btnPix2=0;
-static KPixmap *btnDownPix2=0;
-static KPixmap *iBtnPix2=0;
-static KPixmap *iBtnDownPix2=0;
+static KPixmap *btnPix1;
+static KPixmap *iBtnPix1;
+static KPixmap *btnDownPix1;
+static KPixmap *iBtnDownPix1;
+static KPixmap *btnPix2;
+static KPixmap *btnDownPix2;
+static KPixmap *iBtnPix2;
+static KPixmap *iBtnDownPix2;
 static QColor btnForeground;
 
-static bool pixmaps_created = false;
-
-static int titleHeight = 14; // configurable title height not implemented yet
+static int titleHeight = 14;
 static int btnWidth1 = 17;
 static int btnWidth2 = 27;
 
+static int handleSize = 8;	// the resize handle size in pixels
+
+static bool pixmaps_created = false;
+
+// =====================================
+
+extern "C" KDecorationFactory* create_factory()
+{
+    return new Laptop::LaptopClientFactory();
+}
+
+// =====================================
+
+static inline const KDecorationOptions* options()
+{
+    return KDecoration::options();
+}
 
 static void drawButtonFrame(KPixmap *pix, const QColorGroup &g, bool sunken)
 {
@@ -89,6 +107,14 @@ static void create_pixmaps()
         return;
     pixmaps_created = true;
 
+    titleHeight = QFontMetrics(options()->font(true)).height() + 2;
+    if (titleHeight < handleSize) titleHeight = handleSize;
+    titleHeight &= ~1; // Make title height even
+    if (titleHeight < 14) titleHeight = 14;
+
+    btnWidth1 = titleHeight + 3;
+    btnWidth2 = 3*titleHeight/2 + 6;
+
     // titlebar
     QPainter p;
     QPainter maskPainter;
@@ -102,10 +128,10 @@ static void create_pixmaps()
     maskPainter.setPen(Qt::color1);
     for(i=0, y=2; i < 3; ++i, y+=4){
         for(x=1; x <= 33; x+=3){
-            p.setPen(options->color(Options::TitleBar, true).light(150));
+            p.setPen(options()->color(KDecoration::ColorTitleBar, true).light(150));
             p.drawPoint(x, y);
             maskPainter.drawPoint(x, y);
-            p.setPen(options->color(Options::TitleBar, true).dark(150));
+            p.setPen(options()->color(KDecoration::ColorTitleBar, true).dark(150));
             p.drawPoint(x+1, y+1);
             maskPainter.drawPoint(x+1, y+1);
         }
@@ -119,19 +145,19 @@ static void create_pixmaps()
         aUpperGradient->resize(32, titleHeight+2);
         iUpperGradient = new KPixmap;
         iUpperGradient->resize(32, titleHeight+2);
-        QColor bgColor = options->color(Options::TitleBar, true);
+        QColor bgColor = options()->color(KDecoration::ColorTitleBar, true);
         KPixmapEffect::gradient(*aUpperGradient,
                                 bgColor.light(120),
                                 bgColor.dark(120),
                                 KPixmapEffect::VerticalGradient);
-        bgColor = options->color(Options::TitleBar, false);
+        bgColor = options()->color(KDecoration::ColorTitleBar, false);
         KPixmapEffect::gradient(*iUpperGradient,
                                 bgColor.light(120),
                                 bgColor.dark(120),
                                 KPixmapEffect::VerticalGradient);
     }
     // buttons (active/inactive, sunken/unsunken, 2 sizes each)
-    QColorGroup g = options->colorGroup(Options::ButtonBg, true);
+    QColorGroup g = options()->colorGroup(KDecoration::ColorButtonBg, true);
     QColor c = g.background();
     btnPix1 = new KPixmap;
     btnPix1->resize(btnWidth1, titleHeight);
@@ -158,7 +184,7 @@ static void create_pixmaps()
                                 KPixmapEffect::DiagonalGradient);
         KPixmapEffect::gradient(*btnDownPix2, c.dark(130), c.light(120),
                                 KPixmapEffect::DiagonalGradient);
-        g = options->colorGroup(Options::ButtonBg, false);
+        g = options()->colorGroup(KDecoration::ColorButtonBg, false);
         c = g.background();
         KPixmapEffect::gradient(*iBtnPix1, c.light(120), c.dark(130),
                                 KPixmapEffect::DiagonalGradient);
@@ -174,27 +200,27 @@ static void create_pixmaps()
         btnDownPix1->fill(c.rgb());
         btnPix2->fill(c.rgb());
         btnDownPix2->fill(c.rgb());
-        g = options->colorGroup(Options::ButtonBg, false);
+        g = options()->colorGroup(KDecoration::ColorButtonBg, false);
         c = g.background();
         iBtnPix1->fill(c.rgb());
         iBtnDownPix1->fill(c.rgb());
         iBtnPix2->fill(c.rgb());
         iBtnDownPix2->fill(c.rgb());
     }
-    g = options->colorGroup(Options::ButtonBg, true);
+    g = options()->colorGroup(KDecoration::ColorButtonBg, true);
     c = g.background();
     drawButtonFrame(btnPix1, g, false);
     drawButtonFrame(btnDownPix1, g, true);
     drawButtonFrame(btnPix2, g, false);
     drawButtonFrame(btnDownPix2, g, true);
-    g = options->colorGroup(Options::ButtonBg, false);
+    g = options()->colorGroup(KDecoration::ColorButtonBg, false);
     c = g.background();
     drawButtonFrame(iBtnPix1, g, false);
     drawButtonFrame(iBtnDownPix1, g, true);
     drawButtonFrame(iBtnPix2, g, false);
     drawButtonFrame(iBtnDownPix2, g, true);
 
-    if(qGray(options->color(Options::ButtonBg, true).rgb()) > 128)
+    if(qGray(options()->color(KDecoration::ColorButtonBg, true).rgb()) > 128)
         btnForeground = Qt::black;
     else
         btnForeground = Qt::white;
@@ -218,39 +244,45 @@ static void delete_pixmaps()
     pixmaps_created = false;
 }
 
+// =====================================
 
-LaptopClientButton::LaptopClientButton(int w, int h, Client *parent, 
-        const char *name, const unsigned char *bitmap, 
-        const QString& tip)
-    : KWinButton(parent, name, tip)
+LaptopButton::LaptopButton(int w, int h, LaptopClient *parent,
+        const char *name, const unsigned char *bitmap,
+        const QString& tip, const int realizeBtns)
+    : QButton(parent->widget(), name), client(parent)
 {
-    client = parent;
+    realizeButtons = realizeBtns;
+
+    setCursor( arrowCursor );
     defaultSize = QSize(w, h);
     setFixedHeight(h);
     resize(defaultSize);
     if(bitmap)
         setBitmap(bitmap);
+
     //setBackgroundMode(QWidget::NoBackground);
+
+    QToolTip::add(this, tip);
 }
 
-QSize LaptopClientButton::sizeHint() const
+QSize LaptopButton::sizeHint() const
 {
     return(defaultSize);
 }
 
-void LaptopClientButton::reset()
+void LaptopButton::reset()
 {
     repaint(false);
 }
 
-void LaptopClientButton::setBitmap(const unsigned char *bitmap)
+void LaptopButton::setBitmap(const unsigned char *bitmap)
 {
     deco = QBitmap(8, 8, bitmap, true);
     deco.setMask(deco);
     repaint();
 }
 
-void LaptopClientButton::drawButton(QPainter *p)
+void LaptopButton::drawButton(QPainter *p)
 {
     bool smallBtn = width() == btnWidth1;
     if(btnPix1){
@@ -268,7 +300,7 @@ void LaptopClientButton::drawButton(QPainter *p)
         }
     }
     else{
-        QColorGroup g = options->colorGroup(Options::ButtonBg,
+        QColorGroup g = options()->colorGroup(KDecoration::ColorButtonBg,
                                             client->isActive());
         int w = width();
         int h = height();
@@ -287,34 +319,48 @@ void LaptopClientButton::drawButton(QPainter *p)
     p->drawPixmap(isDown() ? xOff+1: xOff, isDown() ? yOff+1 : yOff, deco);
 }
 
-void LaptopClient::slotReset()
+// =====================================
+
+void LaptopClient::reset(unsigned long)
 {
-    int i;
-    for(i=0; i < 5; ++i){
-        if(button[i])
+    for (int i = 0; i < 5; ++i) {
+        if (button[i])
             button[i]->reset();
     }
-    repaint();
+    widget()->repaint();
 }
 
-LaptopClient::LaptopClient( Workspace *ws, WId w, QWidget *parent,
-                            const char *name )
-    : Client( ws, w, parent, name, WResizeNoErase | WNorthWestGravity )
+LaptopClient::LaptopClient(KDecorationBridge *b, KDecorationFactory *f)
+    : KDecoration(b, f)
 {
+}
+
+void LaptopClient::init()
+{
+    createMainWidget(WResizeNoErase | WStaticContents);
+    widget()->installEventFilter(this);
+
     lastButtonWidth = 0;
     lastBufferWidth = 0;
 
-    connect(options, SIGNAL(resetClients()), this, SLOT(slotReset()));
+    // XXX Check how to do this...
+    // connect(options(), SIGNAL(resetClients()), this, SLOT(slotReset()));
     bool help = providesContextHelp();
 
-
-    QGridLayout* g = new QGridLayout(this, 0, 0, 0);
+    g = new QGridLayout(widget(), 0, 0, 0);
     g->setResizeMode(QLayout::FreeResize);
     g->addRowSpacing(0, 3);
     g->addRowSpacing(2, 1);
-    g->addWidget(windowWrapper(), 3, 1);
+    if (isPreview())
+	g->addWidget(new QLabel(i18n("<center><b></b>Laptop preview</center>"),
+		    widget()), 3, 1);
+    else
+	g->addItem( new QSpacerItem( 0, 0 ), 3, 1); // no widget in the middle
+
     g->setRowStretch(3, 10);
-    g->addRowSpacing(4, 8); // bottom handles
+    spacer = new QSpacerItem(10, mustDrawHandle() ? handleSize : 4,
+			QSizePolicy::Expanding, QSizePolicy::Minimum);
+    g->addItem(spacer, 4, 1);
     g->addColSpacing(0, 4);
     g->addColSpacing(2, 4);
 
@@ -322,32 +368,32 @@ LaptopClient::LaptopClient( Workspace *ws, WId w, QWidget *parent,
     if ( isTool() )
 	th -= 2;
 
-    button[BtnClose] = new LaptopClientButton(27, th, this, "close", 
+    button[BtnClose] = new LaptopButton(btnWidth2, th, this, "close",
                                  close_bits, i18n("Close"));
-    button[BtnSticky] = new LaptopClientButton(17, th, this, "sticky",
-                                 NULL, i18n("Sticky"));
-    if(isSticky())
+    button[BtnSticky] = new LaptopButton(btnWidth1, th, this, "sticky",
+                                 NULL, isOnAllDesktops()?i18n("Not on all desktops"):i18n("On all desktops"));
+    if(isOnAllDesktops())
         button[BtnSticky]->setBitmap(unsticky_bits);
     else
         button[BtnSticky]->setBitmap(sticky_bits);
-    button[BtnIconify] = new LaptopClientButton(27, th, this, "iconify",
+    button[BtnIconify] = new LaptopButton(btnWidth2, th, this, "iconify",
                                           iconify_bits, i18n("Minimize"));
-    button[BtnMax] = new LaptopClientButton(27, th, this, "maximize",
-                                      maximize_bits, i18n("Maximize"));
-    if(help){
-        button[BtnHelp] = new LaptopClientButton(17, th, this, "help",
+    button[BtnMax] = new LaptopButton(btnWidth2, th, this, "maximize",
+                                      maximize_bits, i18n("Maximize"), LeftButton|MidButton|RightButton);
+    if (help) {
+        button[BtnHelp] = new LaptopButton(btnWidth1, th, this, "help",
                                      question_bits, i18n("Help"));
-        connect(button[BtnHelp], SIGNAL( clicked() ), this, ( SLOT( contextHelp() ) ) );
+        connect(button[BtnHelp], SIGNAL( clicked() ), this, SLOT( showContextHelp() ) );
     }
     else
         button[BtnHelp] = NULL;
 
-    connect( button[BtnClose], SIGNAL( clicked() ), this, ( SLOT( closeWindow() ) ) );
-    connect( button[BtnSticky], SIGNAL( clicked() ), this, ( SLOT( toggleSticky() ) ) );
-    connect( button[BtnIconify], SIGNAL( clicked() ), this, ( SLOT( iconify() ) ) );
-    connect( button[BtnMax], SIGNAL( clicked() ), this, ( SLOT( slotMaximize() ) ) );
+    connect( button[BtnClose], SIGNAL( clicked() ), this, SLOT( closeWindow() ) );
+    connect( button[BtnSticky], SIGNAL( clicked() ), this, SLOT( toggleOnAllDesktops() ) );
+    connect( button[BtnIconify], SIGNAL( clicked() ), this, SLOT( minimize() ) );
+    connect( button[BtnMax], SIGNAL( clicked() ), this, SLOT( slotMaximize() ) );
 
-    hb = new QHBoxLayout();
+    hb = new QBoxLayout(0, QBoxLayout::LeftToRight, 0, 0, 0);
     hb->setResizeMode(QLayout::FreeResize);
     g->addLayout( hb, 1, 1 );
     hb->addWidget( button[BtnClose]);
@@ -356,8 +402,8 @@ LaptopClient::LaptopClient( Workspace *ws, WId w, QWidget *parent,
                                QSizePolicy::Minimum);
     hb->addItem(titlebar);
     hb->addSpacing(1);
-    if(help){
-        hb->addWidget( button[BtnHelp]);
+    if (help) {
+        hb->addWidget(button[BtnHelp]);
     }
     hb->addWidget( button[BtnSticky]);
     hb->addWidget( button[BtnIconify]);
@@ -369,6 +415,8 @@ LaptopClient::LaptopClient( Workspace *ws, WId w, QWidget *parent,
 	button[BtnIconify]->hide();
     if ( !isMaximizable() )
 	button[BtnMax]->hide();
+    if ( !isCloseable() )
+        button[BtnClose]->hide();
 
     hiddenItems = false;
     bufferDirty = true;
@@ -376,51 +424,46 @@ LaptopClient::LaptopClient( Workspace *ws, WId w, QWidget *parent,
 
 void LaptopClient::slotMaximize()
 {
-    if ( button[BtnMax]->last_button == MidButton )
-	maximize( MaximizeVertical );
-    else if ( button[BtnMax]->last_button == RightButton )
-	maximize( MaximizeHorizontal );
-    else
-	maximize();
+    maximize(button[BtnMax]->last_button);
 }
 
-void LaptopClient::resizeEvent( QResizeEvent* e)
+void LaptopClient::resizeEvent(QResizeEvent* e)
 {
-    Client::resizeEvent( e );
-
     doShape();
     calcHiddenButtons();
-    if ( isVisibleToTLW() ) {
+    if ( widget()->isVisibleToTLW() ) {
 	int dx = 0;
 	int dy = 0;
 	if ( e->oldSize().width() != width() )
 	    dx = 32 + QABS( e->oldSize().width() -  width() );
 	if ( e->oldSize().height() != height() )
-	    dy = 8 + QABS( e->oldSize().height() -  height() );
+	    dy = mustDrawHandle() ? handleSize : 4 +
+		QABS( e->oldSize().height() -  height() );
 	if ( dy )
-	    update( 0, height() - dy + 1, width(), dy );
+	    widget()->update( 0, height() - dy + 1, width(), dy );
 	if ( dx ) {
-	    update( width() - dx + 1, 0, dx, height() );
-	    update( QRect( QPoint(4,4), titlebar->geometry().bottomLeft() - QPoint(1,0) ) );
-	    update( QRect( titlebar->geometry().topRight(), QPoint( width() - 4, titlebar->geometry().bottom() ) ) );
-	    // titlebar needs no background
-	    QApplication::postEvent( this, new QPaintEvent( titlebar->geometry(), FALSE ) );
+	    widget()->update( width() - dx + 1, 0, dx, height() );
+	    widget()->update( QRect( QPoint(4,4),
+			titlebar->geometry().bottomLeft() - QPoint(1,0) ) );
+	    widget()->update( QRect( titlebar->geometry().topRight(),
+			QPoint( width() - 4, titlebar->geometry().bottom() ) ) );
+	    widget()->update(titlebar->geometry());
 	}
     }
 }
 
-void LaptopClient::captionChange( const QString& )
+void LaptopClient::captionChange()
 {
     bufferDirty = true;
-    repaint( titlebar->geometry(), false );
+    widget()->repaint(titlebar->geometry(), false);
 }
 
 void LaptopClient::paintEvent( QPaintEvent* )
 {
-    QPainter p(this);
-    QColorGroup g = options->colorGroup(Options::Frame, isActive());
+    QPainter p(widget());
+    QColorGroup g = options()->colorGroup(KDecoration::ColorFrame, isActive());
 
-    QRect r(rect());
+    QRect r(widget()->rect());
     p.setPen(Qt::black);
     p.drawRect(r);
     // outer frame
@@ -432,29 +475,37 @@ void LaptopClient::paintEvent( QPaintEvent* )
     p.drawLine(r.x()+1, r.bottom()-1, r.right()-1, r.bottom()-1);
 
     int th = titleHeight;
+    int bb = handleSize + 2; // Bottom border
+    int bs = handleSize - 2; // inner size of bottom border
+    if (!mustDrawHandle()) {
+	bb = 6;
+	bs = 0;
+    }
     if ( isTool() )
 	th -= 2;
 
     // inner rect
-    p.drawRect(r.x()+3, r.y()+th+3, r.width()-6,
-               r.height()-th-10);
-    // handles
-    if(r.width() > 44){
-        qDrawShadePanel(&p, r.x()+1, r.bottom()-6, 20,
-                        6, g, false, 1, &g.brush(QColorGroup::Mid));
-        qDrawShadePanel(&p, r.x()+21, r.bottom()-6, r.width()-42, 6,
-                        g, false, 1, isActive() ?
-                        &g.brush(QColorGroup::Background) :
-                        &g.brush(QColorGroup::Mid));
-        qDrawShadePanel(&p, r.right()-20, r.bottom()-6, 20, 6,
-                        g, false, 1, &g.brush(QColorGroup::Mid));
-    }
-    else
-        qDrawShadePanel(&p, r.x()+1, r.bottom()-6, r.width()-2, 6, g,
-                        false, 1, isActive() ?
-                        &g.brush(QColorGroup::Background) :
-                        &g.brush(QColorGroup::Mid));
+    p.drawRect(r.x() + 3, r.y() + th + 3, r.width() - 6, r.height() - th - bb);
 
+    // handles
+    if (mustDrawHandle()) {
+	if (r.width() > 3*handleSize + 20) {
+	    int range = 8 + 3*handleSize/2;
+	    qDrawShadePanel(&p, r.x() + 1, r.bottom() - bs, range, 
+		    handleSize - 2, g, false, 1, &g.brush(QColorGroup::Mid));
+	    qDrawShadePanel(&p, r.x() + range + 1, r.bottom() - bs,
+		    r.width() - 2*range - 2, handleSize - 2, g, false, 1,
+		    isActive() ? &g.brush(QColorGroup::Background) :
+				 &g.brush(QColorGroup::Mid));
+	    qDrawShadePanel(&p, r.right() - range, r.bottom() - bs,
+		    range, bs, g, false, 1, &g.brush(QColorGroup::Mid));
+	} else {
+	    qDrawShadePanel(&p, r.x() + 1, r.bottom() - bs,
+		    r.width() - 2, bs, g, false, 1,
+		    isActive() ?  &g.brush(QColorGroup::Background) :
+				  &g.brush(QColorGroup::Mid));
+	}
+    }
     r = titlebar->geometry();
     r.setRight(r.right()-1);
 
@@ -468,11 +519,11 @@ void LaptopClient::paintEvent( QPaintEvent* )
                               *iUpperGradient);
         else
             p.fillRect(r.x(), r.y(), r.width(), r.height()-1,
-                       options->color(Options::TitleBar, false));
+                       options()->color(KDecoration::ColorTitleBar, false));
 
-        p.setFont(options->font(false, isTool() ));
-        QFontMetrics fm(options->font(false));
-        g = options->colorGroup(Options::TitleBar, false);
+        p.setFont(options()->font(false, isTool() ));
+        QFontMetrics fm(options()->font(false));
+        g = options()->colorGroup(KDecoration::ColorTitleBar, false);
         if(iUpperGradient)
             p.drawTiledPixmap(r.x()+((r.width()-fm.width(caption()))/2)-4,
                               r.y(), fm.width(caption())+8, r.height()-1,
@@ -487,10 +538,10 @@ void LaptopClient::paintEvent( QPaintEvent* )
         p.setPen(g.button());
         p.drawLine(r.right(), r.y(), r.right(), r.bottom());
         p.drawLine(r.x(), r.bottom(), r.right(), r.bottom());
-        p.setPen(options->color(Options::Font, false));
+        p.setPen(options()->color(KDecoration::ColorFont, false));
         p.drawText(r.x(), r.y(), r.width(), r.height()-1,
                    AlignCenter, caption() );
-        g = options->colorGroup(Options::Frame, true);
+        g = options()->colorGroup(KDecoration::ColorFrame, true);
         p.setPen(g.background());
         p.drawPoint(r.x(), r.y());
         p.drawPoint(r.right(), r.y());
@@ -511,45 +562,58 @@ void LaptopClient::doShape()
     setMask(mask);
 }
 
-void LaptopClient::showEvent(QShowEvent *ev)
-{
-    Client::showEvent(ev);
-    doShape();
-    repaint();
-}
-
-void LaptopClient::windowWrapperShowEvent( QShowEvent* )
+void LaptopClient::showEvent(QShowEvent *)
 {
     doShape();
+    widget()->repaint();
 }
 
 void LaptopClient::mouseDoubleClickEvent( QMouseEvent * e )
 {
     if (titlebar->geometry().contains( e->pos() ) )
-	workspace()->performWindowOperation( this, options->operationTitlebarDblClick() );
-    workspace()->requestFocus( this );
+        titlebarDblClickOperation();
 }
 
-void LaptopClient::stickyChange(bool on)
+bool LaptopClient::mustDrawHandle() const 
+{ 
+    bool drawSmallBorders = !options()->moveResizeMaximizedWindows();
+    if (drawSmallBorders && (maximizeMode() & MaximizeVertical)) {
+	return false;
+    } else {
+	return isResizable();
+    }
+}
+
+void LaptopClient::iconChange()
 {
+    // There is no icon support in this theme
+}
+
+void LaptopClient::desktopChange()
+{
+    bool on = isOnAllDesktops();
     button[BtnSticky]->setBitmap(on ? unsticky_bits : sticky_bits);
-    button[BtnSticky]->setTipText(on ? i18n("Un-Sticky") : i18n("Sticky"));
+    QToolTip::remove(button[BtnSticky]);
+    QToolTip::add(button[BtnSticky],
+	    on ? i18n("Not on all desktops") : i18n("On all desktops"));
 }
 
-void LaptopClient::maximizeChange(bool m)
+void LaptopClient::maximizeChange()
 {
+    bool m = (maximizeMode() == MaximizeFull);
     button[BtnMax]->setBitmap(m ? minmax_bits : maximize_bits);
-    button[BtnMax]->setTipText(m ? i18n("Restore") : i18n("Maximize"));
+    QToolTip::remove(button[BtnMax]);
+    QToolTip::add(button[BtnMax], m ? i18n("Restore") : i18n("Maximize"));
+    spacer->changeSize(10, mustDrawHandle() ? handleSize : 4,
+	    QSizePolicy::Expanding, QSizePolicy::Minimum);
+    g->activate();
+    doShape();
+    widget()->repaint(false);
 }
 
-void LaptopClient::init()
+void LaptopClient::activeChange()
 {
-    //
-}
-
-void LaptopClient::activeChange(bool)
-{
-    repaint(false);
+    widget()->repaint(false);
     int i;
     for(i=0; i < 5; ++i){
         if(button[i])
@@ -591,10 +655,10 @@ void LaptopClient::calcHiddenButtons()
                 if(button[i]->sizeHint().width() + totalSize <= width()){
                     totalSize+=button[i]->sizeHint().width();
                     if(button[i]->isHidden() &&
-		       ( !isTransient() || !isTransient() || i != BtnSticky ) &&
+		       ( !isTransient() || i != BtnSticky ) &&
 		       ( isMinimizable() || i != BtnIconify ) &&
 		       ( isMaximizable() || ( i != BtnIconify && i != BtnSticky && i != BtnMax ) )
-		
+
 		       ) {
                         button[i]->resize(button[i]->sizeHint());
                         button[i]->show();
@@ -629,14 +693,14 @@ void LaptopClient::updateActiveBuffer( )
         p.drawTiledPixmap(r, *aUpperGradient);
     }
     else{
-        p.fillRect(r, options->color(Options::TitleBar, true));
+        p.fillRect(r, options()->color(KDecoration::ColorTitleBar, true));
     }
     if(titlePix)
         p.drawTiledPixmap(r, *titlePix);
 
-    p.setFont(options->font(true, isTool() ));
-    QFontMetrics fm(options->font(true));
-    QColorGroup g = options->colorGroup(Options::TitleBar, true);
+    p.setFont(options()->font(true, isTool() ));
+    QFontMetrics fm(options()->font(true));
+    QColorGroup g = options()->colorGroup(KDecoration::ColorTitleBar, true);
     if(aUpperGradient)
         p.drawTiledPixmap(r.x()+((r.width()-fm.width(caption()))/2)-4,
                           r.y(), fm.width(caption())+8, r.height()-1,
@@ -651,10 +715,10 @@ void LaptopClient::updateActiveBuffer( )
     p.setPen(g.button());
     p.drawLine(r.right(), r.y(), r.right(), r.bottom());
     p.drawLine(r.x(), r.bottom(), r.right(), r.bottom());
-    p.setPen(options->color(Options::Font, true));
+    p.setPen(options()->color(KDecoration::ColorFont, true));
     p.drawText(r.x(), r.y(), r.width(), r.height()-1,
                AlignCenter, caption() );
-    g = options->colorGroup(Options::Frame, true);
+    g = options()->colorGroup(KDecoration::ColorFrame, true);
     p.setPen(g.background());
     p.drawPoint(r.x(), r.y());
     p.drawPoint(r.right(), r.y());
@@ -662,47 +726,156 @@ void LaptopClient::updateActiveBuffer( )
     p.end();
 }
 
-  Client::MousePosition
-LaptopClient::mousePosition( const QPoint& p ) const
+LaptopClient::Position LaptopClient::mousePosition(const QPoint & p) const
 {
-  MousePosition m = Nowhere;
+  Position m = PositionCenter;
+  int range = 8 + 3*handleSize/2;
 
-  if (p.y() < (height() - 7))
-    m = Client::mousePosition(p);
+  if (p.y() < (height() - handleSize + 1))
+    m = KDecoration::mousePosition(p);
 
   else {
-    if (p.x() >= (width() - 20))
-      m = BottomRight;
-    else if (p.x() <= 20)
-      m = BottomLeft;
+    if (p.x() >= (width() - range))
+      m = PositionBottomRight;
+    else if (p.x() <= range)
+      m = PositionBottomLeft;
     else
-      m = Bottom;
+      m = PositionBottom;
   }
 
   return m;
 }
 
-extern "C"
+void LaptopClient::borders(int &left, int &right, int &top, int &bottom) const
 {
-    Client *allocate(Workspace *ws, WId w, int)
-    {
-        return(new LaptopClient(ws, w));
+    left = right = 4;
+    top = titleHeight + 4;
+    bottom = mustDrawHandle() ? handleSize : 4;
+}
+
+void LaptopClient::shadeChange()
+{
+}
+
+QSize LaptopClient::minimumSize() const
+{
+    return QSize(4 * handleSize, handleSize);
+}
+
+void LaptopClient::resize(const QSize& s)
+{
+    widget()->resize(s);
+    widget()->repaint(); //there is some strange wrong repaint of the frame without
+}
+
+static const int SUPPORTED_WINDOW_TYPES_MASK = NET::NormalMask |
+    NET::DesktopMask | NET::DockMask | NET::ToolbarMask | NET::MenuMask |
+    NET::DialogMask | NET::OverrideMask | NET::TopMenuMask |
+    NET::UtilityMask | NET::SplashMask;
+
+bool LaptopClient::isTransient() const
+{
+    NET::WindowType type = windowType(SUPPORTED_WINDOW_TYPES_MASK);
+    return type == NET::Dialog;
+}
+
+bool LaptopClient::isTool() const
+{
+    NET::WindowType type = windowType(SUPPORTED_WINDOW_TYPES_MASK);
+    return type == NET::Toolbar || type == NET::Utility || type == NET::Menu;
+}
+
+bool LaptopClient::eventFilter(QObject *o, QEvent *e)
+{
+    if (o != widget())
+	return false;
+    switch (e->type()) {
+    case QEvent::Resize:
+	resizeEvent(static_cast< QResizeEvent* >( e ));
+	return true;
+    case QEvent::Paint:
+	paintEvent(static_cast< QPaintEvent* >( e ));
+	return true;
+    case QEvent::MouseButtonDblClick:
+	mouseDoubleClickEvent(static_cast< QMouseEvent* >( e ));
+	return true;
+    case QEvent::MouseButtonPress:
+	processMousePressEvent(static_cast< QMouseEvent* >( e ));
+	return true;
+    case QEvent::Show:
+	showEvent(static_cast< QShowEvent* >( e ));
+	return true;
+    default:
+	break;
     }
-    void init()
-    {
-       create_pixmaps();
-    }
-    void reset()
-    {
-       delete_pixmaps();
-       create_pixmaps();
-       // Ensure change in tooltip state gets applied
-       Workspace::self()->slotResetAllClientsDelayed();
-    }
-    void deinit()
-    {
-       delete_pixmaps();
+    return false;
+}
+
+// =====================================
+
+LaptopClientFactory::LaptopClientFactory()
+{
+    create_pixmaps();
+}
+
+LaptopClientFactory::~LaptopClientFactory()
+{
+    delete_pixmaps();
+}
+
+KDecoration *LaptopClientFactory::createDecoration(KDecorationBridge *b)
+{
+    findPreferredHandleSize();
+    return new Laptop::LaptopClient(b, this);
+}
+
+bool LaptopClientFactory::reset(unsigned long /*changed*/)
+{
+    findPreferredHandleSize();
+
+    // TODO Do not recreate decorations if it is not needed. Look at
+    // ModernSystem for how to do that
+    Laptop::delete_pixmaps();
+    Laptop::create_pixmaps();
+    // For now just return true.
+    return true;
+}
+
+QValueList< LaptopClientFactory::BorderSize >
+LaptopClientFactory::borderSizes() const
+{
+    // the list must be sorted
+    return QValueList< BorderSize >() << BorderNormal << BorderLarge <<
+	BorderVeryLarge <<  BorderHuge << BorderVeryHuge << BorderOversized;
+}
+
+void LaptopClientFactory::findPreferredHandleSize()
+{
+    switch (options()->preferredBorderSize(this)) {
+    case KDecoration::BorderLarge:
+	handleSize = 11;
+	break;
+    case KDecoration::BorderVeryLarge:
+	handleSize = 16;
+	break;
+    case KDecoration::BorderHuge:
+	handleSize = 24;
+	break;
+    case KDecoration::BorderVeryHuge:
+	handleSize = 32;
+	break;
+    case KDecoration::BorderOversized:
+	handleSize = 40;
+	break;
+    case KDecoration::BorderTiny:
+    case KDecoration::BorderNormal:
+    default:
+	handleSize = 8;
     }
 }
 
+} // Laptop namespace
+
 #include "laptopclient.moc"
+
+// vim: sw=4
