@@ -20,6 +20,7 @@ License. See the file "COPYING" for the exact licensing terms.
 #include "atoms.h"
 #include "tabbox.h"
 #include "group.h"
+#include "rules.h"
 
 #include <qwhatsthis.h>
 #include <kkeynative.h>
@@ -65,7 +66,7 @@ void WinInfo::changeState( unsigned long state, unsigned long mask )
         m_client->setMaximize( m_client->maximizeMode() & Client::MaximizeVertical, state & NET::MaxHoriz );
 
     if ( mask & NET::Shaded )
-        m_client->setShade( state & NET::Shaded ? Client::ShadeNormal : Client::ShadeNone );
+        m_client->setShade( state & NET::Shaded ? ShadeNormal : ShadeNone );
     if ( mask & NET::KeepAbove)
         m_client->setKeepAbove( (state & NET::KeepAbove) != 0 );
     if ( mask & NET::KeepBelow)
@@ -89,7 +90,7 @@ void WinInfo::changeState( unsigned long state, unsigned long mask )
 // ****************************************
 
 RootInfo::RootInfo( Workspace* ws, Display *dpy, Window w, const char *name, unsigned long pr[], int pr_num, int scr )
-    : NETRootInfo2( dpy, w, name, pr, pr_num, scr )
+    : NETRootInfo3( dpy, w, name, pr, pr_num, scr )
     {
     workspace = ws;
     }
@@ -110,10 +111,10 @@ void RootInfo::changeActiveWindow( Window w, NET::RequestSource src, Time timest
         {
         if( timestamp == CurrentTime )
             timestamp = c->userTime();
-        if( src == NET::FromUnknown )
-            src = NET::FromTool; // KWIN_FOCUS, use qt_x_time as timestamp?
+        if( src != NET::FromApplication && src != FromTool )
+            src = NET::FromTool;
         if( src == NET::FromTool )
-            workspace->activateClient( c );
+            workspace->activateClient( c, true ); // force
         else // NET::FromApplication
             {
             Client* c2;
@@ -131,6 +132,24 @@ void RootInfo::changeActiveWindow( Window w, NET::RequestSource src, Time timest
         }
     }
 
+void RootInfo::restackWindow( Window w, RequestSource src, Window above, int detail, Time timestamp )
+    {
+    if( Client* c = workspace->findClient( WindowMatchPredicate( w )))
+        {
+        if( timestamp == CurrentTime )
+            timestamp = c->userTime();
+        if( src != NET::FromApplication && src != FromTool )
+            src = NET::FromTool;
+        c->restackWindow( above, detail, src, timestamp, true );
+        }
+    }
+
+void RootInfo::gotTakeActivity( Window w, Time timestamp, long flags )
+    {
+    if( Client* c = workspace->findClient( WindowMatchPredicate( w )))
+        workspace->handleTakeActivity( c, timestamp, flags );
+    }
+
 void RootInfo::closeWindow(Window w)
     {
     Client* c = workspace->findClient( WindowMatchPredicate( w ));
@@ -161,12 +180,6 @@ void RootInfo::gotPing( Window w, Time timestamp )
         c->gotPing( timestamp );
     }
 
-void RootInfo::restackWindow( Window w, Window above, int detail )
-    {
-    if( Client* c = workspace->findClient( WindowMatchPredicate( w )))
-        c->restackWindow( above, detail, NET::FromTool, true );
-    }
-
 // ****************************************
 // Workspace
 // ****************************************
@@ -414,16 +427,20 @@ bool Workspace::workspaceEvent( XEvent * e )
                 return FALSE;
             break;
         case FocusIn:
-            if( e->xfocus.window == rootWin() && e->xfocus.detail == NotifyDetailNone )
+            if( e->xfocus.window == rootWin()
+                && ( e->xfocus.detail == NotifyDetailNone || e->xfocus.detail == NotifyPointerRoot ))
                 {
                 updateXTime(); // focusToNull() uses qt_x_time, which is old now (FocusIn has no timestamp)
                 Window focus;
                 int revert;
                 XGetInputFocus( qt_xdisplay(), &focus, &revert );
-                if( focus == None )
+                if( focus == None || focus == PointerRoot )
                     {
-                    kdWarning( 1212 ) << "X focus set to None, reseting focus" << endl;
-                    focusToNull();
+                    //kdWarning( 1212 ) << "X focus set to None/PointerRoot, reseting focus" << endl;
+                    if( mostRecentlyActivatedClient() != NULL )
+                        requestFocus( mostRecentlyActivatedClient(), true );
+                    else
+                        focusToNull();
                     }
                 }
             // fall through
@@ -501,7 +518,8 @@ bool Client::windowEvent( XEvent* e )
             fetchName();
         if ( ( dirty[ WinInfo::PROTOCOLS ] & NET::WMIconName ) != 0 )
             fetchIconicName();
-        if ( ( dirty[ WinInfo::PROTOCOLS ] & NET::WMStrut ) != 0 )
+        if ( ( dirty[ WinInfo::PROTOCOLS ] & NET::WMStrut ) != 0
+            || ( dirty[ WinInfo::PROTOCOLS2 ] & NET::WM2ExtendedStrut ) != 0 )
             {
             if( isTopMenu())  // the fallback mode of KMenuBar may alter the strut
                 checkWorkspacePosition();  // restore it
@@ -767,8 +785,12 @@ void Client::configureRequestEvent( XConfigureRequestEvent* e )
     if ( isResize() || isMove())
         return; // we have better things to do right now
 
-    if( isFullScreen() // refuse resizing of fullscreen windows
-        || isSplash() // no manipulations with splashscreens either
+    if( fullscreen_mode == FullScreenNormal ) // refuse resizing of fullscreen windows
+        { // but allow resizing fullscreen hacks in order to let them cancel fullscreen mode
+        sendSyntheticConfigureNotify();
+        return;
+        }
+    if( isSplash() // no manipulations with splashscreens either
         || isTopMenu()) // topmenus neither
         {
         sendSyntheticConfigureNotify();
@@ -787,15 +809,16 @@ void Client::configureRequestEvent( XConfigureRequestEvent* e )
         }
 
     if( e->value_mask & ( CWX | CWY | CWHeight | CWWidth ))
-        configureRequest( e->value_mask, e->x, e->y, e->width, e->height );
+        configureRequest( e->value_mask, e->x, e->y, e->width, e->height, 0, false );
 
     if ( e->value_mask & CWStackMode )
-        restackWindow( e->above, e->detail, NET::FromApplication );
+        restackWindow( e->above, e->detail, NET::FromApplication, userTime(), false );
 
     // TODO sending a synthetic configure notify always is fine, even in cases where
     // the ICCCM doesn't require this - it can be though of as 'the WM decided to move
-    // the window later'. Perhaps those unnecessary ones could be saved though.
-    // See also Client::setGeometry()/plainResize()/move().
+    // the window later'. The client should not cause that many configure request,
+    // so this should not have any significant impact. With user moving/resizing
+    // the it should be optimized though (see also Client::setGeometry()/plainResize()/move()).
     sendSyntheticConfigureNotify();
 
     // SELI TODO accept configure requests for isDesktop windows (because kdesktop
@@ -834,7 +857,9 @@ void Client::propertyNotifyEvent( XPropertyEvent* e )
             else if (e->atom == atoms->wm_client_leader )
                 getWmClientLeader();
             else if( e->atom == qt_window_role )
-                window_role = getStringProperty( window(), qt_window_role );
+                window_role = staticWindowRole( window());
+            else if( e->atom == atoms->motif_wm_hints )
+                getMotifHints();
             break;
         }
     }
@@ -872,8 +897,11 @@ void Client::enterNotifyEvent( XCrossingEvent* e )
 
         if ( options->focusPolicy !=  Options::FocusStrictlyUnderMouse && ( isDesktop() || isDock() || isTopMenu() ) )
             return;
+        if ( options->delayFocus )
+            workspace()->requestDelayFocus( this );
+        else
+            workspace()->requestFocus( this );
 
-        workspace()->requestFocus( this );
         return;
         }
     }
@@ -909,6 +937,7 @@ void Client::leaveNotifyEvent( XCrossingEvent* e )
         if ( lostMouse ) 
             {
             cancelAutoRaise();
+            workspace()->cancelDelayFocus();
             delete shadeHoverTimer;
             shadeHoverTimer = 0;
             if ( shade_mode == ShadeHover && !moveResizeMode && !buttonDown )
@@ -966,8 +995,8 @@ void Client::ungrabButton( int modifier )
   (Motif, AWT, Tk, ...)
  */
 void Client::updateMouseGrab()
-    {
-    if( isActive() )
+    {                   // see Workspace::establishTabBoxGrab()
+    if( isActive() && !workspace()->forcedGlobalMouseGrab())
         {
         // remove the grab for no modifiers only if the window
         // is unobscured or if the user doesn't want click raise
@@ -1076,7 +1105,7 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
         uint keyModX = (options->keyCmdAllModKey() == Qt::Key_Meta) ?
             KKeyNative::modX(KKey::WIN) :
             KKeyNative::modX(KKey::ALT);
-        bool bModKeyHeld = ( state & KKeyNative::accelModMaskX()) == keyModX;
+        bool bModKeyHeld = keyModX != 0 && ( state & KKeyNative::accelModMaskX()) == keyModX;
 
         if( isSplash()
             && button == Button1 && !bModKeyHeld )
@@ -1087,15 +1116,9 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
             return true;
             }
 
-        if ( isActive() && w == wrapperId()
-             && ( options->clickRaise && !bModKeyHeld ) ) 
-            {
-            if ( button < 4 ) // exclude wheel
-                autoRaise();
-            }
-
         Options::MouseCommand com = Options::MouseNothing;
         bool was_action = false;
+        bool perform_handled = false;
         if ( bModKeyHeld )
             {
             was_action = true;
@@ -1117,6 +1140,7 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
             if( !isActive() && w == wrapperId())
                 {
                 was_action = true;
+                perform_handled = true;
                 switch (button) 
                     {
                     case Button1:
@@ -1132,10 +1156,18 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
                         com = Options::MouseActivateAndPassClick;
                     }
                 }
+            // active inner window
+            if( isActive() && w == wrapperId()
+                && options->clickRaise && button < 4 ) // exclude wheel
+                {
+                com = Options::MouseActivateRaiseAndPassClick;
+                was_action = true;
+                perform_handled = true;
+                }
             }
         if( was_action )
             {
-            bool replay = performMouseCommand( com, QPoint( x_root, y_root) );
+            bool replay = performMouseCommand( com, QPoint( x_root, y_root), perform_handled );
 
             if ( isSpecialWindow() && !isOverride())
                 replay = TRUE;
