@@ -29,7 +29,7 @@ License. See the file "COPYING" for the exact licensing terms.
  the clients were created.
 
  Every window has one layer assigned in which it is. There are 6 layers,
- from bottom : DesktopLayer, BelowLayer, NormalLayer, DockLayer, AboveLayer
+ from bottom : DesktopLayer, BelowLayer, NormalLayer, AboveLayer, DockLayer
  and ActiveLayer (see also NETWM sect.7.10.). The layer a window is in depends
  on the window type, and on other things like whether the window is active.
 
@@ -64,13 +64,17 @@ License. See the file "COPYING" for the exact licensing terms.
 
 #include <assert.h>
 
+#include <kdebug.h>
+
 #include "utils.h"
 #include "client.h"
 #include "workspace.h"
 #include "tabbox.h"
 #include "popupinfo.h"
 #include "group.h"
-#include <kdebug.h>
+#include "rules.h"
+
+extern Time qt_x_time;
 
 namespace KWinInternal
 {
@@ -141,11 +145,11 @@ void Workspace::propagateClients( bool propagate_new_clients )
          --it )
         {
         new_stack[ pos++ ] = (*it)->frameId();
-        if( (*it)->isTopMenu())
+        if( (*it)->belongsToLayer() >= DockLayer )
             topmenu_space_pos = pos;
         }
     if( topmenu_space != NULL )
-        { // make sure the topmenu space is below all topmenus, if there are any
+        { // make sure the topmenu space is below all topmenus, fullscreens, etc.
         for( int i = pos;
              i > topmenu_space_pos;
              --i )
@@ -268,6 +272,8 @@ void Workspace::lowerClient( Client* c )
     {
     if ( !c )
         return;
+    if( c->isTopMenu())
+        return;
 
     c->cancelAutoRaise();
 
@@ -293,6 +299,8 @@ void Workspace::lowerClientWithinApplication( Client* c )
     {
     if ( !c )
         return;
+    if( c->isTopMenu())
+        return;
 
     c->cancelAutoRaise();
 
@@ -319,6 +327,8 @@ void Workspace::raiseClient( Client* c )
     {
     if ( !c )
         return;
+    if( c->isTopMenu())
+        return;
 
     c->cancelAutoRaise();
 
@@ -337,13 +347,18 @@ void Workspace::raiseClient( Client* c )
     unconstrained_stacking_order.append( c );
 
     if( !c->isSpecialWindow())
+        {
         most_recently_raised = c;
+        pending_take_activity = NULL;
+        }
     }
 
 void Workspace::raiseClientWithinApplication( Client* c )
     {
     if ( !c )
         return;
+    if( c->isTopMenu())
+        return;
 
     c->cancelAutoRaise();
 
@@ -367,9 +382,9 @@ void Workspace::raiseClientWithinApplication( Client* c )
         }
     }
 
-void Workspace::raiseClientRequest( Client* c )
+void Workspace::raiseClientRequest( Client* c, NET::RequestSource src, Time timestamp )
     {
-    if( allowFullClientRaising( c ))
+    if( src == NET::FromTool || allowFullClientRaising( c, timestamp ))
         raiseClient( c );
     else
         {
@@ -378,20 +393,22 @@ void Workspace::raiseClientRequest( Client* c )
         }
     }
 
-void Workspace::lowerClientRequest( Client* c )
+void Workspace::lowerClientRequest( Client* c, NET::RequestSource src, Time /*timestamp*/ )
     {
     // If the client has support for all this focus stealing prevention stuff,
     // do only lowering within the application, as that's the more logical
     // variant of lowering when application requests it.
     // No demanding of attention here of course.
-    if( c->hasUserTimeSupport())
-        lowerClientWithinApplication( c );
-    else
+    if( src == NET::FromTool || !c->hasUserTimeSupport())
         lowerClient( c );
+    else
+        lowerClientWithinApplication( c );
     }
 
 void Workspace::restackClientUnderActive( Client* c )
     {
+    if( c->isTopMenu())
+        return;
     if( !active_client || active_client == c )
         {
         raiseClient( c );
@@ -590,6 +607,15 @@ bool Workspace::keepTransientAbove( const Client* mainwindow, const Client* tran
     // group transients way too high.
     if( mainwindow->isTopMenu() && transient->groupTransient())
         return false;
+    // #93832 - don't keep splashscreens above dialogs
+    if( transient->isSplash() && mainwindow->isDialog())
+        return false;
+    // This is rather a hack for #76026. Don't keep non-modal dialogs above
+    // the mainwindow, but only if they're group transient (since only such dialogs
+    // have taskbar entry in Kicker). A proper way of doing this (both kwin and kicker)
+    // needs to be found.
+    if( transient->isDialog() && !transient->isModal() && transient->groupTransient())
+        return false;
     return true;
     // #63223 - don't keep transients above docks, because the dock is kept high,
     // and e.g. dialogs for them would be too high too
@@ -605,24 +631,18 @@ bool Workspace::keepTransientAbove( const Client* mainwindow, const Client* tran
 // Client
 //*******************************
 
-void Client::restackWindow( Window /*above TODO */, int detail, NET::RequestSource source, bool send_event )
+void Client::restackWindow( Window /*above TODO */, int detail, NET::RequestSource src, Time timestamp, bool send_event )
     {
     switch ( detail )
         {
         case Above:
         case TopIf:
-            if( source == NET::FromTool )
-                workspace()->raiseClient( this );
-            else
-                workspace()->raiseClientRequest( this );
-            break;
+            workspace()->raiseClientRequest( this, src, timestamp );
+          break;
         case Below:
         case BottomIf:
-            if( source == NET::FromTool )
-                workspace()->lowerClient( this );
-            else
-                workspace()->lowerClientRequest( this );
-            break;
+            workspace()->lowerClientRequest( this, src, timestamp );
+          break;
         case Opposite:
         default:
             break;
@@ -633,23 +653,42 @@ void Client::restackWindow( Window /*above TODO */, int detail, NET::RequestSour
     
 void Client::setKeepAbove( bool b )
     {
-    if ( b == keepAbove() )
+    b = rules()->checkKeepAbove( b );
+    if( b )
+        setKeepBelow( false );
+    if ( b == keepAbove()
+        || ( b && keepBelow())) // forced below
+        { // force hint change if different
+        if( bool( info->state() & NET::KeepAbove ) != keepAbove())
+            info->setState( keepAbove() ? NET::KeepAbove : 0, NET::KeepAbove );
         return;
-    setKeepBelow( false );
+        }
     keep_above = b;
-    info->setState( b ? NET::KeepAbove : 0, NET::KeepAbove );
-    // TODO emit a signal about the change to the style plugin
+    info->setState( keepAbove() ? NET::KeepAbove : 0, NET::KeepAbove );
+    if( decoration != NULL )
+        decoration->emitKeepAboveChanged( keepAbove());
     workspace()->updateClientLayer( this );
+    updateWindowRules();
     }
 
 void Client::setKeepBelow( bool b )
     {
-    if ( b == keepBelow() )
+    b = rules()->checkKeepBelow( b );
+    if( b )
+        setKeepAbove( false );
+    if ( b == keepBelow()
+        || ( b && keepAbove())) // forced above
+        { // force hint change if different
+        if( bool( info->state() & NET::KeepBelow ) != keepBelow())
+            info->setState( keepBelow() ? NET::KeepBelow : 0, NET::KeepBelow );
         return;
-    setKeepAbove( false );
+        }
     keep_below = b;
-    info->setState( b ? NET::KeepBelow : 0, NET::KeepBelow );
+    info->setState( keepBelow() ? NET::KeepBelow : 0, NET::KeepBelow );
+    if( decoration != NULL )
+        decoration->emitKeepBelowChanged( keepBelow());
     workspace()->updateClientLayer( this );
+    updateWindowRules();
     }
 
 Layer Client::layer() const
@@ -677,14 +716,14 @@ Layer Client::belongsToLayer() const
     if( isTopMenu())
         return DockLayer;
     // only raise fullscreen above docks if it's the topmost window in unconstrained stacking order,
-    // i.e. the window set to be topmost by the user
-    bool raise_special_active_windows = ( workspace()->topClientOnDesktop( desktop(), true ) == this );
+    // i.e. the window set to be topmost by the user (also includes transients of the fullscreen window)
+    const Client* ac = workspace()->mostRecentlyActivatedClient(); // instead of activeClient() - avoids flicker
+    if( isFullScreen() && ac != NULL
+        && workspace()->topClientOnDesktop( desktop(), true ) == ac
+        && ( ac == this || this->hasTransient( ac, true )))
+        return ActiveLayer;
     if( keepAbove())
         return AboveLayer;
-    if( isFullScreen() && workspace()->activeClient() != NULL
-        && ( workspace()->activeClient() == this || this->hasTransient( workspace()->activeClient(), true ))
-        && raise_special_active_windows )
-        return ActiveLayer;
     return NormalLayer;
     }
 
