@@ -33,6 +33,8 @@ License. See the file "COPYING" for the exact licensing terms.
 #undef INT8
 #undef INT32
 
+extern Time qt_x_time;
+
 namespace KWinInternal
 {
 
@@ -55,7 +57,7 @@ int x11ErrorHandler(Display *d, XErrorEvent *e)
          e->request_code == X_ChangeWindowAttributes
          || e->request_code == X_GrabKey
          )
-        && (e->error_code == BadAccess))
+        && (e->error_code == BadAccess)) 
         {
         fputs(i18n("kwin: it looks like there's already a window manager running. kwin not started.\n").local8Bit(), stderr);
         exit(1);
@@ -70,7 +72,7 @@ int x11ErrorHandler(Display *d, XErrorEvent *e)
 
     fprintf(stderr, "kwin: %s(0x%lx): %s\n", req, e->resourceid, msg);
 
-    if (initting)
+    if (initting) 
         {
         fputs(i18n("kwin: failure during initialization; aborting").local8Bit(), stderr);
         exit(1);
@@ -97,6 +99,9 @@ Application::Application( )
         ::exit(1);
         }
     connect( &owner, SIGNAL( lostOwnership()), SLOT( lostSelection()));
+    
+    // if there was already kwin running, it saved its configuration after loosing the selection -> reread
+    config()->reparseConfiguration();
 
     initting = TRUE; // startup....
 
@@ -123,7 +128,8 @@ Application::Application( )
 Application::~Application()
     {
     delete Workspace::self();
-    XSetInputFocus( qt_xdisplay(), PointerRoot, RevertToPointerRoot, CurrentTime );
+    if( owner.ownerWindow() != None ) // if there was no --replace (no new WM)
+        XSetInputFocus( qt_xdisplay(), PointerRoot, RevertToPointerRoot, qt_x_time );
     delete options;
     }
 
@@ -142,7 +148,7 @@ bool Application::x11EventFilter( XEvent *e )
     return KApplication::x11EventFilter( e );
     }
 
-static void sighandler(int)
+static void sighandler(int) 
     {
     QApplication::exit();
     }
@@ -151,12 +157,12 @@ static void sighandler(int)
 } // namespace
 
 static const char version[] = "3.0";
-static const char description[] = I18N_NOOP( "The KDE window manager." );
+static const char description[] = I18N_NOOP( "KDE window manager" );
 
 static KCmdLineOptions args[] =
     {
         { "lock", I18N_NOOP("Disable configuration options"), 0 },
-        { "replace", I18N_NOOP("Replace already running ICCCM2.0 compliant window manager."), 0 },
+        { "replace", I18N_NOOP("Replace already-running ICCCM2.0-compliant window manager"), 0 },
         KCmdLineLastOption
     };
 
@@ -164,26 +170,26 @@ extern "C"
 int kdemain( int argc, char * argv[] )
     {
     bool restored = false;
-    for (int arg = 1; arg < argc; arg++)
+    for (int arg = 1; arg < argc; arg++) 
         {
-        if (! qstrcmp(argv[arg], "-session"))
+        if (! qstrcmp(argv[arg], "-session")) 
             {
             restored = true;
             break;
             }
         }
 
-    if (! restored)
+    if (! restored) 
         {
         // we only do the multihead fork if we are not restored by the session
 	// manager, since the session manager will register multiple kwins,
         // one for each screen...
         QCString multiHead = getenv("KDE_MULTIHEAD");
-        if (multiHead.lower() == "true")
+        if (multiHead.lower() == "true") 
             {
 
             Display* dpy = XOpenDisplay( NULL );
-            if ( !dpy )
+            if ( !dpy ) 
                 {
                 fprintf(stderr, "%s: FATAL ERROR while trying to open display %s\n",
                         argv[0], XDisplayName(NULL ) );
@@ -201,13 +207,13 @@ int kdemain( int argc, char * argv[] )
                 display_name.remove(pos,10); // 10 is enough to be sure we removed ".s"
 
             QCString envir;
-            if (number_of_screens != 1)
+            if (number_of_screens != 1) 
                 {
-                for (int i = 0; i < number_of_screens; i++ )
+                for (int i = 0; i < number_of_screens; i++ ) 
                     {
 		    // if execution doesn't pass by here, then kwin
 		    // acts exactly as previously
-                    if ( i != KWinInternal::screen_number && fork() == 0 )
+                    if ( i != KWinInternal::screen_number && fork() == 0 ) 
                         {
                         KWinInternal::screen_number = i;
 			// break here because we are the child process, we don't
@@ -219,7 +225,7 @@ int kdemain( int argc, char * argv[] )
 		//   number. If it had it, it was removed at the "pos" check
                 envir.sprintf("DISPLAY=%s.%d", display_name.data(), KWinInternal::screen_number);
 
-                if (putenv( strdup(envir.data())) )
+                if (putenv( strdup(envir.data())) ) 
                     {
                     fprintf(stderr,
                             "%s: WARNING: unable to set DISPLAY environment variable\n",
