@@ -36,6 +36,7 @@ License. See the file "COPYING" for the exact licensing terms.
 #include "placement.h"
 #include "notifications.h"
 #include "group.h"
+#include "rules.h"
 
 #include <X11/extensions/shape.h>
 #include <X11/keysym.h>
@@ -49,8 +50,6 @@ namespace KWinInternal
 
 extern int screen_number;
 
-static Window null_focus_window = 0;
-
 Workspace *Workspace::_self = 0;
 
 // Rikkus: This class is too complex. It needs splitting further.
@@ -67,10 +66,13 @@ Workspace::Workspace( bool restore )
     number_of_desktops(0),
     popup_client      (0),
     desktop_widget    (0),
+    temporaryRulesMessages( "_KDE_NET_WM_TEMPORARY_RULES", NULL, false ),
     active_client     (0),
     last_active_client     (0),
     most_recently_raised (0),
     movingClient(0),
+    pending_take_activity ( NULL ),
+    delayfocus_client (0),
     was_user_interaction (false),
     session_saving    (false),
     control_grab      (false),
@@ -96,8 +98,15 @@ Workspace::Workspace( bool restore )
     layoutX(-1),
     layoutY(2),
     workarea(NULL),
+    screenarea(NULL),
+    topmenu_selection( NULL ),
+    topmenu_watcher( NULL ),
+    topmenu_height( 0 ),
+    managing_topmenus( false ),
+    topmenu_space( NULL ),
     set_active_client_recursion( 0 ),
-    block_stacking_updates( 0 )
+    block_stacking_updates( 0 ),
+    forced_global_mouse_grab( false )
     {
     _self = this;
     mgr = new PluginMgr;
@@ -106,15 +115,21 @@ Workspace::Workspace( bool restore )
     installed_colormap = default_colormap;
     session.setAutoDelete( TRUE );
 
+    connect( &temporaryRulesMessages, SIGNAL( gotMessage( const QString& )),
+        this, SLOT( gotTemporaryRulesMessage( const QString& )));
+    connect( &rulesUpdatedTimer, SIGNAL( timeout()), this, SLOT( writeWindowRules()));
+
     updateXTime(); // needed for proper initialization of user_time in Client ctor
 
+    delayFocusTimer = 0; 
+    
     electric_time_first = qt_x_time;
     electric_time_last = qt_x_time;
 
     if ( restore )
       loadSessionInfo();
 
-    loadFakeSessionInfo();
+    loadWindowRules();
 
     (void) QApplication::desktop(); // trigger creation of desktop widget
 
@@ -136,7 +151,8 @@ Workspace::Workspace( bool restore )
                  PropertyChangeMask |
                  ColormapChangeMask |
                  SubstructureRedirectMask |
-                 SubstructureNotifyMask
+                 SubstructureNotifyMask |
+                 FocusChangeMask // for NotifyDetailNone
                  );
 
     Shape::init();
@@ -169,12 +185,17 @@ Workspace::Workspace( bool restore )
 
 void Workspace::init()
     {
-    if (options->electricBorders() == Options::ElectricAlways)
-       createBorderWindows();
+    checkElectricBorders();
 
     supportWindow = new QWidget;
     XLowerWindow( qt_xdisplay(), supportWindow->winId()); // see usage in layers.cpp
 
+    XSetWindowAttributes attr;
+    attr.override_redirect = 1;
+    null_focus_window = XCreateWindow( qt_xdisplay(), qt_xrootwin(), -1,-1, 1, 1, 0, CopyFromParent,
+        InputOnly, CopyFromParent, CWOverrideRedirect, &attr );
+    XMapWindow(qt_xdisplay(), null_focus_window);
+
     unsigned long protocols[ 5 ] =
         {
         NET::Supported |
@@ -235,6 +256,8 @@ void Workspace::init()
         NET::WM2AllowedActions |
         NET::WM2RestackWindow |
         NET::WM2MoveResizeWindow |
+        NET::WM2ExtendedStrut |
+        NET::WM2KDETemporaryRules |
         0
         ,
         NET::ActionMove |
@@ -271,10 +294,6 @@ void Workspace::init()
     // now we know how many desktops we'll, thus, we initialise the positioning object
     initPositioning = new Placement(this);
 
-    unsigned int i, nwins;
-    Window root_return, parent_return, *wins;
-    XWindowAttributes attr;
-
     connect(&reconfigureTimer, SIGNAL(timeout()), this,
             SLOT(slotReconfigure()));
     connect( &updateToolWindowsTimer, SIGNAL( timeout()), this, SLOT( slotUpdateToolWindows()));
@@ -295,9 +314,6 @@ void Workspace::init()
     Atom topmenu_atom = XInternAtom( qt_xdisplay(), nm, False );
     topmenu_selection = new KSelectionOwner( topmenu_atom );
     topmenu_watcher = new KSelectionWatcher( topmenu_atom );
-    topmenu_height = 0;
-    managing_topmenus = false;
-    topmenu_space = NULL;
 // TODO grabXServer(); - where exactly put this? topmenu selection claiming down belong must be before
 
         { // begin updates blocker block
@@ -308,9 +324,12 @@ void Workspace::init()
         else
             lostTopMenuSelection();
 
+        unsigned int i, nwins;
+        Window root_return, parent_return, *wins;
         XQueryTree(qt_xdisplay(), root, &root_return, &parent_return, &wins, &nwins);
         for (i = 0; i < nwins; i++) 
             {
+            XWindowAttributes attr;
             XGetWindowAttributes(qt_xdisplay(), wins[i], &attr);
             if (attr.override_redirect )
                 continue;
@@ -383,9 +402,8 @@ Workspace::~Workspace()
          ++it )
         {
 	// only release the window
-        if( !(*it)->isDesktop()) // TODO ?
-            storeFakeSessionInfo( *it );
         (*it)->releaseWindow( true );
+        // no removeClient() is called !
         }
     delete desktop_widget;
     delete tab_box;
@@ -394,18 +412,25 @@ Workspace::~Workspace()
     if ( root == qt_xrootwin() )
         XDeleteProperty(qt_xdisplay(), qt_xrootwin(), atoms->kwin_running);
 
-    writeFakeSessionInfo();
+    writeWindowRules();
     KGlobal::config()->sync();
 
     delete rootInfo;
     delete supportWindow;
     delete mgr;
     delete[] workarea;
+    delete[] screenarea;
     delete startup;
     delete initPositioning;
     delete topmenu_watcher;
     delete topmenu_selection;
     delete topmenu_space;
+    while( !rules.isEmpty())
+        {
+        delete rules.front();
+        rules.pop_front();
+        }
+    XDestroyWindow( qt_xdisplay(), null_focus_window );
 // TODO    ungrabXServer();
     _self = 0;
     }
@@ -443,6 +468,8 @@ void Workspace::addClient( Client* c, allowed_t )
         }
     if( !unconstrained_stacking_order.contains( c ))
         unconstrained_stacking_order.append( c );
+    if( !stacking_order.contains( c )) // it'll be updated later, and updateToolWindows() requires
+        stacking_order.append( c );    // c to be in stacking_order
     if( c->isTopMenu())
         addTopMenu( c );
     updateClientArea(); // this cannot be in manage(), because the client got added only now
@@ -454,10 +481,10 @@ void Workspace::addClient( Client* c, allowed_t )
         if( activeClient() == NULL && should_get_focus.count() == 0 )
             activateClient( findDesktop( true, currentDesktop()));
         }
-    if( c->isUtility() || c->isMenu() || c->isToolbar())
-        updateToolWindows( true );
     checkTransients( c->window()); // SELI does this really belong here?
     updateStackingOrder( true ); // propagate new client
+    if( c->isUtility() || c->isMenu() || c->isToolbar())
+        updateToolWindows( true );
     }
 
 /*
@@ -475,8 +502,6 @@ void Workspace::removeClient( Client* c, allowed_t )
     if( c->isNormalWindow())
         Notify::raise( Notify::Delete );
 
-    storeFakeSessionInfo( c );
-
     Q_ASSERT( clients.contains( c ) || desktops.contains( c ));
     clients.remove( c );
     desktops.remove( c );
@@ -496,6 +521,10 @@ void Workspace::removeClient( Client* c, allowed_t )
     Q_ASSERT( c != active_client );
     if ( c == last_active_client )
         last_active_client = 0;
+    if( c == pending_take_activity )
+        pending_take_activity = NULL;
+    if( c == delayfocus_client )
+        cancelDelayFocus();
 
     updateStackingOrder( true );
 
@@ -652,10 +681,25 @@ void Workspace::updateToolWindows( bool also_hide )
                 else
                     show = false;
                 }
+            if( !show && also_hide )
+                {
+                const ClientList mainclients = (*it)->mainClients();
+                // don't hide utility windows which are standalone(?) or
+                // have e.g. kicker as mainwindow
+                if( mainclients.isEmpty())
+                    show = true;
+                for( ClientList::ConstIterator it2 = mainclients.begin();
+                     it2 != mainclients.end();
+                     ++it2 )
+                    {
+                    if( (*it2)->isSpecialWindow())
+                        show = true;
+                    }
+                if( !show )
+                    to_hide.append( *it );
+                }
             if( show )
                 to_show.append( *it );
-            else if( also_hide )
-                to_hide.append( *it );
             }
         } // first show new ones, then hide
     for( ClientList::ConstIterator it = to_show.fromLast();
@@ -714,6 +758,7 @@ void Workspace::slotSettingsChanged(int category)
   Reread settings
  */
 KWIN_PROCEDURE( CheckBorderSizesProcedure, cl->checkBorderSizes() );
+KWIN_PROCEDURE( ResetupRulesProcedure, cl->setupWindowRules( true ) );
 
 void Workspace::slotReconfigure()
     {
@@ -748,10 +793,7 @@ void Workspace::slotReconfigure()
         forEachClient( CheckBorderSizesProcedure());
         }
 
-    if (options->electricBorders() == Options::ElectricAlways)
-       createBorderWindows();
-    else
-       destroyBorderWindows();
+    checkElectricBorders();
 
     if( options->topMenuEnabled() && !managingTopMenus())
         {
@@ -771,28 +813,32 @@ void Workspace::slotReconfigure()
         updateTopMenuGeometry();
         updateCurrentTopMenu();
         }
+
+    loadWindowRules();
+    forEachClient( ResetupRulesProcedure());
     }
 
 void Workspace::loadDesktopSettings()
     {
-    KConfig c("kwinrc");
-
+    KConfig* c = KGlobal::config();
     QCString groupname;
     if (screen_number == 0)
         groupname = "Desktops";
     else
         groupname.sprintf("Desktops-screen-%d", screen_number);
-    c.setGroup(groupname);
+    KConfigGroupSaver saver(c,groupname);
 
-    int n = c.readNumEntry("Number", 4);
+    int n = c->readNumEntry("Number", 4);
     number_of_desktops = n;
     delete workarea;
     workarea = new QRect[ n + 1 ];
+    delete screenarea;
+    screenarea = NULL;
     rootInfo->setNumberOfDesktops( number_of_desktops );
     desktop_focus_chain.resize( n );
     for(int i = 1; i <= n; i++) 
         {
-        QString s = c.readEntry(QString("Name_%1").arg(i),
+        QString s = c->readEntry(QString("Name_%1").arg(i),
                                 i18n("Desktop %1").arg(i));
         rootInfo->setDesktopName( i, s.utf8().data() );
         desktop_focus_chain[i-1] = i;
@@ -801,16 +847,15 @@ void Workspace::loadDesktopSettings()
 
 void Workspace::saveDesktopSettings()
     {
-    KConfig c("kwinrc");
-
+    KConfig* c = KGlobal::config();
     QCString groupname;
     if (screen_number == 0)
         groupname = "Desktops";
     else
         groupname.sprintf("Desktops-screen-%d", screen_number);
-    c.setGroup(groupname);
+    KConfigGroupSaver saver(c,groupname);
 
-    c.writeEntry("Number", number_of_desktops );
+    c->writeEntry("Number", number_of_desktops );
     for(int i = 1; i <= number_of_desktops; i++) 
         {
         QString s = desktopName( i );
@@ -823,13 +868,13 @@ void Workspace::saveDesktopSettings()
 
         if (s != defaultvalue) 
             {
-            c.writeEntry( QString("Name_%1").arg(i), s );
+            c->writeEntry( QString("Name_%1").arg(i), s );
             }
         else 
             {
-            QString currentvalue = c.readEntry(QString("Name_%1").arg(i));
+            QString currentvalue = c->readEntry(QString("Name_%1").arg(i));
             if (currentvalue != defaultvalue)
-                c.writeEntry( QString("Name_%1").arg(i), "" );
+                c->writeEntry( QString("Name_%1").arg(i), "" );
             }
         }
     }
@@ -841,7 +886,7 @@ QStringList Workspace::configModules(bool controlCenter)
     if (controlCenter)
         args << "kde-kwinoptions.desktop";
     else if (kapp->authorizeControlModule("kde-kwinoptions.desktop"))
-        args  << "kwinactions" << "kwinfocus" <<  "kwinmoving" << "kwinadvanced";
+        args  << "kwinactions" << "kwinfocus" <<  "kwinmoving" << "kwinadvanced" << "kwinrules";
     return args;
     }
 
@@ -1066,6 +1111,8 @@ bool Workspace::setCurrentDesktop( int new_desktop )
             requestFocus( findDesktop( true, currentDesktop()));
         }
 
+    updateCurrentTopMenu();
+
     // Update focus chain:
     //  If input: chain = { 1, 2, 3, 4 } and current_desktop = 3,
     //   Output: chain = { 3, 1, 2, 4 }.
@@ -1156,11 +1203,10 @@ void Workspace::setNumberOfDesktops( int n )
  */
 void Workspace::sendClientToDesktop( Client* c, int desk, bool dont_activate )
     {
-    if ( c->desktop() == desk )
-        return;
-
     bool was_on_desktop = c->isOnDesktop( desk ) || c->isOnAllDesktops();
     c->setDesktop( desk );
+    if ( c->desktop() != desk ) // no change or desktop forced
+        return;
     desk = c->desktop(); // Client did range checking
 
     if ( c->isOnDesktop( currentDesktop() ) )
@@ -1321,6 +1367,12 @@ void Workspace::sendPingToWindow( Window window, Time timestamp )
     rootInfo->sendPing( window, timestamp );
     }
 
+void Workspace::sendTakeActivity( Client* c, Time timestamp, long flags )
+    {
+    rootInfo->takeActivity( c->window(), timestamp, flags );
+    pending_take_activity = c;
+    }
+
 
 /*!
   Takes a screenshot of the current window and puts it in the clipboard.
@@ -1625,7 +1677,27 @@ QWidget* Workspace::desktopWidget()
     return desktop_widget;
     }
 
-
+//Delayed focus functions
+void Workspace::delayFocus()
+    {
+    requestFocus( delayfocus_client );
+    cancelDelayFocus();
+    }
+    
+void Workspace::requestDelayFocus( Client* c )
+    {
+    delayfocus_client = c;
+    delete delayFocusTimer;
+    delayFocusTimer = new QTimer( this );
+    connect( delayFocusTimer, SIGNAL( timeout() ), this, SLOT( delayFocus() ) );
+    delayFocusTimer->start( options->delayFocusInterval, TRUE  );
+    }
+    
+void Workspace::cancelDelayFocus()
+    {
+    delete delayFocusTimer;
+    delayFocusTimer = 0;
+    }
 
 // Electric Borders
 //========================================================================//
@@ -1634,12 +1706,11 @@ QWidget* Workspace::desktopWidget()
 // borders. Technically this is done with input only windows. Since
 // electric borders can be switched on and off, we have these two
 // functions to create and destroy them.
-void Workspace::createBorderWindows()
+void Workspace::checkElectricBorders( bool force )
     {
-    if ( electric_have_borders )
-        return;
-
-    electric_have_borders = true;
+    if( force )
+        destroyBorderWindows();
+    
     electric_current_border = 0;
 
     QRect r = QApplication::desktop()->geometry();
@@ -1648,6 +1719,20 @@ void Workspace::createBorderWindows()
     electricLeft = r.left();
     electricRight = r.right();
 
+    if (options->electricBorders() == Options::ElectricAlways)
+       createBorderWindows();
+    else
+       destroyBorderWindows();
+    }
+
+void Workspace::createBorderWindows()
+    {
+    if ( electric_have_borders )
+        return;
+
+    electric_have_borders = true;
+
+    QRect r = QApplication::desktop()->geometry();
     XSetWindowAttributes attributes;
     unsigned long valuemask;
     attributes.override_redirect = True;
@@ -1924,6 +2009,10 @@ void Workspace::setupTopMenuHandling()
     disconnect( topmenu_watcher, SIGNAL( lostOwner()), this, SLOT( lostTopMenuOwner()));
     managing_topmenus = true;
     topmenu_space = new QWidget;
+    Window stack[ 2 ];
+    stack[ 0 ] = supportWindow->winId();
+    stack[ 1 ] = topmenu_space->winId();
+    XRestackWindows(qt_xdisplay(), stack, 2);
     updateTopMenuGeometry();
     topmenu_space->show();
     updateClientArea();
@@ -1951,9 +2040,9 @@ QString Workspace::desktopName( int desk ) const
     return QString::fromUtf8( rootInfo->desktopName( desk ) );
     }
 
-bool Workspace::checkStartupNotification( Window w, KStartupInfoData& data )
+bool Workspace::checkStartupNotification( Window w, KStartupInfoId& id, KStartupInfoData& data )
     {
-    return startup->checkStartup( w, data ) == KStartupInfo::Match;
+    return startup->checkStartup( w, id, data ) == KStartupInfo::Match;
     }
 
 /*!
@@ -1962,16 +2051,6 @@ bool Workspace::checkStartupNotification( Window w, KStartupInfoData& data )
  */
 void Workspace::focusToNull()
     {
-    int mask;
-    XSetWindowAttributes attr;
-    if (null_focus_window == 0) 
-        {
-        mask = CWOverrideRedirect;
-        attr.override_redirect = 1;
-        null_focus_window = XCreateWindow(qt_xdisplay(), qt_xrootwin(), -1,-1, 1, 1, 0, CopyFromParent,
-                          InputOnly, CopyFromParent, mask, &attr);
-        XMapWindow(qt_xdisplay(), null_focus_window);
-        }
     XSetInputFocus(qt_xdisplay(), null_focus_window, RevertToPointerRoot, qt_x_time );
     }
 
@@ -1981,23 +2060,27 @@ void Workspace::helperDialog( const QString& message, const Client* c )
     QString type;
     if( message == "noborderaltf3" )
         {
+        QString shortcut = QString( "%1 (%2)" ).arg( keys->label( "Window Operations Menu" ))
+            .arg( keys->shortcut( "Window Operations Menu" ).seq( 0 ).toString());
         args << "--msgbox" <<
               i18n( "You have selected to show a window without its border.\n"
-                    "Without the border, you won't be able to enable the border "
-                    "again using the mouse. Use the window operations menu instead, "
+                    "Without the border, you will not be able to enable the border "
+                    "again using the mouse: use the window operations menu instead, "
                     "activated using the %1 keyboard shortcut." )
-                .arg( keys->shortcut( "Window Operations Menu" ).seq( 0 ).toString());
+                .arg( shortcut );
         type = "altf3warning";
         }
     else if( message == "fullscreenaltf3" )
         {
+        QString shortcut = QString( "%1 (%2)" ).arg( keys->label( "Window Operations Menu" ))
+            .arg( keys->shortcut( "Window Operations Menu" ).seq( 0 ).toString());
         args << "--msgbox" <<
               i18n( "You have selected to show a window in fullscreen mode.\n"
-                    "If the application itself doesn't have an option to turn the fullscreen "
-                    "mode off, you won't be able to disable it "
-                    "again using the mouse. Use the window operations menu instead, "
+                    "If the application itself does not have an option to turn the fullscreen "
+                    "mode off you will not be able to disable it "
+                    "again using the mouse: use the window operations menu instead, "
                     "activated using the %1 keyboard shortcut." )
-                .arg( keys->shortcut( "Window Operations Menu" ).seq( 0 ).toString());
+                .arg( shortcut );
         type = "altf3warning";
         }
     else
