@@ -129,7 +129,7 @@ KonqPopupMenu::KonqPopupMenu( KBookmarkManager *mgr, const KFileItemList &items,
                               KURL viewURL,
                               KActionCollection & actions,
                               KNewMenu * newMenu,
-			      QWidget * parentWidget,
+                              QWidget * parentWidget,
                               bool showProperties )
   : QPopupMenu( parentWidget, "konq_popupmenu" ), m_actions( actions ), m_ownActions( static_cast<QObject *>( 0 ), "KonqPopupMenu::m_ownActions" ), m_pMenuNew( newMenu ), m_sViewURL(viewURL), m_lstItems(items), m_pManager(mgr)
 {
@@ -269,6 +269,7 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
     bool sDeleting      = true;
     bool sMoving        = true;
     m_sMimeType         = m_lstItems.first()->mimetype();
+    QString mimeGroup   = m_sMimeType.left(m_sMimeType.find('/'));
     mode_t mode         = m_lstItems.first()->mode();
     bool isDirectory    = m_sMimeType == "inode/directory";
     bool bTrashIncluded = false;
@@ -304,8 +305,13 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
 
         // Determine if common mimetype among all URLs
         if ( m_sMimeType != (*it)->mimetype() )
+        {
             m_sMimeType = QString::null; // mimetypes are different => null
 
+            if ( mimeGroup != (*it)->mimetype().left((*it)->mimetype().find('/')))
+                mimeGroup = QString::null; // mimetype groups are different as well!
+        }
+
         if ( !bTrashIncluded &&
              (*it)->url().isLocalFile() &&
              (*it)->url().path( 1 ) == KGlobalSettings::trashPath() )
@@ -376,7 +382,7 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
 
     // Either 'newview' is in the actions we're given (probably in the tabhandling group)
     // or we need to insert it ourselves (e.g. for kdesktop). In the first case, actNewWindow must remain 0.
-    if ( kpf & ShowNewWindow )
+    if ( ((kpf & ShowNewWindow) != 0) && sReading )
     {
         QString openStr = isKDesktop ? i18n( "&Open" ) : i18n( "Open in New &Window" );
         actNewWindow = new KAction( openStr, "window_new", 0, this, SLOT( slotPopupNewView() ), &m_ownActions, "newview" );
@@ -487,6 +493,13 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
         }
         addGroup( "editactions" );
     }
+
+    if (d->m_itemFlags & KParts::BrowserExtension::ShowTextSelectionItems) {
+      addMerge( 0 );
+      m_factory->addClient( this );
+      return;
+    }
+
     if ( !isCurrentTrash && !isIntoTrash && (d->m_itemFlags & KParts::BrowserExtension::ShowBookmark))
     {
         addSeparator();
@@ -530,7 +543,7 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
         user = KDEDesktopMimeType::userDefinedServices( m_lstItems.first()->url().path(), url.isLocalFile() );
     }
 
-    if ( !isCurrentTrash && !isIntoTrash)
+    if ( !isCurrentTrash && !isIntoTrash && sReading)
     {
 
         // 2 - Look for "servicesmenus" bindings (konqueror-specific user-defined services)
@@ -590,10 +603,11 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
                     QStringList types = cfg.readListEntry( "ServiceTypes" );
                     QStringList excludeTypes = cfg.readListEntry( "ExcludeServiceTypes" );
                     bool ok = false;
-                    QString mimeGroup = m_sMimeType.left(m_sMimeType.find('/'));
 
                     // check for exact matches or a typeglob'd mimetype if we have a mimetype
-                    for (QStringList::iterator it = types.begin(); it != types.end(); ++it)
+                    for (QStringList::iterator it = types.begin();
+                         it != types.end() && !ok;
+                         ++it)
                     {
                         // we could cram the following three if statements into
                         // one gigantic boolean statement but that would be a
@@ -603,63 +617,55 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
                         if (*it == "all/all" ||
                             *it == "allfiles" /*compat with KDE up to 3.0.3*/)
                         {
-			  ok = true;
-			  for (QStringList::iterator itex = excludeTypes.begin(); itex != excludeTypes.end(); ++itex)
-			    {
-			      if( (*itex).right(1) == "*" &&(*itex).left((*itex).find('/'))== mimeGroup )
-				{
-				  ok = false;
-				  break;
-				}
-			      else if ( (*itex).contains( m_sMimeType))
-				{
-				  ok = false;
-				  break;
-				}
-			    }
-                        }
-                        // next, do we match all files?
-                        if (*it == "all/allfiles" &&
-                            !isDirectory) // ## or inherits from it
-                        {
-			  ok = true;
-			  for (QStringList::iterator itex = excludeTypes.begin(); itex != excludeTypes.end(); ++itex)
-			    {
-			      if( (*itex).right(1) == "*" &&(*itex).left((*itex).find('/'))== mimeGroup )
-				{
-				  ok = false;
-				  break;
-				}
-			      else if ( (*itex).contains( m_sMimeType))
-				{
-				  ok = false;
-				  break;
-				}
-			    }
+                            ok = true;
+                            for (QStringList::iterator itex = excludeTypes.begin(); itex != excludeTypes.end(); ++itex)
+                            {
+                                if( ((*itex).right(1) == "*" && (*itex).left((*itex).find('/')) == mimeGroup) ||
+                                    ((*itex) == m_sMimeType) )
+                                {
+                                    ok = false;
+                                    break;
+                                }
+                            }
                         }
 
-                        // if we have a mimetype, see if we have an exact or type
-                        // globbed match
-                        if (!m_sMimeType.isNull() &&
-                            (*it == m_sMimeType ||
+                        // next, do we match all files?
+                        if (!ok &&
+                            !isDirectory /* ## or inherits from it */ &&
+                            *it == "all/allfiles")
+                        {
+                            ok = true;
+                            for (QStringList::iterator itex = excludeTypes.begin(); itex != excludeTypes.end(); ++itex)
+                            {
+                                if( ((*itex).right(1) == "*" && (*itex).left((*itex).find('/')) == mimeGroup) ||
+                                    ((*itex) == m_sMimeType) )
+                                {
+                                    ok = false;
+                                    break;
+                                }
+                            }
+
+                        }
+
+                        // if we have a mimetype, see if we have an exact or a type globbed match
+                        if (!ok &&
+                            (!m_sMimeType.isEmpty() &&
+                              *it == m_sMimeType) ||
+                            (!mimeGroup.isEmpty() &&
                              ((*it).right(1) == "*" &&
                               (*it).left((*it).find('/')) == mimeGroup)))
                         {
-			  ok = true;
-			  for (QStringList::iterator itex = excludeTypes.begin(); itex != excludeTypes.end(); ++itex)
-			    {
-			      if( (*itex).right(1) == "*" &&(*itex).left((*itex).find('/'))== mimeGroup )
-				{
-				  ok = false;
-				  break;
-				}
-			      else if ( (*itex).contains( m_sMimeType))
-				{
-				  ok = false;
-				  break;
-				}
-			    }
-			}
+                            ok = true;
+                            for (QStringList::iterator itex = excludeTypes.begin(); itex != excludeTypes.end(); ++itex)
+                            {
+                                if( ((*itex).right(1) == "*" && (*itex).left((*itex).find('/')) == mimeGroup) ||
+                                    ((*itex) == m_sMimeType) )
+                                {
+                                    ok = false;
+                                    break;
+                                }
+                            }
+                        }
                     }
 
                     if ( ok )
@@ -814,7 +820,7 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
         addSeparator();
     }
 
-    if ( !isCurrentTrash && !isIntoTrash && !devicesFile)
+    if ( !isCurrentTrash && !isIntoTrash && !devicesFile && sReading)
         addPlugins( ); // now it's time to add plugins
 
     if ( KPropertiesDialog::canDisplay( m_lstItems ) && (kpf & ShowProperties) )
@@ -828,15 +834,12 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
             m_menuElement.lastChild().toElement().tagName().lower() == "separator" )
         m_menuElement.removeChild( m_menuElement.lastChild() );
 
-    if( bCanChangeSharing && !isCurrentTrash && !isIntoTrash)
+    if( bCanChangeSharing && !isCurrentTrash && !isIntoTrash )
     {
         if(KFileShare::authorization()==KFileShare::Authorized)
         {
             addSeparator();
-            QString label;
-            label=i18n("Share");
-
-            act = new KAction( label, 0, this, SLOT( slotOpenShareFileDialog() ),
+            act = new KAction( i18n("Share"), 0, this, SLOT( slotOpenShareFileDialog() ),
                                &m_ownActions, "sharefile" );
             addAction( act );
         }
@@ -912,17 +915,18 @@ void KonqPopupMenu::slotPopupOpenWith()
 
 void KonqPopupMenu::slotPopupAddToBookmark()
 {
-  KBookmarkGroup root = m_pManager->root();
+  KBookmarkGroup root;
   if ( m_lstPopupURLs.count() == 1 ) {
     KURL url = m_lstPopupURLs.first();
     QString title = d->m_urlTitle.isEmpty() ? url.prettyURL() : d->m_urlTitle;
-    root.addBookmark( m_pManager, title, url.url() );
+    root = m_pManager->addBookmarkDialog( url.prettyURL(), title );
   }
   else
   {
+    root = m_pManager->root();
     KURL::List::ConstIterator it = m_lstPopupURLs.begin();
     for ( ; it != m_lstPopupURLs.end(); it++ )
-      root.addBookmark( m_pManager, (*it).prettyURL(), (*it).url() );
+      root.addBookmark( m_pManager, (*it).prettyURL(), (*it) );
   }
   m_pManager->emitChanged( root );
 }
@@ -1000,36 +1004,36 @@ KonqPopupMenu::ProtocolInfo KonqPopupMenu::protocolInfo() const
   return m_info;
 }
 void KonqPopupMenu::addPlugins( ){
-	// search for Konq_PopupMenuPlugins inspired by simons kpropsdlg
-	//search for a plugin with the right protocol
-	KTrader::OfferList plugin_offers;
+        // search for Konq_PopupMenuPlugins inspired by simons kpropsdlg
+        //search for a plugin with the right protocol
+        KTrader::OfferList plugin_offers;
         unsigned int pluginCount = 0;
-	plugin_offers = KTrader::self()->query( m_sMimeType.isNull() ? QString::fromLatin1( "all/all" ) : m_sMimeType , "'KonqPopupMenu/Plugin' in ServiceTypes");
+        plugin_offers = KTrader::self()->query( m_sMimeType.isNull() ? QString::fromLatin1( "all/all" ) : m_sMimeType , "'KonqPopupMenu/Plugin' in ServiceTypes");
         if ( plugin_offers.isEmpty() )
-	  return; // no plugins installed do not bother about it
+          return; // no plugins installed do not bother about it
 
-	KTrader::OfferList::ConstIterator iterator = plugin_offers.begin( );
-	KTrader::OfferList::ConstIterator end = plugin_offers.end( );
+        KTrader::OfferList::ConstIterator iterator = plugin_offers.begin( );
+        KTrader::OfferList::ConstIterator end = plugin_offers.end( );
 
-	addGroup( "plugins" );
-	// travers the offerlist
-	for(; iterator != end; ++iterator, ++pluginCount ){
-		KonqPopupMenuPlugin *plugin =
-			KParts::ComponentFactory::
-			createInstanceFromLibrary<KonqPopupMenuPlugin>( (*iterator)->library().local8Bit(),
-									this,
-									(*iterator)->name().latin1() );
-		if ( !plugin )
-			continue;
+        addGroup( "plugins" );
+        // travers the offerlist
+        for(; iterator != end; ++iterator, ++pluginCount ){
+                KonqPopupMenuPlugin *plugin =
+                        KParts::ComponentFactory::
+                        createInstanceFromLibrary<KonqPopupMenuPlugin>( (*iterator)->library().local8Bit(),
+                                                                        this,
+                                                                        (*iterator)->name().latin1() );
+                if ( !plugin )
+                        continue;
                 QString pluginClientName = QString::fromLatin1( "Plugin%1" ).arg( pluginCount );
                 addMerge( pluginClientName );
                 plugin->domDocument().documentElement().setAttribute( "name", pluginClientName );
-		m_pluginList.append( plugin );
-		insertChildClient( plugin );
-	}
+                m_pluginList.append( plugin );
+                insertChildClient( plugin );
+        }
 
-	addMerge( "plugins" );
-	addSeparator();
+        addMerge( "plugins" );
+        addSeparator();
 }
 KURL KonqPopupMenu::url( ) const {
   return m_sViewURL;
@@ -1041,7 +1045,7 @@ KURL::List KonqPopupMenu::popupURLList( ) const {
   return m_lstPopupURLs;
 }
 /**
-	Plugin
+        Plugin
 */
 
 KonqPopupMenuPlugin::KonqPopupMenuPlugin( KonqPopupMenu *parent, const char *name )
