@@ -1,5 +1,5 @@
 /* This file is part of the KDE Project
-   Copyright (c) 2001 Malte Starostik <malte.starostik@t-online.de>
+   Copyright (c) 2001 Malte Starostik <malte@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -16,36 +16,18 @@
    Boston, MA 02111-1307, USA.
 */
 
-// $Id: konq_sound.h,v 1.5 2001/07/25 18:43:17 wester Exp $
-
 #ifndef __konq_sound_h__
 #define __konq_sound_h__
 
-#include <qobject.h>
-
 #include <klibloader.h>
-#include <soundserver.h>
-#include <kartsdispatcher.h>
 
-class KURL;
-class KPlayObject;
-class KPlayObjectFactory;
-
-class KonqSoundFactory : public KLibFactory
+class KonqSoundPlayer : public QObject
 {
-	Q_OBJECT
 public:
-	KonqSoundFactory(QObject *parent = 0, const char *name = 0);
-	virtual ~KonqSoundFactory();
-
-protected:
-	virtual QObject *createObject(QObject * = 0, const char * = 0,
-		const char *className = "QObject", const QStringList &args = QStringList());
-
-private:
-	KArtsDispatcher     m_dispatcher;
-	Arts::SoundServerV2 m_soundServer;
-	KPlayObjectFactory *m_factory;
+	virtual const QStringList &mimeTypes() = 0;
+	virtual void play(const QString &fileName) = 0;
+	virtual void stop() = 0;
+	virtual bool isPlaying() = 0;
 };
 
 #endif
