@@ -61,7 +61,6 @@
 #include <X11/Constraint.h>
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
-#include <Xm/DrawingA.h>
 
 // provide these symbols when compiling with gcc 3.x
 
@@ -480,10 +479,10 @@ NSPluginInstance::forwarder(Widget w, XtPointer cl_data, XEvent * event, Boolean
   Q_UNUSED(w);
   NSPluginInstance *inst = (NSPluginInstance*)cl_data;
   *cont = True;
-  if (inst->_area == 0 || event->xkey.window == XtWindow(inst->_area))
+  if (inst->_form == 0 || event->xkey.window == XtWindow(inst->_form))
     return;
   *cont = False;
-  event->xkey.window = XtWindow(inst->_area);
+  event->xkey.window = XtWindow(inst->_form);
   event->xkey.subwindow = None;
   XtDispatchEvent(event);
 }
@@ -530,15 +529,16 @@ NSPluginInstance::NSPluginInstance(NPP privateData, NPPluginFuncs *pluginFuncs,
       height = 1200;
 
    // create drawing area
-   Arg args[5];
-   Cardinal nargs=0;
+   Arg args[7];
+   Cardinal nargs = 0;
    XtSetArg(args[nargs], XtNwidth, width); nargs++;
    XtSetArg(args[nargs], XtNheight, height); nargs++;
+   XtSetArg(args[nargs], XtNborderWidth, 0); nargs++;
 
    String n, c;
    XtGetApplicationNameAndClass(qt_xdisplay(), &n, &c);
 
-   _toplevel = XtAppCreateShell("pane", c, topLevelShellWidgetClass,
+   _toplevel = XtAppCreateShell("drawingArea", c, applicationShellWidgetClass,
                                 qt_xdisplay(), args, nargs);
 
    // What exactly does widget mapping mean? Without this call the widget isn't
@@ -548,20 +548,31 @@ NSPluginInstance::NSPluginInstance(NPP privateData, NPPluginFuncs *pluginFuncs,
    XtRealizeWidget(_toplevel);
 
    // Create form window that is searched for by flash plugin
-   _form = XtCreateManagedWidget("form", compositeWidgetClass, _toplevel, args, nargs);
+   _form = XtVaCreateWidget("form", compositeWidgetClass, _toplevel, NULL);
+   XtSetArg(args[nargs], XtNvisual, QPaintDevice::x11AppVisual()); nargs++;
+   XtSetArg(args[nargs], XtNdepth, QPaintDevice::x11AppDepth()); nargs++;
+   XtSetArg(args[nargs], XtNcolormap, QPaintDevice::x11AppColormap()); nargs++;
+   XtSetValues(_form, args, nargs);
+   XSync(qt_xdisplay(), false);
+
+   // From mozilla - not sure if it's needed yet, nor what to use for embedder
+#if 0
+   /* this little trick seems to finish initializing the widget */
+#if XlibSpecificationRelease >= 6
+   XtRegisterDrawable(qt_xdisplay(), embedderid, _toplevel);
+#else
+   _XtRegisterWindow(embedderid, _toplevel);
+#endif
+#endif
    XtRealizeWidget(_form);
+   XtManageChild(_form);
 
-   // Create widget that is passed to the plugin
-   XtInitializeWidgetClass(xmDrawingAreaWidgetClass);
-   _area = XmCreateDrawingArea( _form, (char*)("drawingArea"), args, nargs);
-   XtRealizeWidget(_area);
-   XtMapWidget(_area);
-   
    // Register forwarder
    XtAddEventHandler(_toplevel, (KeyPressMask|KeyReleaseMask), 
                      False, forwarder, (XtPointer)this );
    XtAddEventHandler(_form, (KeyPressMask|KeyReleaseMask), 
                      False, forwarder, (XtPointer)this );
+   XSync(qt_xdisplay(), false);
 }
 
 NSPluginInstance::~NSPluginInstance()
@@ -616,8 +627,6 @@ void NSPluginInstance::destroy()
                              False, forwarder, (XtPointer)this);
         XtRemoveEventHandler(_toplevel, (KeyPressMask|KeyReleaseMask), 
                              False, forwarder, (XtPointer)this);
-        XtDestroyWidget(_area);
-	_area = 0;
         XtDestroyWidget(_form);
 	_form = 0;
         XtDestroyWidget(_toplevel);
@@ -660,7 +669,8 @@ void NSPluginInstance::timer()
         if ( req.url.left(11).lower()=="javascript:" )
             url = req.url;
         else if ( KURL::isRelativeURL(req.url) ) {
-            KURL absUrl( _baseURL, req.url );
+            KURL bu( _baseURL );
+            KURL absUrl( bu, req.url );
             url = absUrl.url();
         } else if ( req.url[0]=='/' && KURL(_baseURL).hasHost() ) {
             KURL absUrl( _baseURL );
@@ -699,23 +709,14 @@ void NSPluginInstance::timer()
 
                     emitStatus( i18n("Submitting data to %1").arg(url) );
                     s->post( url, req.data, req.mime, req.notify, req.args );
-                } else if (url.lower().startsWith("javascript:document.location") ||
-                     url.lower().startsWith("javascript:window.location.href")) {
-                    // hack to get java vm and HyperCosm 3d working
-                    NSPluginBufStream *s = new NSPluginBufStream( this );
-                    connect( s, SIGNAL(finished(NSPluginStreamBase*)),
-                             SLOT(streamFinished(NSPluginStreamBase*)) );
-                    _streams.append( s );
-
-                    QByteArray buf;
-                    buf.setRawData( _baseURL.latin1(), _baseURL.length()+1 );
-                    s->get( url, "text/html", buf, req.notify, true );
                 } else if (url.lower().startsWith("javascript:")){
                     if (_callback) {
                         static int _jsrequestid = 0;
                         _callback->evalJavaScript( _jsrequestid, url.mid(11) );
                         if ( req.notify )
                             _jsrequests.insert(_jsrequestid++, new Request( req ));
+                    } else {
+                        kdDebug() << "No callback for javascript: url!" << endl;
                     }
                 } else {
                     // create stream
@@ -738,7 +739,8 @@ void NSPluginInstance::timer()
 
 
 QString NSPluginInstance::normalizedURL(const QString& url) const {
-    KURL inURL(_baseURL, url);
+    KURL bu( _baseURL );
+    KURL inURL(bu, url);
     KConfig cfg("kcmnspluginrc", true);
     cfg.setGroup("Misc");
 
@@ -829,14 +831,14 @@ int NSPluginInstance::setWindow(int remove)
    _win.clipRect.bottom = _height;
    _win.clipRect.right = _width;
 
-   _win.window = (void*) XtWindow(_area);
+   _win.window = (void*) XtWindow(_form);
    kdDebug(1431) << "Window ID = " << _win.window << endl;
 
    _win_info.type = NP_SETWINDOW;
-   _win_info.display = XtDisplay( _area );
-   _win_info.visual = DefaultVisualOfScreen( XtScreen(_area) );
-   _win_info.colormap = DefaultColormapOfScreen( XtScreen(_area) );
-   _win_info.depth = DefaultDepthOfScreen( XtScreen(_area) );
+   _win_info.display = XtDisplay(_form);
+   _win_info.visual = DefaultVisualOfScreen(XtScreen(_form));
+   _win_info.colormap = DefaultColormapOfScreen(XtScreen(_form));
+   _win_info.depth = DefaultDepthOfScreen(XtScreen(_form));
 
    _win.ws_info = &_win_info;
 
@@ -875,15 +877,18 @@ void NSPluginInstance::resizePlugin(int w, int h)
    _width = w;
    _height = h;
 
-   Arg args[5];
-   Cardinal nargs=0;
+   Arg args[7];
+   Cardinal nargs = 0;
    XtSetArg(args[nargs], XtNwidth, _width); nargs++;
    XtSetArg(args[nargs], XtNheight, _height); nargs++;
+   XtSetArg(args[nargs], XtNvisual, QPaintDevice::x11AppVisual()); nargs++;
+   XtSetArg(args[nargs], XtNdepth, QPaintDevice::x11AppDepth()); nargs++;
+   XtSetArg(args[nargs], XtNcolormap, QPaintDevice::x11AppColormap()); nargs++;
+   XtSetArg(args[nargs], XtNborderWidth, 0); nargs++;
 
    XtSetValues(_form, args, nargs);
-   XtSetValues(_area, args, nargs);
 
-   resizeWidgetTree(XtWindow(_area), _width, _height);
+   resizeWidgetTree(XtWindow(_form), _width, _height);
 
    setWindow();
 
@@ -901,14 +906,23 @@ void NSPluginInstance::javascriptResult(int id, QString result) {
                  SLOT(streamFinished(NSPluginStreamBase*)) );
         _streams.append( s );
 
-        s->create( req->url, QString("text/plain"), req->notify );
         int len = result.length();
-        QByteArray data( len + 1 );
-        if (len)
-            memcpy( data.data(), result.latin1(), len );
-        data[len] = 0;
-        s->process( data, 0 );
-        s->finish( false );
+        s->create( req->url, QString("text/plain"), req->notify );
+        kdDebug(1431) << "javascriptResult has been called with: "<<result<<endl;
+        if (len > 0) {
+            QByteArray data(len + 1);
+            memcpy(data.data(), result.latin1(), len);
+            data[len] = 0;
+            s->process(data, 0);
+        } else {
+            len = 7; //  "unknown"
+            QByteArray data(len + 1);
+            memcpy(data.data(), "unknown", len);
+            data[len] = 0;
+            s->process(data, 0);
+        }
+        s->finish(false);
+
         delete req;
     }
 }
@@ -1449,39 +1463,40 @@ bool NSPluginStreamBase::create( const QString& url, const QString& mimeType, vo
 int NSPluginStreamBase::process( const QByteArray &data, int start )
 {
    int32 max, sent, to_sent, len;
-   char *d = data.data()+start;
+   char *d = data.data() + start;
 
-   to_sent = data.size()-start;
-   while (to_sent>0)
+   to_sent = data.size() - start;
+   while (to_sent > 0)
    {
       inform();
 
-      max = _instance->NPWriteReady( _stream );
+      max = _instance->NPWriteReady(_stream);
+      //kdDebug(1431) << "to_sent == " << to_sent << " and max = " << max << endl;
       len = QMIN(max, to_sent);
 
-      kdDebug(1431) << "-> Feeding stream to plugin: offset=" << _pos << ", len=" << len << endl;
+      //kdDebug(1431) << "-> Feeding stream to plugin: offset=" << _pos << ", len=" << len << endl;
       sent = _instance->NPWrite( _stream, _pos, len, d );
-      kdDebug(1431) << "<- Feeding stream: sent = " << sent << endl;
+      //kdDebug(1431) << "<- Feeding stream: sent = " << sent << endl;
 
-      if (sent==0) // interrupt the stream for a few ms
+      if (sent == 0) // interrupt the stream for a few ms
           break;
 
-      if ( sent<0 ) {
+      if (sent < 0) {
           // stream data rejected/error
           kdDebug(1431) << "stream data rejected/error" << endl;
           _error = true;
           break;
       }
 
-      if ( _tempFile )
-          _tempFile->dataStream()->writeRawBytes( d, sent );
+      if (_tempFile)
+          _tempFile->dataStream()->writeRawBytes(d, sent);
 
       to_sent -= sent;
       _pos += sent;
       d += sent;
    }
 
-   return data.size()-to_sent;
+   return data.size() - to_sent;
 }
 
 
@@ -1525,10 +1540,12 @@ void NSPluginStreamBase::queue( const QByteArray &data )
     _queuePos = 0;
     _tries = 0;
 
+/*
     kdDebug(1431) << "new queue size=" << data.size()
                   << " data=" << (void*)data.data() 
                   << " queue=" << (void*)_queue.data() << " qsize="
                   << _queue.size() << endl;
+*/
 }
 
 
@@ -1556,7 +1573,7 @@ void NSPluginStreamBase::finish( bool err )
 
         _instance->NPDestroyStream( _stream, NPRES_DONE );
         if ( _notifyData )
-            _instance->NPURLNotify( _url, NPRES_DONE, _notifyData );
+            _instance->NPURLNotify( _url.url(), NPRES_DONE, _notifyData );
     } else {
         // close temp file
         if ( _tempFile )
@@ -1565,7 +1582,7 @@ void NSPluginStreamBase::finish( bool err )
         // destroy stream
         _instance->NPDestroyStream( _stream, NPRES_NETWORK_ERR );
         if ( _notifyData )
-            _instance->NPURLNotify( _url, NPRES_NETWORK_ERR, _notifyData );
+            _instance->NPURLNotify( _url.url(), NPRES_NETWORK_ERR, _notifyData );
     }
 
     // delete stream
@@ -1648,7 +1665,7 @@ bool NSPluginStream::get( const QString& url, const QString& mimeType,
     // create new stream
     if ( create( url, mimeType, notify ) ) {
         // start the kio job
-        _job = KIO::get(url, false, false);
+        _job = KIO::get(KURL( url ), false, false);
         _job->addMetaData("errorPage", "false");
         _job->addMetaData("AllowCompressedPage", "false");
         connect(_job, SIGNAL(data(KIO::Job *, const QByteArray &)),
@@ -1670,7 +1687,7 @@ bool NSPluginStream::post( const QString& url, const QByteArray& data,
     // create new stream
     if ( create( url, mimeType, notify ) ) {
         // start the kio job
-        _job = KIO::http_post(url, data, false);
+        _job = KIO::http_post(KURL( url ), data, false);
         _job->addMetaData("content-type", args.contentType());
         _job->addMetaData("errorPage", "false");
         _job->addMetaData("AllowCompressedPage", "false");
@@ -1689,7 +1706,7 @@ bool NSPluginStream::post( const QString& url, const QByteArray& data,
 
 void NSPluginStream::data(KIO::Job * job, const QByteArray &data)
 {
-    kdDebug(1431) << "NSPluginStream::data - job=" << (void*)job << " data size=" << data.size() << endl;
+    //kdDebug(1431) << "NSPluginStream::data - job=" << (void*)job << " data size=" << data.size() << endl;
     queue( data );
     if ( !pump() ) {
         _job->suspend();
@@ -1738,3 +1755,4 @@ void NSPluginStream::result(KIO::Job *job)
 }
 
 #include "nsplugin.moc"
+// vim: ts=4 sw=4 et
